@@ -5,7 +5,7 @@
  *
  * Luis Ontanon 2005 - Context and Transaction Tracing
  *
- * $Id: packet-h248-template.c 21990 2007-05-29 18:52:42Z guy $
+ * $Id: packet-h248-template.c 24078 2008-01-13 17:03:06Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -109,6 +109,7 @@ static dissector_handle_t h248_tpkt_handle;
 /* Forward declarations */
 static int dissect_h248_ServiceChangeReasonStr(gboolean implicit_tag, tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index);
 
+/* http://www.iana.org/assignments/megaco-h248 last updated 2007-11-28*/
 static const value_string package_name_vals[] = {
   {   0x0000, "Media stream properties H.248.1 Annex C" },
   {   0x0001, "Generic H.248.1 Annex E" },
@@ -252,6 +253,65 @@ static const value_string package_name_vals[] = {
   {   0x008f, "NAT Traversal Package" },							/* Annex E of ETSI TS 102 333 */
   {   0x0090, "MPLS Package" },										/* Annex F of ETSI TS 102 333 */
   {   0x0091, "VLAN Package" },										/* Annex G of ETSI TS 102 333 */
+  {   0x0092, "Detailed Congestion Reporting Package" },			/* H.248.32 */
+  {   0x0093, "Stimulus Analogue Lines Package" },					/* H.248.34 */
+  {   0x0094, "icascgen" },											/* H.248.29 Annex B */
+  {   0x0095, "Coin Operated Phone Control Package" },				/* H.248.35 */ 
+  {   0x0096, "Metering Pulse Detection Package" },					/* H.248.26 Amendment 1 */
+  {   0x0097, "Trace Package" },									/* 3GPP TS 29.232 v6.3.0 */
+  {   0x0098, "Hanging Termination Package" },						/* H.248.36 */
+  {   0x0099, "IP NAPT Traversal Package" },						/* H.248.37 */
+  {   0x009a, "Notification Behaviour Package" },					/* H.248.1v3 */
+  {   0x009b, "Base Context Package" },								/* H.248.38 */
+  {   0x009c, "Application Data Inactivity Detection Package" },	/* H.248.40 */
+  {   0x009d, "Domain Connection Package " },						/* H.248.41 */
+  {   0x009e, "Digital Circuit Multiplication Equipment Package" }, /* H.248.42 */
+  {   0x009f, "Multi-level Precedence and Pre-emption Package" },	/* H.248.44 */
+  {   0x00a0, "MGC Information Package" },							/* H.248.45 */
+  {   0x00a1, "Text Overlay Package" },								/* H.248.19 Amendment 1 */
+  {   0x00a2, "Border and Background Package" },					/* H.248.19 Amendment 1 */
+  {   0x00a3, "Segmentation Package" },								/* H.248.1v3 */ 
+  {   0x00a4, "ETSI notification behaviour package" },				/* ETSI ES 283 039-3 */
+  {   0x00a5, "ETSI notification rate package" },					/* ETSI ES 283 039-4 */
+  {   0x00a6, "Automatic Speech Recognition Package" },				/* H.248.9 Amendment 1 */ 
+  {   0x00a7, "Set extension to basic syntax for TTS enhancement Package" },/* H.248.9 Amendment 1 */
+  {   0x00a8, "Advanced audio server base package for TTS enhancement" },	/* H.248.9 Amendment 1 */ 
+  {   0x00a9, "Multimedia Play Package" },							/* H.248.9 Amendment 1 */ 
+  {   0x00aa, "Floor Status Detection Package" },					/* H.248.19 Amendment 2 */
+  {   0x00ab, "Floor Control Policy Package" },						/* H.248.19 Amendment 2 */ 
+  {   0x00ac, "Address Reporting Package" },						/* H.248.37 Amendment 1 */ 
+  {   0x00ad, "Connection Capability Control Package" },			/* H.248.46 */ 
+  {   0x00ae, "Statistic Conditional Reporting Package" },			/* H.248.47 Amendment 1 */
+  {   0x00af, "RTCP HR QoS Statistics Package" },					/* H.248.48 */ 
+  {   0x00b0, "Received RTCP XR Package" },							/* H.248.30 (01/2007) */
+  {   0x00b1, "Received RTCP XR Burst Metrics Package" },			/* H.248.30 (01/2007) */
+  {   0x00b2, "ASCI Group call package" },							/* 3GPP TS 29.232 v7.4.0 */
+  {   0x00b3, "Multimedia Recording Package" },						/* H.248.9 Amendment 1 */
+  {   0x00b4, "H.245 Transport Package" },							/* H.248.12 Amendment 2 */
+  {   0x00b5, "RTCP Handling package" },							/* H.248.57 */
+  {   0x00b6, "Gate Management - Outgoing Destination Address/Port Filtering Package" },/* H.248.43 */
+  {   0x00b7, "Gate Management - Incoming Protocol Filtering Package" },/* H.248.43 */
+  {   0x00b8, "Gate Management - Outgoing Protocol Filtering Package" },/* H.248.43 */
+  {   0x00b9, "Gate Management - Incoming Filtering Behaviour Package" },/* H.248.43 */
+  {   0x00ba, "Gate Management - Outgoing Filtering Behaviour Package" },	/* H.248.43 */
+  {   0x00bb, "Session Description Protocol RFC Package" },					/* H.248.49 */
+  {   0x00bc, "Session Description Protocol Capabilities Package" },		/* H.248.49 */
+  {   0x00bd, "NAT Traversal Toolkit - STUN Base Package" },				/* H.248.50 */
+  {   0x00be, "NAT Traversal Toolkit - MG STUN Client Package" },			/* H.248.50 */
+  {   0x00bf, "NAT Traversal Toolkit - MG TURN Client Package" },			/* H.248.50 */
+  {   0x00c0, "NAT Traversal Toolkit - MGC STUN Client Package" },			/* H.248.50 */
+  {   0x00c1, "NAT Traversal Toolkit - STUN Information Package" },				/* H.248.50 */
+  {   0x00c2, "NAT Traversal Toolkit - MG Act-as STUN Server Package" },			/* H.248.50 */
+  {   0x00c3, "NAT Traversal Toolkit - Originate STUN Continuity Check Package" },	/* H.248.50 */
+  {   0x00c4, "NAT Traversal Toolkit - MGC Originated STUN Request Package" },		/* H.248.50 */
+  {   0x00c5, "NAT Traversal Toolkit - RTP NOOP Request Package" },					/* H.248.50 */
+  {   0x00c6, "Termination Connection Model Package" },					/* H.248.51 */
+  {   0x00c7, "QoS Class Package" },									/* H.248.52 */
+  {   0x00c8, "Traffic Policing Statistics Package" },					/* H.248.53 */
+  {   0x00c9, "Packet Size Package" },									/* H.248.53 */
+  {   0x00ca, "Pull Mode Package" },									/* H.248.55 */
+  {   0x00cb, "RTP Application Data Package" },							/* H.248.58 */
+  {   0x00cc, "Event Timestamp Notification Package" },					/* H.248.59 */
   {   0x8000, "Ericsson IU" },
   {   0x8001, "Ericsson UMTS and GSM Circuit" },
   {   0x8002, "Ericsson Tone Generator Package" },
@@ -263,6 +323,10 @@ static const value_string package_name_vals[] = {
   {   0x8008, "Ericsson  V5.2 Layer" },
   {   0x8009, "Ericsson Detailed Termination Information Package" },
   {   0x800a, "Nokia Bearer Characteristics Package" },
+  {   0x800b, "Nokia Test Call Package" },
+  {   0x800c, "Nokia Extended Continuity Package" },
+  {   0x800d, "Nokia IPnwR Package" },
+  {   0x800e, "Ericsson Tracing Enhancements Package" },
 	{0,     NULL}
 };
 /*
@@ -534,7 +598,7 @@ extern void h248_param_ber_octetstring(proto_tree* tree, tvbuff_t* tvb, packet_i
 extern void h248_param_ber_boolean(proto_tree* tree, tvbuff_t* tvb, packet_info* pinfo, int hfid, h248_curr_info_t* u _U_, void* implicit) {
 	asn1_ctx_t asn1_ctx;
 	asn1_ctx_init(&asn1_ctx, ASN1_ENC_BER, TRUE, pinfo);
-	dissect_ber_boolean(implicit ? *((gboolean*)implicit) : FALSE, &asn1_ctx, tree, tvb, 0, hfid);
+	dissect_ber_boolean(implicit ? *((gboolean*)implicit) : FALSE, &asn1_ctx, tree, tvb, 0, hfid, NULL);
 }
 
 extern void h248_param_item(proto_tree* tree,
@@ -579,13 +643,13 @@ extern void h248_param_PkgdName(proto_tree* tree, tvbuff_t* tvb, packet_info* pi
     name_minor=tvb_get_ntohs(new_tvb, 2);
 
     /* do the prettification */
-    proto_item_append_text(ber_last_created_item, "  %s (%04x)", val_to_str(name_major, package_name_vals, "Unknown Package"), name_major);
+    proto_item_append_text(asn1_ctx.created_item, "  %s (%04x)", val_to_str(name_major, package_name_vals, "Unknown Package"), name_major);
 
     if(tree){
 	proto_item* pi;
 	const gchar* strval;
 	    
-	package_tree = proto_item_add_subtree(ber_last_created_item, ett_packagename);
+	package_tree = proto_item_add_subtree(asn1_ctx.created_item, ett_packagename);
 	proto_tree_add_uint(package_tree, hf_h248_pkg_name, tvb, offset-4, 2, name_major);
 
 	for(i=0; i < packages->len; i++) {
@@ -731,10 +795,10 @@ static int dissect_h248_PkgdName(gboolean implicit_tag, tvbuff_t *tvb, int offse
     packageandid=(name_major<<16)|name_minor;
 
     /* do the prettification */
-    proto_item_append_text(ber_last_created_item, "  %s (%04x)", val_to_str(name_major, package_name_vals, "Unknown Package"), name_major);
+    proto_item_append_text(actx->created_item, "  %s (%04x)", val_to_str(name_major, package_name_vals, "Unknown Package"), name_major);
 
     if(tree){
-		package_tree = proto_item_add_subtree(ber_last_created_item, ett_packagename);
+		package_tree = proto_item_add_subtree(actx->created_item, ett_packagename);
 		proto_tree_add_uint(package_tree, hf_h248_pkg_name, tvb, offset-4, 2, name_major);
     }
 
@@ -790,9 +854,9 @@ static int dissect_h248_EventName(gboolean implicit_tag, tvbuff_t *tvb, int offs
     packageandid=(name_major<<16)|name_minor;
 
     /* do the prettification */
-    proto_item_append_text(ber_last_created_item, "  %s (%04x)", val_to_str(name_major, package_name_vals, "Unknown Package"), name_major);
+    proto_item_append_text(actx->created_item, "  %s (%04x)", val_to_str(name_major, package_name_vals, "Unknown Package"), name_major);
     if(tree){
-      package_tree = proto_item_add_subtree(ber_last_created_item, ett_packagename);
+      package_tree = proto_item_add_subtree(actx->created_item, ett_packagename);
     }
     proto_tree_add_uint(package_tree, hf_h248_pkg_name, tvb, offset-4, 2, name_major);
 
@@ -867,9 +931,9 @@ static int dissect_h248_SignalName(gboolean implicit_tag , tvbuff_t *tvb, int of
     packageandid=(name_major<<16)|name_minor;
 
     /* do the prettification */
-    proto_item_append_text(ber_last_created_item, "  %s (%04x)", val_to_str(name_major, package_name_vals, "Unknown Package"), name_major);
+    proto_item_append_text(actx->created_item, "  %s (%04x)", val_to_str(name_major, package_name_vals, "Unknown Package"), name_major);
     if(tree){
-      package_tree = proto_item_add_subtree(ber_last_created_item, ett_packagename);
+      package_tree = proto_item_add_subtree(actx->created_item, ett_packagename);
     }
     proto_tree_add_uint(package_tree, hf_h248_pkg_name, tvb, offset-4, 2, name_major);
 
@@ -980,7 +1044,7 @@ static int dissect_h248_SigParameterName(gboolean implicit_tag _U_, tvbuff_t *tv
 	proto_item* pi;
 	
 	offset = dissect_ber_octet_string(implicit_tag, actx, tree, tvb, offset,  hf_index, &next_tvb);
-	pi = get_ber_last_created_item();
+	pi = actx->created_item;
 	
 	switch(tvb_length(next_tvb)) {
 		case 4: param_id = tvb_get_ntohl(next_tvb,0); break;
@@ -1049,7 +1113,7 @@ static int dissect_h248_EventParameterName(gboolean implicit_tag _U_, tvbuff_t *
 	proto_item* pi;
 
 	offset = dissect_ber_octet_string(implicit_tag, actx, tree, tvb, offset, hf_index, &next_tvb);
-	pi = get_ber_last_created_item();
+	pi = actx->created_item;
 
 	if (next_tvb) {
 		switch(tvb_length(next_tvb)) {
@@ -1134,9 +1198,9 @@ static int dissect_h248_MtpAddress(gboolean implicit_tag, tvbuff_t *tvb, int off
     }
 
     /* do the prettification */
-    proto_item_append_text(ber_last_created_item, "  NI = %d, PC = %d ( %d-%d )", val&0x03,val>>2,val&0x03,val>>2);
+    proto_item_append_text(actx->created_item, "  NI = %d, PC = %d ( %d-%d )", val&0x03,val>>2,val&0x03,val>>2);
     if(tree){
-      mtp_tree = proto_item_add_subtree(ber_last_created_item, ett_mtpaddress);
+      mtp_tree = proto_item_add_subtree(actx->created_item, ett_mtpaddress);
     }
     proto_tree_add_uint(mtp_tree, hf_h248_mtpaddress_ni, tvb, old_offset, offset-old_offset, val&0x03);
     proto_tree_add_uint(mtp_tree, hf_h248_mtpaddress_pc, tvb, old_offset, offset-old_offset, val>>2);
