@@ -3,7 +3,7 @@
  *
  * See RFC 1777 (LDAP v2), RFC 2251 (LDAP v3), and RFC 2222 (SASL).
  *
- * $Id: packet-ldap-template.c 19818 2006-11-06 11:41:02Z jake $
+ * $Id: packet-ldap-template.c 24664 2008-03-17 12:03:36Z morriss $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -82,15 +82,17 @@
 #include <epan/prefs.h>
 #include <epan/tap.h>
 #include <epan/emem.h>
-#include <epan/oid_resolv.h>
+#include <epan/oids.h>
 #include <epan/strutil.h>
 #include <epan/dissectors/packet-tcp.h>
 #include <epan/dissectors/packet-windows-common.h>
 #include <epan/dissectors/packet-dcerpc.h>
+#include <epan/asn1.h>
 
 #include "packet-frame.h"
 #include "packet-ldap.h"
 #include "packet-ntlmssp.h"
+#include "packet-ssl.h"
 
 #include "packet-ber.h"
 #include "packet-per.h"
@@ -159,21 +161,31 @@ static gint ett_mscldap_netlogon_flags = -1;
 static dissector_table_t ldap_name_dissector_table=NULL;
 static const char *object_identifier_id = NULL; /* LDAP OID */
 
-/* desegmentation of LDAP */
-static gboolean ldap_desegment = TRUE;
-static guint    ldap_tcp_port = 389;
-
 static gboolean do_protocolop = FALSE;
 static gchar    *attr_type = NULL;
 static gboolean is_binary_attr_type = FALSE;
+static guint32 last_frame_seen = 0;
 
 #define TCP_PORT_LDAP			389
+#define TCP_PORT_LDAPS			636
 #define UDP_PORT_CLDAP			389
 #define TCP_PORT_GLOBALCAT_LDAP         3268 /* Windows 2000 Global Catalog */
 
-static dissector_handle_t gssapi_handle;
-static dissector_handle_t gssapi_wrap_handle;
+/* desegmentation of LDAP */
+static gboolean ldap_desegment = TRUE;
+static guint global_ldap_tcp_port = TCP_PORT_LDAP;
+static guint global_ldaps_tcp_port = TCP_PORT_LDAPS;
+static guint tcp_port = 0;
+static guint ssl_port = 0;
+
+static dissector_handle_t gssapi_handle = NULL;
+static dissector_handle_t gssapi_wrap_handle = NULL;
 static dissector_handle_t ntlmssp_handle = NULL;
+static dissector_handle_t spnego_handle = NULL;
+static dissector_handle_t ssl_handle = NULL;
+static dissector_handle_t ldap_handle = NULL;
+
+void prefs_register_ldap(void); /* forward declaration for use in preferences registration */
 
 
 /* different types of rpc calls ontop of ms cldap */
@@ -218,6 +230,8 @@ typedef struct ldap_conv_info_t {
   GHashTable *matched;
   gboolean is_mscldap;
   guint32  num_results;
+  gboolean start_tls_pending;
+  guint32  start_tls_frame;
 } ldap_conv_info_t;
 static ldap_conv_info_t *ldap_info_items;
 
@@ -274,7 +288,7 @@ static char *ldapvalue_string=NULL;
  * display it as a string, othervise just display it in hex.
  */
 static int
-dissect_ldap_AssertionValue(gboolean implicit_tag, tvbuff_t *tvb, int offset, packet_info *pinfo _U_, proto_tree *tree, int hf_index)
+dissect_ldap_AssertionValue(gboolean implicit_tag, tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index)
 {
 	gint8 class;
 	gboolean pc, ind, is_ascii;
@@ -284,7 +298,7 @@ dissect_ldap_AssertionValue(gboolean implicit_tag, tvbuff_t *tvb, int offset, pa
 
 	if(!implicit_tag){
 		offset=get_ber_identifier(tvb, offset, &class, &pc, &tag);
-		offset=get_ber_length(NULL, tvb, offset, &len, &ind);
+		offset=get_ber_length(tvb, offset, &len, &ind);
 	} else {
 		len=tvb_length_remaining(tvb,offset);
 	}
@@ -318,7 +332,7 @@ dissect_ldap_AssertionValue(gboolean implicit_tag, tvbuff_t *tvb, int offset, pa
 		e_uuid_t uuid;
 
 		/* This octet string contained a GUID */
-		dissect_dcerpc_uuid_t(tvb, offset, pinfo, tree, drep, hf_ldap_guid, &uuid);
+		dissect_dcerpc_uuid_t(tvb, offset, actx->pinfo, tree, drep, hf_ldap_guid, &uuid);
 
 		ldapvalue_string=ep_alloc(1024);
 		g_snprintf(ldapvalue_string, 1023, "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
@@ -428,6 +442,7 @@ ldap_match_call_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gu
         case LDAP_REQ_DELETE:
         case LDAP_REQ_MODRDN:
         case LDAP_REQ_COMPARE:
+        case LDAP_REQ_EXTENDED:
           lcr.is_request=TRUE;
           lcr.req_frame=pinfo->fd->num;
           lcr.rep_frame=0;
@@ -441,6 +456,7 @@ ldap_match_call_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gu
         case LDAP_RES_DELETE:
         case LDAP_RES_MODRDN:
         case LDAP_RES_COMPARE:
+        case LDAP_RES_EXTENDED:
           lcr.is_request=FALSE;
           lcr.req_frame=0;
           lcr.rep_frame=pinfo->fd->num;
@@ -464,6 +480,7 @@ ldap_match_call_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gu
       case LDAP_REQ_DELETE:
       case LDAP_REQ_MODRDN:
       case LDAP_REQ_COMPARE:
+      case LDAP_REQ_EXTENDED:
 
 		/* this a a request - add it to the unmatched list */
 
@@ -497,6 +514,7 @@ ldap_match_call_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gu
       case LDAP_RES_DELETE:
       case LDAP_RES_MODRDN:
       case LDAP_RES_COMPARE:
+      case LDAP_RES_EXTENDED:
 
 		/* this is a result - it should be in our unmatched list */
 
@@ -543,7 +561,7 @@ ldap_match_call_response(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gu
 static void
 dissect_ldap_payload(tvbuff_t *tvb, packet_info *pinfo,
 		     proto_tree *tree, ldap_conv_info_t *ldap_info,
-		     gboolean rest_is_pad, gboolean is_mscldap)
+		     gboolean is_mscldap)
 {
   int offset = 0;
   guint length_remaining;
@@ -556,18 +574,24 @@ dissect_ldap_payload(tvbuff_t *tvb, packet_info *pinfo,
   gboolean pc, ind = 0;
   gint32 ber_tag;
 
+
+one_more_pdu:
+
     length_remaining = tvb_ensure_length_remaining(tvb, offset);
 
-    if (rest_is_pad && length_remaining < 6) return;
+    if (length_remaining < 6) return;
 
     /*
      * OK, try to read the "Sequence Of" header; this gets the total
      * length of the LDAP message.
      */
 	messageOffset = get_ber_identifier(tvb, offset, &class, &pc, &ber_tag);
-	messageOffset = get_ber_length(tree, tvb, messageOffset, &msg_len, &ind);
+	messageOffset = get_ber_length(tvb, messageOffset, &msg_len, &ind);
 
-    if (ber_tag == BER_UNI_TAG_SEQUENCE) {
+    /* sanity check */
+    if((msg_len<4) || (msg_len>10000000)) return;
+
+    if ( (class==BER_CLASS_UNI) && (ber_tag==BER_UNI_TAG_SEQUENCE) ) {
       	/*
       	 * Add the length of the "Sequence Of" header to the message
       	 * length.
@@ -615,14 +639,22 @@ dissect_ldap_payload(tvbuff_t *tvb, packet_info *pinfo,
     /*
      * Now dissect the LDAP message.
      */
-
-	ldap_info->is_mscldap = is_mscldap;
-	pinfo->private_data = ldap_info;
-	dissect_LDAPMessage_PDU(msg_tvb, pinfo, tree);
-
+    ldap_info->is_mscldap = is_mscldap;
+    pinfo->private_data = ldap_info;
+    dissect_LDAPMessage_PDU(msg_tvb, pinfo, tree);
 
     offset += msg_len;
 
+    /* If this was a sasl blob there might be another PDU following in the
+     * same blob
+     */
+    if(tvb_length_remaining(tvb, offset)>=6){
+        tvb = tvb_new_subset(tvb, offset, tvb_length_remaining(tvb, offset), -1);
+	offset = 0;
+
+        goto one_more_pdu;
+    }
+
 }
 
 static void
@@ -660,13 +692,15 @@ dissect_ldap_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean i
     /* No.  Attach that information to the conversation, and add
      * it to the list of information structures.
      */
-    ldap_info = se_alloc(sizeof(ldap_conv_info_t));
+    ldap_info = g_malloc(sizeof(ldap_conv_info_t));
     ldap_info->auth_type = 0;
     ldap_info->auth_mech = 0;
     ldap_info->first_auth_frame = 0;
     ldap_info->matched=g_hash_table_new(ldap_info_hash_matched, ldap_info_equal_matched);
     ldap_info->unmatched=g_hash_table_new(ldap_info_hash_unmatched, ldap_info_equal_unmatched);
     ldap_info->num_results = 0;
+    ldap_info->start_tls_frame = 0;
+    ldap_info->start_tls_pending = FALSE;
 
     conversation_add_proto_data(conversation, proto_ldap, ldap_info);
 
@@ -674,7 +708,7 @@ dissect_ldap_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean i
     ldap_info_items = ldap_info;
 
   }
-
+  
   switch (ldap_info->auth_type) {
     case LDAP_AUTH_SASL:
     /*
@@ -707,7 +741,17 @@ dissect_ldap_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean i
      * Info column.
      */
     if (check_col(pinfo->cinfo, COL_PROTOCOL)) col_set_str(pinfo->cinfo, COL_PROTOCOL, pinfo->current_proto);
-    if (check_col(pinfo->cinfo, COL_INFO)) col_clear(pinfo->cinfo, COL_INFO);
+
+    if(last_frame_seen == pinfo->fd->num) {
+      /* we have already dissected an ldap PDU in this frame - add a separator and set a fence */
+      if (check_col(pinfo->cinfo, COL_INFO)) {
+	col_append_str(pinfo->cinfo, COL_INFO, "| ");
+	col_set_fence(pinfo->cinfo, COL_INFO);
+      }
+    } else
+      if (check_col(pinfo->cinfo, COL_INFO)) col_clear(pinfo->cinfo, COL_INFO);
+
+    last_frame_seen = pinfo->fd->num;
 
     ldap_item = proto_tree_add_item(tree, is_mscldap?proto_cldap:proto_ldap, tvb, 0, -1, FALSE);
     ldap_tree = proto_item_add_subtree(ldap_item, ett_ldap);
@@ -781,12 +825,14 @@ dissect_ldap_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean i
         proto_tree_add_uint(ldap_tree, hf_ldap_sasl_buffer_length, sasl_tvb, 0, 4,
                             sasl_len);
 
-        sasl_item = proto_tree_add_text(ldap_tree, sasl_tvb, 0,  sasl_msg_len, "SASL buffer");
+        sasl_item = proto_tree_add_text(ldap_tree, sasl_tvb, 0,  sasl_msg_len, "SASL Buffer");
         sasl_tree = proto_item_add_subtree(sasl_item, ett_ldap_sasl_blob);
       }
 
       if (ldap_info->auth_mech != NULL &&
-          strcmp(ldap_info->auth_mech, "GSS-SPNEGO") == 0) {
+          ((strcmp(ldap_info->auth_mech, "GSS-SPNEGO") == 0) ||
+	   /* auth_mech may have been set from the bind */
+	   (strcmp(ldap_info->auth_mech, "GSSAPI") == 0))) {
 	  tvbuff_t *gssapi_tvb, *plain_tvb = NULL, *decr_tvb= NULL;
 	  int ver_len;
 	  int length;
@@ -847,6 +893,10 @@ dissect_ldap_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean i
              * The LDAP message was encrypted in the packet, and has
              * been decrypted; dissect the decrypted LDAP message.
              */
+            if (check_col(pinfo->cinfo, COL_INFO)) {
+				col_set_str(pinfo->cinfo, COL_INFO, "SASL GSS-API Privacy (decrypted): ");
+            }
+
             if (sasl_tree) {
 	      enc_item = proto_tree_add_text(sasl_tree, gssapi_tvb, ver_len, -1,
                                 "GSS-API Encrypted payload (%d byte%s)",
@@ -854,7 +904,7 @@ dissect_ldap_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean i
                                 plurality(sasl_len - ver_len, "", "s"));
 	      enc_tree = proto_item_add_subtree(enc_item, ett_ldap_payload);
             }
-	    dissect_ldap_payload(decr_tvb, pinfo, enc_tree, ldap_info, TRUE, is_mscldap);
+	    dissect_ldap_payload(decr_tvb, pinfo, enc_tree, ldap_info, is_mscldap);
           } else if (plain_tvb) {
 	    proto_item *plain_item = NULL;
 	    proto_tree *plain_tree = NULL;
@@ -863,6 +913,10 @@ dissect_ldap_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean i
 	     * The LDAP message wasn't encrypted in the packet;
 	     * dissect the plain LDAP message.
              */
+            if (check_col(pinfo->cinfo, COL_INFO)) {
+				col_set_str(pinfo->cinfo, COL_INFO, "SASL GSS-API Integrity: ");
+            }
+
 	    if (sasl_tree) {
               plain_item = proto_tree_add_text(sasl_tree, gssapi_tvb, ver_len, -1,
                                 "GSS-API payload (%d byte%s)",
@@ -871,14 +925,14 @@ dissect_ldap_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean i
 	      plain_tree = proto_item_add_subtree(plain_item, ett_ldap_payload);
             }
 
-           dissect_ldap_payload(plain_tvb, pinfo, plain_tree, ldap_info, TRUE, is_mscldap);
+           dissect_ldap_payload(plain_tvb, pinfo, plain_tree, ldap_info, is_mscldap);
 	  } else {
             /*
              * The LDAP message was encrypted in the packet, and was
              * not decrypted; just show it as encrypted data.
              */
             if (check_col(pinfo->cinfo, COL_INFO)) {
-        	    col_add_fstr(pinfo->cinfo, COL_INFO, "LDAP GSS-API Encrypted payload (%d byte%s)",
+        	    col_add_fstr(pinfo->cinfo, COL_INFO, "SASL GSS-API Privacy: payload (%d byte%s)",
                                  sasl_len - ver_len,
                                  plurality(sasl_len - ver_len, "", "s"));
             }
@@ -893,7 +947,7 @@ dissect_ldap_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, gboolean i
       offset += sasl_msg_len;
     } else {
 	/* plain LDAP, so dissect the payload */
-	dissect_ldap_payload(tvb, pinfo, ldap_tree, ldap_info, FALSE, is_mscldap);
+	dissect_ldap_payload(tvb, pinfo, ldap_tree, ldap_info, is_mscldap);
     }
 }
 
@@ -1051,7 +1105,7 @@ static int dissect_mscldap_netlogon_flags(proto_tree *parent_tree, tvbuff_t *tvb
   return offset;
 }
 
-static void dissect_NetLogon_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+static void dissect_NetLogon_PDU(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
 {
   int old_offset, offset=0;
   char str[256];
@@ -1152,7 +1206,7 @@ get_normal_ldap_pdu_len(packet_info *pinfo _U_, tvbuff_t *tvb, int offset)
 	 * offset is where the tag is
 	 * offset+1 is where length starts
 	 */
-	data_offset=get_ber_length(NULL, tvb, offset+1, &len, &ind);
+	data_offset=get_ber_length(tvb, offset+1, &len, &ind);
 	return len+data_offset-offset;
 }
 
@@ -1164,7 +1218,7 @@ dissect_normal_ldap_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 }
 
 static void
-dissect_ldap_oid(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+dissect_ldap_oid(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
 {
 	char *oid;
 	const char *oidname;
@@ -1179,7 +1233,7 @@ dissect_ldap_oid(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		return;
 	}
 
-	oidname=get_oid_str_name(oid);
+	oidname=oid_resolved_from_string(oid);
 
 	if(oidname){
 		proto_tree_add_text(tree, tvb, 0, tvb_length(tvb), "OID: %s (%s)",oid,oidname);
@@ -1270,7 +1324,7 @@ dissect_ldap_nt_sec_desc(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 }
 
 static void
-dissect_ldap_sid(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
+dissect_ldap_sid(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
 {
 	char *tmpstr;
 
@@ -1301,16 +1355,31 @@ static void
 dissect_ldap_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 	guint32 sasl_len;
-	guint32 gss_len;
 	guint32 ldap_len;
 	int offset;
 	gboolean ind;
+        conversation_t *conversation;
+	ldap_conv_info_t *ldap_info = NULL;
+ 
+	/*
+	 * Do we have a conversation for this connection?
+	 */
+	conversation = find_conversation(pinfo->fd->num, 
+				&pinfo->src, &pinfo->dst,
+				pinfo->ptype, pinfo->srcport,
+				pinfo->destport, 0);
+	if(conversation){
+		ldap_info = conversation_get_proto_data(conversation, proto_ldap);
+	}
 
         ldm_tree = NULL;
 
 	/* This is a bit tricky. We have to find out whether SASL is used
 	 * so that we know how big a header we are supposed to pass
 	 * to tcp_dissect_pdus()
+	 * We must also cope with the case when a client connects to LDAP
+	 * and performs a few unauthenticated searches of LDAP before
+	 * it performs the bind on the same tcp connection.
 	 */
 	/* check for a SASL header, i.e. assume it is SASL if 
 	 * 1, first four bytes (SASL length) is an integer 
@@ -1319,16 +1388,8 @@ dissect_ldap_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	 *        "random" tcp payload)
 	 * (no SASL ldap PDUs are ever going to be >64k in size?)
 	 *
-	 * Following the SASL header is a GSSAPI blob so the next byte
-	 * is always 0x60. (only true for MS SASL LDAP, there are other
-	 * blobs that may follow in real-world)
-	 *
-	 * 2, Then one byte with the value 0x60 indicating the GSSAPI blob
-	 *
-	 * 3, Then X bytes describing the BER encoded lengtyh of the blob.
-	 *    This length should point to the same end-of-pdu as 1,
-	 *
-	 * 4, finally a byte 0x06 indicating that the next object is an OID
+	 * 2, we must have a conversation and the auth type must
+	 *    be LDAP_AUTH_SASL
 	 */
 	sasl_len=tvb_get_ntohl(tvb, 0);
  
@@ -1336,21 +1397,16 @@ dissect_ldap_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		goto this_was_not_sasl;
 	}
 
-	if(tvb_get_guint8(tvb, 4)!=0x60){
+	if( sasl_len>65535 ){
 		goto this_was_not_sasl;
 	}
 		
-	offset=get_ber_length(NULL, tvb, 5, &gss_len, &ind);
-	if(sasl_len!=(gss_len+offset-4)){
-		goto this_was_not_sasl;
-	}
-
-	if(tvb_get_guint8(tvb, offset)!=0x06){
+	if((!ldap_info) || (ldap_info->auth_type!=LDAP_AUTH_SASL) ){
 		goto this_was_not_sasl;
 	}
 
 	tcp_dissect_pdus(tvb, pinfo, tree, ldap_desegment, 4, get_sasl_ldap_pdu_len, dissect_sasl_ldap_pdu);
-
+	return;
 
 this_was_not_sasl:
 	/* check if it is a normal BER encoded LDAP packet
@@ -1363,7 +1419,7 @@ this_was_not_sasl:
 	}
 
 	/* check that length makes sense */
-	offset=get_ber_length(NULL, tvb, 1, &ldap_len, &ind);
+	offset=get_ber_length(tvb, 1, &ldap_len, &ind);
 
 	/* dont check ind since indefinite length is never used for ldap (famous last words)*/
 	if(ldap_len<2){
@@ -1372,9 +1428,38 @@ this_was_not_sasl:
 
 	tcp_dissect_pdus(tvb, pinfo, tree, ldap_desegment, 4, get_normal_ldap_pdu_len, dissect_normal_ldap_pdu);
 
+	goto end;
 
 this_was_not_normal_ldap:
 
+	/* perhaps it was SSL? */
+	if(ldap_info && 
+	   ldap_info->start_tls_frame && 
+	   ( pinfo->fd->num >= ldap_info->start_tls_frame)) {
+
+	  /* we have started TLS and so this may be an SSL layer */
+	  guint32 old_start_tls_frame;
+
+	  /* temporarily dissect this port as SSL */
+	  dissector_delete("tcp.port", tcp_port, ldap_handle); 
+	  ssl_dissector_add(tcp_port, "ldap", TRUE);
+    
+	  old_start_tls_frame = ldap_info->start_tls_frame;
+	  ldap_info->start_tls_frame = 0; /* make sure we don't call SSL again */
+	  pinfo->can_desegment++; /* ignore this LDAP layer so SSL can use the TCP resegment */
+
+	  offset = call_dissector(ssl_handle, tvb, pinfo, tree);
+
+	  ldap_info->start_tls_frame = old_start_tls_frame;
+	  ssl_dissector_delete(tcp_port, "ldap", TRUE);
+
+	  /* restore ldap as the dissector for this port */
+	  dissector_add("tcp.port", tcp_port, ldap_handle);
+
+	  /* we are done */
+	  return;
+	}
+ end:
 	return;
 }
 
@@ -1392,7 +1477,9 @@ ldap_reinit(void)
   ldap_conv_info_t *ldap_info;
 
   /* Free up state attached to the ldap_info structures */
-  for (ldap_info = ldap_info_items; ldap_info != NULL; ldap_info = ldap_info->next) {
+  for (ldap_info = ldap_info_items; ldap_info != NULL; ) {
+    ldap_conv_info_t *last;
+
     if (ldap_info->auth_mech != NULL) {
       g_free(ldap_info->auth_mech);
       ldap_info->auth_mech=NULL;
@@ -1401,9 +1488,14 @@ ldap_reinit(void)
     ldap_info->matched=NULL;
     g_hash_table_destroy(ldap_info->unmatched);
     ldap_info->unmatched=NULL;
+
+    last = ldap_info;
+    ldap_info = ldap_info->next;
+    g_free(ldap_info);
   }
 
   ldap_info_items = NULL;
+  last_frame_seen = 0;
 
 }
 
@@ -1618,7 +1710,7 @@ void proto_register_ldap(void) {
 
   register_dissector("ldap", dissect_ldap_tcp, proto_ldap);
 
-  ldap_module = prefs_register_protocol(proto_ldap, NULL);
+  ldap_module = prefs_register_protocol(proto_ldap, prefs_register_ldap);
   prefs_register_bool_preference(ldap_module, "desegment_ldap_messages",
     "Reassemble LDAP messages spanning multiple TCP segments",
     "Whether the LDAP dissector should reassemble messages spanning multiple TCP segments."
@@ -1627,7 +1719,11 @@ void proto_register_ldap(void) {
 
   prefs_register_uint_preference(ldap_module, "tcp.port", "LDAP TCP Port",
 				 "Set the port for LDAP operations",
-				 10, &ldap_tcp_port);
+				 10, &global_ldap_tcp_port);
+
+  prefs_register_uint_preference(ldap_module, "ssl.port", "LDAPS TCP Port",
+				 "Set the port for LDAP operations over SSL",
+				 10, &global_ldaps_tcp_port);
 
   prefs_register_obsolete_preference(ldap_module, "max_pdu");
 
@@ -1647,48 +1743,62 @@ void proto_register_ldap(void) {
 void
 proto_reg_handoff_ldap(void)
 {
-	dissector_handle_t ldap_handle, cldap_handle;
+	dissector_handle_t cldap_handle;
 	ldap_handle = create_dissector_handle(dissect_ldap_tcp, proto_ldap);
 
-	dissector_add("tcp.port", ldap_tcp_port, ldap_handle);
+	dissector_add("tcp.port", global_ldap_tcp_port, ldap_handle);
 	dissector_add("tcp.port", TCP_PORT_GLOBALCAT_LDAP, ldap_handle);
 
+	ssl_dissector_add(global_ldaps_tcp_port, "ldap", TRUE);
+
 	cldap_handle = create_dissector_handle(dissect_mscldap, proto_cldap);
 	dissector_add("udp.port", UDP_PORT_CLDAP, cldap_handle);
 
 	gssapi_handle = find_dissector("gssapi");
 	gssapi_wrap_handle = find_dissector("gssapi_verf");
+	spnego_handle = find_dissector("spnego");
 
 	ntlmssp_handle = find_dissector("ntlmssp");
 
+	ssl_handle = find_dissector("ssl");
+
 /*  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dsml/dsml/ldap_controls_and_session_support.asp */
-	add_oid_str_name("1.2.840.113556.1.4.319","LDAP_PAGED_RESULT_OID_STRING");
-	add_oid_str_name("1.2.840.113556.1.4.417","LDAP_SERVER_SHOW_DELETED_OID");
-	add_oid_str_name("1.2.840.113556.1.4.473","LDAP_SERVER_SORT_OID");
-	add_oid_str_name("1.2.840.113556.1.4.474","LDAP_CONTROL_SORT_RESP_OID");
-	add_oid_str_name("1.2.840.113556.1.4.521","LDAP_SERVER_CROSSDOM_MOVE_TARGET_OID");
-	add_oid_str_name("1.2.840.113556.1.4.528","LDAP_SERVER_NOTIFICATION_OID");
-	add_oid_str_name("1.2.840.113556.1.4.529","LDAP_SERVER_EXTENDED_DN_OID");
-	add_oid_str_name("1.2.840.113556.1.4.619","LDAP_SERVER_LAZY_COMMIT_OID");
-	add_oid_str_name("1.2.840.113556.1.4.800","LDAP_CAP_ACTIVE_DIRECTORY_OID");
-	add_oid_str_name("1.2.840.113556.1.4.801","LDAP_SERVER_SD_FLAGS_OID");
-	add_oid_str_name("1.2.840.113556.1.4.804","LDAP_OID_COMPARATOR_OR");
-	add_oid_str_name("1.2.840.113556.1.4.805","LDAP_SERVER_TREE_DELETE_OID");
-	add_oid_str_name("1.2.840.113556.1.4.841","LDAP_SERVER_DIRSYNC_OID");
-	add_oid_str_name("1.2.840.113556.1.4.970 ","None");
-	add_oid_str_name("1.2.840.113556.1.4.1338","LDAP_SERVER_VERIFY_NAME_OID");
-	add_oid_str_name("1.2.840.113556.1.4.1339","LDAP_SERVER_DOMAIN_SCOPE_OID");
-	add_oid_str_name("1.2.840.113556.1.4.1340","LDAP_SERVER_SEARCH_OPTIONS_OID");
-	add_oid_str_name("1.2.840.113556.1.4.1413","LDAP_SERVER_PERMISSIVE_MODIFY_OID");
-	add_oid_str_name("1.2.840.113556.1.4.1504","LDAP_SERVER_ASQ_OID");
-	add_oid_str_name("1.2.840.113556.1.4.1670","LDAP_CAP_ACTIVE_DIRECTORY_V51_OID");
-	add_oid_str_name("1.2.840.113556.1.4.1781","LDAP_SERVER_FAST_BIND_OID");
-	add_oid_str_name("1.2.840.113556.1.4.1791","LDAP_CAP_ACTIVE_DIRECTORY_LDAP_INTEG_OID");
-	add_oid_str_name("1.2.840.113556.1.4.1851","LDAP_CAP_ACTIVE_DIRECTORY_ADAM_OID");
-	add_oid_str_name("1.3.6.1.4.1.1466.101.119.1","None");
-	add_oid_str_name("1.3.6.1.4.1.1466.20037","LDAP_START_TLS_OID");
-	add_oid_str_name("2.16.840.1.113730.3.4.9","LDAP_CONTROL_VLVREQUEST VLV");
-	add_oid_str_name("2.16.840.1.113730.3.4.10","LDAP_CONTROL_VLVRESPONSE VLV");
+	oid_add_from_string("LDAP_PAGED_RESULT_OID_STRING","1.2.840.113556.1.4.319");
+	oid_add_from_string("LDAP_SERVER_SHOW_DELETED_OID","1.2.840.113556.1.4.417");
+	oid_add_from_string("LDAP_SERVER_SORT_OID","1.2.840.113556.1.4.473");
+	oid_add_from_string("LDAP_CONTROL_SORT_RESP_OID","1.2.840.113556.1.4.474");
+	oid_add_from_string("LDAP_SERVER_CROSSDOM_MOVE_TARGET_OID","1.2.840.113556.1.4.521");
+	oid_add_from_string("LDAP_SERVER_NOTIFICATION_OID","1.2.840.113556.1.4.528");
+	oid_add_from_string("LDAP_SERVER_EXTENDED_DN_OID","1.2.840.113556.1.4.529");
+	oid_add_from_string("meetingAdvertiseScope","1.2.840.113556.1.4.582");
+	oid_add_from_string("LDAP_SERVER_LAZY_COMMIT_OID","1.2.840.113556.1.4.619");
+	oid_add_from_string("mhsORAddress","1.2.840.113556.1.4.650");
+	oid_add_from_string("managedObjects","1.2.840.113556.1.4.654");
+	oid_add_from_string("LDAP_CAP_ACTIVE_DIRECTORY_OID","1.2.840.113556.1.4.800");
+	oid_add_from_string("LDAP_SERVER_SD_FLAGS_OID","1.2.840.113556.1.4.801");
+	oid_add_from_string("LDAP_OID_COMPARATOR_OR","1.2.840.113556.1.4.804");
+	oid_add_from_string("LDAP_SERVER_TREE_DELETE_OID","1.2.840.113556.1.4.805");
+	oid_add_from_string("LDAP_SERVER_DIRSYNC_OID","1.2.840.113556.1.4.841");
+	oid_add_from_string("None","1.2.840.113556.1.4.970");
+	oid_add_from_string("LDAP_SERVER_VERIFY_NAME_OID","1.2.840.113556.1.4.1338");
+	oid_add_from_string("LDAP_SERVER_DOMAIN_SCOPE_OID","1.2.840.113556.1.4.1339");
+	oid_add_from_string("LDAP_SERVER_SEARCH_OPTIONS_OID","1.2.840.113556.1.4.1340");
+	oid_add_from_string("LDAP_SERVER_PERMISSIVE_MODIFY_OID","1.2.840.113556.1.4.1413");
+	oid_add_from_string("LDAP_SERVER_ASQ_OID","1.2.840.113556.1.4.1504");
+	oid_add_from_string("LDAP_CAP_ACTIVE_DIRECTORY_V51_OID","1.2.840.113556.1.4.1670");
+	oid_add_from_string("LDAP_SERVER_FAST_BIND_OID","1.2.840.113556.1.4.1781");
+	oid_add_from_string("LDAP_CAP_ACTIVE_DIRECTORY_LDAP_INTEG_OID","1.2.840.113556.1.4.1791");
+	oid_add_from_string("msDS-ObjectReference","1.2.840.113556.1.4.1840");
+	oid_add_from_string("msDS-QuotaEffective","1.2.840.113556.1.4.1848");
+	oid_add_from_string("LDAP_CAP_ACTIVE_DIRECTORY_ADAM_OID","1.2.840.113556.1.4.1851");
+	oid_add_from_string("msDS-PortSSL","1.2.840.113556.1.4.1860");
+	oid_add_from_string("msDS-isRODC","1.2.840.113556.1.4.1960");
+	oid_add_from_string("msDS-SDReferenceDomain","1.2.840.113556.1.4.1711");
+	oid_add_from_string("msDS-AdditionalDnsHostName","1.2.840.113556.1.4.1717");
+	oid_add_from_string("None","1.3.6.1.4.1.1466.101.119.1");
+	oid_add_from_string("LDAP_START_TLS_OID","1.3.6.1.4.1.1466.20037");
+	oid_add_from_string("LDAP_CONTROL_VLVREQUEST VLV","2.16.840.1.113730.3.4.9");
+	oid_add_from_string("LDAP_CONTROL_VLVRESPONSE VLV","2.16.840.1.113730.3.4.10");
 
 	register_ldap_name_dissector("netlogon", dissect_NetLogon_PDU, proto_cldap);
 	register_ldap_name_dissector("objectGUID", dissect_ldap_guid, proto_ldap);
@@ -1702,4 +1812,29 @@ proto_reg_handoff_ldap(void)
 
 }
 
+void prefs_register_ldap(void) {
 
+  if(tcp_port != global_ldap_tcp_port) {
+    if(tcp_port)
+      dissector_delete("tcp.port", tcp_port, ldap_handle);
+
+    /* Set our port number for future use */
+    tcp_port = global_ldap_tcp_port;
+
+    if(tcp_port) 
+      dissector_add("tcp.port", tcp_port, ldap_handle);
+
+  }
+
+  if(ssl_port != global_ldaps_tcp_port) {
+    if(ssl_port)
+      ssl_dissector_delete(ssl_port, "ldap", TRUE);
+
+    /* Set our port number for future use */
+    ssl_port = global_ldaps_tcp_port;
+
+    if(ssl_port) 
+      ssl_dissector_add(ssl_port, "ldap", TRUE);
+  }
+
+}
