@@ -2,7 +2,7 @@
  * Routines for STANAG 4406 (X.400 Military Message Extensions)  packet dissection
  * Graeme Lunt 2005
  *
- * $Id: packet-s4406-template.c 20414 2007-01-13 12:59:27Z gal $
+ * $Id: packet-s4406-template.c 23252 2007-10-23 05:50:00Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -30,6 +30,7 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include <epan/conversation.h>
+#include <epan/asn1.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -67,6 +68,8 @@ dissect_s4406(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	int offset = 0;
 	proto_item *item=NULL;
 	proto_tree *tree=NULL;
+	asn1_ctx_t asn1_ctx;
+	asn1_ctx_init(&asn1_ctx, ASN1_ENC_BER, TRUE, pinfo);
 
 	if(parent_tree){
 		item = proto_tree_add_item(parent_tree, proto_s4406, tvb, 0, -1, FALSE);
@@ -76,9 +79,9 @@ dissect_s4406(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	if (check_col(pinfo->cinfo, COL_PROTOCOL))
 		col_set_str(pinfo->cinfo, COL_PROTOCOL, "S4406");
 	if (check_col(pinfo->cinfo, COL_INFO))
-	  col_add_str(pinfo->cinfo, COL_INFO, "Military");
+	  col_set_str(pinfo->cinfo, COL_INFO, "Military");
 
-	dissect_s4406_InformationObject(TRUE, tvb, offset, pinfo , tree, -1);
+	dissect_s4406_InformationObject(TRUE, tvb, offset, &asn1_ctx , tree, -1);
 }
 
 
