@@ -1,7 +1,7 @@
 /* capture_sync.c
  * Synchronisation between Wireshark capture parent and child instances
  *
- * $Id: capture_sync.c 24041 2008-01-09 11:50:22Z guy $
+ * $Id: capture_sync.c 24507 2008-02-29 18:48:29Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -291,6 +291,8 @@ sync_pipe_start(capture_options *capture_opts) {
         simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "We don't know where to find dumpcap.");
         return FALSE;
     }
+    
+    g_log(LOG_DOMAIN_CAPTURE, G_LOG_LEVEL_DEBUG, "argv[0]: %s", argv[0]);
 
     argv = sync_pipe_add_arg(argv, &argc, "-i");
     argv = sync_pipe_add_arg(argv, &argc, capture_opts->iface);
@@ -854,10 +856,22 @@ sync_interface_list_open(gchar **msg) {
     argv = sync_pipe_add_arg(argv, &argc, "-D");
     argv = sync_pipe_add_arg(argv, &argc, "-M");
 
-    /* dumpcap should be running in capture child mode (hidden feature) */
+#if 0
+    /* dumpcap should be running in capture child mode (hidden feature)                   */
+    /* XXX: Actually: don't run dumpcap in capture_child_mode.                            */
+    /*     Instead run dumpcap in 'normal' mode so that dumpcap err msgs are sent to      */
+    /*     stderr in normal format and are then sent to whereever our stderr goes.        */
+    /*     Note: Using 'dumpcap -D -M -Z' (capture_child mode) changes only the format of */
+    /*           dumpcap err msgs. That is: dumpcap in capture_child mode outputs err     */
+    /*           msgs to stderr in a special type/len/string format which would then      */
+    /*           currently be sent as is to stderr resulting in garbled output.           */
+    /*     ToDo: Revise this code to be similar to sync_pipe_start so that 'dumpcap -Z'   */
+    /*     special format error messages to stderr are captured and returned to caller    */
+    /*     (eg: so can be processed and displayed in a pop-up box).                       */ 
 #ifndef DEBUG_CHILD
     argv = sync_pipe_add_arg(argv, &argc, "-Z");
     argv = sync_pipe_add_arg(argv, &argc, SIGNAL_PIPE_CTRL_ID_NONE);
+#endif
 #endif
 
     return sync_pipe_run_command(argv, msg);
@@ -870,7 +884,7 @@ sync_interface_list_open(gchar **msg) {
  * exit value.  In either case, msg must be freed with g_free().
  */
 int
-sync_linktype_list_open(gchar *ifname, gchar **msg) {
+sync_linktype_list_open(const gchar *ifname, gchar **msg) {
     int argc;
     const char **argv;
 
@@ -894,12 +908,23 @@ sync_linktype_list_open(gchar *ifname, gchar **msg) {
     argv = sync_pipe_add_arg(argv, &argc, "-L");
     argv = sync_pipe_add_arg(argv, &argc, "-M");
 
-    /* dumpcap should be running in capture child mode (hidden feature) */
+#if 0
+    /* dumpcap should be running in capture child mode (hidden feature)                   */
+    /* XXX: Actually: don't run dumpcap in capture_child_mode.                            */
+    /*     Instead run dumpcap in 'normal' mode so that dumpcap err msgs are sent to      */
+    /*     stderr in normal format and are then sent to whereever our stderr goes.        */
+    /*     Note: Using 'dumpcap -L -M -Z' (capture_child mode) changes only the format of */
+    /*           dumpcap err msgs. That is: dumpcap in capture_child mode outputs err     */
+    /*           msgs to stderr in a special type/len/string format which would then      */
+    /*           currently be sent as is to stderr resulting in garbled output.           */
+    /*     ToDo: Revise this code to be similar to sync_pipe_start so that 'dumpcap -Z'   */
+    /*     special format error messages to stderr are captured and returned to caller    */
+    /*     (eg: so can be processed and displayed in a pop-up box).                       */ 
 #ifndef DEBUG_CHILD
     argv = sync_pipe_add_arg(argv, &argc, "-Z");
     argv = sync_pipe_add_arg(argv, &argc, SIGNAL_PIPE_CTRL_ID_NONE);
 #endif
-
+#endif
     return sync_pipe_run_command(argv, msg);
 }
 
@@ -919,7 +944,7 @@ sync_interface_stats_open(int *read_fd, int *fork_child, gchar **msg) {
         return -1;
     }
 
-    g_log(LOG_DOMAIN_CAPTURE, G_LOG_LEVEL_DEBUG, "sync_linktype_list_open");
+    g_log(LOG_DOMAIN_CAPTURE, G_LOG_LEVEL_DEBUG, "sync_interface_stats_open");
 
     argv = init_pipe_args(&argc);
 
@@ -928,16 +953,27 @@ sync_interface_stats_open(int *read_fd, int *fork_child, gchar **msg) {
         return CANT_RUN_DUMPCAP;
     }
 
-    /* Ask for the linktype list */
+    /* Ask for the interface statistics */
     argv = sync_pipe_add_arg(argv, &argc, "-S");
     argv = sync_pipe_add_arg(argv, &argc, "-M");
 
-    /* dumpcap should be running in capture child mode (hidden feature) */
+#if 0
+    /* dumpcap should be running in capture child mode (hidden feature)                   */
+    /* XXX: Actually: don't run dumpcap in capture_child_mode.                            */
+    /*     Instead run dumpcap in 'normal' mode so that dumpcap err msgs are sent to      */
+    /*     stderr in normal format and are then sent to whereever our stderr goes.        */
+    /*     Note: Using 'dumpcap -S -M -Z' (capture_child mode) changes only the format of */
+    /*           dumpcap err msgs. That is: dumpcap in capture_child mode outputs err     */
+    /*           msgs to stderr in a special type/len/string format which would then      */
+    /*           currently be sent as is to stderr resulting in garbled output.           */
+    /*     ToDo: Revise this code to be similar to sync_pipe_start so that 'dumpcap -Z'   */
+    /*     special format error messages to stderr are captured and returned to caller    */
+    /*     (eg: so can be processed and displayed in a pop-up box).                       */ 
 #ifndef DEBUG_CHILD
     argv = sync_pipe_add_arg(argv, &argc, "-Z");
     argv = sync_pipe_add_arg(argv, &argc, SIGNAL_PIPE_CTRL_ID_NONE);
 #endif
-
+#endif
     return sync_pipe_open_command(argv, read_fd, fork_child, msg);
 }
 
@@ -1105,6 +1141,7 @@ pipe_read_block(int pipe, char *indicator, int len, char *msg) {
         return -1;
     }
 
+    /* XXX If message is "2part", the msg probably won't be sent to debug log correctly */
     g_log(LOG_DOMAIN_CAPTURE, G_LOG_LEVEL_DEBUG,
           "read %d ok indicator: %c len: %u msg: %s", pipe, *indicator,
           len, msg);
