@@ -1,7 +1,7 @@
 /* capture_ui_utils.c
  * Utilities for capture user interfaces
  *
- * $Id: capture_ui_utils.c 22788 2007-09-04 22:13:26Z morriss $
+ * $Id: capture_ui_utils.c 24231 2008-01-31 16:47:05Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -28,6 +28,7 @@
 
 #ifdef HAVE_LIBPCAP
 
+#include <stdio.h>
 #include <string.h>
 #include <glib.h>
 
@@ -225,9 +226,7 @@ build_capture_combo_list(GList *if_list, gboolean do_hide)
 
       /* Is this interface hidden and, if so, should we include it
          anyway? */
-      if (prefs.capture_devices_hide == NULL ||
-	  strstr(prefs.capture_devices_hide, if_info->name) == NULL ||
-	  !do_hide) {
+      if (!prefs_is_capture_device_hidden(if_info->name) || !do_hide) {
 	/* It's not hidden, or it is but we should include it in the list. */
 
 	/* Do we have a user-supplied description? */
@@ -318,11 +317,13 @@ get_if_name(const char *if_text)
         * it'll be followed by a blank if it separates the description
         * and the interface name.  (We don't wire in "rpcap", in case we
         * support other protocols in the same syntax.)
+        * Unfortunately, another colon can be used in "rpcap://host:port/"
+        * before port. Check if colon is followed by digit.
         */
-       if (strncmp(if_name, "://", 3) != 0) {
+       if ((strncmp(if_name, "://", 3) != 0) && !isdigit(if_name[1])) {
          /*
-          * OK, we've found a colon not followed by "//".  Skip blanks
-          * following it.
+          * OK, we've found a colon followed neither by "//" nor by digit.  
+          * Skip blanks following it.
           */
          if_name++;
          while (*if_name == ' ')
