@@ -1,5 +1,5 @@
 /*
- * $Id: dfvm.c 20195 2006-12-22 09:01:12Z sahlberg $
+ * $Id: dfvm.c 23659 2007-11-28 22:44:37Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -36,6 +36,7 @@ dfvm_insn_new(dfvm_opcode_t op)
 	insn->arg1 = NULL;
 	insn->arg2 = NULL;
 	insn->arg3 = NULL;
+	insn->arg4 = NULL;
 	return insn;
 }
 
@@ -68,6 +69,9 @@ dfvm_insn_free(dfvm_insn_t *insn)
 	if (insn->arg3) {
 		dfvm_value_free(insn->arg3);
 	}
+	if (insn->arg4) {
+		dfvm_value_free(insn->arg4);
+	}
 	g_free(insn);
 }
 
@@ -342,8 +346,10 @@ free_register_overhead(dfilter_t* df)
 	int i;
 
 	for (i = 0; i < df->num_registers; i++) {
+		df->attempted_load[i] = FALSE;
 		if (df->registers[i]) {
 			g_list_free(df->registers[i]);
+			df->registers[i] = NULL;
 		}
 	}
 }
@@ -380,7 +386,7 @@ mk_range(dfilter_t *df, int from_reg, int to_reg, drange *drange)
 gboolean
 dfvm_apply(dfilter_t *df, proto_tree *tree)
 {
-	int		i, id, length;
+	int		id, length;
 	gboolean	accum = TRUE;
 	dfvm_insn_t	*insn;
 	dfvm_value_t	*arg1;
@@ -393,13 +399,6 @@ dfvm_apply(dfilter_t *df, proto_tree *tree)
 
 	g_assert(tree);
 
-
-	/* Clear registers */
-	for (i = 0; i < df->num_registers; i++) {
-		df->registers[i] = NULL;
-		df->attempted_load[i] = FALSE;
-	}
-
 	length = df->insns->len;
 
 	for (id = 0; id < length; id++) {
@@ -414,7 +413,7 @@ dfvm_apply(dfilter_t *df, proto_tree *tree)
 				hfinfo = arg1->value.hfinfo;
 				while(hfinfo) {
 					accum = proto_check_for_protocol_or_field(tree,
-							arg1->value.hfinfo->id);
+							hfinfo->id);
 					if (accum) {
 						break;
 					}
@@ -444,11 +443,6 @@ dfvm_apply(dfilter_t *df, proto_tree *tree)
                     &df->registers[arg2->value.numeric]); 
 				break;
 
-			case PUT_FVALUE:
-				accum = put_fvalue(df,
-						arg1->value.fvalue, arg2->value.numeric);
-				break;
-
 			case MK_RANGE:
 				arg3 = insn->arg3;
 				mk_range(df,
@@ -523,6 +517,12 @@ dfvm_apply(dfilter_t *df, proto_tree *tree)
 				}
 				break;
 
+			case PUT_FVALUE:
+#if 0
+				accum = put_fvalue(df,
+						arg1->value.fvalue, arg2->value.numeric);
+				break;
+#endif
 
 			default:
 				g_assert_not_reached();
@@ -533,3 +533,50 @@ dfvm_apply(dfilter_t *df, proto_tree *tree)
 	g_assert_not_reached();
 	return FALSE; /* to appease the compiler */
 }
+
+void
+dfvm_init_const(dfilter_t *df)
+{
+	int		id, length;
+	dfvm_insn_t	*insn;
+	dfvm_value_t	*arg1;
+	dfvm_value_t	*arg2;
+
+	length = df->consts->len;
+
+	for (id = 0; id < length; id++) {
+
+		insn = g_ptr_array_index(df->consts, id);
+		arg1 = insn->arg1;
+		arg2 = insn->arg2;
+
+		switch (insn->op) {
+			case PUT_FVALUE:
+				put_fvalue(df,
+						arg1->value.fvalue, arg2->value.numeric);
+				break;
+			case CHECK_EXISTS:
+			case READ_TREE:
+			case CALL_FUNCTION:
+			case MK_RANGE:
+			case ANY_EQ:
+			case ANY_NE:
+			case ANY_GT:
+			case ANY_GE:
+			case ANY_LT:
+			case ANY_LE:
+			case ANY_BITWISE_AND:
+			case ANY_CONTAINS:
+			case ANY_MATCHES:
+			case NOT:
+			case RETURN:
+			case IF_TRUE_GOTO:
+			case IF_FALSE_GOTO:
+			default:
+				g_assert_not_reached();
+				break;
+		}
+	}
+
+	return;
+}
