@@ -11,7 +11,7 @@
  * Routines for Mobile IP dissection
  * Copyright 2000, Stefan Raab <sraab@cisco.com>
  *
- * $Id: packet-3g-a11.c 18757 2006-07-18 06:15:22Z etxrab $
+ * $Id: packet-3g-a11.c 24047 2008-01-09 20:49:33Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -21,15 +21,18 @@
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *Ref:
+ * http://www.3gpp2.org/Public_html/specs/A.S0009-C_v1.0_070801.pdf
  */
 
 #ifdef HAVE_CONFIG_H
@@ -103,6 +106,7 @@ static int hf_a11_ase_gre_key = -1;
 static int hf_a11_ase_pcf_addr_key = -1;
 
 /* Foward QoS Information */
+static int hf_a11_fqi_length = -1;
 static int hf_a11_fqi_srid = -1;
 static int hf_a11_fqi_flags = -1;
 static int hf_a11_fqi_flowcount = -1;
@@ -116,6 +120,7 @@ static int hf_a11_fqi_granted_qoslen = -1;
 static int hf_a11_fqi_granted_qos = -1;
 
 /* Reverse QoS Information */
+static int hf_a11_rqi_length = -1;
 static int hf_a11_rqi_srid = -1;
 static int hf_a11_rqi_flowcount = -1;
 static int hf_a11_rqi_flowid = -1;
@@ -186,7 +191,7 @@ static const value_string a11_reply_codes[]= {
   {9, "Connection Update"},
 #if 0
   {1, "Reg Accepted, but Simultaneous Bindings Unsupported."},
-  {64, "Reg Deny (FA)- Unspecified Reason"}, 
+  {64, "Reg Deny (FA)- Unspecified Reason"},
   {65, "Reg Deny (FA)- Administratively Prohibited"},
   {66, "Reg Deny (FA)- Insufficient Resources"},
   {67, "Reg Deny (FA)- MN failed Authentication"},
@@ -221,18 +226,24 @@ static const value_string a11_reply_codes[]= {
   {137, "Reg Deny (HA)- Requested Reverse Tunnel Unavailable"},
   {138, "Reg Deny (HA)- Reverse Tunnel is Mandatory and 'T' Bit Not Set"},
   {139, "Reg Deny (HA)- Requested Encapsulation Unavailable"},
+  {140, "Registration Denied - no CID available"},
   {141, "Reg Deny (HA)- unsupported Vendor ID / Application Type in CVSE"},
+  {142, "Registration Denied - nonexistent A10 or IP flow"},
   {0, NULL},
 };
 
 
 static const value_string a11_ack_status[]= {
   {0, "Update Accepted"},
+  {1, "Partial QoS updated"},
   {128, "Update Deny - Unspecified"},
   {131, "Update Deny - Sending Node Failed Authentication"},
   {133, "Update Deny - Registration ID Mismatch"},
   {134, "Update Deny - Poorly Formed Request"},
-  {201, "Update Deny - Session Parameter Not Updated"},
+  {193, "Update Deny - Session Parameter Not Updated"},
+  {253, "Update Denied - QoS profileID not supported"},
+  {254, "Update Denied - insufficient resources"},
+  {255, "Update Denied - handoff in progress"},
   {0, NULL},
 };
 
@@ -321,14 +332,16 @@ static const value_string a11_ext_app[]= {
   {0x0901, "Service Option (Service Option Value)"},
   {0x0A01, "PDSN Enabled Features (Flow Control Enabled)"},
   {0x0A02, "PDSN Enabled Features (Packet Boundary Enabled)"},
+  {0x0A03, "PDSN Enabled Features (GRE Segmentation Enabled)"},
   {0x0B01, "PCF Enabled Features (Short Data Indication Supported)"},
   {0x0B02, "PCF Enabled Features (GRE Segmentation Enabled)"},
   {0x0C01, "Additional Session Info"},
-  {0x0D01, "Forward QoS Information"},
-  {0x0D02, "Reverse QoS Information"},
-  {0x0D03, "Subscriber QoS Profile"},
-  {0x0DFE, "Forward QoS Update Information"},
-  {0x0DFF, "Reverse QoS Update Information"},
+  {0x0D01, "QoS Information (Forward QoS Information)"},
+  {0x0D02, "QoS Information (Reverse QoS Information)"},
+  {0x0D03, "QoS Information (Subscriber QoS Profile)"},
+  {0x0DFE, "QoS Information (Forward QoS Update Information)"},
+  {0x0DFF, "QoS Information (Reverse QoS Update Information)"},
+  {0x0E01, "Header Compression (ROHC Configuration Parameters)"},
   {0, NULL},
 };
 
@@ -342,11 +355,11 @@ static const value_string a11_airlink_types[]= {
 
 #define ATTRIBUTE_NAME_LEN_MAX 128
 #define ATTR_TYPE_NULL 0
-#define ATTR_TYPE_INT 1 
-#define ATTR_TYPE_STR 2 
+#define ATTR_TYPE_INT 1
+#define ATTR_TYPE_STR 2
 #define ATTR_TYPE_IPV4 3
-#define ATTR_TYPE_TYPE 4 
-#define ATTR_TYPE_MSID 5 
+#define ATTR_TYPE_TYPE 4
+#define ATTR_TYPE_MSID 5
 #define A11_MSG_MSID_ELEM_LEN_MAX 8
 #define A11_MSG_MSID_LEN_MAX 15
 
@@ -391,6 +404,11 @@ static const struct radius_attribute attrs[]={
   {"SDB Octet Count (Term.)", 26, 31,  4, ATTR_TYPE_INT},
   {"SDB Octet Count (Orig.)", 26, 32,  4, ATTR_TYPE_INT},
   {"ESN (Integer)",           26, 48,  4, ATTR_TYPE_INT},
+  {"Sublink",	              26, 108,  4, ATTR_TYPE_STR},
+  {"Reverse PDCH RC",	      26, 114,  2, ATTR_TYPE_INT},
+  {"Flow ID Parameter",       26, 144,  4, ATTR_TYPE_INT},
+  {"Granted QoS Parameters",  26, 132,  4, ATTR_TYPE_INT},
+  {"Flow Status",			  26, 145,  4, ATTR_TYPE_INT},
   {"Unknown",                 -1, -1, -1, ATTR_TYPE_NULL},
 };
 #define NUM_ATTR (sizeof(attrs)/sizeof(struct radius_attribute))
@@ -400,7 +418,7 @@ static const struct radius_attribute attrs[]={
 
 /* decode MSID from SSE */
 
-/* MSID is encoded in Binary Coded Decimal format 
+/* MSID is encoded in Binary Coded Decimal format
    Fisrt Byte: [odd-indicator] [Digit 1]
    Seond Byte: [Digit 3] [Digit 2]
    ..
@@ -417,7 +435,7 @@ decode_sse(proto_tree* ext_tree, tvbuff_t* tvb, int offset, size_t ext_len)
     guint8 msid_num_digits = 0;
     guint8 msid_index = 0;
     guint8 msid_digits[A11_MSG_MSID_LEN_MAX+2];
-    guint8* p_msid = 0;
+    const guint8* p_msid = 0;
     gboolean odd_even_ind = 0;
 
     /* Decode Protocol Type */
@@ -497,6 +515,9 @@ decode_sse(proto_tree* ext_tree, tvbuff_t* tvb, int offset, size_t ext_len)
     if(msid_len > A11_MSG_MSID_ELEM_LEN_MAX)
     {
         p_msid = "MSID is too long";
+    }else if(msid_len < 1)
+    {
+        p_msid = "MSID is too short";
     }else
     {
        /* Decode the BCD digits */
@@ -505,14 +526,14 @@ decode_sse(proto_tree* ext_tree, tvbuff_t* tvb, int offset, size_t ext_len)
            guint8 msid_digit = tvb_get_guint8(tvb, offset);
            offset += 1;
            ext_len -= 1;
-   
+
            msid_digits[msid_index*2] = (msid_digit & 0x0F) + '0';
            msid_digits[(msid_index*2) + 1] = ((msid_digit & 0xF0) >> 4) + '0';
        }
-   
+
        odd_even_ind = (msid_digits[0] == '1');
        msid_num_digits = 0;
-   
+
        if(odd_even_ind)
        {
            msid_num_digits = ((msid_len-1) * 2) + 1;
@@ -520,11 +541,11 @@ decode_sse(proto_tree* ext_tree, tvbuff_t* tvb, int offset, size_t ext_len)
        {
            msid_num_digits = (msid_len-1) * 2;
        }
-    
+
        msid_digits[msid_num_digits + 1] = 0;
        p_msid = msid_digits + 1;
     }
-   
+
 
     proto_tree_add_string
       (ext_tree, hf_a11_ses_msid, tvb, msid_start_offset, msid_len, p_msid);
@@ -590,7 +611,7 @@ dissect_a11_radius( tvbuff_t *tvb, int offset, proto_tree *tree, int app_len)
         offset += radius_len;
         continue;
       }
-      radius_vendor_id = tvb_get_ntohl(tvb, offset +2); 
+      radius_vendor_id = tvb_get_ntohl(tvb, offset +2);
 
       if(radius_vendor_id != VENDOR_THE3GPP2)
       {
@@ -633,7 +654,7 @@ dissect_a11_radius( tvbuff_t *tvb, int offset, proto_tree *tree, int app_len)
       offset += radius_len;
       continue;
     }
-    
+
     offset += SKIP_HDR_LEN;
     radius_len -= SKIP_HDR_LEN;
     radius_offset = 0;
@@ -651,7 +672,7 @@ dissect_a11_radius( tvbuff_t *tvb, int offset, proto_tree *tree, int app_len)
         return;
       }
 
-      radius_subtype = tvb_get_guint8(tvb, offset + radius_offset);	
+      radius_subtype = tvb_get_guint8(tvb, offset + radius_offset);
       attribute_len = tvb_get_guint8(tvb, offset + radius_offset + 1);
       if (attribute_len < 2)
       {
@@ -746,7 +767,7 @@ static void dissect_ase(tvbuff_t* tvb, int offset, int ase_len, proto_tree* ext_
       proto_item* ti = proto_tree_add_text
               (ext_tree, tvb, offset+clen, 0x0D+1, "GRE Key Entry (SRID: %d)", srid);
       proto_tree* exts_tree = proto_item_add_subtree(ti, ett_a11_ase);
-   
+
       /* Entry Length */
       proto_tree_add_item(exts_tree, hf_a11_ase_len_type, tvb, offset+clen, 1, FALSE);
       clen++;
@@ -858,9 +879,15 @@ static void dissect_fwd_qosinfo(tvbuff_t* tvb, int offset, proto_tree* ext_tree)
    guint8 flow_index = 0;
    guint8 dscp_enabled = 0;
 
+   /* 
+    * Starts with a length field 
+    * http://www.3gpp2.org/Public_html/specs/A.S0009-C_v1.0_070801.pdf
+    */
+	proto_tree_add_item(ext_tree, hf_a11_fqi_length, tvb, offset+clen, 2, FALSE);
+	clen = clen + 2;
 
    /* SR Id */
-   srid = tvb_get_guint8(tvb, offset+clen); 
+   srid = tvb_get_guint8(tvb, offset+clen);
    proto_tree_add_item(ext_tree, hf_a11_fqi_srid, tvb, offset+clen, 1, FALSE);
    clen++;
 
@@ -886,7 +913,7 @@ static void dissect_fwd_qosinfo(tvbuff_t* tvb, int offset, proto_tree* ext_tree)
           (ext_tree, tvb, offset+clen, entry_len+1, "Forward Flow Entry (Flow Id: %d)", flow_id);
 
       proto_tree* exts_tree = proto_item_add_subtree(ti, ett_a11_fqi_flowentry);
-   
+
       /* Entry Length */
       proto_tree_add_item(exts_tree, hf_a11_fqi_entrylen, tvb, offset+clen, 1, FALSE);
       clen++;
@@ -910,7 +937,7 @@ static void dissect_fwd_qosinfo(tvbuff_t* tvb, int offset, proto_tree* ext_tree)
       if(requested_qos_len)
       {
         proto_tree_add_item
-          (exts_tree, hf_a11_fqi_requested_qos, tvb, offset+clen, 
+          (exts_tree, hf_a11_fqi_requested_qos, tvb, offset+clen,
               requested_qos_len, FALSE);
         clen += requested_qos_len;
       }
@@ -938,8 +965,15 @@ static void dissect_rev_qosinfo(tvbuff_t* tvb, int offset, proto_tree* ext_tree)
    guint8 flow_count = 0;
    guint8 flow_index = 0;
 
-   /* SR Id */
-   srid = tvb_get_guint8(tvb, offset+clen); 
+   /* 
+    * Starts with a length field 
+    * http://www.3gpp2.org/Public_html/specs/A.S0009-C_v1.0_070801.pdf
+    */
+	proto_tree_add_item(ext_tree, hf_a11_fqi_length, tvb, offset+clen, 2, FALSE);
+	clen = clen + 2;
+
+	/* SR Id */
+   srid = tvb_get_guint8(tvb, offset+clen);
    proto_tree_add_item(ext_tree, hf_a11_rqi_srid, tvb, offset+clen, 1, FALSE);
    clen++;
 
@@ -961,7 +995,7 @@ static void dissect_rev_qosinfo(tvbuff_t* tvb, int offset, proto_tree* ext_tree)
           (ext_tree, tvb, offset+clen, entry_len+1, "Reverse Flow Entry (Flow Id: %d)", flow_id);
 
       proto_tree* exts_tree = proto_item_add_subtree(ti, ett_a11_rqi_flowentry);
-   
+
       /* Entry Length */
       proto_tree_add_item(exts_tree, hf_a11_rqi_entrylen, tvb, offset+clen, 1, FALSE);
       clen++;
@@ -1010,9 +1044,9 @@ static void dissect_subscriber_qos_profile(tvbuff_t* tvb, int offset, int ext_le
 
     int qos_profile_len = ext_len;
 
-    proto_item* ti = 
-       proto_tree_add_text (ext_tree, tvb, offset, 0, 
-                            "Subscriber Qos Profile (%d bytes)", 
+    proto_item* ti =
+       proto_tree_add_text (ext_tree, tvb, offset, 0,
+                            "Subscriber Qos Profile (%d bytes)",
                             qos_profile_len);
 
     exts_tree = proto_item_add_subtree(ti, ett_a11_subscriber_profile);
@@ -1021,7 +1055,7 @@ static void dissect_subscriber_qos_profile(tvbuff_t* tvb, int offset, int ext_le
     if(qos_profile_len)
     {
         proto_tree_add_item
-          (exts_tree,  hf_a11_subsciber_profile, tvb, offset, 
+          (exts_tree,  hf_a11_subsciber_profile, tvb, offset,
               qos_profile_len, FALSE);
     }
 }
@@ -1050,7 +1084,7 @@ static void dissect_fwd_qosupdate_info(tvbuff_t* tvb, int offset, proto_tree* ex
 
       clen++;
       exts_tree = proto_item_add_subtree(ti, ett_a11_fqui_flowentry);
-   
+
       /* Forward QoS Sub Blob Length */
       granted_qos_len = tvb_get_guint8(tvb, offset+clen);
       proto_tree_add_item
@@ -1061,7 +1095,7 @@ static void dissect_fwd_qosupdate_info(tvbuff_t* tvb, int offset, proto_tree* ex
       if(granted_qos_len)
       {
         proto_tree_add_item
-          (exts_tree, hf_a11_fqui_updated_qos, tvb, offset+clen, 
+          (exts_tree, hf_a11_fqui_updated_qos, tvb, offset+clen,
               granted_qos_len, FALSE);
         clen += granted_qos_len;
       }
@@ -1092,7 +1126,7 @@ static void dissect_rev_qosupdate_info(tvbuff_t* tvb, int offset, proto_tree* ex
           (ext_tree, tvb, offset+clen, 1, "Reverse Flow Entry (Flow Id: %d)", flow_id);
       clen++;
       exts_tree = proto_item_add_subtree(ti, ett_a11_rqui_flowentry);
-  
+
       /* Reverse QoS Sub Blob Length */
       granted_qos_len = tvb_get_guint8(tvb, offset+clen);
       proto_tree_add_item
@@ -1146,13 +1180,13 @@ dissect_a11_extensions( tvbuff_t *tvb, int offset, proto_tree *tree)
       hdrLen = 4;
     } else if (ext_type == CVSE_EXT || ext_type == OLD_CVSE_EXT) {
       ext_len = tvb_get_ntohs(tvb, offset + 2);
-      ext_subtype = tvb_get_guint8(tvb, offset + 8);    
+      ext_subtype = tvb_get_guint8(tvb, offset + 8);
       hdrLen = 4;
     } else {
       ext_len = tvb_get_guint8(tvb, offset + 1);
       hdrLen = 2;
     }
-    
+
     ti = proto_tree_add_text(exts_tree, tvb, offset, ext_len + hdrLen,
                  "Extension: %s",
                  val_to_str(ext_type, a11_ext_types,
@@ -1202,7 +1236,7 @@ dissect_a11_extensions( tvbuff_t *tvb, int offset, proto_tree *tree)
     case MN_NAI_EXT:
       if (ext_len == 0)
         break;
-      proto_tree_add_item(ext_tree, hf_a11_next_nai, tvb, offset, 
+      proto_tree_add_item(ext_tree, hf_a11_next_nai, tvb, offset,
                           ext_len, FALSE);
       break;
 
@@ -1226,7 +1260,7 @@ dissect_a11_extensions( tvbuff_t *tvb, int offset, proto_tree *tree)
         break;
       proto_tree_add_item(ext_tree, hf_a11_aext_auth, tvb, offset,
                           ext_len, FALSE);
-      
+
       break;
     case OLD_CVSE_EXT:      /* RFC 3115 */
     case CVSE_EXT:          /* RFC 3115 */
@@ -1349,7 +1383,7 @@ dissect_a11( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint8         flags;
   size_t         offset=0;
   const guint8  *reftime;
-  
+
   if (!tvb_bytes_exist(tvb, offset, 1))
 	return 0;	/* not enough data to check message type */
 
@@ -1358,27 +1392,27 @@ dissect_a11( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	return 0;	/* not a known message type */
 
   /* Make entries in Protocol column and Info column on summary display */
-  
-  if (check_col(pinfo->cinfo, COL_PROTOCOL)) 
+
+  if (check_col(pinfo->cinfo, COL_PROTOCOL))
 	col_set_str(pinfo->cinfo, COL_PROTOCOL, "3GPP2 A11");
-  if (check_col(pinfo->cinfo, COL_INFO)) 
+  if (check_col(pinfo->cinfo, COL_INFO))
 	col_clear(pinfo->cinfo, COL_INFO);
 
   switch (type) {
   case REGISTRATION_REQUEST:
-	if (check_col(pinfo->cinfo, COL_INFO)) 
-	  col_add_fstr(pinfo->cinfo, COL_INFO, "Reg Request: PDSN=%s PCF=%s", 
+	if (check_col(pinfo->cinfo, COL_INFO))
+	  col_add_fstr(pinfo->cinfo, COL_INFO, "Reg Request: PDSN=%s PCF=%s",
 				   ip_to_str(tvb_get_ptr(tvb, 8, 4)),
 				   ip_to_str(tvb_get_ptr(tvb,12,4)));
-	
+
 	if (tree) {
 	  ti = proto_tree_add_item(tree, proto_a11, tvb, offset, -1, FALSE);
 	  a11_tree = proto_item_add_subtree(ti, ett_a11);
-	  
+
 	  /* type */
 	  proto_tree_add_uint(a11_tree, hf_a11_type, tvb, offset, 1, type);
 	  offset++;
-	  
+
 	  /* flags */
 	  flags = tvb_get_guint8(tvb, offset);
 	  tf = proto_tree_add_uint(a11_tree, hf_a11_flags, tvb,
@@ -1396,15 +1430,15 @@ dissect_a11( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	  /* lifetime */
 	  proto_tree_add_item(a11_tree, hf_a11_life, tvb, offset, 2, FALSE);
 	  offset +=2;
-	  
+
 	  /* home address */
 	  proto_tree_add_item(a11_tree, hf_a11_homeaddr, tvb, offset, 4, FALSE);
 	  offset += 4;
-	  
+
 	  /* home agent address */
 	  proto_tree_add_item(a11_tree, hf_a11_haaddr, tvb, offset, 4, FALSE);
 	  offset += 4;
-	  
+
 	  /* Care of Address */
 	  proto_tree_add_item(a11_tree, hf_a11_coa, tvb, offset, 4, FALSE);
 	  offset += 4;
@@ -1416,23 +1450,23 @@ dissect_a11( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					    "%s",
 					    ntp_fmt_ts(reftime));
 	  offset += 8;
-		
+
 	} /* if tree */
 	break;
   case REGISTRATION_REPLY:
-	if (check_col(pinfo->cinfo, COL_INFO)) 
-	  col_add_fstr(pinfo->cinfo, COL_INFO, "Reg Reply:   PDSN=%s, Code=%u", 
+	if (check_col(pinfo->cinfo, COL_INFO))
+	  col_add_fstr(pinfo->cinfo, COL_INFO, "Reg Reply:   PDSN=%s, Code=%u",
 				   ip_to_str(tvb_get_ptr(tvb,8,4)), tvb_get_guint8(tvb,1));
-	
+
 	if (tree) {
 	  /* Add Subtree */
 	  ti = proto_tree_add_item(tree, proto_a11, tvb, offset, -1, FALSE);
 	  a11_tree = proto_item_add_subtree(ti, ett_a11);
-	  
+
 	  /* Type */
   	  proto_tree_add_uint(a11_tree, hf_a11_type, tvb, offset, 1, type);
 	  offset++;
-	  
+
 	  /* Reply Code */
 	  proto_tree_add_item(a11_tree, hf_a11_code, tvb, offset, 1, FALSE);
 	  offset++;
@@ -1458,24 +1492,24 @@ dissect_a11( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 					    ntp_fmt_ts(reftime));
 	  offset += 8;
 	} /* if tree */
-	
+
 	break;
   case REGISTRATION_UPDATE:
-	if (check_col(pinfo->cinfo, COL_INFO)) 
-	  col_add_fstr(pinfo->cinfo, COL_INFO,"Reg Update:  PDSN=%s", 
+	if (check_col(pinfo->cinfo, COL_INFO))
+	  col_add_fstr(pinfo->cinfo, COL_INFO,"Reg Update:  PDSN=%s",
 				   ip_to_str(tvb_get_ptr(tvb,8,4)));
 	if (tree) {
 	  /* Add Subtree */
 	  ti = proto_tree_add_item(tree, proto_a11, tvb, offset, -1, FALSE);
 	  a11_tree = proto_item_add_subtree(ti, ett_a11);
-	  
+
 	  /* Type */
   	  proto_tree_add_uint(a11_tree, hf_a11_type, tvb, offset, 1, type);
 	  offset++;
 
 	  /* Reserved */
 	  offset+=3;
-	
+
 	  /* Home address */
 	  proto_tree_add_item(a11_tree, hf_a11_homeaddr, tvb, offset, 4, FALSE);
 	  offset += 4;
@@ -1496,19 +1530,19 @@ dissect_a11( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	} /* if tree */
 	break;
   case REGISTRATION_ACK:
-	if (check_col(pinfo->cinfo, COL_INFO)) 
-	  col_add_fstr(pinfo->cinfo, COL_INFO, "Reg Ack:     PCF=%s Status=%u", 
+	if (check_col(pinfo->cinfo, COL_INFO))
+	  col_add_fstr(pinfo->cinfo, COL_INFO, "Reg Ack:     PCF=%s Status=%u",
 				   ip_to_str(tvb_get_ptr(tvb, 8, 4)),
 				   tvb_get_guint8(tvb,3));
 	if (tree) {
 	  /* Add Subtree */
 	  ti = proto_tree_add_item(tree, proto_a11, tvb, offset, -1, FALSE);
 	  a11_tree = proto_item_add_subtree(ti, ett_a11);
-	  
+
 	  /* Type */
   	  proto_tree_add_uint(a11_tree, hf_a11_type, tvb, offset, 1, type);
 	  offset++;
-	
+
 	  /* Reserved */
 	  offset+=2;
 
@@ -1536,21 +1570,21 @@ dissect_a11( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	} /* if tree */
 	break;
   case SESSION_UPDATE: /* IOS4.3 */
-	if (check_col(pinfo->cinfo, COL_INFO)) 
-	  col_add_fstr(pinfo->cinfo, COL_INFO,"Ses Update:  PDSN=%s", 
+	if (check_col(pinfo->cinfo, COL_INFO))
+	  col_add_fstr(pinfo->cinfo, COL_INFO,"Ses Update:  PDSN=%s",
 				   ip_to_str(tvb_get_ptr(tvb,8,4)));
 	if (tree) {
 	  /* Add Subtree */
 	  ti = proto_tree_add_item(tree, proto_a11, tvb, offset, -1, FALSE);
 	  a11_tree = proto_item_add_subtree(ti, ett_a11);
-	  
+
 	  /* Type */
   	  proto_tree_add_uint(a11_tree, hf_a11_type, tvb, offset, 1, type);
 	  offset++;
 
 	  /* Reserved */
 	  offset+=3;
-	
+
 	  /* Home address */
 	  proto_tree_add_item(a11_tree, hf_a11_homeaddr, tvb, offset, 4, FALSE);
 	  offset += 4;
@@ -1571,19 +1605,19 @@ dissect_a11( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	} /* if tree */
 	break;
   case SESSION_ACK: /* IOS4.3 */
-	if (check_col(pinfo->cinfo, COL_INFO)) 
-	  col_add_fstr(pinfo->cinfo, COL_INFO, "Ses Upd Ack: PCF=%s, Status=%u", 
+	if (check_col(pinfo->cinfo, COL_INFO))
+	  col_add_fstr(pinfo->cinfo, COL_INFO, "Ses Upd Ack: PCF=%s, Status=%u",
 				   ip_to_str(tvb_get_ptr(tvb, 8, 4)),
 				   tvb_get_guint8(tvb,3));
 	if (tree) {
 	  /* Add Subtree */
 	  ti = proto_tree_add_item(tree, proto_a11, tvb, offset, -1, FALSE);
 	  a11_tree = proto_item_add_subtree(ti, ett_a11);
-	  
+
 	  /* Type */
   	  proto_tree_add_uint(a11_tree, hf_a11_type, tvb, offset, 1, type);
 	  offset++;
-	
+
 	  /* Reserved */
 	  offset+=2;
 
@@ -1623,13 +1657,13 @@ dissect_a11( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 void proto_register_a11(void);
 
 void proto_register_a11(void)
-{                 
+{
 
 /* Setup list of header fields */
 	static hf_register_info hf[] = {
 	  { &hf_a11_type,
 		 { "Message Type",           "a11.type",
-			FT_UINT8, BASE_DEC, VALS(a11_types), 0,          
+			FT_UINT8, BASE_DEC, VALS(a11_types), 0,
 			"A11 Message type.", HFILL }
 	  },
 	  { &hf_a11_flags,
@@ -1640,83 +1674,83 @@ void proto_register_a11(void)
 	  { &hf_a11_s,
 		 {"Simultaneous Bindings",           "a11.s",
 
-		   FT_BOOLEAN, 8, NULL, 128,          
+		   FT_BOOLEAN, 8, NULL, 128,
 		   "Simultaneous Bindings Allowed", HFILL }
 	  },
 	  { &hf_a11_b,
 		 {"Broadcast Datagrams",           "a11.b",
-		   FT_BOOLEAN, 8, NULL, 64,          
+		   FT_BOOLEAN, 8, NULL, 64,
 		   "Broadcast Datagrams requested", HFILL }
 	  },
 	  { &hf_a11_d,
 		 { "Co-located Care-of Address",           "a11.d",
-		   FT_BOOLEAN, 8, NULL, 32,          
+		   FT_BOOLEAN, 8, NULL, 32,
 		   "MN using Co-located Care-of address", HFILL }
 	  },
 	  { &hf_a11_m,
 		 {"Minimal Encapsulation",           "a11.m",
-		   FT_BOOLEAN, 8, NULL, 16,          
+		   FT_BOOLEAN, 8, NULL, 16,
 		   "MN wants Minimal encapsulation", HFILL }
 	  },
 	  { &hf_a11_g,
 		 {"GRE",           "a11.g",
-		   FT_BOOLEAN, 8, NULL, 8,          
+		   FT_BOOLEAN, 8, NULL, 8,
 		   "MN wants GRE encapsulation", HFILL }
 	  },
 	  { &hf_a11_v,
 		 { "Van Jacobson",           "a11.v",
-		   FT_BOOLEAN, 8, NULL, 4,          
+		   FT_BOOLEAN, 8, NULL, 4,
 		   "Van Jacobson", HFILL }
 	  },
 	  { &hf_a11_t,
 		 { "Reverse Tunneling",           "a11.t",
-		   FT_BOOLEAN, 8, NULL, 2,          
+		   FT_BOOLEAN, 8, NULL, 2,
 		   "Reverse tunneling requested", HFILL }
 	  },
 	  { &hf_a11_code,
 		 { "Reply Code",           "a11.code",
-			FT_UINT8, BASE_DEC, VALS(a11_reply_codes), 0,          
+			FT_UINT8, BASE_DEC, VALS(a11_reply_codes), 0,
 			"A11 Registration Reply code.", HFILL }
 	  },
 	  { &hf_a11_status,
 		 { "Reply Status",           "a11.ackstat",
-			FT_UINT8, BASE_DEC, VALS(a11_ack_status), 0,          
+			FT_UINT8, BASE_DEC, VALS(a11_ack_status), 0,
 			"A11 Registration Ack Status.", HFILL }
 	  },
 	  { &hf_a11_life,
 		 { "Lifetime",           "a11.life",
-			FT_UINT16, BASE_DEC, NULL, 0,          
+			FT_UINT16, BASE_DEC, NULL, 0,
 			"A11 Registration Lifetime.", HFILL }
 	  },
 	  { &hf_a11_homeaddr,
 		 { "Home Address",           "a11.homeaddr",
-			FT_IPv4, BASE_NONE, NULL, 0,          
+			FT_IPv4, BASE_NONE, NULL, 0,
 			"Mobile Node's home address.", HFILL }
 	  },
-	  
+
 	  { &hf_a11_haaddr,
 		 { "Home Agent",           "a11.haaddr",
-			FT_IPv4, BASE_NONE, NULL, 0,          
+			FT_IPv4, BASE_NONE, NULL, 0,
 			"Home agent IP Address.", HFILL }
 	  },
 	  { &hf_a11_coa,
 		 { "Care of Address",           "a11.coa",
-			FT_IPv4, BASE_NONE, NULL, 0,          
+			FT_IPv4, BASE_NONE, NULL, 0,
 			"Care of Address.", HFILL }
 	  },
 	  { &hf_a11_ident,
 		 { "Identification",           "a11.ident",
-			FT_BYTES, BASE_NONE, NULL, 0,          
+			FT_BYTES, BASE_NONE, NULL, 0,
 			"MN Identification.", HFILL }
 	  },
 	  { &hf_a11_ext_type,
 		 { "Extension Type",           "a11.ext.type",
-			FT_UINT8, BASE_DEC, VALS(a11_ext_types), 0,          
+			FT_UINT8, BASE_DEC, VALS(a11_ext_types), 0,
 			"Mobile IP Extension Type.", HFILL }
 	  },
 	  { &hf_a11_ext_stype,
 		 { "Gen Auth Ext SubType",           "a11.ext.auth.subtype",
-			FT_UINT8, BASE_DEC, VALS(a11_ext_stypes), 0,          
+			FT_UINT8, BASE_DEC, VALS(a11_ext_stypes), 0,
 			"Mobile IP Auth Extension Sub Type.", HFILL }
 	  },
 	  { &hf_a11_ext_len,
@@ -1791,17 +1825,17 @@ void proto_register_a11(void)
 	  },
 	  { &hf_a11_vse_ppaddr,
 		 { "Anchor P-P Address",           "a11.ext.ppaddr",
-			FT_IPv4, BASE_NONE, NULL, 0,          
+			FT_IPv4, BASE_NONE, NULL, 0,
 			"Anchor P-P Address.", HFILL }
 	  },
 	  { &hf_a11_vse_dormant,
 		 { "All Dormant Indicator",           "a11.ext.dormant",
-			FT_UINT16, BASE_HEX, VALS(a11_ext_dormant), 0,          
+			FT_UINT16, BASE_HEX, VALS(a11_ext_dormant), 0,
 			"All Dormant Indicator.", HFILL }
 	  },
 	  { &hf_a11_vse_code,
 		 { "Reply Code",           "a11.ext.code",
-			FT_UINT8, BASE_DEC, VALS(a11_reply_codes), 0,          
+			FT_UINT8, BASE_DEC, VALS(a11_reply_codes), 0,
 			"PDSN Code.", HFILL }
 	  },
 	  { &hf_a11_vse_pdit,
@@ -1859,6 +1893,11 @@ void proto_register_a11(void)
 			FT_IPv4, BASE_NONE, NULL, 0,
 			"PCF IP Address.", HFILL }
 	  },
+	  { &hf_a11_fqi_length,
+		 { "Length",   "a11.ext.fqi.length",
+			FT_UINT16, BASE_DEC, NULL, 0,
+			NULL, HFILL }
+	  },
 	  { &hf_a11_fqi_srid,
 		 { "SRID",   "a11.ext.fqi.srid",
 			FT_UINT8, BASE_DEC, NULL, 0,
@@ -1914,6 +1953,11 @@ void proto_register_a11(void)
 			FT_BYTES, BASE_NONE, NULL, 0,
 			"Forward Granted QoS.", HFILL }
 	  },
+	  { &hf_a11_rqi_length,
+		 { "Length",   "a11.ext.rqi.length",
+			FT_UINT16, BASE_DEC, NULL, 0,
+			NULL, HFILL }
+	  },
 	  { &hf_a11_rqi_srid,
 		 { "SRID",   "a11.ext.rqi.srid",
 			FT_UINT8, BASE_DEC, NULL, 0,
