@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2005 by Alejandro Vaquero <alejandro.vaquero@verso.com>
  *
- * $Id: packet-actrace.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-actrace.c 23192 2007-10-15 20:11:25Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -497,7 +497,7 @@ static void dissect_actrace(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		}
 	} else {
 		if (check_col(pinfo->cinfo, COL_INFO))
-			col_add_str(pinfo->cinfo, COL_INFO, "Non CAS or ISDN AudioCodes trace message");
+			col_set_str(pinfo->cinfo, COL_INFO, "Non CAS or ISDN AudioCodes trace message");
 	}
 }
 
