@@ -1,7 +1,8 @@
 /* packet-asap.c
  * Routines for Aggregate Server Access Protocol (ASAP)
  * It is hopefully (needs testing) compilant to
- * http://www.ietf.org/internet-drafts/draft-ietf-rserpool-common-param-09.txt
+ * http://www.ietf.org/internet-drafts/draft-ietf-rserpool-common-param-15.txt
+ * http://www.ietf.org/internet-drafts/draft-ietf-rserpool-policies-04.txt
  * http://www.ietf.org/internet-drafts/draft-ietf-rserpool-asap-12.txt
  *
  * The code is not as simple as possible for the current protocol
@@ -11,9 +12,9 @@
  * TODO:
  *   - check message lengths
  *
- * Copyright 2004, 2005 Michael Tuexen <tuexen [AT] fh-muenster.de>
+ * Copyright 2004, 2005, 2006, 2007 Michael Tuexen <tuexen [AT] fh-muenster.de>
  *
- * $Id: packet-asap.c 18265 2006-05-31 13:41:35Z tuexen $
+ * $Id: packet-asap.c 23939 2007-12-22 20:30:23Z tuexen $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -58,24 +59,30 @@ static int hf_parameter_value = -1;
 static int hf_parameter_padding = -1;
 static int hf_parameter_ipv4_address = -1;
 static int hf_parameter_ipv6_address = -1;
+static int hf_dccp_port = -1;
+static int hf_dccp_reserved = -1;
+static int hf_dccp_service_code = -1;
 static int hf_sctp_port = -1;
 static int hf_transport_use = -1;
 static int hf_tcp_port = -1;
 static int hf_udp_port = -1;
 static int hf_udp_reserved = -1;
+static int hf_udp_lite_port = -1;
+static int hf_udp_lite_reserved = -1;
 static int hf_policy_type = -1;
 static int hf_policy_value = -1;
+static int hf_policy_weight = -1;
+static int hf_policy_priority = -1;
+static int hf_policy_load = -1;
+static int hf_policy_degradation = -1;
 static int hf_pool_handle = -1;
 static int hf_pe_pe_identifier = -1;
 static int hf_home_enrp_id = -1;
 static int hf_reg_life = -1;
 static int hf_server_identifier = -1;
-static int hf_m_bit = -1;
-static int hf_reserved = -1;
 static int hf_cookie = -1;
 static int hf_pe_identifier = -1;
 static int hf_pe_checksum = -1;
-static int hf_pe_checksum_reserved = -1;
 static int hf_home_enrp_server_bit = -1;
 static int hf_reject_bit = -1;
 
@@ -90,13 +97,14 @@ dissect_parameters(tvbuff_t *, proto_tree *);
 static void
 dissect_parameter(tvbuff_t *, proto_tree *);
 static void
-dissect_asap_message(tvbuff_t *, packet_info *, proto_tree *);
+dissect_asap(tvbuff_t *, packet_info *, proto_tree *);
 
 #define NETWORK_BYTE_ORDER     FALSE
 #define ADD_PADDING(x) ((((x) + 3) >> 2) << 2)
 
 #define ASAP_UDP_PORT 3863
 #define ASAP_TCP_PORT 3863
+#define ASAP_SCTP_PORT 3863
 
 /* Dissectors for error causes. This is common for ASAP and ENRP. */
 
@@ -123,27 +131,29 @@ dissect_unknown_cause(tvbuff_t *cause_tvb, proto_tree *cause_tree, proto_item *c
   proto_item_append_text(cause_item, " (code %u and %u byte%s information)", code, cause_info_length, plurality(cause_info_length, "", "s"));
 }
 
-#define UNRECOGNIZED_PARAMETER_CAUSE_CODE                  1
-#define UNRECONGNIZED_MESSAGE_CAUSE_CODE                   2
-#define INVALID_VALUES                                     3
-#define NON_UNIQUE_PE_IDENTIFIER                           4
-#define POOLING_POLICY_INCONSISTENT_CAUSE_CODE             5
-#define LACK_OF_RESOURCES_CAUSE_CODE                       6
-#define INCONSISTENT_TRANSPORT_TYPE_CAUSE_CODE             7
-#define INCONSISTENT_DATA_CONTROL_CONFIGURATION_CAUSE_CODE 8
-#define UNKNOWN_POOL_HANDLE                                9
+#define UNRECOGNIZED_PARAMETER_CAUSE_CODE                  0x1
+#define UNRECONGNIZED_MESSAGE_CAUSE_CODE                   0x2
+#define INVALID_VALUES                                     0x3
+#define NON_UNIQUE_PE_IDENTIFIER                           0x4
+#define POOLING_POLICY_INCONSISTENT_CAUSE_CODE             0x5
+#define LACK_OF_RESOURCES_CAUSE_CODE                       0x6
+#define INCONSISTENT_TRANSPORT_TYPE_CAUSE_CODE             0x7
+#define INCONSISTENT_DATA_CONTROL_CONFIGURATION_CAUSE_CODE 0x8
+#define UNKNOWN_POOL_HANDLE                                0x9
+#define REJECTION_DUE_TO_SECURITY_CAUSE_CODE               0xa
 
 static const value_string cause_code_values[] = {
-  { UNRECOGNIZED_PARAMETER_CAUSE_CODE,                  "Unrecognized parameter"         },
-  { UNRECONGNIZED_MESSAGE_CAUSE_CODE,                   "Unrecognized message"           },
-  { INVALID_VALUES,                                     "Invalid values"                 },
-  { NON_UNIQUE_PE_IDENTIFIER,                           "Non-unique PE identifier"       },
-  { POOLING_POLICY_INCONSISTENT_CAUSE_CODE,             "Pooling policy inconsistent"    },
-  { LACK_OF_RESOURCES_CAUSE_CODE,                       "Lack of resources"              },
-  { INCONSISTENT_TRANSPORT_TYPE_CAUSE_CODE,             "Inconsistent transport type"    },
-  { INCONSISTENT_DATA_CONTROL_CONFIGURATION_CAUSE_CODE, "Inconsistent data/control type" },
-  { UNKNOWN_POOL_HANDLE,                                "Unknown pool handle"            },
-  { 0,                                                  NULL                             } };
+  { UNRECOGNIZED_PARAMETER_CAUSE_CODE,                  "Unrecognized parameter"                  },
+  { UNRECONGNIZED_MESSAGE_CAUSE_CODE,                   "Unrecognized message"                    },
+  { INVALID_VALUES,                                     "Invalid values"                          },
+  { NON_UNIQUE_PE_IDENTIFIER,                           "Non-unique PE identifier"                },
+  { POOLING_POLICY_INCONSISTENT_CAUSE_CODE,             "Pooling policy inconsistent"             },
+  { LACK_OF_RESOURCES_CAUSE_CODE,                       "Lack of resources"                       },
+  { INCONSISTENT_TRANSPORT_TYPE_CAUSE_CODE,             "Inconsistent transport type"             },
+  { INCONSISTENT_DATA_CONTROL_CONFIGURATION_CAUSE_CODE, "Inconsistent data/control type"          },
+  { UNKNOWN_POOL_HANDLE,                                "Unknown pool handle"                     },
+  { REJECTION_DUE_TO_SECURITY_CAUSE_CODE,               "Rejected due to security considerations" },
+  { 0,                                                  NULL                                      } };
 
 static void
 dissect_error_cause(tvbuff_t *cause_tvb, proto_tree *parameter_tree)
@@ -166,33 +176,35 @@ dissect_error_cause(tvbuff_t *cause_tvb, proto_tree *parameter_tree)
   switch(code) {
   case UNRECOGNIZED_PARAMETER_CAUSE_CODE:
     parameter_tvb = tvb_new_subset(cause_tvb, CAUSE_INFO_OFFSET, -1, -1);
-    dissect_parameter(parameter_tvb, parameter_tree);
+    dissect_parameter(parameter_tvb, cause_tree);
     break;
   case UNRECONGNIZED_MESSAGE_CAUSE_CODE:
     message_tvb = tvb_new_subset(cause_tvb, CAUSE_INFO_OFFSET, -1, -1);
-    dissect_asap_message(message_tvb, NULL, parameter_tree);
+    dissect_asap(message_tvb, NULL, cause_tree);
     break;
     break;
   case INVALID_VALUES:
     parameter_tvb = tvb_new_subset(cause_tvb, CAUSE_INFO_OFFSET, -1, -1);
-    dissect_parameter(parameter_tvb, parameter_tree);
+    dissect_parameter(parameter_tvb, cause_tree);
     break;
   case NON_UNIQUE_PE_IDENTIFIER:
     break;
   case POOLING_POLICY_INCONSISTENT_CAUSE_CODE:
     parameter_tvb = tvb_new_subset(cause_tvb, CAUSE_INFO_OFFSET, -1, -1);
-    dissect_parameter(parameter_tvb, parameter_tree);
+    dissect_parameter(parameter_tvb, cause_tree);
     break;
   case LACK_OF_RESOURCES_CAUSE_CODE:
     break;
   case INCONSISTENT_TRANSPORT_TYPE_CAUSE_CODE:
     parameter_tvb = tvb_new_subset(cause_tvb, CAUSE_INFO_OFFSET, -1, -1);
-    dissect_parameter(parameter_tvb, parameter_tree);
+    dissect_parameter(parameter_tvb, cause_tree);
     break;
   case INCONSISTENT_DATA_CONTROL_CONFIGURATION_CAUSE_CODE:
     break;
   case UNKNOWN_POOL_HANDLE:
     break;
+  case REJECTION_DUE_TO_SECURITY_CAUSE_CODE:
+    break;
   default:
     dissect_unknown_cause(cause_tvb, cause_tree, cause_item);
     break;
@@ -249,6 +261,27 @@ dissect_ipv6_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree, prot
   proto_item_append_text(parameter_item, " (%s)", ip6_to_str((const struct e_in6_addr *)tvb_get_ptr(parameter_tvb, IPV6_ADDRESS_OFFSET, IPV6_ADDRESS_LENGTH)));
 }
 
+#define DCCP_PORT_LENGTH         2
+#define DCCP_RESERVED_LENGTH     2
+#define DCCP_SERVICE_CODE_LENGTH 4
+#define DCCP_PORT_OFFSET         PARAMETER_VALUE_OFFSET
+#define DCCP_RESERVED_OFFSET     (DCCP_PORT_OFFSET + DCCP_PORT_LENGTH)
+#define DCCP_SERVICE_CODE_OFFSET (DCCP_RESERVED_OFFSET + DCCP_RESERVED_LENGTH)
+#define DCCP_ADDRESS_OFFSET      (DCCP_SERVICE_CODE_OFFSET + DCCP_SERVICE_CODE_LENGTH)
+
+static void
+dissect_dccp_transport_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree)
+{
+  tvbuff_t *parameters_tvb;
+
+  proto_tree_add_item(parameter_tree, hf_dccp_port,         parameter_tvb, DCCP_PORT_OFFSET,         DCCP_PORT_LENGTH,         NETWORK_BYTE_ORDER);
+  proto_tree_add_item(parameter_tree, hf_dccp_reserved,     parameter_tvb, DCCP_RESERVED_OFFSET,     DCCP_RESERVED_LENGTH,     NETWORK_BYTE_ORDER);
+  proto_tree_add_item(parameter_tree, hf_dccp_service_code, parameter_tvb, DCCP_SERVICE_CODE_OFFSET, DCCP_SERVICE_CODE_LENGTH, NETWORK_BYTE_ORDER);
+
+  parameters_tvb = tvb_new_subset(parameter_tvb, DCCP_ADDRESS_OFFSET, -1, -1);
+  dissect_parameters(parameters_tvb, parameter_tree);
+}
+
 #define TRANSPORT_USE_DATA_ONLY         0
 #define TRANSPORT_USE_DATA_PLUS_CONTROL 1
 
@@ -311,29 +344,102 @@ dissect_udp_transport_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_t
   dissect_parameters(parameters_tvb, parameter_tree);
 }
 
-#define POLICY_TYPE_LENGTH   1
-#define POLICY_VALUE_LENGTH  3
+#define UDP_LITE_PORT_LENGTH     2
+#define UDP_LITE_RESERVED_LENGTH 2
+#define UDP_LITE_PORT_OFFSET     PARAMETER_VALUE_OFFSET
+#define UDP_LITE_RESERVED_OFFSET (UDP_LITE_PORT_OFFSET + UDP_LITE_PORT_LENGTH)
+#define UDP_LITE_ADDRESS_OFFSET  (UDP_LITE_RESERVED_OFFSET + UDP_LITE_RESERVED_LENGTH)
 
-#define POLICY_TYPE_OFFSET   PARAMETER_VALUE_OFFSET
-#define POLICY_VALUE_OFFSET  (POLICY_TYPE_OFFSET + POLICY_TYPE_LENGTH)
+static void
+dissect_udp_lite_transport_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree)
+{
+  tvbuff_t *parameters_tvb;
 
-#define ROUND_ROBIN_POLICY   1
-#define LEAST_USED_POLICY    2
-#define LEAST_USED_WITH_DEG  3
-#define WEIGHTED_ROUND_ROBIN 4
+  proto_tree_add_item(parameter_tree, hf_udp_lite_port,     parameter_tvb, UDP_LITE_PORT_OFFSET,     UDP_LITE_PORT_LENGTH,     NETWORK_BYTE_ORDER);
+  proto_tree_add_item(parameter_tree, hf_udp_lite_reserved, parameter_tvb, UDP_LITE_RESERVED_OFFSET, UDP_LITE_RESERVED_LENGTH, NETWORK_BYTE_ORDER);
+
+  parameters_tvb = tvb_new_subset(parameter_tvb, UDP_LITE_ADDRESS_OFFSET, -1, -1);
+  dissect_parameters(parameters_tvb, parameter_tree);
+}
+
+#define POLICY_TYPE_LENGTH        4
+#define POLICY_WEIGHT_LENGTH      4
+#define POLICY_PRIORITY_LENGTH    4
+#define POLICY_LOAD_LENGTH        4
+#define POLICY_DEGRADATION_LENGTH 4
+
+#define POLICY_TYPE_OFFSET        PARAMETER_VALUE_OFFSET
+#define POLICY_VALUE_OFFSET       (POLICY_TYPE_OFFSET + POLICY_TYPE_LENGTH)
+#define POLICY_WEIGHT_OFFSET      POLICY_VALUE_OFFSET
+#define POLICY_PRIORITY_OFFSET    POLICY_VALUE_OFFSET
+#define POLICY_LOAD_OFFSET        POLICY_VALUE_OFFSET
+#define POLICY_DEGRADATION_OFFSET (POLICY_LOAD_OFFSET + POLICY_LOAD_LENGTH)
+
+#define ROUND_ROBIN_POLICY           0x00000001
+#define WEIGHTED_ROUND_ROBIN_POLICY  0x00000002
+#define RANDOM_POLICY                0x00000003
+#define WEIGHTED_RANDOM_POLICY       0x00000004
+#define PRIORITY_POLICY              0x00000005
+#define LEAST_USED_POLICY            0x40000001
+#define LEAST_USED_WITH_DEG_POLICY   0x40000002
+#define PRIORITY_LEAST_USED_POLICY   0x40000003
+#define RANDOMIZED_LEAST_USED_POLICY 0x40000004
 
 static const value_string policy_type_values[] = {
-  { ROUND_ROBIN_POLICY,   "Round robin" },
-  { LEAST_USED_POLICY,    "Least used" },
-  { LEAST_USED_WITH_DEG,  "Least used with degradation" },
-  { WEIGHTED_ROUND_ROBIN, "Weighted round robin" },
-  { 0,                    NULL } };
+  { ROUND_ROBIN_POLICY,           "Round robin" },
+  { WEIGHTED_ROUND_ROBIN_POLICY,  "Weighted round robin" },
+  { RANDOM_POLICY,                "Random"},
+  { WEIGHTED_RANDOM_POLICY,       "Weighted random" },
+  { PRIORITY_POLICY,              "Priority" },
+  { LEAST_USED_POLICY,            "Least used" },
+  { LEAST_USED_WITH_DEG_POLICY,   "Least used with degradation" },
+  { PRIORITY_LEAST_USED_POLICY,   "Priority least used" },
+  { RANDOMIZED_LEAST_USED_POLICY, "Randomized least used" },
+  { 0,                            NULL } };
 
 static void
 dissect_pool_member_selection_policy_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree)
 {
+  guint32 type;
+  guint length;
+  
   proto_tree_add_item(parameter_tree, hf_policy_type,  parameter_tvb, POLICY_TYPE_OFFSET,  POLICY_TYPE_LENGTH,  NETWORK_BYTE_ORDER);
-  proto_tree_add_item(parameter_tree, hf_policy_value, parameter_tvb, POLICY_VALUE_OFFSET, POLICY_VALUE_LENGTH, NETWORK_BYTE_ORDER);
+  type = tvb_get_ntohl(parameter_tvb, POLICY_TYPE_OFFSET);
+  switch (type) {
+  case ROUND_ROBIN_POLICY:
+    break;
+  case WEIGHTED_ROUND_ROBIN_POLICY:
+    proto_tree_add_item(parameter_tree, hf_policy_weight, parameter_tvb, POLICY_WEIGHT_OFFSET, POLICY_WEIGHT_LENGTH, NETWORK_BYTE_ORDER);
+    break;
+  case RANDOM_POLICY:
+    break;
+  case WEIGHTED_RANDOM_POLICY:
+    proto_tree_add_item(parameter_tree, hf_policy_weight, parameter_tvb, POLICY_WEIGHT_OFFSET, POLICY_WEIGHT_LENGTH, NETWORK_BYTE_ORDER);
+    break;
+  case PRIORITY_POLICY:
+    proto_tree_add_item(parameter_tree, hf_policy_weight, parameter_tvb, POLICY_PRIORITY_OFFSET, POLICY_PRIORITY_LENGTH, NETWORK_BYTE_ORDER);
+    break;
+  case LEAST_USED_POLICY:
+    proto_tree_add_item(parameter_tree, hf_policy_load, parameter_tvb, POLICY_LOAD_OFFSET, POLICY_LOAD_LENGTH, NETWORK_BYTE_ORDER);
+    break;
+  case LEAST_USED_WITH_DEG_POLICY:
+    proto_tree_add_item(parameter_tree, hf_policy_load, parameter_tvb, POLICY_LOAD_OFFSET, POLICY_LOAD_LENGTH, NETWORK_BYTE_ORDER);
+    proto_tree_add_item(parameter_tree, hf_policy_degradation, parameter_tvb, POLICY_DEGRADATION_OFFSET, POLICY_DEGRADATION_LENGTH, NETWORK_BYTE_ORDER);
+    break;
+  case PRIORITY_LEAST_USED_POLICY:
+    proto_tree_add_item(parameter_tree, hf_policy_load, parameter_tvb, POLICY_LOAD_OFFSET, POLICY_LOAD_LENGTH, NETWORK_BYTE_ORDER);
+    proto_tree_add_item(parameter_tree, hf_policy_degradation, parameter_tvb, POLICY_DEGRADATION_OFFSET, POLICY_DEGRADATION_LENGTH, NETWORK_BYTE_ORDER);
+    break;
+  case RANDOMIZED_LEAST_USED_POLICY:
+    proto_tree_add_item(parameter_tree, hf_policy_load, parameter_tvb, POLICY_LOAD_OFFSET, POLICY_LOAD_LENGTH, NETWORK_BYTE_ORDER);
+    break;
+  default:
+    length = tvb_length(parameter_tvb) - POLICY_TYPE_LENGTH;
+    if (length > 0) {
+      proto_tree_add_item(parameter_tree, hf_policy_value, parameter_tvb, POLICY_VALUE_OFFSET, length, NETWORK_BYTE_ORDER);
+    }
+    break;
+  }
 }
 
 #define POOL_HANDLE_OFFSET PARAMETER_VALUE_OFFSET
@@ -370,14 +476,8 @@ dissect_pool_element_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tr
 }
 
 #define SERVER_ID_LENGTH         4
-#define RESERVED_LENGTH          4
-
 #define SERVER_ID_OFFSET         PARAMETER_VALUE_OFFSET
-#define RESERVED_OFFSET          (SERVER_ID_OFFSET + SERVER_ID_LENGTH)
-#define SERVER_TRANSPORT_OFFSET  (RESERVED_OFFSET + RESERVED_LENGTH)
-
-#define M_BIT_MASK               0x80000000
-#define RESERVED_MASK            0x7fffffff
+#define SERVER_TRANSPORT_OFFSET  (SERVER_ID_OFFSET + SERVER_ID_LENGTH)
 
 static void
 dissect_server_information_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree)
@@ -385,8 +485,6 @@ dissect_server_information_parameter(tvbuff_t *parameter_tvb, proto_tree *parame
   tvbuff_t *parameters_tvb;
 
   proto_tree_add_item(parameter_tree, hf_server_identifier, parameter_tvb, SERVER_ID_OFFSET, SERVER_ID_LENGTH, NETWORK_BYTE_ORDER);
-  proto_tree_add_item(parameter_tree, hf_m_bit,             parameter_tvb, RESERVED_OFFSET,  RESERVED_LENGTH,  NETWORK_BYTE_ORDER);
-  proto_tree_add_item(parameter_tree, hf_reserved,          parameter_tvb, RESERVED_OFFSET,  RESERVED_LENGTH,  NETWORK_BYTE_ORDER);
 
   parameters_tvb = tvb_new_subset(parameter_tvb, SERVER_TRANSPORT_OFFSET, -1, -1);
   dissect_parameters(parameters_tvb, parameter_tree);
@@ -427,16 +525,13 @@ dissect_pe_identifier_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_t
 }
 
 #define PE_CHECKSUM_LENGTH 2
-#define PE_CHECKSUM_RESERVED_LENGTH 2
 
 #define PE_CHECKSUM_OFFSET PARAMETER_VALUE_OFFSET
-#define PE_CHECKSUM_RESERVED_OFFSET (PE_CHECKSUM_OFFSET + PE_CHECKSUM_LENGTH)
 
 static void
 dissect_pe_checksum_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree, proto_item *parameter_item)
 {
-  proto_tree_add_item(parameter_tree, hf_pe_checksum,          parameter_tvb, PE_CHECKSUM_OFFSET,          PE_CHECKSUM_LENGTH,          NETWORK_BYTE_ORDER);
-  proto_tree_add_item(parameter_tree, hf_pe_checksum_reserved, parameter_tvb, PE_CHECKSUM_RESERVED_OFFSET, PE_CHECKSUM_RESERVED_LENGTH, NETWORK_BYTE_ORDER);
+  proto_tree_add_item(parameter_tree, hf_pe_checksum, parameter_tvb, PE_CHECKSUM_OFFSET, PE_CHECKSUM_LENGTH, NETWORK_BYTE_ORDER);
   proto_item_append_text(parameter_item, " (0x%x)", tvb_get_ntohs(parameter_tvb, PE_CHECKSUM_OFFSET));
 }
 
@@ -456,24 +551,28 @@ dissect_unknown_parameter(tvbuff_t *parameter_tvb, proto_tree *parameter_tree, p
 
 #define IPV4_ADDRESS_PARAMETER_TYPE                 0x01
 #define IPV6_ADDRESS_PARAMETER_TYPE                 0x02
-#define SCTP_TRANSPORT_PARAMETER_TYPE               0x03
-#define TCP_TRANSPORT_PARAMETER_TYPE                0x04
-#define UDP_TRANSPORT_PARAMETER_TYPE                0x05
-#define POOL_MEMBER_SELECTION_POLICY_PARAMETER_TYPE 0x06
-#define POOL_HANDLE_PARAMETER_TYPE                  0x07
-#define POOL_ELEMENT_PARAMETER_TYPE                 0x08
-#define SERVER_INFORMATION_PARAMETER_TYPE           0x09
-#define OPERATION_ERROR_PARAMETER_TYPE              0x0a
-#define COOKIE_PARAMETER_TYPE                       0x0b
-#define PE_IDENTIFIER_PARAMETER_TYPE                0x0c
-#define PE_CHECKSUM_PARAMETER_TYPE                  0x0d
+#define DCCP_TRANSPORT_PARAMETER_TYPE               0x03
+#define SCTP_TRANSPORT_PARAMETER_TYPE               0x04
+#define TCP_TRANSPORT_PARAMETER_TYPE                0x05
+#define UDP_TRANSPORT_PARAMETER_TYPE                0x06
+#define UDP_LITE_TRANSPORT_PARAMETER_TYPE           0x07
+#define POOL_MEMBER_SELECTION_POLICY_PARAMETER_TYPE 0x08
+#define POOL_HANDLE_PARAMETER_TYPE                  0x09
+#define POOL_ELEMENT_PARAMETER_TYPE                 0x0a
+#define SERVER_INFORMATION_PARAMETER_TYPE           0x0b
+#define OPERATION_ERROR_PARAMETER_TYPE              0x0c
+#define COOKIE_PARAMETER_TYPE                       0x0d
+#define PE_IDENTIFIER_PARAMETER_TYPE                0x0e
+#define PE_CHECKSUM_PARAMETER_TYPE                  0x0f
 
 static const value_string parameter_type_values[] = {
   { IPV4_ADDRESS_PARAMETER_TYPE,                 "IPV4 address" },
   { IPV6_ADDRESS_PARAMETER_TYPE,                 "IPV6 address" },
+  { DCCP_TRANSPORT_PARAMETER_TYPE,               "DCCP transport address" },
   { SCTP_TRANSPORT_PARAMETER_TYPE,               "SCTP transport address" },
   { TCP_TRANSPORT_PARAMETER_TYPE,                "TCP transport address" },
   { UDP_TRANSPORT_PARAMETER_TYPE,                "UDP transport address" },
+  { UDP_LITE_TRANSPORT_PARAMETER_TYPE,           "UDP-Lite transport address" },
   { POOL_MEMBER_SELECTION_POLICY_PARAMETER_TYPE, "Pool member selection policy" },
   { POOL_HANDLE_PARAMETER_TYPE,                  "Pool handle" },
   { POOL_ELEMENT_PARAMETER_TYPE,                 "Pool element" },
@@ -512,6 +611,9 @@ dissect_parameter(tvbuff_t *parameter_tvb, proto_tree *asap_tree)
   case IPV6_ADDRESS_PARAMETER_TYPE:
     dissect_ipv6_parameter(parameter_tvb, parameter_tree, parameter_item);
     break;
+  case DCCP_TRANSPORT_PARAMETER_TYPE:
+    dissect_dccp_transport_parameter(parameter_tvb, parameter_tree);
+    break;
   case SCTP_TRANSPORT_PARAMETER_TYPE:
     dissect_sctp_transport_parameter(parameter_tvb, parameter_tree);
     break;
@@ -521,6 +623,9 @@ dissect_parameter(tvbuff_t *parameter_tvb, proto_tree *asap_tree)
   case UDP_TRANSPORT_PARAMETER_TYPE:
     dissect_udp_transport_parameter(parameter_tvb, parameter_tree);
     break;
+  case UDP_LITE_TRANSPORT_PARAMETER_TYPE:
+    dissect_udp_lite_transport_parameter(parameter_tvb, parameter_tree);
+    break;
   case POOL_MEMBER_SELECTION_POLICY_PARAMETER_TYPE:
     dissect_pool_member_selection_policy_parameter(parameter_tvb, parameter_tree);
     break;
@@ -641,9 +746,9 @@ dissect_asap_message(tvbuff_t *message_tvb, packet_info *pinfo, proto_tree *asap
   proto_tree *flags_tree;
   guint8 type;
 
-  /* pinfo is NULL only if dissect_enrp_message is called from dissect_error cause */
 
   type = tvb_get_guint8(message_tvb, MESSAGE_TYPE_OFFSET);
+  /* pinfo is NULL only if dissect_asap_message is called via dissect_error cause */
   if (pinfo && (check_col(pinfo->cinfo, COL_INFO)))
     col_add_fstr(pinfo->cinfo, COL_INFO, "%s ", val_to_str(type, message_type_values, "Unknown ASAP type"));
 
@@ -674,7 +779,7 @@ dissect_asap(tvbuff_t *message_tvb, packet_info *pinfo, proto_tree *tree)
   proto_item *asap_item;
   proto_tree *asap_tree;
 
-  /* pinfo is NULL only if dissect_asap_message is called from dissect_error cause */
+  /* pinfo is NULL only if dissect_asap is called from dissect_error cause */
   if (pinfo && (check_col(pinfo->cinfo, COL_PROTOCOL)))
     col_set_str(pinfo->cinfo, COL_PROTOCOL, "ASAP");
 
@@ -711,23 +816,29 @@ proto_register_asap(void)
     { &hf_parameter_padding,      { "Padding",                     "asap.parameter_padding",                        FT_BYTES,   BASE_NONE, NULL,                             0x0,                       "", HFILL } },
     { &hf_parameter_ipv4_address, { "IP Version 4 address",        "asap.ipv4_address",                             FT_IPv4,    BASE_NONE, NULL,                             0x0,                       "", HFILL } },
     { &hf_parameter_ipv6_address, { "IP Version 6 address",        "asap.ipv6_address",                             FT_IPv6,    BASE_NONE, NULL,                             0x0,                       "", HFILL } },
+    { &hf_dccp_port,              { "Port",                        "enrp.dccp_transport_port",                      FT_UINT16,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
+    { &hf_dccp_reserved,          { "Reserved",                    "enrp.dccp_transport_reserved",                  FT_UINT16,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
+    { &hf_dccp_service_code,      { "Service code",                "enrp.dccp_transport_service_code",              FT_UINT16,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
     { &hf_sctp_port,              { "Port",                        "asap.sctp_transport_port",                      FT_UINT16,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
     { &hf_transport_use,          { "Transport use",               "asap.transport_use",                            FT_UINT16,  BASE_DEC,  VALS(transport_use_values),       0x0,                       "", HFILL } },
     { &hf_tcp_port,               { "Port",                        "asap.tcp_transport_port",                       FT_UINT16,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
     { &hf_udp_port,               { "Port",                        "asap.udp_transport_port",                       FT_UINT16,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
     { &hf_udp_reserved,           { "Reserved",                    "asap.udp_transport_reserved",                   FT_UINT16,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
-    { &hf_policy_type,            { "Policy type",                 "asap.pool_member_slection_policy_type",         FT_UINT8,   BASE_DEC,  VALS(policy_type_values),         0x0,                       "", HFILL } },
-    { &hf_policy_value,           { "Policy value",                "asap.pool_member_slection_policy_value",        FT_INT24,   BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
+    { &hf_udp_lite_port,          { "Port",                        "enrp.udp_lite_transport_port",                  FT_UINT16,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
+    { &hf_udp_lite_reserved,      { "Reserved",                    "enrp.udp_lite_transport_reserved",              FT_UINT16,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
+    { &hf_policy_type,            { "Policy type",                 "asap.pool_member_slection_policy_type",         FT_UINT32,  BASE_DEC,  VALS(policy_type_values),         0x0,                       "", HFILL } },
+    { &hf_policy_weight,          { "Policy weight",               "asap.pool_member_slection_policy_weight",       FT_UINT32,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
+    { &hf_policy_priority,        { "Policy priority",             "asap.pool_member_slection_policy_priority",     FT_UINT32,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
+    { &hf_policy_load,            { "Policy load",                 "asap.pool_member_slection_policy_load",         FT_UINT32,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
+    { &hf_policy_degradation,     { "Policy degradation",          "asap.pool_member_slection_policy_degradation",  FT_UINT32,  BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
+    { &hf_policy_value,           { "Policy value",                "asap.pool_member_slection_policy_value",        FT_BYTES,   BASE_HEX,  NULL,                             0x0,                       "", HFILL } },
     { &hf_pool_handle,            { "Pool handle",                 "asap.pool_handle_pool_handle",                  FT_BYTES,   BASE_HEX,  NULL,                             0x0,                       "", HFILL } },
     { &hf_pe_pe_identifier,       { "PE identifier",               "asap.pool_element_pe_identifier",               FT_UINT32,  BASE_HEX,  NULL,                             0x0,                       "", HFILL } },
     { &hf_home_enrp_id,           { "Home ENRP server identifier", "asap.pool_element_home_enrp_server_identifier", FT_UINT32,  BASE_HEX,  NULL,                             0x0,                       "", HFILL } },
     { &hf_reg_life,               { "Registration life",           "asap.pool_element_registration_life",           FT_INT32,   BASE_DEC,  NULL,                             0x0,                       "", HFILL } },
-    { &hf_m_bit,                  { "M-Bit",                       "asap.server_information_m_bit",                 FT_BOOLEAN, 32,        NULL,                             M_BIT_MASK,                "", HFILL } },
-    { &hf_reserved,               { "Reserved",                    "asap.server_information_reserved",              FT_UINT32,  BASE_HEX,  NULL,                             RESERVED_MASK,             "", HFILL } },
     { &hf_cookie,                 { "Cookie",                      "asap.cookie",                                   FT_BYTES,   BASE_HEX,  NULL,                             0x0,                       "", HFILL } },
     { &hf_pe_identifier,          { "PE identifier",               "asap.pe_identifier",                            FT_UINT32,  BASE_HEX,  NULL,                             0x0,                       "", HFILL } },
     { &hf_pe_checksum,            { "PE checksum",                 "asap.pe_checksum",                              FT_UINT32,  BASE_HEX,  NULL,                             0x0,                       "", HFILL } },
-    { &hf_pe_checksum_reserved,   { "Reserved",                    "asap.pe_checksum_reserved",                     FT_UINT16,  BASE_HEX,  NULL,                             0x0,                       "", HFILL } },
     { &hf_server_identifier,      { "Server identifier",           "asap.server_identifier",                        FT_UINT32,  BASE_HEX,  NULL,                             0x0,                       "", HFILL } },
     { &hf_home_enrp_server_bit,   { "H bit",                       "asap.h_bit",                                    FT_BOOLEAN, 8,         TFS(&home_enrp_server_bit_value), HOME_ENRP_SERVER_BIT_MASK, "", HFILL } },
     { &hf_reject_bit,             { "R bit",                       "asap.r_bit",                                    FT_BOOLEAN, 8,         TFS(&reject_bit_value),           REJECT_BIT_MASK,           "", HFILL } },
@@ -758,9 +869,6 @@ proto_reg_handoff_asap(void)
   asap_handle = create_dissector_handle(dissect_asap, proto_asap);
   dissector_add("sctp.ppi",  ASAP_PAYLOAD_PROTOCOL_ID, asap_handle);
   dissector_add("udp.port",  ASAP_UDP_PORT,  asap_handle);
-  /* FIXME */
-  /*
   dissector_add("tcp.port",  ASAP_TCP_PORT,  asap_handle);
   dissector_add("sctp.port", ASAP_SCTP_PORT, asap_handle);
-  */
 }
