@@ -1,7 +1,7 @@
 /* packet-dcom.c
  * Routines for DCOM generics
  *
- * $Id: packet-dcom.c 19844 2006-11-07 07:39:16Z jake $
+ * $Id: packet-dcom.c 24249 2008-02-02 17:25:40Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -87,6 +87,7 @@
 #include <epan/emem.h>
 #include <epan/addr_resolv.h>
 #include <epan/inet_aton.h>
+#include <ctype.h>
 #include <isprint.h>
 #include "packet-dcerpc.h"
 #include "packet-dcom.h"
@@ -252,7 +253,7 @@ void dcom_interface_dump(void) {
 
         for(objects = machine->objects; objects != NULL; objects = g_list_next(objects)) {
             object = objects->data;
-            g_warning(" Object(#%4u): OID:0x%" PRIx64 " private:%p", object->first_packet, object->oid, object->private_data);
+            g_warning(" Object(#%4u): OID:0x%" G_GINT64_MODIFIER "x private:%p", object->first_packet, object->oid, object->private_data);
 
             for(interfaces = object->interfaces; interfaces != NULL; interfaces = g_list_next(interfaces)) {
                 interf = interfaces->data;
@@ -265,7 +266,7 @@ void dcom_interface_dump(void) {
 }
 
 
-dcom_interface_t *dcom_interface_find(packet_info *pinfo, const guint8 *ip, e_uuid_t *ipid) 
+dcom_interface_t *dcom_interface_find(packet_info *pinfo _U_, const guint8 *ip _U_, e_uuid_t *ipid) 
 {
     dcom_interface_t *interf;
     GList *interfaces;
@@ -331,7 +332,7 @@ dcom_interface_t *dcom_interface_new(packet_info *pinfo, const guint8 *ip, e_uui
     /* create new machine if not found */
     if(dcom_iter == NULL) {
         machine = se_alloc(sizeof(dcom_machine_t));
-        memcpy( (void *) (machine->ip), ip, 4);
+        memcpy(machine->ip, ip, 4);
         machine->objects = NULL;
         machine->first_packet = pinfo->fd->num;
         dcom_machines = g_list_append(dcom_machines, machine);
@@ -1034,6 +1035,30 @@ dissect_dcom_indexed_DWORD(tvbuff_t *tvb, int offset,	packet_info *pinfo,
 }
 	
 
+/* dissect hresult field of a usual DCOM call (create "raw" item) */
+int
+dissect_dcom_HRESULT_item(tvbuff_t *tvb, int offset,	packet_info *pinfo,
+					 proto_tree *tree, guint8 *drep, 
+					 guint32 * pu32HResult, int field_index, proto_item **item)
+{
+    guint32 u32HResult;
+
+	/* dissect the DWORD, but don't add to tree */
+	offset = dissect_dcom_DWORD(tvb, offset, pinfo, NULL /*tree*/, drep, 
+                    field_index, &u32HResult);
+
+    if (tree) {
+		/* special formatted output of indexed value */
+        *item = proto_tree_add_item (tree, field_index, tvb, offset-4, 4, (drep[0] & 0x10));
+    }
+
+    if (pu32HResult)
+        *pu32HResult = u32HResult;
+
+	return offset;
+}
+	
+
 /* dissect hresult field of a usual DCOM call (seperate method, because often used) */
 int
 dissect_dcom_HRESULT(tvbuff_t *tvb, int offset,	packet_info *pinfo,
@@ -1115,9 +1140,9 @@ dissect_dcom_COMVERSION(tvbuff_t *tvb, int offset, packet_info *pinfo,
 }
 	
 
-static int
+int
 dissect_dcom_SAFEARRAY(tvbuff_t *tvb, int offset, packet_info *pinfo,
-						proto_tree *tree, guint8 *drep, int hfindex _U_)
+						proto_tree *tree, guint8 *drep, int hfindex _U_, sa_callback_t sacb)
 {
 	guint32 u32Dims;
 	guint16 u16Dims;
@@ -1190,18 +1215,24 @@ dissect_dcom_SAFEARRAY(tvbuff_t *tvb, int offset, packet_info *pinfo,
                         hf_dcom_sa_vartype32, &u32VarType);
 	offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
                         hf_dcom_sa_elements, &u32Elements);
-
 	offset = dissect_dcom_dcerpc_pointer(tvb, offset, pinfo, sub_tree, drep, &u32Pointer);
-	offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
-                        hf_dcom_sa_bound_elements, &u32BoundElements);
-	offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
-                        hf_dcom_sa_low_bound, &u32LowBound);
+
+	while(u32Dims--) {
+		offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
+							hf_dcom_sa_bound_elements, &u32BoundElements);
+		offset = dissect_dcom_DWORD(tvb, offset, pinfo, sub_tree, drep, 
+							hf_dcom_sa_low_bound, &u32LowBound);
+	}
 
 	offset = dissect_dcom_dcerpc_array_size(tvb, offset, pinfo, sub_tree, drep, &u32ArraySize);
 
     tvb_ensure_bytes_exist(tvb, offset, u32ArraySize * u32ElementSize);
 	u32VariableOffset = offset + u32ArraySize * u32ElementSize;
 
+        if(sacb) {
+            sacb(tvb, offset, pinfo, tree, drep, u32VarType, u32ArraySize);
+        }
+
 	u32Tmp = u32ArraySize;
 	while(u32ArraySize--) {
 		switch(u32VarType) {
@@ -1245,12 +1276,12 @@ dissect_dcom_SAFEARRAY(tvbuff_t *tvb, int offset, packet_info *pinfo,
 				u32VariableOffset = dissect_dcom_tobedone_data(tvb, u32VariableOffset, pinfo, sub_tree, drep, 
 								10000);
 		}
-	}
+	    }
 
-	/* update subtree header */
-	proto_item_append_text(sub_item, ": Elements: %u/%u VarType: %s",
-		u32Elements, u32BoundElements,
-		val_to_str(u32VarType, dcom_variant_type_vals, "Unknown (0x%08x)") );
+	    /* update subtree header */
+	    proto_item_append_text(sub_item, ": Elements: %u/%u VarType: %s",
+		    u32Elements, u32BoundElements,
+		    val_to_str(u32VarType, dcom_variant_type_vals, "Unknown (0x%08x)") );
 
 	proto_item_set_len(sub_item, u32VariableOffset - u32SubStart);
 
@@ -1390,7 +1421,7 @@ dissect_dcom_VARIANT(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			break;
 		case(WIRESHARK_VT_ARRAY):
 			offset = dissect_dcom_SAFEARRAY(tvb, offset, pinfo, sub_tree, drep,
-								0);
+								0, NULL);
 			break;
 		case(WIRESHARK_VT_ERROR):
 			offset = dissect_dcom_HRESULT(tvb, offset, pinfo, sub_tree, drep,
@@ -1581,7 +1612,7 @@ dcom_tvb_get_nwstringz0(tvbuff_t *tvb, gint offset, guint32 inLength, gchar *psz
         for(u32IdxA = 0, u32IdxW = 0; 
             u32IdxW < u32Idx && u32IdxA < outLength-2;
             u32IdxW++, u32IdxA+=2) {
-            sprintf(&pszStr[u32IdxA], "%02X", tvb_get_guint8(tvb, offset+u32IdxW));
+		    g_snprintf(&pszStr[u32IdxA], 3, "%02X", tvb_get_guint8(tvb, offset+u32IdxW));
         }
     }
 
@@ -1941,7 +1972,7 @@ dissect_dcom_OBJREF(tvbuff_t *tvb, gint offset, packet_info *pinfo,
         /* add interface instance to database (we currently only handle IPv4) */
         if(pinfo->net_src.type == AT_IPv4) {
             dcom_if = dcom_interface_new(pinfo, 
-                ip,
+	        ip,
                 &iid, oxid, oid, &ipid);
         }
     }
