@@ -1,7 +1,7 @@
 /* packet-eth.c
  * Routines for ethernet packet disassembly
  *
- * $Id: packet-eth.c 23901 2007-12-17 20:42:10Z gerald $
+ * $Id: packet-eth.c 23880 2007-12-16 00:35:34Z sake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -305,13 +305,17 @@ dissect_eth_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree,
 	}
     }
 
-    if (check_col(pinfo->cinfo, COL_INFO))
+    if (check_col(pinfo->cinfo, COL_INFO)) 
       col_set_str(pinfo->cinfo, COL_INFO, "Ethernet II");
     if (parent_tree) {
-      ti = proto_tree_add_protocol_format(parent_tree, proto_eth, tvb, 0, ETH_HEADER_SIZE,
+      if (PTREE_DATA(parent_tree)->visible) {
+          ti = proto_tree_add_protocol_format(parent_tree, proto_eth, tvb, 0, ETH_HEADER_SIZE,
 		"Ethernet II, Src: %s (%s), Dst: %s (%s)",
 		get_ether_name(src_addr), ether_to_str(src_addr), get_ether_name(dst_addr), ether_to_str(dst_addr));
-
+      }
+      else {
+	    ti = proto_tree_add_item(parent_tree, proto_eth, tvb, 0, ETH_HEADER_SIZE, FALSE);
+      }
       fh_tree = proto_item_add_subtree(ti, ett_ether2);
     }
 
