@@ -11,7 +11,7 @@
  *   Technical realization of Short Message Service (SMS)
  *   (3GPP TS 23.040 version 5.4.0 Release 5)
  *
- * $Id: packet-gsm_sms.c 18955 2006-08-19 02:58:48Z guy $
+ * $Id: packet-gsm_sms.c 24286 2008-02-07 15:44:45Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -59,7 +59,6 @@
 
 #include "packet-gsm_sms.h"
 
-
 /* PROTOTYPES/FORWARDS */
 
 #define	EXTRANEOUS_DATA_CHECK(edc_len, edc_max_len) \
@@ -118,6 +117,9 @@ static gint ett_udh = -1;
 /* Initialize the protocol and registered fields */
 static int proto_gsm_sms = -1;
 
+static gint hf_gsm_sms_coding_group_bits2 = -1;
+static gint hf_gsm_sms_coding_group_bits4 = -1;
+
 static char bigbuf[1024];
 static dissector_handle_t data_handle;
 static packet_info *g_pinfo;
@@ -510,6 +512,34 @@ dis_field_pid(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 oct)
 	break;
     }
 }
+/* 3GPP TS 23.038 version 7.0.0 Release 7
+ * The TP-Data-Coding-Scheme field, defined in 3GPP TS 23.040 [4],
+ * indicates the data coding scheme of the TP-UD field, and may indicate a message class.
+ * Any reserved codings shall be assumed to be the GSM 7 bit default alphabet
+ * (the same as codepoint 00000000) by a receiving entity.
+ * The octet is used according to a coding group which is indicated in bits 7..4.
+ */
+
+/* Coding Group Bits */
+static const value_string gsm_sms_coding_group_bits_vals[] = {
+	{ 0,	"General Data Coding indication" }, /* 00xx */
+	{ 1,	"General Data Coding indication" }, /* 00xx */
+	{ 2,	"General Data Coding indication" }, /* 00xx */
+	{ 3,	"General Data Coding indication" }, /* 00xx */
+	{ 4,	"Message Marked for Automatic Deletion Group" }, /* 01xx */
+	{ 5,	"Message Marked for Automatic Deletion Group" }, /* 01xx */
+	{ 6,	"Message Marked for Automatic Deletion Group" }, /* 01xx */
+	{ 7,	"Message Marked for Automatic Deletion Group" }, /* 01xx */
+	{ 8,	"Reserved coding groups" },			/* 1000..1011  */
+	{ 9,	"Reserved coding groups" },			/* 1000..1011  */
+	{ 10,	"Reserved coding groups" },			/* 1000..1011  */
+	{ 11,	"Reserved coding groups" },			/* 1000..1011  */
+	{ 12,	"Message Waiting Indication Group: Discard Message" },/* 1100  */
+	{ 13,	"Message Waiting Indication Group: Store Message" },  /* 1101  */
+	{ 14,	"Message Waiting Indication Group: Store Message" },  /* 1110  */
+	{ 15,	"Data coding/message class" },  /* 1111  */
+    { 0, NULL },
+};
 
 /* 9.2.3.10 */
 static void
@@ -537,6 +567,11 @@ dis_field_dcs(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 oct,
 	    oct);
 
     subtree = proto_item_add_subtree(item, ett_dcs);
+	if(oct&0x80){
+		proto_tree_add_item(subtree, hf_gsm_sms_coding_group_bits4, tvb, offset, 1, FALSE);
+	}else{
+		proto_tree_add_item(subtree, hf_gsm_sms_coding_group_bits2, tvb, offset, 1, FALSE);
+	}
 
     if (oct == 0x00)
     {
@@ -637,7 +672,9 @@ dis_field_dcs(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 oct,
 	case 0x00: str = "GSM 7 bit default alphabet";
 	    *seven_bit = TRUE;
 	    break;
-	case 0x01: str = "8 bit data"; break;
+	case 0x01: str = "8 bit data";
+		*eight_bit = TRUE;
+		break;
 	case 0x02: str = "UCS2 (16 bit)";
 	    *ucs2 = TRUE;
 	    break;
@@ -1496,21 +1533,21 @@ char_def_alphabet_ext_decode(unsigned char value)
 {
     switch (value)
     {
-    case 0x0a: return 0x0c; break; /* form feed */
-    case 0x14: return '^';  break;
-    case 0x28: return '{';  break;
-    case 0x29: return '}';  break;
-    case 0x2f: return '\\'; break;
-    case 0x3c: return '[';  break;
-    case 0x3d: return '~';  break;
-    case 0x3e: return ']';  break;
-    case 0x40: return '|';  break;
-    case 0x65: return 0x20ac; break; /* euro */
-    default: return '?';    break; /* invalid character */
+    case 0x0a: return 0x0c; /* form feed */
+    case 0x14: return '^';
+    case 0x28: return '{';
+    case 0x29: return '}';
+    case 0x2f: return '\\';
+    case 0x3c: return '[';
+    case 0x3d: return '~';
+    case 0x3e: return ']';
+    case 0x40: return '|';
+    case 0x65: return 0x20ac; /* euro */
+    default: return '?'; /* invalid character */
     }
 }
 
-static gunichar 
+static gunichar
 char_def_alphabet_decode(unsigned char value)
 {
     if (value < GN_CHAR_ALPHABET_SIZE)
@@ -1658,7 +1695,7 @@ dis_field_ud_iei(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 length)
     guint8	iei_len;
 
 
-    while (length > 2)
+    while (length >= 2)
     {
 	iei_fcn = NULL;
 
@@ -1793,6 +1830,12 @@ dis_field_ud(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint32 length, gb
     guint32	out_len;
     char	*ustr;
     char        messagebuf[160];
+#if GLIB_MAJOR_VERSION >= 2
+    proto_item *ucs2_item;
+    gchar *utf8_text = NULL;
+    GIConv cd;
+    GError *l_conv_error = NULL;
+#endif
 
     fill_bits = 0;
 
@@ -1874,10 +1917,30 @@ dis_field_ud(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint32 length, gb
 	        tvb_format_text(tvb, offset, length));
 		}
 		else if (ucs2)
+		{
+#if GLIB_MAJOR_VERSION >= 2
+			if ((cd = g_iconv_open("UTF-8","UCS-2BE")) != (GIConv)-1)
 			{
-			/* tvb_get_ephemeral_faked_unicode takes the lengt in number of guint16's */
-			ustr = tvb_get_ephemeral_faked_unicode(tvb, offset, (length>>1), FALSE);
-			proto_tree_add_text(subtree, tvb, offset, length, "%s", ustr);
+				utf8_text = g_convert_with_iconv(tvb->real_data +  offset, length , cd , NULL , NULL , &l_conv_error);
+				if(!l_conv_error){
+					ucs2_item = proto_tree_add_text(subtree, tvb, offset, length, "UCS-2 to UTF-8 converted text: %s", utf8_text);
+				}else{
+					ucs2_item = proto_tree_add_text(subtree, tvb, offset, length, "%s", "Failed on UCS2 contact wireshark developers");
+				}
+				PROTO_ITEM_SET_GENERATED(ucs2_item);
+				if(utf8_text)
+					g_free(utf8_text);
+				g_iconv_close(cd);
+			}
+			else
+			{
+#endif
+				/* tvb_get_ephemeral_faked_unicode takes the lengt in number of guint16's */
+				ustr = tvb_get_ephemeral_faked_unicode(tvb, offset, (length>>1), FALSE);
+				proto_tree_add_text(subtree, tvb, offset, length, "%s", ustr);
+#if GLIB_MAJOR_VERSION >= 2
+			}
+#endif
 		}
     }
 }
@@ -2625,12 +2688,21 @@ proto_register_gsm_sms(void)
     guint		i;
     guint		last_offset;
 
-#if 0
+
     /* Setup list of header fields */
     static hf_register_info hf[] =
     {
+	    { &hf_gsm_sms_coding_group_bits2,
+	      { "Coding Group Bits", "gsm_sms.coding_group_bits2",
+	        FT_UINT8, BASE_DEC, VALS(gsm_sms_coding_group_bits_vals), 0xc0,
+	        "Coding Group Bits", HFILL }
+		},
+	    { &hf_gsm_sms_coding_group_bits4,
+	      { "Coding Group Bits", "gsm_sms.coding_group_bits4",
+	        FT_UINT8, BASE_DEC, VALS(gsm_sms_coding_group_bits_vals), 0xf0,
+	        "Coding Group Bits", HFILL }
+		},
     };
-#endif
 
     /* Setup protocol subtree array */
 #define	NUM_INDIVIDUAL_PARMS	12
@@ -2668,9 +2740,8 @@ proto_register_gsm_sms(void)
     proto_gsm_sms =
 	proto_register_protocol(gsm_sms_proto_name, gsm_sms_proto_name_short, "gsm_sms");
 
-#if 0
+
     proto_register_field_array(proto_gsm_sms, hf, array_length(hf));
-#endif
 
     proto_register_subtree_array(ett, array_length(ett));
 }
