@@ -4,7 +4,7 @@
  * Copyright 2002, Richard Sharpe <rsharpe@samba.org> Added a few 
  *		   bits and pieces ...
  *
- * $Id: packet-gssapi.c 22627 2007-08-24 07:12:04Z jmayer $
+ * $Id: packet-gssapi.c 24058 2008-01-10 16:35:54Z richardv $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -492,31 +492,31 @@ proto_register_gssapi(void)
 {
 	static hf_register_info hf[] = {
 	{ &hf_gssapi_oid, 
-	    	{ "OID", "gssapi.OID", FT_STRING, BASE_NONE,
+	    	{ "OID", "gss-api.OID", FT_STRING, BASE_NONE,
 		  NULL, 0, "This is a GSS-API Object Identifier", HFILL }},
 	{ &hf_gssapi_segment,
-		{ "GSSAPI Segment", "gssapi.segment", FT_FRAMENUM, BASE_NONE, 
+		{ "GSSAPI Segment", "gss-api.segment", FT_FRAMENUM, BASE_NONE, 
 		  NULL, 0x0, "GSSAPI Segment", HFILL }},
 	{ &hf_gssapi_segments,
-		{ "GSSAPI Segments", "gssapi.segment.segments", FT_NONE, BASE_NONE, 
+		{ "GSSAPI Segments", "gss-api.segment.segments", FT_NONE, BASE_NONE, 
 		  NULL, 0x0, "GSSAPI Segments", HFILL }},
 	{ &hf_gssapi_segment_overlap,
-		{ "Fragment overlap",	"gssapi.segment.overlap", FT_BOOLEAN, BASE_NONE, 
+		{ "Fragment overlap",	"gss-api.segment.overlap", FT_BOOLEAN, BASE_NONE, 
 		   NULL, 0x0, "Fragment overlaps with other fragments", HFILL }},
 	{ &hf_gssapi_segment_overlap_conflict,
-		{ "Conflicting data in fragment overlap",	"gssapi.segment.overlap.conflict", FT_BOOLEAN, BASE_NONE, 
+		{ "Conflicting data in fragment overlap",	"gss-api.segment.overlap.conflict", FT_BOOLEAN, BASE_NONE, 
 		  NULL, 0x0, "Overlapping fragments contained conflicting data", HFILL }},
 	{ &hf_gssapi_segment_multiple_tails,
-		{ "Multiple tail fragments found",	"gssapi.segment.multipletails", FT_BOOLEAN, BASE_NONE, 
+		{ "Multiple tail fragments found",	"gss-api.segment.multipletails", FT_BOOLEAN, BASE_NONE, 
 		  NULL, 0x0, "Several tails were found when defragmenting the packet", HFILL }},
 	{ &hf_gssapi_segment_too_long_fragment,
-		{ "Fragment too long",	"gssapi.segment.toolongfragment", FT_BOOLEAN, BASE_NONE, 
+		{ "Fragment too long",	"gss-api.segment.toolongfragment", FT_BOOLEAN, BASE_NONE, 
 		  NULL, 0x0, "Fragment contained data past end of packet", HFILL }},
 	{ &hf_gssapi_segment_error,
-		{ "Defragmentation error", "gssapi.segment.error", FT_FRAMENUM, BASE_NONE, 
+		{ "Defragmentation error", "gss-api.segment.error", FT_FRAMENUM, BASE_NONE, 
 		  NULL, 0x0, "Defragmentation error due to illegal fragments", HFILL }},
 	{ &hf_gssapi_reassembled_in,
-		{ "Reassembled In", "gssapi.reassembled_in", FT_FRAMENUM, BASE_DEC, 
+		{ "Reassembled In", "gss-api.reassembled_in", FT_FRAMENUM, BASE_DEC, 
 		  NULL, 0x0, "The frame where this pdu is reassembled", HFILL }},
 
 	};
