@@ -2,7 +2,7 @@
  * Routines for H.264 dissection
  * Copyright 2007, Anders Broman <anders.broman[at]ericsson.com>
  *
- * $Id: packet-h264.c 23891 2007-12-16 20:27:55Z stig $
+ * $Id: packet-h264.c 24525 2008-03-01 17:23:39Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -39,6 +39,8 @@
 
 #include <epan/packet.h>
 #include <epan/proto.h>
+#include <epan/asn1.h>
+#include <epan/strutil.h>
 
 #include "prefs.h"
 
@@ -100,12 +102,29 @@ static int hf_h264_num_ref_idx_l1_active_minus1		= -1;
 static int hf_h264_weighted_pred_flag				= -1;
 static int hf_h264_weighted_bipred_idc				= -1;
 static int hf_h264_pic_init_qp_minus26				= -1;
+static int hf_h264_pic_init_qs_minus26				= -1;
 static int hf_h264_chroma_qp_index_offset			= -1;
 static int hf_h264_deblocking_filter_control_present_flag = -1;
 static int hf_h264_constrained_intra_pred_flag		= -1;
 static int hf_h264_redundant_pic_cnt_present_flag	= -1;
 static int hf_h264_transform_8x8_mode_flag			= -1;
 static int hf_h264_pic_scaling_matrix_present_flag	= -1;
+static int hf_h264_second_chroma_qp_index_offset	= -1;
+static int hf_h264_par_profile						= -1;
+static int hf_h264_par_profile_b					= -1;
+static int hf_h264_par_profile_m					= -1;
+static int hf_h264_par_profile_e					= -1;
+static int hf_h264_par_profile_h					= -1;
+static int hf_h264_par_profile_h10					= -1;
+static int hf_h264_par_profile_h4_2_2				= -1;
+static int hf_h264_par_profile_h4_4_4				= -1;
+static int hf_h264_par_add_mode_sup					= -1;
+static int hf_h264_par_AdditionalModesSupported		= -1;
+static int hf_h264_par_add_mode_sup_rcdo			= -1;
+static int hf_h264_par_ProfileIOP					= -1;
+static int hf_h264_par_constraint_set0_flag			= -1;
+static int hf_h264_par_constraint_set1_flag			= -1;
+static int hf_h264_par_constraint_set2_flag			= -1;
 
 /* VUI parameters */
 static int hf_h264_aspect_ratio_info_present_flag	= -1;
@@ -162,12 +181,26 @@ static int ett_h264_profile = -1;
 static int ett_h264_nal = -1;
 static int ett_h264_stream = -1;
 static int ett_h264_nal_unit = -1;
+static int ett_h264_par_profile = -1;
+static int ett_h264_par_AdditionalModesSupported = -1;
+static int ett_h264_par_ProfileIOP				 = -1;
 
 /* The dynamic payload type which will be dissected as H.264 */
 
 static guint dynamic_payload_type = 0;
 static guint temp_dynamic_payload_type = 0;
 
+/* syntax tables in subclause 7.3 is equal to
+ * ue(v), me(v), se(v), or te(v).
+ */
+typedef enum {
+    H264_UE_V = 0,
+    H264_ME_V = 1,
+    H264_SE_V = 2,
+    H264_TE_V = 3
+} h264_golomb_descriptors;
+
+
 static const true_false_string h264_f_bit_vals = {
   "Bit errors or other syntax violations",
   "No bit errors or other syntax violations"
@@ -292,11 +325,12 @@ static const value_string h264_slice_type_vals[] = {
 #define cVALS(x) (const value_string*)(x)
 
 guint32
-dissect_h264_exp_golomb_code(proto_tree *tree, int hf_index, tvbuff_t *tvb, gint *start_bit_offset)
+dissect_h264_exp_golomb_code(proto_tree *tree, int hf_index, tvbuff_t *tvb, gint *start_bit_offset, h264_golomb_descriptors descriptor)
 /*(tvbuff_t *tvb, gint *start_bit_offset) */
 {
 	gint		leading_zero_bits, bit_offset;
 	guint32		codenum, mask, value, tmp;
+	gint32		se_value=0;		
 	gint		b;
 	char *str;
 	int bit;
@@ -313,25 +347,25 @@ dissect_h264_exp_golomb_code(proto_tree *tree, int hf_index, tvbuff_t *tvb, gint
 	str[0]='\0';
 	for(bit=0;bit<((int)(bit_offset&0x07));bit++){
 		if(bit&&(!(bit%4))){
-			strcat(str, " ");
+			g_strlcat(str, " ", 256);
 		}
-		strcat(str,".");
+		g_strlcat(str,".", 256);
 	}
 
 
 	leading_zero_bits = -1;
 	for( b = 0; !b; leading_zero_bits++ ){
 		if(bit&&(!(bit%4))){
-			strcat(str, " ");
+			g_strlcat(str, " ", 256);
 		}
 		if(bit&&(!(bit%8))){
-			strcat(str, " ");
+			g_strlcat(str, " ", 256);
 		}
 		b = tvb_get_bits8(tvb, bit_offset, 1);
 		if(b != 0){
-			strcat(str, "1");
+			g_strlcat(str, "1", 256);
 		} else {
-			strcat(str, "0");
+			g_strlcat(str, "0", 256);
 		}
 		bit++;
 		bit_offset++;
@@ -342,13 +376,45 @@ dissect_h264_exp_golomb_code(proto_tree *tree, int hf_index, tvbuff_t *tvb, gint
 		*start_bit_offset = bit_offset;
 		for(;bit%8;bit++){
 			if(bit&&(!(bit%4))){
-				strcat(str, " ");
+				g_strlcat(str, " ", 256);
 			}
-		strcat(str,".");
+		g_strlcat(str,".", 256);
 		}
 		if(hf_field){
-			strcat(str," = ");
-			strcat(str,hf_field->name);
+			g_strlcat(str," = ", 256);
+			g_strlcat(str,hf_field->name, 256);
+			switch (descriptor){
+			case H264_SE_V:
+				/* if the syntax element is coded as se(v), 
+				 * the value of the syntax element is derived by invoking the
+				 * mapping process for signed Exp-Golomb codes as specified in 
+				 * subclause 9.1.1 with codeNum as the input.
+				 */
+				if(hf_field->type==FT_INT32){
+					if (hf_field->strings) {
+						proto_tree_add_int_format(tree, hf_index, tvb, bit_offset>>3, 1, codenum,
+							  "%s: %s (%d)",
+							  str,
+							  val_to_str(codenum, cVALS(hf_field->strings), "Unknown "),
+							  codenum);
+					}else{
+						switch(hf_field->display){
+							case BASE_DEC:
+								proto_tree_add_int_format(tree, hf_index, tvb, bit_offset>>3, 1, codenum,
+							         "%s: %d",
+									  str,
+									  codenum);
+								break;
+							default:
+								DISSECTOR_ASSERT_NOT_REACHED();
+								break;
+						}
+					}
+				}
+				return codenum;
+			default:
+				break;
+			}
 			if(hf_field->type==FT_UINT32){
 				if (hf_field->strings) {
 					proto_tree_add_uint_format(tree, hf_index, tvb, bit_offset>>3, 1, codenum,
@@ -406,31 +472,61 @@ dissect_h264_exp_golomb_code(proto_tree *tree, int hf_index, tvbuff_t *tvb, gint
 	/* read the bits for the int */
 	for(i=0;i<leading_zero_bits;i++){
 		if(bit&&(!(bit%4))){
-			strcat(str, " ");
+			g_strlcat(str, " ", 256);
 		}
 		if(bit&&(!(bit%8))){
-			strcat(str, " ");
+			g_strlcat(str, " ", 256);
 		}
 		bit++;
 		tmp = value & mask;
 		if(tmp != 0){
-			strcat(str, "1");
+			g_strlcat(str, "1", 256);
 		} else {
-			strcat(str, "0");
+			g_strlcat(str, "0", 256);
 		}
 		mask = mask>>1;
 	}
 	for(;bit%8;bit++){
 		if(bit&&(!(bit%4))){
-			strcat(str, " ");
+			g_strlcat(str, " ", 256);
 		}
-		strcat(str,".");
+		g_strlcat(str,".", 256);
+	}
+
+	switch (descriptor){
+	case H264_SE_V:
+		/* if the syntax element is coded as se(v), 
+		 * the value of the syntax element is derived by invoking the
+		 * mapping process for signed Exp-Golomb codes as specified in 
+		 * subclause 9.1.1 with codeNum as the input.
+		 *		k+1 
+		 * (-1)    Ceil( k2 )
+		 */
+		se_value = (codenum + 1) >> 1;
+		if (!(se_value & 1)){
+			se_value =  - se_value;
+		}
+		break;
+	default:
+		break;
 	}
 
 	if(hf_field){
-		strcat(str," = ");
-		strcat(str,hf_field->name);
-		if(hf_field->type==FT_UINT32){
+		g_strlcat(str," = ", 256);
+		g_strlcat(str,hf_field->name, 256);
+		switch (descriptor){
+		case H264_SE_V:
+			g_strlcat(str,"(se(v))", 256);
+			/* if the syntax element is coded as se(v), 
+			 * the value of the syntax element is derived by invoking the
+			 * mapping process for signed Exp-Golomb codes as specified in 
+			 * subclause 9.1.1 with codeNum as the input.
+			 */
+			break;
+		default:
+		break;
+		}
+		if((hf_field->type==FT_UINT32)&&(descriptor==H264_UE_V)){
 			if (hf_field->strings) {
 				proto_tree_add_uint_format(tree, hf_index, tvb, bit_offset>>3, 1, codenum,
 						  "%s: %s (%u)",
@@ -456,6 +552,29 @@ dissect_h264_exp_golomb_code(proto_tree *tree, int hf_index, tvbuff_t *tvb, gint
 						break;
 				}
 			}
+		}else if((hf_field->type==FT_INT32)&&(descriptor==H264_SE_V)){
+			if (hf_field->strings) {
+				proto_tree_add_int_format(tree, hf_index, tvb, bit_offset>>3, 1, codenum,
+						  "%s: %s (%d)",
+						  str,
+						  val_to_str(codenum, cVALS(hf_field->strings), "Unknown "),
+						  se_value);
+			}else{
+				switch(hf_field->display){
+					case BASE_DEC:
+						proto_tree_add_int_format(tree, hf_index, tvb, bit_offset>>3, 1, codenum,
+					         "%s: %d",
+							  str,
+							  se_value);
+						break;
+					default:
+						DISSECTOR_ASSERT_NOT_REACHED();
+						break;
+				}
+			}
+			*start_bit_offset = bit_offset;
+			return se_value;
+
 		}else{
 			/* Only allow guint32 */
 			DISSECTOR_ASSERT_NOT_REACHED();
@@ -532,13 +651,13 @@ static int
 dissect_h264_slice_header(proto_tree *tree, tvbuff_t *tvb, packet_info *pinfo _U_, gint bit_offset)
 {
 	/* first_mb_in_slice 2 ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_first_mb_in_slice, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_first_mb_in_slice, tvb, &bit_offset, H264_UE_V);
 
 	/* slice_type 2 ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_slice_type, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_slice_type, tvb, &bit_offset, H264_UE_V);
 	
 	/* pic_parameter_set_id 2 ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_pic_parameter_set_id, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_pic_parameter_set_id, tvb, &bit_offset, H264_UE_V);
 	
 	/* frame_num 2 u(v) */
 	/* XXXX Is frame_num allways 4 bits in the bit stream? */
@@ -583,7 +702,7 @@ dissect_h264_hrd_parameters(proto_tree *tree, tvbuff_t *tvb, packet_info *pinfo
 
 
 	/* cpb_cnt_minus1 0 ue(v) */
-	cpb_cnt_minus1 = dissect_h264_exp_golomb_code(tree, hf_h264_cpb_cnt_minus1, tvb, &bit_offset);
+	cpb_cnt_minus1 = dissect_h264_exp_golomb_code(tree, hf_h264_cpb_cnt_minus1, tvb, &bit_offset, H264_UE_V);
 
 	/* bit_rate_scale 0 u(4) */
 	proto_tree_add_bits_item(tree, hf_h264_bit_rate_scale, tvb, bit_offset, 4, FALSE);
@@ -596,10 +715,10 @@ dissect_h264_hrd_parameters(proto_tree *tree, tvbuff_t *tvb, packet_info *pinfo
 	for( SchedSelIdx = 0; SchedSelIdx <= cpb_cnt_minus1; SchedSelIdx++ ) {
 
 		/* bit_rate_value_minus1[ SchedSelIdx ] 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_bit_rate_value_minus1, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_bit_rate_value_minus1, tvb, &bit_offset, H264_UE_V);
 
 		/* cpb_size_value_minus1[ SchedSelIdx ] 0 ue(v)*/
-		dissect_h264_exp_golomb_code(tree, hf_h264_cpb_size_value_minus1, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_cpb_size_value_minus1, tvb, &bit_offset, H264_UE_V);
 
 		/* cbr_flag[ SchedSelIdx ] 0 u(1) */
 		proto_tree_add_bits_item(tree, hf_h264_cbr_flag, tvb, bit_offset, 1, FALSE);
@@ -711,10 +830,10 @@ dissect_h264_vui_parameters(proto_tree *tree, tvbuff_t *tvb, packet_info *pinfo,
 
 	if( chroma_loc_info_present_flag ) {
 		/* chroma_sample_loc_type_top_field 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_chroma_sample_loc_type_top_field, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_chroma_sample_loc_type_top_field, tvb, &bit_offset, H264_UE_V);
 
 		/* chroma_sample_loc_type_bottom_field 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_chroma_sample_loc_type_bottom_field, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_chroma_sample_loc_type_bottom_field, tvb, &bit_offset, H264_UE_V);
 	}
 
 	/* timing_info_present_flag 0 u(1) */
@@ -770,25 +889,26 @@ dissect_h264_vui_parameters(proto_tree *tree, tvbuff_t *tvb, packet_info *pinfo,
 
 	if( bitstream_restriction_flag ) {
 		/* motion_vectors_over_pic_boundaries_flag 0 u(1) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_motion_vectors_over_pic_boundaries_flag, tvb, &bit_offset);
+		proto_tree_add_bits_item(tree, hf_h264_motion_vectors_over_pic_boundaries_flag, tvb, bit_offset, 1, FALSE);
+		bit_offset++;
 
 		/* max_bytes_per_pic_denom 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_max_bytes_per_pic_denom, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_max_bytes_per_pic_denom, tvb, &bit_offset, H264_UE_V);
 		
 		/* max_bits_per_mb_denom 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_max_bits_per_mb_denom, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_max_bits_per_mb_denom, tvb, &bit_offset, H264_UE_V);
 		
 		/* log2_max_mv_length_horizontal 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_log2_max_mv_length_horizontal, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_log2_max_mv_length_horizontal, tvb, &bit_offset, H264_UE_V);
 		
 		/* log2_max_mv_length_vertical 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_log2_max_mv_length_vertical, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_log2_max_mv_length_vertical, tvb, &bit_offset, H264_UE_V);
 		
 		/* num_reorder_frames 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_num_reorder_frames, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_num_reorder_frames, tvb, &bit_offset, H264_UE_V);
 		
 		/* max_dec_frame_buffering 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_max_dec_frame_buffering, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_max_dec_frame_buffering, tvb, &bit_offset, H264_UE_V);
 	}
 
 	return bit_offset;
@@ -872,7 +992,7 @@ dissect_h264_slice_data_partition_a_layer_rbsp(proto_tree *tree, tvbuff_t *tvb,
 	bit_offset = dissect_h264_slice_header(tree, tvb, pinfo, bit_offset);
 
 	/* slice_id All ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_slice_id, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_slice_id, tvb, &bit_offset, H264_UE_V);
 	proto_tree_add_text(tree, tvb, bit_offset>>3, -1, "[Not decoded yet]");
 	return; 
 	/* slice_data( ) * only category 2 parts of slice_data( ) syntax * 2*/
@@ -892,7 +1012,7 @@ dissect_h264_slice_data_partition_b_layer_rbsp(proto_tree *tree, tvbuff_t *tvb,
 	bit_offset = offset <<3;
 
 	/* slice_id All ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_slice_id, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_slice_id, tvb, &bit_offset, H264_UE_V);
 	/* if( redundant_pic_cnt_present_flag )	*/
 	/* redundant_pic_cnt All ue(v) */
 	/* slice_data( ) * only category 3 parts of slice_data( ) syntax * 3 */
@@ -913,7 +1033,7 @@ dissect_h264_slice_data_partition_c_layer_rbsp(proto_tree *tree, tvbuff_t *tvb,
 	bit_offset = offset <<3;
 
 	/* slice_id All ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_slice_id, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_slice_id, tvb, &bit_offset, H264_UE_V);
 	/* if( redundant_pic_cnt_present_flag ) */
 	/* redundant_pic_cnt All ue(v) */
 	/* slice_data( ) * only category 4 parts of slice_data( ) syntax * 4 */
@@ -986,14 +1106,14 @@ dissect_h264_seq_parameter_set_rbsp(proto_tree *tree, tvbuff_t *tvb, packet_info
 	 * The parsing process for this descriptor is specified in subclause 9.1.
 	 */
 	bit_offset = offset<<3;
-	dissect_h264_exp_golomb_code(tree, hf_h264_seq_parameter_set_id, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_seq_parameter_set_id, tvb, &bit_offset, H264_UE_V);
 
 
 	if( profile_idc == 100 || profile_idc == 110 ||
 		profile_idc == 122 || profile_idc == 144 ) {
 
 		/* chroma_format_idc 0 ue(v) */
-		chroma_format_idc = dissect_h264_exp_golomb_code(tree, hf_h264_chroma_format_idc, tvb, &bit_offset);
+		chroma_format_idc = dissect_h264_exp_golomb_code(tree, hf_h264_chroma_format_idc, tvb, &bit_offset, H264_UE_V);
 		if( chroma_format_idc == 3 ){
 			/* residual_colour_transform_flag 0 u(1) */
 			proto_tree_add_bits_item(tree, hf_h264_residual_colour_transform_flag, tvb, bit_offset, 1, FALSE);
@@ -1001,16 +1121,19 @@ dissect_h264_seq_parameter_set_rbsp(proto_tree *tree, tvbuff_t *tvb, packet_info
 		}
 
 		/* bit_depth_luma_minus8 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_bit_depth_luma_minus8, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_bit_depth_luma_minus8, tvb, &bit_offset, H264_UE_V);
 
 		/* bit_depth_chroma_minus8 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_bit_depth_chroma_minus8, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_bit_depth_chroma_minus8, tvb, &bit_offset, H264_UE_V);
 
 		/* qpprime_y_zero_transform_bypass_flag 0 u(1) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_qpprime_y_zero_transform_bypass_flag, tvb, &bit_offset);
+		proto_tree_add_bits_item(tree, hf_h264_qpprime_y_zero_transform_bypass_flag, tvb, bit_offset, 1, FALSE);
+		bit_offset++;
 
 		/* seq_scaling_matrix_present_flag 0 u(1) */
-		seq_scaling_matrix_present_flag = dissect_h264_exp_golomb_code(tree, hf_h264_seq_scaling_matrix_present_flag, tvb, &bit_offset);
+		seq_scaling_matrix_present_flag = tvb_get_bits8(tvb, bit_offset, 1);
+		proto_tree_add_bits_item(tree, hf_h264_seq_scaling_matrix_present_flag, tvb, bit_offset, 1, FALSE);
+		bit_offset++;
 		
 		if( seq_scaling_matrix_present_flag ){
 			/*
@@ -1034,45 +1157,45 @@ dissect_h264_seq_parameter_set_rbsp(proto_tree *tree, tvbuff_t *tvb, packet_info
 	}
 
 	/* log2_max_frame_num_minus4 0 ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_log2_max_frame_num_minus4, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_log2_max_frame_num_minus4, tvb, &bit_offset, H264_UE_V);
 
 	/* pic_order_cnt_type 0 ue(v) */
 	offset = bit_offset>>3;
-	pic_order_cnt_type = dissect_h264_exp_golomb_code(tree,hf_h264_pic_order_cnt_type, tvb, &bit_offset);
+	pic_order_cnt_type = dissect_h264_exp_golomb_code(tree,hf_h264_pic_order_cnt_type, tvb, &bit_offset, H264_UE_V);
 
 	if(pic_order_cnt_type == 0){
 		/* log2_max_pic_order_cnt_lsb_minus4 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_log2_max_pic_order_cnt_lsb_minus4, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_log2_max_pic_order_cnt_lsb_minus4, tvb, &bit_offset, H264_UE_V);
 	}else if(pic_order_cnt_type == 1) {
 		/* delta_pic_order_always_zero_flag 0 u(1) */
 		proto_tree_add_bits_item(tree, hf_h264_delta_pic_order_always_zero_flag, tvb, bit_offset, 1, FALSE);
 		bit_offset++;
 
 		/* offset_for_non_ref_pic 0 se(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_offset_for_non_ref_pic, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_offset_for_non_ref_pic, tvb, &bit_offset, H264_SE_V);
 		
 		/* offset_for_top_to_bottom_field 0 se(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_offset_for_top_to_bottom_field, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_offset_for_top_to_bottom_field, tvb, &bit_offset, H264_SE_V);
 		
 		/* num_ref_frames_in_pic_order_cnt_cycle 0 ue(v) */
-		num_ref_frames_in_pic_order_cnt_cycle = dissect_h264_exp_golomb_code(tree, hf_264_num_ref_frames_in_pic_order_cnt_cycle, tvb, &bit_offset);
+		num_ref_frames_in_pic_order_cnt_cycle = dissect_h264_exp_golomb_code(tree, hf_264_num_ref_frames_in_pic_order_cnt_cycle, tvb, &bit_offset, H264_UE_V);
 		for( i = 0; i < num_ref_frames_in_pic_order_cnt_cycle; i++ ){
 			/*offset_for_ref_frame[ i ] 0 se(v)*/
-			dissect_h264_exp_golomb_code(tree, hf_h264_offset_for_ref_frame, tvb, &bit_offset);
+			dissect_h264_exp_golomb_code(tree, hf_h264_offset_for_ref_frame, tvb, &bit_offset, H264_SE_V);
 		}
 	} 
 	/* num_ref_frames 0 ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_num_ref_frames, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_num_ref_frames, tvb, &bit_offset, H264_UE_V);
 
 	/* 	gaps_in_frame_num_value_allowed_flag 0 u(1) */
 	proto_tree_add_bits_item(tree, hf_h264_gaps_in_frame_num_value_allowed_flag, tvb, bit_offset, 1, FALSE);
 	bit_offset++;
 	
 	/* 	pic_width_in_mbs_minus1 0 ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_pic_width_in_mbs_minus1, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_pic_width_in_mbs_minus1, tvb, &bit_offset, H264_UE_V);
 	
 	/* pic_height_in_map_units_minus1 0 ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_pic_height_in_map_units_minus1, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_pic_height_in_map_units_minus1, tvb, &bit_offset, H264_UE_V);
 
 	/* frame_mbs_only_flag 0 u(1) */
 	frame_mbs_only_flag = tvb_get_bits8(tvb, bit_offset, 1);
@@ -1095,10 +1218,10 @@ dissect_h264_seq_parameter_set_rbsp(proto_tree *tree, tvbuff_t *tvb, packet_info
 
 	if(frame_cropping_flag) {
 		/* frame_crop_left_offset 0 ue(v) */
-		dissect_h264_exp_golomb_code(tree, hf_h264_frame_crop_left_offset, tvb, &bit_offset);
-		dissect_h264_exp_golomb_code(tree,hf_h264_frame_crop_right_offset, tvb, &bit_offset);
-		dissect_h264_exp_golomb_code(tree, hf_h264_frame_crop_top_offset, tvb, &bit_offset);
-		dissect_h264_exp_golomb_code(tree, hf_h264_frame_crop_bottom_offset, tvb, &bit_offset);
+		dissect_h264_exp_golomb_code(tree, hf_h264_frame_crop_left_offset, tvb, &bit_offset, H264_UE_V);
+		dissect_h264_exp_golomb_code(tree,hf_h264_frame_crop_right_offset, tvb, &bit_offset, H264_UE_V);
+		dissect_h264_exp_golomb_code(tree, hf_h264_frame_crop_top_offset, tvb, &bit_offset, H264_UE_V);
+		dissect_h264_exp_golomb_code(tree, hf_h264_frame_crop_bottom_offset, tvb, &bit_offset, H264_UE_V);
 
 	}
 
@@ -1122,15 +1245,15 @@ dissect_h264_pic_parameter_set_rbsp(proto_tree *tree, tvbuff_t *tvb, packet_info
 {
 
 	gint bit_offset;
-	guint32 num_slice_groups_minus1, slice_group_map_type;
+	guint32 num_slice_groups_minus1, slice_group_map_type, pic_scaling_matrix_present_flag;
 
 	bit_offset = offset<<3;
 
 	/* pic_parameter_set_id 1 ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_pic_parameter_set_id, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_pic_parameter_set_id, tvb, &bit_offset, H264_UE_V);
 
 	/* seq_parameter_set_id 1 ue(v) */
-	dissect_h264_exp_golomb_code(tree, hf_h264_seq_parameter_set_id, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_seq_parameter_set_id, tvb, &bit_offset, H264_UE_V);
 
 	/* entropy_coding_mode_flag 1 u(1) */
 	proto_tree_add_bits_item(tree, hf_h264_entropy_coding_mode_flag, tvb, bit_offset, 1, FALSE);
@@ -1141,10 +1264,10 @@ dissect_h264_pic_parameter_set_rbsp(proto_tree *tree, tvbuff_t *tvb, packet_info
 	bit_offset++;
 
 	/* num_slice_groups_minus1 1 ue(v)*/
-	num_slice_groups_minus1 = dissect_h264_exp_golomb_code(tree, hf_h264_num_slice_groups_minus1, tvb, &bit_offset);
+	num_slice_groups_minus1 = dissect_h264_exp_golomb_code(tree, hf_h264_num_slice_groups_minus1, tvb, &bit_offset, H264_UE_V);
 	if( num_slice_groups_minus1 > 0 ) {
 		/* slice_group_map_type 1 ue(v)*/
-		slice_group_map_type = dissect_h264_exp_golomb_code(tree, hf_h264_slice_group_map_type, tvb, &bit_offset);
+		slice_group_map_type = dissect_h264_exp_golomb_code(tree, hf_h264_slice_group_map_type, tvb, &bit_offset, H264_UE_V);
 	/* if( slice_group_map_type = = 0 )*/
 	/* for( iGroup = 0; iGroup <= num_slice_groups_minus1; iGroup++ )*/
 	/* run_length_minus1[ iGroup ] 1 ue(v)*/
@@ -1168,10 +1291,10 @@ dissect_h264_pic_parameter_set_rbsp(proto_tree *tree, tvbuff_t *tvb, packet_info
 		return;
 	}
 	/* num_ref_idx_l0_active_minus1 1 ue(v)*/
-	dissect_h264_exp_golomb_code(tree, hf_h264_num_ref_idx_l0_active_minus1, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_num_ref_idx_l0_active_minus1, tvb, &bit_offset, H264_UE_V);
 	
 	/* num_ref_idx_l1_active_minus1 1 ue(v)*/
-	dissect_h264_exp_golomb_code(tree, hf_h264_num_ref_idx_l1_active_minus1, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_num_ref_idx_l1_active_minus1, tvb, &bit_offset, H264_UE_V);
 	
 	/* weighted_pred_flag 1 u(1)*/
 	proto_tree_add_bits_item(tree, hf_h264_weighted_pred_flag, tvb, bit_offset, 1, FALSE);
@@ -1182,13 +1305,13 @@ dissect_h264_pic_parameter_set_rbsp(proto_tree *tree, tvbuff_t *tvb, packet_info
 	bit_offset= bit_offset+2;
 
 	/* pic_init_qp_minus26  * relative to 26 * 1 se(v)*/
-	dissect_h264_exp_golomb_code(tree, hf_h264_pic_init_qp_minus26, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_pic_init_qp_minus26, tvb, &bit_offset, H264_SE_V);
 
 	/* pic_init_qs_minus26  * relative to 26 *  1 se(v)*/
-	dissect_h264_exp_golomb_code(tree, hf_h264_pic_init_qp_minus26, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_pic_init_qs_minus26, tvb, &bit_offset, H264_SE_V);
 
 	/* chroma_qp_index_offset 1 se(v)*/
-	dissect_h264_exp_golomb_code(tree, hf_h264_chroma_qp_index_offset, tvb, &bit_offset);
+	dissect_h264_exp_golomb_code(tree, hf_h264_chroma_qp_index_offset, tvb, &bit_offset, H264_SE_V);
 
 	/* deblocking_filter_control_present_flag 1 u(1)*/
 	proto_tree_add_bits_item(tree, hf_h264_deblocking_filter_control_present_flag, tvb, bit_offset, 1, FALSE);
@@ -1208,11 +1331,13 @@ dissect_h264_pic_parameter_set_rbsp(proto_tree *tree, tvbuff_t *tvb, packet_info
 		bit_offset++;
 
 		/* pic_scaling_matrix_present_flag 1 u(1)*/
-		/* pic_scaling_matrix_present_flag = tvb_get_bits8(tvb, bit_offset, 1); */
+		pic_scaling_matrix_present_flag = tvb_get_bits8(tvb, bit_offset, 1);
 		proto_tree_add_bits_item(tree, hf_h264_pic_scaling_matrix_present_flag, tvb, bit_offset, 1, FALSE);
 		bit_offset++;
 
-		/* if( pic_scaling_matrix_present_flag ){ */
+		if( pic_scaling_matrix_present_flag ){
+			proto_tree_add_text(tree, tvb, bit_offset>>3, -1, "[Not decoded yet]");
+			return;
 			/* for( i = 0; i < 6 + 2* transform_8x8_mode_flag; i++ ) {*/
 				/* pic_scaling_list_present_flag[ i ] 1 u(1)*/
 				/* if( pic_scaling_list_present_flag[ i ] )*/
@@ -1221,9 +1346,10 @@ dissect_h264_pic_parameter_set_rbsp(proto_tree *tree, tvbuff_t *tvb, packet_info
 				/* else*/
 					/* scaling_list( ScalingList8x8[ i - 6 ], 64, UseDefaultScalingMatrix8x8Flag[ i - 6 ] )*/
 			/* }*/
-		/* second_chroma_qp_index_offset 1 se(v)*/
-		proto_tree_add_text(tree, tvb, bit_offset>>3, -1, "[Not decoded yet]");
+			}
 
+		/* second_chroma_qp_index_offset 1 se(v)*/
+		dissect_h264_exp_golomb_code(tree, hf_h264_second_chroma_qp_index_offset, tvb, &bit_offset, H264_SE_V);
 	}
 	bit_offset = dissect_h264_rbsp_trailing_bits(tree, tvb, pinfo, bit_offset);
 
@@ -1457,6 +1583,167 @@ dissect_h264(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 }
 
 
+/* Capability */
+
+static const int *profile_fields[] = {
+  &hf_h264_par_profile_b,
+  &hf_h264_par_profile_m,
+  &hf_h264_par_profile_e,
+  &hf_h264_par_profile_h,
+  &hf_h264_par_profile_h10,
+  &hf_h264_par_profile_h4_2_2,
+  &hf_h264_par_profile_h4_4_4,
+  NULL
+};
+
+static int
+dissect_h264_par_profile(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
+{
+  int offset = 0;
+
+  proto_tree_add_bitmask(tree, tvb, offset, 
+                         hf_h264_par_profile, ett_h264_par_profile,
+                         profile_fields, FALSE);
+  offset += 1;
+  return offset;
+}
+
+static const int *AdditionalModesSupported_fields[] = {
+  &hf_h264_par_add_mode_sup_rcdo,
+  NULL
+};
+static int
+dissect_h264_par_AdditionalModesSupported(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
+{
+  int offset = 0;
+
+  proto_tree_add_bitmask(tree, tvb, offset, 
+                         hf_h264_par_AdditionalModesSupported, ett_h264_par_AdditionalModesSupported,
+                         AdditionalModesSupported_fields, FALSE);
+  offset += 1;
+  return offset;
+}
+
+
+static const int *ProfileIOP_fields[] = {
+  &hf_h264_par_constraint_set0_flag,
+  &hf_h264_par_constraint_set1_flag,
+  &hf_h264_par_constraint_set2_flag,
+  NULL
+};
+
+
+static int
+dissect_h264_ProfileIOP(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree)
+{
+  int offset = 0;
+
+  proto_tree_add_bitmask(tree, tvb, offset, 
+                         hf_h264_par_ProfileIOP, ett_h264_par_ProfileIOP,
+                         ProfileIOP_fields, FALSE);
+  offset += 1;
+  return offset;
+}
+
+static const value_string h264_par_level_values[] = {
+  { 15,		"1" },
+  { 19,		"1b" },
+  { 22,		"1.1" },
+  { 29,		"1.2" },
+  { 36,		"1.3" },
+  { 43,		"2" },
+  { 50,		"2.1" },
+  { 57,		"2.2" },
+  { 64,		"3" },
+  { 71,		"3.1" },
+  { 78,		"3.2" },
+  { 85,		"4" },
+  { 92,		"4.1" },
+  { 99,		"4.2" },
+  { 106,	"5" },
+  { 113 ,	"5.1" },
+  { 0,	NULL }
+};
+
+static int
+dissect_h264_par_level(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree _U_)
+{
+  int offset = 0;
+  guint16 lvl;
+  const gchar *p = NULL;
+  asn1_ctx_t *actx;
+
+  actx = get_asn1_ctx(pinfo->private_data);
+  DISSECTOR_ASSERT(actx);
+
+  lvl = tvb_get_ntohs(tvb, offset);
+  p = match_strval(lvl, VALS(h264_par_level_values));
+  if (p) {
+    proto_item_append_text(actx->created_item, " - Level %s", p);
+  }
+  offset += 2;
+  return offset;
+}
+
+typedef struct _h264_capability_t {
+  const gchar *id;
+  const gchar *name;
+  new_dissector_t content_pdu;
+} h264_capability_t;
+
+static h264_capability_t h264_capability_tab[] = {
+  /* ITU-T H.241 (05/2006), 8.3 H.264 capabilities */
+  { "GenericCapability/0.0.8.241.0.0.1", "ITU-T Rec. H.241 H.264 Video Capabilities", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/41", "Profile", dissect_h264_par_profile },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/42", "Level", dissect_h264_par_level },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/3" , "CustomMaxMBPS", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/4" , "CustomMaxFS", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/5" , "CustomMaxDPB", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/6" , "CustomMaxBRandCPB", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/7" , "MaxStaticMBPS", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/8" , "max-rcmd-nal-unit-size", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/9" , "max-nal-unit-size", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/10", "SampleAspectRatiosSupported", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/11", "AdditionalModesSupported", dissect_h264_par_AdditionalModesSupported },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/12", "AdditionalDisplayCapabilities", NULL },
+  /* TS 26.111  H.264 */
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/43" , "DecoderConfigurationInformation", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/44" , "AcceptRedundantSlices", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/45" , "NalAlignedMode", NULL },
+  { "GenericCapability/0.0.8.241.0.0.1/collapsing/46" , "ProfileIOP", dissect_h264_ProfileIOP },
+  { NULL, NULL, NULL },
+};                                 
+
+static h264_capability_t *find_cap(const gchar *id) {
+  h264_capability_t *ftr = NULL;
+  h264_capability_t *f;
+
+  for (f=h264_capability_tab; f->id; f++) {
+    if (!strcmp(id, f->id)) { ftr = f; break; }
+  }
+  return ftr;
+}
+
+static void
+dissect_h264_name(tvbuff_t *tvb _U_, packet_info *pinfo, proto_tree *tree) 
+{
+  asn1_ctx_t *actx;
+  h264_capability_t *ftr = NULL;
+
+  actx = get_asn1_ctx(pinfo->private_data);
+  DISSECTOR_ASSERT(actx);
+  if (tree) {
+    ftr = find_cap(pinfo->match_string);
+    if (ftr) {
+      proto_item_append_text(actx->created_item, " - %s", ftr->name);
+      proto_item_append_text(proto_item_get_parent(proto_tree_get_parent(tree)), ": %s", ftr->name);
+    } else {
+      proto_item_append_text(actx->created_item, " - unknown(%s)", pinfo->match_string);
+    }
+  }
+}
+
+
 /* Register the protocol with Wireshark */
 /* If this dissector uses sub-dissector registration add a registration routine.
    This format is required because a script is used to find these routines and
@@ -1466,6 +1753,9 @@ void
 proto_reg_handoff_h264(void)
 {
 	dissector_handle_t h264_handle;
+	dissector_handle_t h264_name_handle;
+	h264_capability_t *ftr;
+
 	static int h264_prefs_initialized = FALSE;
 	
 	h264_handle = create_dissector_handle(dissect_h264, proto_h264);
@@ -1484,6 +1774,14 @@ proto_reg_handoff_h264(void)
 	}
 	dissector_add_string("rtp_dyn_payload_type","H264", h264_handle);
 
+  h264_name_handle = create_dissector_handle(dissect_h264_name, proto_h264);
+  for (ftr=h264_capability_tab; ftr->id; ftr++) {
+    if (ftr->name) 
+      dissector_add_string("h245.gef.name", ftr->id, h264_name_handle);
+    if (ftr->content_pdu)
+      dissector_add_string("h245.gef.content", ftr->id, new_create_dissector_handle(ftr->content_pdu, proto_h264));
+  }
+
 }
 
 /* this format is require because a script is used to build the C function
@@ -1544,7 +1842,7 @@ proto_register_h264(void)
 			"Constraint_set1_flag", HFILL }
 		},
 		{ &hf_h264_constraint_set2_flag,
-			{ "Constraint_set1_flag",           "h264.constraint_set2_flag",
+			{ "Constraint_set2_flag",           "h264.constraint_set2_flag",
 			FT_UINT8, BASE_DEC, NULL, 0x20,          
 			"NRI", HFILL }
 		},
@@ -1640,12 +1938,12 @@ proto_register_h264(void)
 		},
 		{ &hf_h264_offset_for_non_ref_pic,
 			{ "offset_for_non_ref_pic",           "h264.offset_for_non_ref_pic",
-			FT_UINT32, BASE_DEC, NULL, 0x0,          
+			FT_INT32, BASE_DEC, NULL, 0x0,          
 			"offset_for_non_ref_pic", HFILL }
 		},
 		{ &hf_h264_offset_for_top_to_bottom_field,
 			{ "offset_for_top_to_bottom_field",           "h264.offset_for_top_to_bottom_field",
-			FT_UINT32, BASE_DEC, NULL, 0x0,          
+			FT_INT32, BASE_DEC, NULL, 0x0,          
 			"offset_for_top_to_bottom_field", HFILL }
 		},
 		{ &hf_264_num_ref_frames_in_pic_order_cnt_cycle,
@@ -1655,7 +1953,7 @@ proto_register_h264(void)
 		},
 		{ &hf_h264_offset_for_ref_frame,
 			{ "offset_for_ref_frame",           "h264.offset_for_ref_frame",
-			FT_UINT32, BASE_DEC, NULL, 0x0,          
+			FT_INT32, BASE_DEC, NULL, 0x0,          
 			"offset_for_ref_frame", HFILL }
 		},
 		{ &hf_h264_num_ref_frames,
@@ -1770,12 +2068,17 @@ proto_register_h264(void)
 		},
 		{ &hf_h264_pic_init_qp_minus26,
 			{ "pic_init_qp_minus26",           "h264.pic_init_qp_minus26",
-			FT_UINT32, BASE_DEC, NULL, 0x0,          
+			FT_INT32, BASE_DEC, NULL, 0x0,          
 			"pic_init_qp_minus26", HFILL }
 		},
+		{ &hf_h264_pic_init_qs_minus26,
+			{ "pic_init_qs_minus26",           "h264.pic_init_qs_minus26",
+			FT_INT32, BASE_DEC, NULL, 0x0,          
+			"pic_init_qs_minus26", HFILL }
+		},
 		{ &hf_h264_chroma_qp_index_offset,
 			{ "chroma_qp_index_offset",           "h264.chroma_qp_index_offset",
-			FT_UINT32, BASE_DEC, NULL, 0x0,          
+			FT_INT32, BASE_DEC, NULL, 0x0,          
 			"chroma_qp_index_offset", HFILL }
 		},
 		{ &hf_h264_deblocking_filter_control_present_flag,
@@ -1803,6 +2106,11 @@ proto_register_h264(void)
 			FT_UINT8, BASE_DEC, NULL, 0x0,          
 			"pic_scaling_matrix_present_flag", HFILL }
 		},
+		{ &hf_h264_second_chroma_qp_index_offset,
+			{ "second_chroma_qp_index_offset",           "h264.second_chroma_qp_index_offset",
+			FT_INT32, BASE_DEC, NULL, 0x0,          
+			"second_chroma_qp_index_offset", HFILL }
+		},
 
 		{ &hf_h264_aspect_ratio_info_present_flag,
 			{ "aspect_ratio_info_present_flag",           "h264.aspect_ratio_info_present_flag",
@@ -2034,6 +2342,67 @@ proto_register_h264(void)
 			FT_UINT8, BASE_DEC, NULL, 0x0,          
 			"frame_num", HFILL }
 		},
+
+    { &hf_h264_par_profile,
+      { "Profile", "h264.profile",
+        FT_UINT8, BASE_HEX, NULL, 0x00,
+        NULL, HFILL}},
+    { &hf_h264_par_profile_b,
+      { "Baseline Profile", "h264.profile.base",
+        FT_BOOLEAN, 8, NULL, 0x40,
+        NULL, HFILL}},
+    { &hf_h264_par_profile_m,
+      { "Main Profile", "h264.profile.main",
+        FT_BOOLEAN, 8, NULL, 0x20,
+        NULL, HFILL}},
+    { &hf_h264_par_profile_e,
+      { "Extended Profile.", "h264.profile.ext",
+        FT_BOOLEAN, 8, NULL, 0x10,
+        NULL, HFILL}},
+    { &hf_h264_par_profile_h,
+      { "High Profile", "h264.profile.high",
+        FT_BOOLEAN, 8, NULL, 0x08,
+        NULL, HFILL}},
+    { &hf_h264_par_profile_h10,
+      { "High 10 Profile", "h264.profile.high10",
+        FT_BOOLEAN, 8, NULL, 0x04,
+        NULL, HFILL}},
+    { &hf_h264_par_profile_h4_2_2,
+      { "High 4:2:2 Profile", "h264.profile.high4_2_2",
+        FT_BOOLEAN, 8, NULL, 0x02,
+        NULL, HFILL}},
+    { &hf_h264_par_profile_h4_4_4,
+      { "High 4:4:4 Profile", "h264.profile.high4_4_4",
+        FT_BOOLEAN, 8, NULL, 0x01,
+        NULL, HFILL}},
+	{ &hf_h264_par_AdditionalModesSupported,
+      { "AdditionalModesSupported", "h264.AdditionalModesSupported",
+        FT_UINT8, BASE_HEX, NULL, 0x00,
+        NULL, HFILL}},
+	{ &hf_h264_par_add_mode_sup,
+      { "Additional Modes Supported", "h264.add_mode_sup",
+        FT_UINT8, BASE_HEX, NULL, 0x00,
+        NULL, HFILL}},
+	{ &hf_h264_par_add_mode_sup_rcdo,
+      { "Reduced Complexity Decoding Operation (RCDO) support", "h264.add_mode_sup.rcdo",
+        FT_BOOLEAN, 8, NULL, 0x40,
+        NULL, HFILL}},
+	{ &hf_h264_par_ProfileIOP,
+      { "ProfileIOP", "h264.ProfileIOP",
+        FT_UINT8, BASE_HEX, NULL, 0x00,
+        NULL, HFILL}},
+	{ &hf_h264_par_constraint_set0_flag,
+      { "constraint_set0_flag", "h264.par.constraint_set0_flag",
+        FT_BOOLEAN, 8, NULL, 0x80,
+        NULL, HFILL}},
+	{ &hf_h264_par_constraint_set1_flag,
+      { "constraint_set1_flag", "h264.par.constraint_set1_flag",
+        FT_BOOLEAN, 8, NULL, 0x40,
+        NULL, HFILL}},
+	{ &hf_h264_par_constraint_set2_flag,
+      { "constraint_set2_flag", "h264.par.constraint_set2_flag",
+        FT_BOOLEAN, 8, NULL, 0x20,
+        NULL, HFILL}},
 	};
 
 /* Setup protocol subtree array */
@@ -2043,6 +2412,9 @@ proto_register_h264(void)
 		&ett_h264_nal,
 		&ett_h264_stream,
 		&ett_h264_nal_unit,
+		&ett_h264_par_profile,
+		&ett_h264_par_AdditionalModesSupported,
+		&ett_h264_par_ProfileIOP,
 	};
 
 /* Register the protocol name and description */
