@@ -9,7 +9,7 @@
  * Copyright 2002, Tim Potter <tpot@samba.org>
  * Copyright 1999, Andrew Tridgell <tridge@samba.org>
  *
- * $Id: packet-http.c 19968 2006-11-24 02:59:43Z sfisher $
+ * $Id: packet-http.c 24586 2008-03-08 21:17:43Z morriss $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -44,10 +44,12 @@
 #include <epan/base64.h>
 #include <epan/emem.h>
 #include <epan/stats_tree.h>
+#include <epan/ws_strsplit.h>
 
 #include <epan/req_resp_hdrs.h>
 #include "packet-http.h"
 #include "packet-tcp.h"
+#include "packet-ssl.h"
 #include <epan/prefs.h>
 #include <epan/expert.h>
 
@@ -60,7 +62,12 @@ typedef enum _http_type {
 
 #include <epan/tap.h>
 
+#ifdef NEED_G_ASCII_STRCASECMP_H
+#include "g_ascii_strcasecmp.h"
+#endif
+
 static int http_tap = -1;
+static int http_eo_tap = -1;
 
 static int proto_http = -1;
 static int hf_http_notification = -1;
@@ -74,6 +81,8 @@ static int hf_http_response_code = -1;
 static int hf_http_authorization = -1;
 static int hf_http_proxy_authenticate = -1;
 static int hf_http_proxy_authorization = -1;
+static int hf_http_proxy_connect_host = -1;
+static int hf_http_proxy_connect_port = -1;
 static int hf_http_www_authenticate = -1;
 static int hf_http_content_type = -1;
 static int hf_http_content_length = -1;
@@ -133,7 +142,6 @@ static gboolean http_decompress_body = TRUE;
 static gboolean http_decompress_body = FALSE;
 #endif
 
-
 #define TCP_PORT_HTTP			80
 #define TCP_PORT_PROXY_HTTP		3128
 #define TCP_PORT_PROXY_ADMIN_HTTP	3132
@@ -149,10 +157,18 @@ static gboolean http_decompress_body = FALSE;
 #define UDP_PORT_SSDP			1900
 
 /*
- * tcp alternate port
+ * tcp and ssl ports
  */
-static guint http_alternate_tcp_port = 0;
-static guint alternate_tcp_port = 0;
+
+#define TCP_DEFAULT_RANGE "80,3128,3132,8080,8088,11371,3689,1900"
+#define SSL_DEFAULT_RANGE "443"
+
+static range_t *global_http_tcp_range = NULL;
+static range_t *global_http_ssl_range = NULL;
+
+static range_t *http_tcp_range = NULL;
+static range_t *http_ssl_range = NULL;
+
 
 /*
  * Protocols implemented atop HTTP.
@@ -164,7 +180,7 @@ typedef enum {
 } http_proto_t;
 
 typedef void (*ReqRespDissector)(tvbuff_t*, proto_tree*, int, const guchar*,
-    const guchar*);
+				 const guchar*, http_conv_t *);
 
 /*
  * Structure holding information from headers needed by main
@@ -180,13 +196,13 @@ typedef struct {
 } headers_t;
 
 static int is_http_request_or_reply(const gchar *data, int linelen,
-    http_type_t *type, ReqRespDissector *reqresp_dissector);
+				    http_type_t *type, ReqRespDissector
+				    *reqresp_dissector, http_conv_t *conv_data);
 static int chunked_encoding_dissector(tvbuff_t **tvb_ptr, packet_info *pinfo,
 		proto_tree *tree, int offset);
-static void http_payload_subdissector(tvbuff_t *next_tvb, proto_tree *tree, proto_tree *sub_tree, packet_info *pinfo);
 static void process_header(tvbuff_t *tvb, int offset, int next_offset,
     const guchar *line, int linelen, int colon_offset, packet_info *pinfo,
-    proto_tree *tree, headers_t *eh_ptr);
+    proto_tree *tree, headers_t *eh_ptr, http_conv_t *conv_data);
 static gint find_header_hf_value(tvbuff_t *tvb, int offset, guint header_len);
 static gboolean check_auth_ntlmssp(proto_item *hdr_item, tvbuff_t *tvb,
     packet_info *pinfo, gchar *value);
@@ -270,6 +286,7 @@ static int st_node_reqs_by_srv_addr = -1;
 static int st_node_reqs_by_http_host = -1;
 static int st_node_resps_by_srv_addr = -1;
 
+/* HTTP/Load Distribution stats init function */
 static void http_reqs_stats_tree_init(stats_tree* st) {
 	st_node_reqs = stats_tree_create_node(st, st_str_reqs, 0, TRUE);
 	st_node_reqs_by_srv_addr = stats_tree_create_node(st, st_str_reqs_by_srv_addr, st_node_reqs, TRUE);
@@ -277,6 +294,7 @@ static void http_reqs_stats_tree_init(stats_tree* st) {
 	st_node_resps_by_srv_addr = stats_tree_create_node(st, st_str_resps_by_srv_addr, 0, TRUE);
 }
 
+/* HTTP/Load Distribution stats packet function */
 static int http_reqs_stats_tree_packet(stats_tree* st, packet_info* pinfo, epan_dissect_t* edt _U_, const void* p) {
 	const http_info_value_t* v = p;
 	int reqs_by_this_host;
@@ -325,10 +343,12 @@ static int http_reqs_stats_tree_packet(stats_tree* st, packet_info* pinfo, epan_
 static int st_node_requests_by_host = -1;
 static const guint8* st_str_requests_by_host = "HTTP Requests by HTTP Host";
 
+/* HTTP/Requests stats init function */
 static void http_req_stats_tree_init(stats_tree* st) {
 	st_node_requests_by_host = stats_tree_create_node(st, st_str_requests_by_host, 0, TRUE);
 }
 
+/* HTTP/Requests stats packet function */
 static int http_req_stats_tree_packet(stats_tree* st, packet_info* pinfo _U_, epan_dissect_t* edt _U_, const void* p) {
 	const http_info_value_t* v = p;
 	int reqs_by_this_host;
@@ -373,6 +393,7 @@ static int st_node_resp_500 = -1;
 static int st_node_other = -1;
 
 
+/* HTTP/Packet Counter stats init function */
 static void http_stats_tree_init(stats_tree* st) {
 	st_node_packets = stats_tree_create_node(st, st_str_packets, 0, TRUE);
 	st_node_requests = stats_tree_create_pivot(st, st_str_requests, st_node_packets);
@@ -386,6 +407,7 @@ static void http_stats_tree_init(stats_tree* st) {
 	st_node_other = stats_tree_create_node(st, st_str_other, st_node_packets,FALSE);
 }
 
+/* HTTP/Packet Counter stats packet function */
 static int http_stats_tree_packet(stats_tree* st, packet_info* pinfo _U_, epan_dissect_t* edt _U_, const void* p) {
 	const http_info_value_t* v = p;
 	guint i = v->response_code;
@@ -420,7 +442,7 @@ static int http_stats_tree_packet(stats_tree* st, packet_info* pinfo _U_, epan_d
 
 		tick_stat_node(st, resp_str, st_node_responses, FALSE);
 
-		g_snprintf(str, sizeof(str),"%u %s",i,match_strval(i,vals_status_code));
+		g_snprintf(str, sizeof(str),"%u %s",i,val_to_str(i,vals_status_code, "Unknown (%d)"));
 		tick_stat_node(st, str, resp_grp, FALSE);
 	} else if (v->request_method) {
 		stats_tree_tick_pivot(st,st_node_requests,v->request_method);
@@ -464,6 +486,38 @@ dissect_http_ntlmssp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		call_dissector(gssapi_handle, ntlmssp_tvb, pinfo, tree);
 }
 
+static http_conv_t *
+get_http_conversation_data(packet_info *pinfo)
+{
+	conversation_t  *conversation;
+	http_conv_t	*conv_data;
+
+	conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst, pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
+
+	if(!conversation) {  /* Conversation does not exist yet - create it */
+		conversation = conversation_new(pinfo->fd->num, &pinfo->src, &pinfo->dst, pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
+	}
+
+	/* Retrieve information from conversation
+	 * or add it if it isn't there yet
+	 */
+	conv_data = conversation_get_proto_data(conversation, proto_http);
+	if(!conv_data) {
+		/* Setup the conversation structure itself */
+		conv_data = se_alloc(sizeof(http_conv_t));
+
+		conv_data->response_code = 0;
+		conv_data->request_method = NULL;
+	 	conv_data->request_uri = NULL;
+		conv_data->startframe = 0;
+
+		conversation_add_proto_data(conversation, proto_http,
+					    conv_data);
+	}
+
+	return conv_data;
+}
+
 /*
  * TODO: remove this ugly global variable.
  * XXX: do we really want to have to pass this from one function to another?
@@ -472,7 +526,7 @@ static http_info_value_t	*stat_info;
 
 static int
 dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
-    proto_tree *tree)
+    proto_tree *tree, http_conv_t *conv_data)
 {
 	http_proto_t	proto;
 	const char	*proto_tag;
@@ -498,7 +552,7 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	gboolean	dissected;
 	/*guint		i;*/
 	/*http_info_value_t *si;*/
-	conversation_t  *conversation;
+	http_eo_t       *eo_info;
 
 	/*
 	 * Is this a request or response?
@@ -516,7 +570,7 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	line = tvb_get_ptr(tvb, offset, first_linelen);
 	http_type = HTTP_OTHERS;	/* type not known yet */
 	is_request_or_reply = is_http_request_or_reply((const gchar *)line,
-	    first_linelen, &http_type, NULL);
+	    first_linelen, &http_type, NULL, conv_data);
 	if (is_request_or_reply) {
 		/*
 		 * Yes, it's a request or response.
@@ -533,25 +587,12 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		}
 	}
 
-	conversation = find_conversation(pinfo->fd->num, &pinfo->src, &pinfo->dst, pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
-
-	if(!conversation) {  /* Conversation does not exist yet - create it */
-		conversation = conversation_new(pinfo->fd->num, &pinfo->src, &pinfo->dst, pinfo->ptype, pinfo->srcport, pinfo->destport, 0);
-	}
-
-	/* Retrieve information from conversation
-	 * or add it if it isn't there yet
-	 */
-	stat_info = conversation_get_proto_data(conversation, proto_http);
-	if(!stat_info) {
-		stat_info = se_alloc(sizeof(http_info_value_t));
-		stat_info->response_code = 0;
-		stat_info->request_method = NULL;
-		stat_info->request_uri = NULL;
-		stat_info->http_host = NULL;
-
-		conversation_add_proto_data(conversation, proto_http, stat_info);
-	}
+	stat_info = ep_alloc(sizeof(http_info_value_t));
+	stat_info->framenum = pinfo->fd->num;
+	stat_info->response_code = 0;
+	stat_info->request_method = NULL;
+	stat_info->request_uri = NULL;
+	stat_info->http_host = NULL;
 
 	switch (pinfo->match_port) {
 
@@ -584,10 +625,10 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		 * is not longer than what's in the buffer, so the
 		 * "tvb_get_ptr()" call won't throw an exception.
 		 */
-		line = tvb_get_ptr(tvb, offset, first_linelen);
-		if (is_request_or_reply)
-			col_add_str(pinfo->cinfo, COL_INFO,
-			    format_text(line, first_linelen));
+		if (is_request_or_reply) {
+		    line = tvb_get_ptr(tvb, offset, first_linelen);
+			col_add_fstr(pinfo->cinfo, COL_INFO, "%s ", format_text(line, first_linelen));
+		}
 		else
 			col_set_str(pinfo->cinfo, COL_INFO, "Continuation or non-HTTP traffic");
 	}
@@ -606,6 +647,7 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	headers.content_type = NULL;	/* content type not known yet */
 	headers.content_type_parameters = NULL;	/* content type parameters too */
 	headers.have_content_length = FALSE;	/* content length not known yet */
+	headers.content_length = 0;		/* content length set to 0 (avoid a gcc warning) */
 	headers.content_encoding = NULL; /* content encoding not known yet */
 	headers.transfer_encoding = NULL; /* transfer encoding not known yet */
 	saw_req_resp_or_header = FALSE;	/* haven't seen anything yet */
@@ -636,7 +678,7 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		reqresp_dissector = NULL;
 		is_request_or_reply =
 		    is_http_request_or_reply((const gchar *)line,
-		    linelen, &http_type, &reqresp_dissector);
+		    linelen, &http_type, &reqresp_dissector, conv_data);
 		if (is_request_or_reply)
 			goto is_http;
 
@@ -776,27 +818,26 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		 */
 		saw_req_resp_or_header = TRUE;
 		if (is_request_or_reply) {
+		    	char *text = tvb_format_text(tvb, offset, next_offset - offset);
 			if (tree) {
 				hdr_item = proto_tree_add_text(http_tree, tvb,
-				    offset, next_offset - offset, "%s",
-				    tvb_format_text(tvb, offset,
-				      next_offset - offset));
+				    offset, next_offset - offset, "%s", text);
 			}
-			expert_add_info_format(pinfo, hdr_item, PI_SEQUENCE, PI_CHAT,
-				"%s",
-				tvb_format_text(tvb, offset, next_offset - offset));
+			expert_add_info_format(pinfo, hdr_item, PI_SEQUENCE, PI_CHAT, "%s", text);
 			if (reqresp_dissector) {
 				if (tree) req_tree = proto_item_add_subtree(hdr_item, ett_http_request);
 				else req_tree = NULL;
 
-				reqresp_dissector(tvb, req_tree, offset, line, lineend);
+				reqresp_dissector(tvb, req_tree, offset, line,
+						  lineend, conv_data);
 			}
+
 		} else {
 			/*
 			 * Header.
 			 */
 			process_header(tvb, offset, next_offset, line, linelen,
-			    colon_offset, pinfo, http_tree, &headers);
+			    colon_offset, pinfo, http_tree, &headers, conv_data);
 		}
 		offset = next_offset;
 	}
@@ -947,9 +988,9 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		 * Handle *transfer* encodings other than "identity".
 		 */
 		if (headers.transfer_encoding != NULL &&
-		    strcasecmp(headers.transfer_encoding, "identity") != 0) {
+		    g_ascii_strcasecmp(headers.transfer_encoding, "identity") != 0) {
 			if (http_dechunk_body &&
-			    (strncasecmp(headers.transfer_encoding, "chunked", 7)
+			    (g_ascii_strncasecmp(headers.transfer_encoding, "chunked", 7)
 			    == 0)) {
 
 				chunks_decoded = chunked_encoding_dissector(
@@ -995,7 +1036,7 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		 * we handle it in any case).
 		 */
 		if (headers.content_encoding != NULL &&
-		    strcasecmp(headers.content_encoding, "identity") != 0) {
+		    g_ascii_strcasecmp(headers.content_encoding, "identity") != 0) {
 			/*
 			 * We currently can't handle, for example, "compress";
 			 * just handle them as data for now.
@@ -1010,8 +1051,8 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 			proto_tree *e_tree = NULL;
 
 			if (http_decompress_body &&
-			    (strcasecmp(headers.content_encoding, "gzip") == 0 ||
-			    strcasecmp(headers.content_encoding, "deflate")
+			    (g_ascii_strcasecmp(headers.content_encoding, "gzip") == 0 ||
+			    g_ascii_strcasecmp(headers.content_encoding, "deflate")
 			    == 0)) {
 
 				uncomp_tvb = tvb_uncompress(next_tvb, 0,
@@ -1047,12 +1088,6 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 				add_new_data_source(pinfo, next_tvb,
 				    "Uncompressed entity body");
 			} else {
-				if (chunks_decoded > 1) {
-					tvb_set_child_real_data_tvbuff(tvb,
-					    next_tvb);
-					add_new_data_source(pinfo, next_tvb,
-					    "Compressed entity body");
-				}
 				call_dissector(data_handle, next_tvb, pinfo,
 				    e_tree);
 
@@ -1064,12 +1099,28 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		 * only if it was content-encoded and/or transfer-encoded.
 		 */
 
+		/* Save values for the Export Object GUI feature if we have
+		 * an active listener to process it (which happens when
+		 * the export object window is open). */
+		if(have_tap_listener(http_eo_tap)) {
+			eo_info = ep_alloc(sizeof(http_eo_t));
+
+			eo_info->hostname = conv_data->http_host;
+			eo_info->filename = conv_data->request_uri;
+			eo_info->content_type = headers.content_type;
+			eo_info->payload_len = next_tvb->length;
+			eo_info->payload_data = next_tvb->real_data;
+
+			tap_queue_packet(http_eo_tap, pinfo, eo_info);
+		}
+
 		/*
 		 * Do subdissector checks.
 		 *
 		 * First, check whether some subdissector asked that they
 		 * be called if something was on some particular port.
 		 */
+
 		handle = dissector_get_port_handle(port_subdissector_table,
 		    pinfo->match_port);
 		if (handle == NULL && headers.content_type != NULL) {
@@ -1125,8 +1176,8 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
 				 */
 				call_dissector(media_handle, next_tvb, pinfo, tree);
 			} else {
-				/* Call the subdissector (defaults to data), otherwise. */
-				http_payload_subdissector(next_tvb, tree, http_tree, pinfo);
+				/* Call the default data dissector */
+				call_dissector(data_handle, next_tvb, pinfo, http_tree);
 			}
 		}
 
@@ -1157,9 +1208,11 @@ dissect_http_message(tvbuff_t *tvb, int offset, packet_info *pinfo,
  */
 static void
 basic_request_dissector(tvbuff_t *tvb, proto_tree *tree, int offset,
-    const guchar *line, const guchar *lineend)
+			const guchar *line, const guchar *lineend,
+			http_conv_t *conv_data)
 {
 	const guchar *next_token;
+	gchar *request_uri;
 	int tokenlen;
 
 	/* The first token is the method. */
@@ -1175,9 +1228,14 @@ basic_request_dissector(tvbuff_t *tvb, proto_tree *tree, int offset,
 	tokenlen = get_token_len(line, lineend, &next_token);
 	if (tokenlen == 0)
 		return;
-	stat_info->request_uri = se_strdup((gchar*) tvb_get_ephemeral_string(tvb, offset, tokenlen));
+
+	/* Save the request URI for various later uses */
+	request_uri = (gchar *)tvb_get_ephemeral_string(tvb, offset, tokenlen);
+	stat_info->request_uri = ep_strdup(request_uri);
+ 	conv_data->request_uri = se_strdup(request_uri);
+
 	proto_tree_add_string(tree, hf_http_request_uri, tvb, offset, tokenlen,
-	    stat_info->request_uri);
+			      request_uri);
 	offset += next_token - line;
 	line = next_token;
 
@@ -1191,7 +1249,8 @@ basic_request_dissector(tvbuff_t *tvb, proto_tree *tree, int offset,
 
 static void
 basic_response_dissector(tvbuff_t *tvb, proto_tree *tree, int offset,
-    const guchar *line, const guchar *lineend)
+			 const guchar *line, const guchar *lineend,
+			 http_conv_t *conv_data _U_)
 {
 	const guchar *next_token;
 	int tokenlen;
@@ -1213,7 +1272,8 @@ basic_response_dissector(tvbuff_t *tvb, proto_tree *tree, int offset,
 	memcpy(response_chars, line, 3);
 	response_chars[3] = '\0';
 
-	stat_info->response_code = strtoul(response_chars,NULL,10);
+	stat_info->response_code = conv_data->response_code =
+		strtoul(response_chars, NULL, 10);
 
 	proto_tree_add_uint(tree, hf_http_response_code, tvb, offset, 3,
 	    stat_info->response_code);
@@ -1327,9 +1387,13 @@ chunked_encoding_dissector(tvbuff_t **tvb_ptr, packet_info *pinfo,
 		tvb_memcpy(tvb, (guint8 *)(raw_data + raw_len),
 			    chunk_offset, chunk_size);
 
-		new_tvb = tvb_new_real_data(raw_data,
-			    chunked_data_size, chunked_data_size);
-		tvb_set_free_cb(new_tvb, g_free);
+		/* Don't create a new tvb if we have a single chunk with
+		 * a size of zero (meaning it is the end of the chunks). */
+		if(chunked_data_size > 0) {
+			new_tvb = tvb_new_real_data(raw_data,
+			      chunked_data_size, chunked_data_size);
+			tvb_set_free_cb(new_tvb, g_free);
+		}
 
 
 		if (subtree) {
@@ -1403,63 +1467,66 @@ chunked_encoding_dissector(tvbuff_t **tvb_ptr, packet_info *pinfo,
 
 }
 
+/* Call a subdissector to handle HTTP CONNECT's traffic */
 static void
-http_payload_subdissector(tvbuff_t *next_tvb, proto_tree *tree, proto_tree *sub_tree, packet_info *pinfo)
+http_payload_subdissector(tvbuff_t *tvb, proto_tree *tree,
+			  packet_info *pinfo, http_conv_t *conv_data)
 {
-	/* tree = the main protocol tree that the subdissector would be listed in
-	 * sub_tree = the http protocol tree
-	 */
- 	guint32 *ptr;
-	struct tcpinfo *tcpinfo = pinfo->private_data;
-	struct tcp_analysis *tcpd=NULL;
+	guint32 *ptr = NULL;
+ 	guint32 dissect_as, saved_port;
 	gchar **strings; /* An array for splitting the request URI into hostname and port */
+	proto_item *item;
+	proto_tree *proxy_tree;
 
-	/* Response code 200 means "OK" and strncmp() == 0 means the strings match exactly */
-	if(stat_info->request_uri && stat_info->response_code == 200 &&
-	   stat_info->request_method && strncmp(stat_info->request_method, "CONNECT", 7) == 0) {
+	/* Grab the destination port number from the request URI to find the right subdissector */
+	strings = g_strsplit(conv_data->request_uri, ":", 2);
 
-		/* Call a subdissector to handle HTTP CONNECT's traffic */
-		tcpd=get_tcp_conversation_data(pinfo);
+	if(strings[0] != NULL && strings[1] != NULL) {
+		/*
+		 * The string was successfuly split in two
+		 * Create a proxy-connect subtree
+		 */
+		if(tree) {
+			item = proto_tree_add_item(tree, proto_http, tvb, 0, -1, FALSE);
+			proxy_tree = proto_item_add_subtree(item, ett_http);
 
-		/* Grab the destination port number from the request URI to find the right subdissector */
-		strings = g_strsplit(stat_info->request_uri, ":", 2);
+			item = proto_tree_add_string(proxy_tree, hf_http_proxy_connect_host,
+		    	    tvb, 0, 0, strings[0]);
+			PROTO_ITEM_SET_GENERATED(item);
 
-		if(strings[0] != NULL && strings[1] != NULL) { /* The string was successfuly split in two */
-			proto_tree_add_text(sub_tree, next_tvb, 0, 0, "Proxy connect hostname: %s", strings[0]);
-			proto_tree_add_text(sub_tree, next_tvb, 0, 0, "Proxy connect port: %s", strings[1]);
+			item = proto_tree_add_uint(proxy_tree, hf_http_proxy_connect_port,
+			    tvb, 0, 0, strtol(strings[1], NULL, 10) );
+			PROTO_ITEM_SET_GENERATED(item);
+		}
 
-			/* Replaces the pinfo->destport or srcport with the port the http connect was done to by
-			 * checking to see which is the http port. */
-			if (pinfo->destport == TCP_PORT_HTTP             || pinfo->destport == TCP_PORT_PROXY_HTTP ||
-			    pinfo->destport == TCP_PORT_PROXY_ADMIN_HTTP || pinfo->destport == TCP_ALT_PORT_HTTP   ||
-			    pinfo->destport == TCP_RADAN_HTTP            || pinfo->destport == TCP_PORT_HKP        ||
-			    pinfo->destport == TCP_PORT_DAAP             || pinfo->destport == http_alternate_tcp_port)
+		/* We're going to get stuck in a loop if we let process_tcp_payload call
+		   us, so call the data dissector instead for proxy connections to http ports. */
+		dissect_as = (int)strtol(strings[1], NULL, 10); /* Convert string to a base-10 integer */
+
+		if (value_is_in_range(http_tcp_range, dissect_as)) {
+			call_dissector(data_handle, tvb, pinfo, tree);
+		} else {
+			/* set pinfo->{src/dst port} and call the TCP sub-dissector lookup */
+			if ( !ptr && value_is_in_range(http_tcp_range, pinfo->destport) )
 				ptr = &pinfo->destport;
 			else
 				ptr = &pinfo->srcport;
 
-			*ptr = (int)strtol(strings[1], NULL, 10); /* Convert string to a base-10 integer */
+                        /* Increase pinfo->can_desegment because we are traversing
+                         * http and want to preserve desegmentation functionality for
+                         * the proxied protocol
+                         */
+			if( pinfo->can_desegment>0 )
+				pinfo->can_desegment++;
 
-			/* We're going to get stuck in a loop if we call let dissect_tcp_payload call
-			   us, so call the data dissector instead for proxy connections to http ports. */
-			if(*ptr == TCP_PORT_HTTP             || *ptr == TCP_PORT_PROXY_HTTP ||
-			   *ptr == TCP_PORT_PROXY_ADMIN_HTTP || *ptr == TCP_ALT_PORT_HTTP   ||
-			   *ptr == TCP_RADAN_HTTP            || *ptr == TCP_PORT_HKP        ||
-			   *ptr == TCP_PORT_DAAP             || *ptr == http_alternate_tcp_port) {
-				call_dissector(data_handle, next_tvb, pinfo, tree);
-			} else {
-				dissect_tcp_payload(next_tvb, pinfo, 0, tcpinfo->seq, /* 0 = offset */
-						    tcpinfo->nxtseq, pinfo->srcport, pinfo->destport,
-						    tree, tree, tcpd);
-			}
+			saved_port = *ptr;
+			*ptr = dissect_as;
+			decode_tcp_ports(tvb, 0, pinfo, tree,
+				pinfo->srcport, pinfo->destport, NULL);
+			*ptr = saved_port;
 		}
-
-		g_strfreev(strings); /* Free the result of g_strsplit() above */
-
-	} else {
-		/* Call the default data dissector */
-		   call_dissector(data_handle, next_tvb, pinfo, sub_tree);
 	}
+	g_strfreev(strings); /* Free the result of g_strsplit() above */
 }
 
 
@@ -1470,7 +1537,8 @@ http_payload_subdissector(tvbuff_t *next_tvb, proto_tree *tree, proto_tree *sub_
  */
 static int
 is_http_request_or_reply(const gchar *data, int linelen, http_type_t *type,
-		ReqRespDissector *reqresp_dissector)
+			 ReqRespDissector *reqresp_dissector,
+			 http_conv_t *conv_data)
 {
 	int isHttpRequestOrReply = FALSE;
 	int prefix_len = 0;
@@ -1639,9 +1707,13 @@ is_http_request_or_reply(const gchar *data, int linelen, http_type_t *type,
 
 		if (isHttpRequestOrReply && reqresp_dissector) {
 			*reqresp_dissector = basic_request_dissector;
-			if (!stat_info->request_method)
-				stat_info->request_method = se_strndup(data, index+1);
+
+			stat_info->request_method = ep_strndup(data, index+1);
+			conv_data->request_method = se_strndup(data, index+1);
 		}
+
+
+
 	}
 
 	return isHttpRequestOrReply;
@@ -1694,7 +1766,8 @@ static const header_info headers[] = {
 static void
 process_header(tvbuff_t *tvb, int offset, int next_offset,
     const guchar *line, int linelen, int colon_offset,
-    packet_info *pinfo, proto_tree *tree, headers_t *eh_ptr)
+    packet_info *pinfo, proto_tree *tree, headers_t *eh_ptr,
+    http_conv_t *conv_data)
 {
 	int len;
 	int line_end_offset;
@@ -1852,7 +1925,8 @@ process_header(tvbuff_t *tvb, int offset, int next_offset,
 			break;
 
 		case HDR_HOST:
-			stat_info->http_host = se_strndup(value, value_len);
+			stat_info->http_host = ep_strndup(value, value_len);
+			conv_data->http_host = se_strndup(value, value_len);
 			break;
 
 		}
@@ -1950,41 +2024,76 @@ check_auth_basic(proto_item *hdr_item, tvbuff_t *tvb, gchar *value)
 static void
 dissect_http(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
+	http_conv_t	*conv_data;
 	int		offset = 0;
 	int		len;
 
-	while (tvb_reported_length_remaining(tvb, offset) != 0) {
-		len = dissect_http_message(tvb, offset, pinfo, tree);
-		if (len == -1)
-			break;
-		offset += len;
+	/*
+	 * Check if this is proxied connection and if so, hand of dissection to the
+	 * payload-dissector.
+	 * Response code 200 means "OK" and strncmp() == 0 means the strings match exactly */
+	conv_data = get_http_conversation_data(pinfo);
+	if(pinfo->fd->num >= conv_data->startframe &&
+	   conv_data->response_code == 200 &&
+	   conv_data->request_method &&
+	   strncmp(conv_data->request_method, "CONNECT", 7) == 0 &&
+	   conv_data->request_uri) {
+		if(conv_data->startframe == 0 && !pinfo->fd->flags.visited)
+			conv_data->startframe = pinfo->fd->num;
+		http_payload_subdissector(tvb, tree, pinfo, conv_data);
+	} else {
+		while (tvb_reported_length_remaining(tvb, offset) != 0) {
+			len = dissect_http_message(tvb, offset, pinfo, tree, conv_data);
+			if (len == -1)
+				break;
+			offset += len;
 
-		/*
-		 * OK, we've set the Protocol and Info columns for the
-		 * first HTTP message; make the columns non-writable,
-		 * so that we don't change it for subsequent HTTP messages.
-		 */
-		col_set_writable(pinfo->cinfo, FALSE);
+			/*
+			 * OK, we've set the Protocol and Info columns for the
+			 * first HTTP message; set a fence so that subsequent
+			 * HTTP messages don't overwrite the Info column.
+			 */
+			if (check_col(pinfo->cinfo, COL_INFO))
+				col_set_fence(pinfo->cinfo, COL_INFO);
+		}
 	}
 }
 
 static void
 dissect_http_udp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-	dissect_http_message(tvb, 0, pinfo, tree);
+	http_conv_t	*conv_data;
+
+	conv_data = get_http_conversation_data(pinfo);
+	dissect_http_message(tvb, 0, pinfo, tree, conv_data);
+}
+
+static void range_delete_http_tcp_callback(guint32 port) {
+  dissector_delete("tcp.port", port, http_handle);
+}
+
+static void range_add_http_tcp_callback(guint32 port) {
+  dissector_add("tcp.port", port, http_handle);
+}
+
+static void range_delete_http_ssl_callback(guint32 port) {
+  ssl_dissector_delete(port, "http", TRUE);
+}
+
+static void range_add_http_ssl_callback(guint32 port) {
+  ssl_dissector_add(port, "http", TRUE);
 }
 
 static void reinit_http(void) {
-	if ( http_alternate_tcp_port != alternate_tcp_port ) {
+	range_foreach(http_tcp_range, range_delete_http_tcp_callback);
+	g_free(http_tcp_range);
+	http_tcp_range = range_copy(global_http_tcp_range);
+	range_foreach(http_tcp_range, range_add_http_tcp_callback);
 
-		if (alternate_tcp_port)
-			dissector_delete("tcp.port", alternate_tcp_port, http_handle );
-
-		if (http_alternate_tcp_port)
-			dissector_add("tcp.port", http_alternate_tcp_port, http_handle);
-
-		alternate_tcp_port = http_alternate_tcp_port;
-	}
+	range_foreach(http_ssl_range, range_delete_http_ssl_callback);
+	g_free(http_ssl_range);
+	http_ssl_range = range_copy(global_http_ssl_range);
+	range_foreach(http_ssl_range, range_add_http_ssl_callback);
 }
 
 void
@@ -2034,6 +2143,14 @@ proto_register_http(void)
 	      { "Proxy-Authorization",	"http.proxy_authorization",
 		FT_STRING, BASE_NONE, NULL, 0x0,
 		"HTTP Proxy-Authorization header", HFILL }},
+	    { &hf_http_proxy_connect_host,
+	      { "Proxy-Connect-Hostname", "http.proxy_connect_host",
+		FT_STRING, BASE_NONE, NULL, 0x0,
+		"HTTP Proxy Connect Hostname", HFILL }},
+	    { &hf_http_proxy_connect_port,
+	      { "Proxy-Connect-Port",	"http.proxy_connect_port",
+		FT_UINT16, BASE_DEC, NULL, 0x0,
+		"HTTP Proxy Connect Port", HFILL }},
 	    { &hf_http_www_authenticate,
 	      { "WWW-Authenticate",	"http.www_authenticate",
 		FT_STRING, BASE_NONE, NULL, 0x0,
@@ -2159,10 +2276,19 @@ proto_register_http(void)
 	    "using \"Content-Encoding: \"",
 	    &http_decompress_body);
 #endif
-	prefs_register_uint_preference(http_module, "tcp_alternate_port",
-								   "Alternate TCP port",
-								   "Decode packets on this TCP port as HTTP",
-								   10,&http_alternate_tcp_port);
+	prefs_register_obsolete_preference(http_module, "tcp_alternate_port");
+
+	range_convert_str(&global_http_tcp_range, TCP_DEFAULT_RANGE, 65535);
+	http_tcp_range = range_empty();
+	prefs_register_range_preference(http_module, "tcp.port", "TCP Ports",
+									"TCP Ports range",
+									&global_http_tcp_range, 65535);
+
+	range_convert_str(&global_http_ssl_range, SSL_DEFAULT_RANGE, 65535);
+	http_ssl_range = range_empty();
+	prefs_register_range_preference(http_module, "ssl.port", "SSL/TLS Ports",
+									"SSL/TLS Ports range",
+									&global_http_ssl_range, 65535);
 
 	http_handle = create_dissector_handle(dissect_http, proto_http);
 
@@ -2197,7 +2323,8 @@ proto_register_http(void)
 	/*
 	 * Register for tapping
 	 */
-	http_tap = register_tap("http");
+	http_tap = register_tap("http"); /* HTTP statistics tap */
+      	http_eo_tap = register_tap("http_eo"); /* HTTP Export Object tap */
 }
 
 /*
@@ -2226,18 +2353,10 @@ proto_reg_handoff_http(void)
 	data_handle = find_dissector("data");
 	media_handle = find_dissector("media");
 
-	dissector_add("tcp.port", TCP_PORT_HTTP, http_handle);
-	dissector_add("tcp.port", TCP_PORT_PROXY_HTTP, http_handle);
-	dissector_add("tcp.port", TCP_ALT_PORT_HTTP, http_handle);
-	dissector_add("tcp.port", TCP_RADAN_HTTP, http_handle);
-	dissector_add("tcp.port", TCP_PORT_PROXY_ADMIN_HTTP, http_handle);
-	dissector_add("tcp.port", TCP_PORT_HKP, http_handle);
-
 	/*
 	 * XXX - is there anything to dissect in the body of an SSDP
 	 * request or reply?  I.e., should there be an SSDP dissector?
 	 */
-	dissector_add("tcp.port", TCP_PORT_SSDP, http_handle);
 	http_udp_handle = create_dissector_handle(dissect_http_udp, proto_http);
 	dissector_add("udp.port", UDP_PORT_SSDP, http_udp_handle);
 
@@ -2307,4 +2426,6 @@ proto_reg_handoff_message_http(void)
 			proto_message_http);
 
 	dissector_add_string("media_type", "message/http", message_http_handle);
+
+	reinit_http();
 }
