@@ -9,7 +9,7 @@
  * Significantly based on packet-fcip.c by
  *       Copyright 2001, Dinesh G Dutt (ddutt@cisco.com)
  *
- * $Id: packet-ifcp.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-ifcp.c 19751 2006-10-31 09:29:07Z sahlberg $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -504,7 +504,7 @@ dissect_ifcp_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 }
 
 static guint
-get_ifcp_pdu_len(tvbuff_t *tvb, int offset)
+get_ifcp_pdu_len(packet_info *pinfo _U_, tvbuff_t *tvb, int offset)
 {
 	guint pdu_len;
  
