@@ -1,7 +1,7 @@
 /* packet-ipv6.h
  * Definitions for IPv6 packet disassembly
  *
- * $Id: packet-ipv6.h 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-ipv6.h 24672 2008-03-17 21:21:48Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -62,14 +62,14 @@ struct ip6_hdr {
 #define ip6_hops	ip6_ctlun.ip6_un1.ip6_un1_hlim
 
 /* Offsets of fields within an IPv6 header. */
-#define	IP6H_CTL	0
+#define	IP6H_CTL		0
 #define	IP6H_CTL_FLOW	0
 #define	IP6H_CTL_PLEN	4
 #define	IP6H_CTL_NXT	6
 #define	IP6H_CTL_HLIM	7
 #define	IP6H_CTL_VFC	0
-#define	IP6H_SRC	8
-#define	IP6H_DST	24
+#define	IP6H_SRC		8
+#define	IP6H_DST		24
 
 #define IPV6_FLOWINFO_MASK	0x0fffffff	/* flow info (28 bits) */
 #define IPV6_FLOWLABEL_MASK	0x000fffff	/* flow label (20 bits) */
@@ -100,17 +100,17 @@ struct ip6_dest {
 };
 
 /* Option types and related macros */
-#define IP6OPT_PAD1		0x00	/* 00 0 00000 */
-#define IP6OPT_PADN		0x01	/* 00 0 00001 */
-#define IP6OPT_JUMBO		0xC2	/* 11 0 00010 = 194 */
-#define IP6OPT_JUMBO_LEN	6
-#define IP6OPT_RTALERT		0x05	/* 00 0 00101 */
+#define IP6OPT_PAD1				0x00	/* 00 0 00000 */
+#define IP6OPT_PADN				0x01	/* 00 0 00001 */
+#define IP6OPT_JUMBO			0xC2	/* 11 0 00010 = 194 */
+#define IP6OPT_JUMBO_LEN		6
+#define IP6OPT_RTALERT			0x05	/* 00 0 00101 */
 
-#define IP6OPT_RTALERT_LEN	4
-#define IP6OPT_RTALERT_MLD	0	/* Datagram contains MLD message */
-#define IP6OPT_RTALERT_RSVP	1	/* Datagram contains RSVP message */
+#define IP6OPT_RTALERT_LEN		4
+#define IP6OPT_RTALERT_MLD		0	/* Datagram contains MLD message */
+#define IP6OPT_RTALERT_RSVP		1	/* Datagram contains RSVP message */
 #define IP6OPT_RTALERT_ACTNET	2	/* contains an Active Networks msg */
-#define IP6OPT_MINLEN		2
+#define IP6OPT_MINLEN			2
 
 #define IP6OPT_HOME_ADDRESS	0xC9  /* 11 0 01001 */
 
@@ -150,6 +150,61 @@ struct ip6_frag {
 	guint32 ip6f_ident;		/* identification */
 };
 
+/* SHIM6 control message types */
+#define SHIM6_TYPE_I1 			0x01	/* 0 000 0001 */
+#define SHIM6_TYPE_R1 			0x02	/* 0 000 0010 */
+#define SHIM6_TYPE_I2			0x03	/* 0 000 0011 */
+#define SHIM6_TYPE_R2 			0x04	/* 0 000 0100 */
+#define SHIM6_TYPE_R1BIS		0x05	/* 0 000 0101 */
+#define SHIM6_TYPE_I2BIS		0x06	/* 0 000 0110 */
+#define SHIM6_TYPE_UPD_REQ		0x40	/* 0 100 0000 = 64 */
+#define SHIM6_TYPE_UPD_ACK		0x41	/* 0 100 0001 = 65 */
+#define SHIM6_TYPE_KEEPALIVE	0x42	/* 0 100 0010 = 66 */
+#define SHIM6_TYPE_PROBE 		0x43	/* 0 100 0011 = 67 */
+
+/* SHIM6 Options */
+#define SHIM6_OPT_RESPVAL       0x01    /* 0 000 0001 */
+#define SHIM6_OPT_LOCLIST       0x02    /* 0 000 0010 */
+#define SHIM6_OPT_LOCPREF       0x03    /* 0 000 0011 */
+#define SHIM6_OPT_CGAPDM        0x04    /* 0 000 0100 */
+#define SHIM6_OPT_CGASIG        0x05    /* 0 000 0101 */
+#define SHIM6_OPT_ULIDPAIR      0x06    /* 0 000 0110 */
+#define SHIM6_OPT_FII           0x07    /* 0 000 0111 */
+
+/* SHIM6 Bitmasks */
+#define SHIM6_BITMASK_P			0x80	/* 1 000 0000 */
+#define SHIM6_BITMASK_TYPE		0x7F	/* 0 111 1111 */
+#define SHIM6_BITMASK_PROTOCOL	0x01	/* 0 000 0001 */
+#define SHIM6_BITMASK_SPECIFIC	0xFE	/* 1 111 1110 */
+#define SHIM6_BITMASK_R			0x80	/* 1 000 0000 */
+#define SHIM6_BITMASK_CT		0x7F	/* 0 111 1111 */
+#define SHIM6_BITMASK_OPT_TYPE	0xFFFE	/* 1 111 1111    1 111 1110 */
+#define SHIM6_BITMASK_CRITICAL	0x01	/* 0 000 0001 */
+#define SHIM6_BITMASK_PRECVD	0xF0	/* 1 111 0000 */
+#define SHIM6_BITMASK_PSENT		0x0F	/* 0 000 1111 */
+#define SHIM6_BITMASK_STA		0xC0	/* 1 100 0000 */
+
+/* SHIM6 Verification Methods */
+#define SHIM6_VERIF_HBA			0x01	/* 0 000 0001 */
+#define SHIM6_VERIF_CGA			0x02	/* 0 000 0010 */
+
+/* SHIM6 Flags */
+#define SHIM6_FLAG_BROKEN		0x01	/* 0 000 0001 */
+#define SHIM6_FLAG_TEMPORARY	0x02	/* 0 000 0010 */
+
+/* SHIM6 REAP States */
+#define SHIM6_REAP_OPERATIONAL	0x00	/* 0 000 0000 */
+#define SHIM6_REAP_EXPLORING	0x01	/* 0 000 0001 */
+#define SHIM6_REAP_INBOUNDOK	0x02	/* 0 000 0010 */
+
+/* SHIM6 header */
+struct ip6_shim {
+	guint8  ip6s_nxt;		/* next header */
+	guint8  ip6s_len;		/* header extension length */
+	guint8  ip6s_p;			/* P field and first 7 bits of remainder */
+	/* followed by shim6 specific data*/
+};
+
 #define IP6F_OFF_MASK		0xfff8	/* mask out offset from _offlg */
 #define IP6F_RESERVED_MASK	0x0006	/* reserved bits in ip6f_offlg */
 #define IP6F_MORE_FRAG		0x0001	/* more-fragments flag */
@@ -187,53 +242,54 @@ struct icmp6_hdr {
 #define ICMP6_TIME_EXCEEDED		3	/* time exceeded, code: */
 #define ICMP6_PARAM_PROB		4	/* ip6 header bad */
 
-#define ICMP6_ECHO_REQUEST		128	/* echo service */
-#define ICMP6_ECHO_REPLY		129	/* echo reply */
+#define ICMP6_ECHO_REQUEST			128	/* echo service */
+#define ICMP6_ECHO_REPLY			129	/* echo reply */
 #define ICMP6_MEMBERSHIP_QUERY		130	/* group membership query */
-#define MLD6_LISTENER_QUERY		130 	/* multicast listener query */
+#define MLD6_LISTENER_QUERY			130 	/* multicast listener query */
 #define ICMP6_MEMBERSHIP_REPORT		131	/* group membership report */
 #define MLD6_LISTENER_REPORT		131	/* multicast listener report */
 #define ICMP6_MEMBERSHIP_REDUCTION	132	/* group membership termination */
-#define MLD6_LISTENER_DONE		132	/* multicast listener done */
+#define MLD6_LISTENER_DONE			132	/* multicast listener done */
 
 #define ND_ROUTER_SOLICIT		133	/* router solicitation */
 #define ND_ROUTER_ADVERT		134	/* router advertisment */
 #define ND_NEIGHBOR_SOLICIT		135	/* neighbor solicitation */
 #define ND_NEIGHBOR_ADVERT		136	/* neighbor advertisment */
-#define ND_REDIRECT			137	/* redirect */
+#define ND_REDIRECT				137	/* redirect */
 
 #define ICMP6_ROUTER_RENUMBERING	138	/* router renumbering */
 
-#define ICMP6_WRUREQUEST		139	/* who are you request */
-#define ICMP6_WRUREPLY			140	/* who are you reply */
-#define ICMP6_FQDN_QUERY		139	/* FQDN query */
-#define ICMP6_FQDN_REPLY		140	/* FQDN reply */
-#define ICMP6_NI_QUERY			139	/* node information request */
-#define ICMP6_NI_REPLY			140	/* node information reply */
-#define ICMP6_IND_SOLICIT		141	/* Inverse ND Solicitation */
-#define ICMP6_IND_ADVERT		142	/* Inverse ND advertisement */
-#define ICMP6_MLDV2_REPORT		143	/* MLD v2 report message : draft-vida-mld-v2-08.txt */
+#define ICMP6_WRUREQUEST			139	/* who are you request */
+#define ICMP6_WRUREPLY				140	/* who are you reply */
+#define ICMP6_FQDN_QUERY			139	/* FQDN query */
+#define ICMP6_FQDN_REPLY			140	/* FQDN reply */
+#define ICMP6_NI_QUERY				139	/* node information request */
+#define ICMP6_NI_REPLY				140	/* node information reply */
+#define ICMP6_IND_SOLICIT			141	/* Inverse ND Solicitation */
+#define ICMP6_IND_ADVERT			142	/* Inverse ND advertisement */
+#define ICMP6_MLDV2_REPORT			143	/* MLD v2 report message : draft-vida-mld-v2-08.txt */
 
 #define ICMP6_MIP6_DHAAD_REQUEST	144	/* Mobile IPv6 DHAAD */
 #define ICMP6_MIP6_DHAAD_REPLY		145	/* Mobile IPv6 DHAAD */
-#define ICMP6_MIP6_MPS			146	/* Mobile IPv6 MPS */
-#define ICMP6_MIP6_MPA			147	/* Mobile IPv6 MPA */
+#define ICMP6_MIP6_MPS				146	/* Mobile IPv6 MPS */
+#define ICMP6_MIP6_MPA				147	/* Mobile IPv6 MPA */
+#define ICMP6_CERT_PATH_SOL			148 /* Certification Path Solicitation Message          [RFC3971] */
+#define ICMP6_CERT_PATH_AD			149 /* Certification Path Advertisement Message         [RFC3971] */
+#define ICMP6_MAXTYPE				153
 
-#define ICMP6_MAXTYPE			153
-
-#define ICMP6_DST_UNREACH_NOROUTE	0	/* no route to destination */
-#define ICMP6_DST_UNREACH_ADMIN	 	1	/* administratively prohibited */
+#define ICMP6_DST_UNREACH_NOROUTE		0	/* no route to destination */
+#define ICMP6_DST_UNREACH_ADMIN	 		1	/* administratively prohibited */
 #define ICMP6_DST_UNREACH_NOTNEIGHBOR	2	/* not a neighbor(obsolete) */
 #define ICMP6_DST_UNREACH_BEYONDSCOPE	2	/* beyond scope of source address */
-#define ICMP6_DST_UNREACH_ADDR		3	/* address unreachable */
-#define ICMP6_DST_UNREACH_NOPORT	4	/* port unreachable */
+#define ICMP6_DST_UNREACH_ADDR			3	/* address unreachable */
+#define ICMP6_DST_UNREACH_NOPORT		4	/* port unreachable */
 
-#define ICMP6_TIME_EXCEED_TRANSIT 	0	/* ttl==0 in transit */
+#define ICMP6_TIME_EXCEED_TRANSIT 		0	/* ttl==0 in transit */
 #define ICMP6_TIME_EXCEED_REASSEMBLY	1	/* ttl==0 in reass */
 
-#define ICMP6_PARAMPROB_HEADER 	 	0	/* erroneous header field */
-#define ICMP6_PARAMPROB_NEXTHEADER	1	/* unrecognized next header */
-#define ICMP6_PARAMPROB_OPTION		2	/* unrecognized option */
+#define ICMP6_PARAMPROB_HEADER 	 		0	/* erroneous header field */
+#define ICMP6_PARAMPROB_NEXTHEADER		1	/* unrecognized next header */
+#define ICMP6_PARAMPROB_OPTION			2	/* unrecognized option */
 
 #define ICMP6_INFOMSG_MASK		0x80	/* all informational messages */
 
@@ -349,16 +405,45 @@ struct nd_opt_hdr {		/* Neighbor discovery option header */
 	/* followed by option specific data*/
 };
 
+/* http://www.iana.org/assignments/icmpv6-parameters */
 #define ND_OPT_SOURCE_LINKADDR		1
 #define ND_OPT_TARGET_LINKADDR		2
 #define ND_OPT_PREFIX_INFORMATION	3
 #define ND_OPT_REDIRECTED_HEADER	4
-#define ND_OPT_MTU			5
-#define ND_OPT_ADVINTERVAL		7
+#define ND_OPT_MTU					5
+#define ND_OPT_ADVINTERVAL			7
 #define ND_OPT_HOMEAGENT_INFO		8
 #define ND_OPT_SOURCE_ADDRLIST		9
 #define ND_OPT_TARGET_ADDRLIST		10
 
+#define ND_OPT_CGA					11
+#define ND_OPT_RSA					12
+#define ND_OPT_TIMESTAMP			13
+#define ND_OPT_NONCE				14
+#define ND_OPT_TRUST_ANCHOR			15
+#define ND_OPT_CERTIFICATE			16
+
+/*
+17      IP Address Option                       [RFC4068]  
+18      New Router Prefix Information Option    [RFC4068]
+19      Link-layer Address Option               [RFC4068]
+20      Neighbor Advertisement Acknowledgment   [RFC4068]
+        Option
+21      CARD Request option                     [RFC4065]
+22      CARD Reply option                       [RFC4065]
+23      MAP Option                              [RFC4140]
+24      Route Information Option                [RFC4191]
+*/
+#define ND_OPT_RECURSIVE_DNS_SERVER	25	/* Recursive DNS Server Option             [RFC5006] */
+/*
+26      RA Flags Extension Option               [RFC5075]
+27      Handover Key Request Option             [RFC-ietf-mipshop-handover-key-03.txt]
+28      Handover Key Reply Option               [RFC-ietf-mipshop-handover-key-03.txt]
+29-252  Unassigned
+253     RFC3692-style Experiment 1 (*)          [RFC4727]
+254     RFC3692-style Experiment 2 (*)          [RFC4727]
+*/
+
 /* draft-ietf-ipngwg-router-preference, not officially assigned yet */
 #define ND_OPT_ROUTE_INFO		200
 /* draft-ietf-mobileip-hmipv6, not officially assigned yet */
