@@ -1,6 +1,6 @@
 /* packet-isup.h
  *
- * $Id: packet-isup.h 19639 2006-10-20 19:29:41Z guy $
+ * $Id: packet-isup.h 21966 2007-05-28 06:13:03Z etxrab $
  *
  * Copyright 2003, Michael Lum <mlum [AT] telostech.com>,
  * In association with Telos Technology Inc.
@@ -133,6 +133,10 @@ WS_VAR_IMPORT const value_string q850_cause_code_vals[];
 #define PARAM_TYPE_COLLECT_CALL_REQ     121
 #define PARAM_TYPE_GENERIC_NR           192
 #define PARAM_TYPE_GENERIC_DIGITS       193
+#define PARAM_TYPE_JURISDICTION			196
+#define PARAM_TYPE_GENERIC_NAME			199
+#define PARAM_TYPE_ORIG_LINE_INFO		234
+#define PARAM_TYPE_CHARGE_NR			235
 
 #define ANSI_ISUP_PARAM_TYPE_OPER_SERV_INF	0xC2
 #define ANSI_ISUP_PARAM_TYPE_EGRESS			0xC3
