@@ -8,7 +8,7 @@
  *
  * Copyright 2002, Michael Tuexen <tuexen [AT] fh-muenster.de>
  *
- * $Id: packet-iua.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-iua.c 21633 2007-05-01 09:45:34Z martinm $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -845,7 +845,7 @@ proto_register_iua(void)
 
   /* Setup list of header fields */
   static hf_register_info hf[] = {
-    { &hf_int_interface_id,      { "Integer interface identifier", "iua.int_interface_identifier",  FT_INT32,   BASE_HEX,  NULL,                           0x0,            "", HFILL } },
+    { &hf_int_interface_id,      { "Integer interface identifier", "iua.int_interface_identifier",  FT_UINT32,  BASE_HEX,  NULL,                           0x0,            "", HFILL } },
     { &hf_text_interface_id,     { "Text interface identifier",    "iua.text_interface_identifier", FT_STRING,  BASE_NONE, NULL,                           0x0,            "", HFILL } },
     { &hf_info_string,           { "Info string",                  "iua.info_string",               FT_STRING,  BASE_NONE, NULL,                           0x0,            "", HFILL } },
     { &hf_dlci_zero_bit,         { "Zero bit",                     "iua.dlci_zero_bit",             FT_BOOLEAN, 8,         NULL,                           ZERO_BIT_MASK,  "", HFILL } },
@@ -892,6 +892,9 @@ proto_register_iua(void)
   proto_register_field_array(proto_iua, hf, array_length(hf));
   proto_register_subtree_array(ett, array_length(ett));
   prefs_register_bool_preference(iua_module, "support_ig", "Support Implementers Guide", "Support Implementers Guide (version 01)", &support_IG);
+
+  /* Allow other dissectors to find this one by name. */
+  register_dissector("iua", dissect_iua, proto_iua);
 }
 
 #define SCTP_PORT_IUA          9900
