@@ -7,7 +7,7 @@
  * Laurent Cazalet <laurent.cazalet@mailclub.net>
  * Thomas Parvais <thomas.parvais@advalvas.be>
  *
- * $Id: packet-l2tp.c 18900 2006-08-14 21:37:12Z jmayer $
+ * $Id: packet-l2tp.c 23405 2007-11-08 22:40:20Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -32,6 +32,9 @@
  * RFC 2661 for L2TPv2
  * I-D draft-ietf-l2tpext-l2tp-base for L2TPv3
  *
+ * L2TPv3
+ * http://www.ietf.org/rfc/rfc3931.txt
+ *
  * Layer Two Tunneling Protocol "L2TP" number assignments:
  *	http://www.iana.org/assignments/l2tp-parameters
  */
@@ -124,6 +127,7 @@ static enum_val_t l2tpv3_cookies[] = {
 #define L2TPv3_PROTOCOL_IP      4
 #define L2TPv3_PROTOCOL_MPLS    5
 #define L2TPv3_PROTOCOL_AAL5    6
+#define L2TPv3_PROTOCOL_LAPD	7
 
 static enum_val_t l2tpv3_protocols[] = {
     {"eth",     "Ethernet",     L2TPv3_PROTOCOL_ETH},
@@ -133,17 +137,20 @@ static enum_val_t l2tpv3_protocols[] = {
     {"ip",      "IP",           L2TPv3_PROTOCOL_IP},
     {"mpls",    "MPLS",         L2TPv3_PROTOCOL_MPLS},
     {"aal5",    "AAL5",         L2TPv3_PROTOCOL_AAL5},
+    {"lapd",	"LAPD",         L2TPv3_PROTOCOL_LAPD},
     {NULL, NULL, 0}
 };
 
 #define L2TPv3_L2_SPECIFIC_NONE         0
 #define L2TPv3_L2_SPECIFIC_DEFAULT      1
 #define L2TPv3_L2_SPECIFIC_ATM          2
+#define L2TPv3_L2_SPECIFIC_LAPD         3
 
 static enum_val_t l2tpv3_l2_specifics[] = {
     {"none",    "None",                 L2TPv3_L2_SPECIFIC_NONE},
     {"default", "Default L2-Specific",  L2TPv3_L2_SPECIFIC_DEFAULT},
     {"atm",     "ATM-Specific",         L2TPv3_L2_SPECIFIC_ATM},
+    {"lapd",    "LAPD-Specific",        L2TPv3_L2_SPECIFIC_LAPD},
     {NULL, NULL, 0}
 };
 
@@ -166,29 +173,38 @@ static gint l2tpv3_l2_specific = L2TPv3_L2_SPECIFIC_DEFAULT;
 #define AVP_Reserved1 13
 #define AVP_CDN       14
 
-#define NUM_CONTROL_CALL_TYPES  20
+#define NUM_CONTROL_CALL_TYPES  27
 static const char *calltypestr[NUM_CONTROL_CALL_TYPES+1] = {
-  "Unknown Call Type           ",
-  "Start_Control_Request       ",
-  "Start_Control_Reply         ",
-  "Start_Control_Connected     ",
-  "Stop_Control_Notification   ",
-  "Reserved                    ",
-  "Hello                       ",
-  "Outgoing_Call_Request       ",
-  "Outgoing_Call_Reply         ",
-  "Outgoing_Call_Connected     ",
-  "Incoming_Call_Request       ",
-  "Incoming_Call_Reply         ",
-  "Incoming_Call_Connected     ",
-  "Reserved                    ",
+  "Unknown Call Type",
+  "Start_Control_Request",
+  "Start_Control_Reply",
+  "Start_Control_Connected",
+  "Stop_Control_Notification",
+  "Reserved",							/* 5*/
+  "Hello",
+  "Outgoing_Call_Request",
+  "Outgoing_Call_Reply",
+  "Outgoing_Call_Connected",
+  "Incoming_Call_Request",				/* 10 */
+  "Incoming_Call_Reply",
+  "Incoming_Call_Connected",
+  "Reserved",							/* 13 */
   "Call_Disconnect_Notification",
-  "WAN_Error_Notify            ",
-  "Set_Link_Info               ",
-  "Modem_Status                ",
-  "Service_Relay_Request_Msg   ",
-  "Service_Relay_Reply_Message ",
-  "Explicit_Acknowledgement    ",
+  "WAN_Error_Notify",					/* 15 */
+  "Set_Link_Info",
+  "Modem_Status",
+  "Service_Relay_Request_Msg",
+  "Service_Relay_Reply_Message",
+  "Explicit_Acknowledgement",			/* 20 */
+  "Failover_Session_Query_Message",		/* 21 [RFC4951] */
+  "Failover_Session_Response_Message",	/* 22 [RFC4951] */
+  /* Multicast Management */
+  "Multicast-Session-Request",			/* 23 [RFC4045]*/
+  "Multicast-Session-Response ",		/* 24 [RFC4045]*/
+  "Multicast-Session-Establishment",	/* 25 [RFC4045]*/
+  "Multicast-Session-Information",		/* 26 [RFC4045]*/
+  "Multicast-Session-End-Notify",		/* 27 [RFC4045]*/
+
 };
 
 static const char *calltype_short_str[NUM_CONTROL_CALL_TYPES+1] = {
@@ -197,22 +213,30 @@ static const char *calltype_short_str[NUM_CONTROL_CALL_TYPES+1] = {
   "SCCRP   ",
   "SCCCN   ",
   "StopCCN ",
-  "Reserved",
+  "Reserved", /* 5 */
   "Hello   ",
   "OCRQ    ",
   "OCRP    ",
   "OCCN    ",
-  "ICRQ    ",
+  "ICRQ    ", /* 10 */
   "ICRP    ",
   "ICCN    ",
   "Reserved",
   "CDN     ",
-  "WEN     ",
+  "WEN     ", /* 15 */
   "SLI     ",
   "MDMST   ",
   "SRRQ    ",
   "SRRP    ",
-  "ACK     ",
+  "ACK     ", /* 20 */
+  "FSQ     ", 
+  "FSR     ", 
+  "MSRQ    ", 
+  "MSRP    ", 
+  "MSE     ", /* 25 */
+  "MSI     ", 
+  "MSEN    ", 
+
 };
 
 
@@ -245,11 +269,13 @@ static const true_false_string l2tp_priority_truth =
 
 static const value_string authen_type_vals[] = {
   { 0, "Reserved" },
-  { 1, "Textual username and password" },
+  { 1, "Textual username/password exchange" },
   { 2, "PPP CHAP" },
   { 3, "PPP PAP" },
   { 4, "No Authentication" },
   { 5, "Microsoft CHAP Version 1" },
+  { 6, "Reserved" },
+  { 7, "EAP" },
   { 0, NULL }
 };
 
@@ -278,7 +304,7 @@ static const value_string result_code_stopccn_vals[] = {
   { 5, "The protocol version of the requester is not supported", },
   { 6, "Requester is being shut down", },
   { 7, "Finite state machine error or timeout", },
-  { 8, "Control connection due to mismatching CCDS value", },
+  { 8, "Control connection due to mismatching CCDS value", }, /* [RFC3308] */
   { 0, NULL }
 };
 
@@ -300,6 +326,18 @@ static const value_string result_code_cdn_vals[] = {
   { 14, "Session not established due to unsupported PW type", },
   { 15, "Session not established, sequencing required without valid L2-Specific Sublayer", },
   { 16, "Finite state machine error or timeout", },
+  { 17, "FR PVC was deleted permanently (no longer provisioned) ", },      /* [RFC4591] */
+  { 18, "FR PVC has been INACTIVE for an extended period of time", },      /* [RFC4591] */
+  { 19, "Mismatched FR Header Length", },                                  /* [RFC4591] */
+  { 20, "HDLC Link was deleted permanently (no longer provisioned)", },    /* [RFC4349] */
+  { 21, "HDLC Link has been INACTIVE for an extended period of time", },   /* [RFC4349] */
+  { 22, "Session not established due to other LCCE can not support the OAM Cell Emulation", },    /* [RFC4454] */
+  { 23, "Mismatching interface MTU", },                                    /* [RFC4667] */
+  { 24, "Attempt to connect to non-existent forwarder", },                 /* [RFC4667] */
+  { 25, "Attempt to connect to unauthorized forwarder", },                 /* [RFC4667] */
+  { 26, "Loop Detected", },                                                /* [draft-ietf-l2tpext-tunnel-switching-06.txt] */
+
+
   { 0, NULL }
 };
 
@@ -314,6 +352,9 @@ static const value_string error_code_vals[] = {
   { 7, "Try another", },
   { 8, "Receipt of an unknown AVP with the M bit set", },
   { 9, "Try another directed", },
+  { 10, "Next hop unreachable", },
+  { 11, "Next hop busy", },
+  { 12, "TSA busy", },
   { 0, NULL }
 };
 
@@ -375,7 +416,7 @@ static const value_string error_code_vals[] = {
 #define  TX_CONNECT_SPEED_V3		74
 #define  RX_CONNECT_SPEED_V3		75
 
-#define NUM_AVP_TYPES  76
+#define NUM_AVP_TYPES  96
 static const value_string avp_type_vals[] = {
   { CONTROL_MESSAGE,           "Control Message" },
   { RESULT_ERROR_CODE,         "Result-Error Code" },
@@ -434,37 +475,60 @@ static const value_string avp_type_vals[] = {
   { CTL_MSG_AUTH_NONCE,        "Control Message Authentication Nonce" },
   { TX_CONNECT_SPEED_V3,       "Tx Connect Speed Version 3" },
   { RX_CONNECT_SPEED_V3,       "Rx Connect Speed Version 3" },
+  { 76,							"Failover Capability" },							/*[RFC4951] */
+  { 77,							"Tunnel Recovery" },								/*[RFC4951] */
+  { 78,							"Suggested Control Sequence" },						/*[RFC4951] */
+  { 79,							"Failover Session State" },							/*[RFC4951] */
+  { 80,							"Multicast Capability" },							/*[RFC4045] */
+  { 81,							"New Outgoing Sessions" },							/*[RFC4045] */
+  { 82,							"New Outgoing Sessions Acknowledgement" },			/*[RFC4045] */
+  { 83,							"Withdraw Outgoing Sessions" },						/*[RFC4045] */
+  { 84,							"Multicast Packets Priority" },						/*[RFC4045] */
+  { 85,							"Frame-Relay Header Length" },						/*[RFC4591] */
+  { 86,							"ATM Maximum Concatenated Cells AVP" },				/*[RFC4454] */
+  { 87,							"OAM Emulation Required AVP" },						/*[RFC4454] */
+  { 88,							"ATM Alarm Status AVP" },							/*[RFC4454] */
+    /*        Also, see ATM Alarm Status AVP Values below */
+  { 89,							"Attachment Group Identifier" },					/*[RFC4667] */
+  { 90,							"Local End Identifier" },							/*[RFC4667] */
+  { 91,							"Interface Maximum Transmission Unit" },			/*[RFC4667] */
+  { 92,							"FCS Retention" },									/*[RFC4720] */
+  { 93,							"Tunnel Switching Aggregator ID AVP" },				/*[draft-ietf-l2tpext-tunnel-switching-06.txt] */
+  { 94,							"Maximum Receive Unit (MRU) AVP" },					/*[RFC4623] */
+  { 95,							"Maximum Reassembled Receive Unit (MRRU) AVP" },	/*[RFC4623] */
+
+
   { 0,                         NULL }
 };
 
 #define CISCO_ASSIGNED_CONNECTION_ID	1
-#define CISCO_PW_CAPABILITY_LIST	2
-#define CISCO_LOCAL_SESSION_ID		3
-#define CISCO_REMOTE_SESSION_ID		4
-#define CISCO_ASSIGNED_COOKIE		5
-#define CISCO_REMOTE_END_ID		6
-#define CISCO_PW_TYPE			7
-#define CISCO_CIRCUIT_STATUS		8
-#define CISCO_SESSION_TIE_BREAKER	9
-#define CISCO_DRAFT_AVP_VERSION		10
-#define CISCO_MESSAGE_DIGEST		12
-#define CISCO_AUTH_NONCE		13
-#define CISCO_INTERFACE_MTU		14
+#define CISCO_PW_CAPABILITY_LIST		2
+#define CISCO_LOCAL_SESSION_ID			3
+#define CISCO_REMOTE_SESSION_ID			4
+#define CISCO_ASSIGNED_COOKIE			5
+#define CISCO_REMOTE_END_ID				6
+#define CISCO_PW_TYPE					7
+#define CISCO_CIRCUIT_STATUS			8
+#define CISCO_SESSION_TIE_BREAKER		9
+#define CISCO_DRAFT_AVP_VERSION			10
+#define CISCO_MESSAGE_DIGEST			12
+#define CISCO_AUTH_NONCE				13
+#define CISCO_INTERFACE_MTU				14
 
 static const value_string cisco_avp_type_vals[] = {
   { CISCO_ASSIGNED_CONNECTION_ID,	"Assigned Connection ID" },
   { CISCO_PW_CAPABILITY_LIST,		"Pseudowire Capabilities List" },
-  { CISCO_LOCAL_SESSION_ID,		"Local Session ID" },
+  { CISCO_LOCAL_SESSION_ID,			"Local Session ID" },
   { CISCO_REMOTE_SESSION_ID,		"Remote Session ID" },
-  { CISCO_ASSIGNED_COOKIE,		"Assigned Cookie" },
-  { CISCO_REMOTE_END_ID,		"Remote End ID" },
-  { CISCO_PW_TYPE,			"Pseudowire Type" },
-  { CISCO_CIRCUIT_STATUS,		"Circuit Status" },
+  { CISCO_ASSIGNED_COOKIE,			"Assigned Cookie" },
+  { CISCO_REMOTE_END_ID,			"Remote End ID" },
+  { CISCO_PW_TYPE,					"Pseudowire Type" },
+  { CISCO_CIRCUIT_STATUS,			"Circuit Status" },
   { CISCO_SESSION_TIE_BREAKER,		"Session Tie Breaker" },
   { CISCO_DRAFT_AVP_VERSION,		"Draft AVP Version" },
   { CISCO_MESSAGE_DIGEST,       	"Message Digest" },
   { CISCO_AUTH_NONCE,        		"Control Message Authentication Nonce" },
-  { CISCO_INTERFACE_MTU,		"Interface MTU" },
+  { CISCO_INTERFACE_MTU,			"Interface MTU" },
   { 0,                         		NULL }
 };
 
@@ -493,6 +557,7 @@ static dissector_handle_t fr_handle;
 static dissector_handle_t ip_handle;
 static dissector_handle_t mpls_handle;
 static dissector_handle_t llc_handle;
+static dissector_handle_t lapd_handle;
 static dissector_handle_t data_handle;
 
 /*
@@ -1343,6 +1408,10 @@ process_l2tpv3_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
           case L2TPv3_L2_SPECIFIC_ATM:
                 next_tvb = tvb_new_subset(tvb, index + l2tpv3_cookie + 4, -1, -1);
                 break;
+		case L2TPv3_L2_SPECIFIC_LAPD:
+				proto_tree_add_text(tree, tvb, index + l2tpv3_cookie + 4, 3,"LAPD info");
+				next_tvb = tvb_new_subset(tvb, index + l2tpv3_cookie+4+3, -1, -1);
+				break;
           case L2TPv3_L2_SPECIFIC_NONE:
           default:
                 next_tvb = tvb_new_subset(tvb, index + l2tpv3_cookie, -1, -1);
@@ -1381,6 +1450,9 @@ process_l2tpv3_data(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
                         call_dissector(llc_handle, next_tvb, pinfo, tree);
                 }
                 break;
+		  case L2TPv3_PROTOCOL_LAPD:
+			  call_dissector(lapd_handle, next_tvb, pinfo, tree);
+			  break;
           default:
                 call_dissector(data_handle, next_tvb, pinfo, tree);
                 break;
@@ -1627,7 +1699,7 @@ dissect_l2tp_udp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	 * as they might not be L2TP packets even though they happen
 	 * to be coming from or going to the L2TP port.
 	 */
-	if (!tvb_bytes_exist(tvb, 0, 2))
+	if (tvb_length(tvb) < 2)
 		return 0;	/* not enough information to check */
 	control = tvb_get_ntohs(tvb, 0);
 	switch (L2TP_VERSION(control)) {
@@ -2052,6 +2124,7 @@ proto_reg_handoff_l2tp(void)
 	ip_handle = find_dissector("ip");
 	mpls_handle = find_dissector("mpls");
 	llc_handle = find_dissector("llc");
+	lapd_handle = find_dissector("lapd");
 
 	data_handle = find_dissector("data");
 }
