@@ -9,7 +9,7 @@
  *          Laboratories, Inc., http://www.PacketCable.com/
  * www.iana.org/assignments/mgcp-localconnectionoptions
  *
- * $Id: packet-mgcp.c 23901 2007-12-17 20:42:10Z gerald $
+ * $Id: packet-mgcp.c 24525 2008-03-01 17:23:39Z stig $
  *
  * Copyright (c) 2000 by Ed Warnicke <hagbard@physics.rutgers.edu>
  * Copyright (c) 2004 by Thomas Anders <thomas.anders [AT] blue-cable.de>
@@ -45,6 +45,7 @@
 #include <epan/prefs.h>
 #include <epan/conversation.h>
 #include <epan/tap.h>
+#include <epan/strutil.h>
 #include "packet-mgcp.h"
 
 #ifdef NEED_G_ASCII_STRCASECMP_H
@@ -1451,15 +1452,14 @@ static void dissect_mgcp_firstline(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 				if (tokenlen > 4)
 					THROW(ReportedBoundsError);
 				code = tvb_format_text(tvb,tvb_previous_offset,tokenlen);
-				strncpy(mi->code,code,4);
-				mi->code[4] = '\0';
+				g_strlcpy(mi->code,code,5);
 				if (is_mgcp_verb(tvb,tvb_previous_offset,tvb_current_len,&verb_description))
 				{
 					mgcp_type = MGCP_REQUEST;
 					if (verb_description != NULL)
 					{
 						/* Can show verb along with code if known */
-						sprintf(code_with_verb, "%s (%s)", code, verb_description);
+						g_snprintf(code_with_verb, 64, "%s (%s)", code, verb_description);
 					}
 
 					proto_tree_add_string_format(tree, hf_mgcp_req_verb, tvb,
@@ -1597,7 +1597,7 @@ static void dissect_mgcp_firstline(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 							mi->request_available = TRUE;
 							mgcp_call->responded = TRUE;
 							mi->req_num = mgcp_call->req_num;
-							strcpy(mi->code,mgcp_call->code);
+							g_strlcpy(mi->code,mgcp_call->code,5);
 							item = proto_tree_add_uint_format(tree, hf_mgcp_req_frame,
 							                                  tvb, 0, 0, mgcp_call->req_num,
 							                                  "This is a response to a request in frame %u",
@@ -1751,7 +1751,7 @@ static void dissect_mgcp_firstline(tvbuff_t *tvb, packet_info *pinfo, proto_tree
 					mgcp_call->transid = mi->transid;
 					mgcp_call->responded = FALSE;
 					mgcp_call->req_time=pinfo->fd->abs_ts;
-					strcpy(mgcp_call->code,mi->code);
+					g_strlcpy(mgcp_call->code,mi->code,5);
 
 					/* Store it */
 					g_hash_table_insert(mgcp_calls, new_mgcp_call_key, mgcp_call);
