@@ -6,7 +6,7 @@
  * Portions Copyright (c) Gilbert Ramirez 2000-2002
  * Portions Copyright (c) Novell, Inc. 2000-2003
  *
- * $Id: packet-ncp-int.h 19073 2006-08-29 08:45:53Z gmorris $
+ * $Id: packet-ncp-int.h 22746 2007-08-30 10:18:29Z gmorris $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -121,6 +121,7 @@ typedef struct {
 	nstime_t		req_frame_time;
     guint16         length;
 	guint32			req_nds_flags;
+	guint32			req_nds_prot_flags;
 	guint8			nds_request_verb;
 	guint8			nds_version;
 	char			object_name[256];
@@ -129,6 +130,7 @@ typedef struct {
     guint32         nds_frag_num;
     guint16         req_mask;
     guint16         req_mask_ext;
+    guint32         nds_frag_flags;
 } ncp_req_hash_value;
 
 void dissect_ncp_request(tvbuff_t*, packet_info*, guint32,
