@@ -5,7 +5,7 @@
  *
  * derived from the packet-nbns.c
  *
- * $Id: packet-netbios.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-netbios.c 23252 2007-10-23 05:50:00Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1149,7 +1149,7 @@ dissect_netbios(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                         break;
 
 		default:
-			col_add_fstr( pinfo->cinfo, COL_INFO, "%s",
+			col_add_str( pinfo->cinfo, COL_INFO, 
 			    command_name);
 			break;
 		}
