@@ -2,7 +2,7 @@
  * Routines for Protocol for carrying Authentication for Network Access dissection
  * Copyright 2006, Peter Racz <racz@ifi.unizh.ch>
  *
- * $Id: packet-pana.c 18894 2006-08-14 08:29:29Z sahlberg $
+ * $Id: packet-pana.c 24643 2008-03-15 22:41:57Z morriss $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -23,7 +23,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 /* This protocol implements PANA as of the internet draft
- * draft-ietf-pana-pana-11  which is a workitem of the ietf workgroup
+ * draft-ietf-pana-pana-18  which is a workitem of the ietf workgroup
  * internet area/pana
  */
 
@@ -44,16 +44,15 @@
 #include <epan/emem.h>
 
 #define PANA_UDP_PORT 3001
-#define PANA_VERSION 1
 
 #define MIN_AVP_SIZE 8
 
 #define PANA_FLAG_R 0x8000
 #define PANA_FLAG_S 0x4000
-#define PANA_FLAG_N 0x2000
-#define PANA_FLAG_L 0x1000
-#define PANA_FLAG_RES4 0x0800
-#define PANA_FLAG_RES5 0x0400
+#define PANA_FLAG_C 0x2000
+#define PANA_FLAG_A 0x1000
+#define PANA_FLAG_P 0x0800
+#define PANA_FLAG_I 0x0400
 #define PANA_FLAG_RES6 0x0200
 #define PANA_FLAG_RES7 0x0100
 #define PANA_FLAG_RES8 0x0080
@@ -67,7 +66,6 @@
 #define PANA_FLAG_RES 0x0fff
 
 #define PANA_AVP_FLAG_V 0x8000
-#define PANA_AVP_FLAG_M 0x4000
 #define PANA_AVP_FLAG_RES2 0x2000
 #define PANA_AVP_FLAG_RES3 0x1000
 #define PANA_AVP_FLAG_RES4 0x0800
@@ -87,10 +85,10 @@
 
 /* Initialize the protocol and registered fields */
 static int proto_pana = -1;
-static int hf_pana_version_type = -1;
 static int hf_pana_reserved_type = -1;
 static int hf_pana_length_type = -1;
 static int hf_pana_msg_type = -1;
+static int hf_pana_session_id = -1;
 static int hf_pana_seqnumber = -1;
 static int hf_pana_response_in = -1;
 static int hf_pana_response_to = -1;
@@ -102,13 +100,14 @@ static dissector_handle_t eap_handle = NULL;
 static int hf_pana_flags = -1;
 static int hf_pana_flag_r = -1;
 static int hf_pana_flag_s = -1;
-static int hf_pana_flag_n = -1;
-static int hf_pana_flag_l = -1;
+static int hf_pana_flag_c = -1;
+static int hf_pana_flag_a = -1;
+static int hf_pana_flag_p = -1;
+static int hf_pana_flag_i = -1;
 static int hf_pana_avp_code = -1;
 static int hf_pana_avp_length = -1;
 static int hf_pana_avp_flags = -1;
 static int hf_pana_avp_flag_v = -1;
-static int hf_pana_avp_flag_m = -1;
 static int hf_pana_avp_reserved = -1;
 static int hf_pana_avp_vendorid = -1;
 
@@ -119,21 +118,12 @@ static int hf_pana_avp_data_int32 = -1;
 static int hf_pana_avp_data_bytes = -1;
 static int hf_pana_avp_data_string = -1;
 static int hf_pana_avp_data_enumerated = -1;
-static int hf_pana_avp_data_addrfamily = -1;
-static int hf_pana_avp_data_ipv4 = -1;
-static int hf_pana_avp_data_ipv6 = -1;
 
 static const value_string msg_type_names[] = {
-       { 1, "PANA-PAA-Discover" },
-       { 2, "PANA-Start" },
-       { 3, "PANA-Auth" },
-       { 4, "PANA-Reauth" },
-       { 5, "PANA-Bind" },
-       { 6, "PANA-Ping" },
-       { 7, "PANA-Termination" },
-       { 8, "PANA-Error" },
-       { 9, "PANA-FirstAuth-End" },
-       { 10, "PANA-Update" },
+       { 1, "PANA-Client-Initiation" },
+       { 2, "PANA-Auth" },
+       { 3, "PANA-Termination" },
+       { 4, "PANA-Notification" },
        { 0, NULL }
 };
 
@@ -144,50 +134,22 @@ static const value_string msg_subtype_names[] = {
 };
 
 static const value_string avp_code_names[] = {
-       { 1, "Algorithm AVP" },
-       { 2, "AUTH AVP" },
-       { 3, "Cookie AVP" },
-       { 4, "Device-Id AVP" },
-       { 5, "EAP-Payload AVP" },
-       { 6, "Failed-AVP AVP" },
-       { 7, "ISP-Information AVP" },
-       { 8, "Key-Id AVP" },
-       { 9, "NAP-Information AVP" },
-       { 10, "Nonce AVP" },
-       { 11, "Notification AVP" },
-       { 12, "PPAC AVP" },
-       { 13, "Protection-Capability AVP" },
-       { 14, "Provider-Identifier AVP" },
-       { 15, "Provider-Name AVP" },
-       { 16, "Result-Code" },
-       { 17, "Session-Id" },
-       { 18, "Session-Lifetime" },
-       { 19, "Termination-Cause" },
+       { 1, "AUTH AVP" },
+       { 2, "EAP-Payload AVP" },
+       { 3, "Integrity-Algorithm AVP" },
+       { 4, "Key-Id AVP" },
+       { 5, "Nonce AVP" },
+       { 6, "PRF-Algorithm AVP" },
+       { 7, "Result-Code" },
+       { 8, "Session-Lifetime" },
+       { 9, "Termination-Cause" },
        { 0, NULL }
 };
 
 static const value_string avp_resultcode_names[] = {
-       { 2001, "PANA_SUCCESS" },
-       { 3001, "PANA_MESSAGE_UNSUPPORTED" },
-       { 3002, "PANA_UNABLE_TO_DELIVER" },
-       { 3008, "PANA_INVALID_HDR_BITS" },
-       { 3009, "PANA_INVALID_AVP_FLAGS" },
-       { 4001, "PANA_AUTHENTICATION_REJECTED" },
-       { 5001, "PANA_AVP_UNSUPPORTED" },
-       { 5002, "PANA_UNKNOWN_SESSION_ID" },
-       { 5003, "PANA_AUTHORIZATION_REJECTED" },
-       { 5004, "PANA_INVALID_AVP_DATA" },
-       { 5005, "PANA_MISSING_AVP" },
-       { 5006, "PANA_RESOURCES_EXCEEDED" },
-       { 5007, "PANA_CONTRADICTING_AVPS" },
-       { 5008, "PANA_AVP_NOT_ALLOWED" },
-       { 5009, "PANA_AVP_OCCURS_TOO_MANY_TIMES" },
-       { 5011, "PANA_UNSUPPORTED_VERSION" },
-       { 5012, "PANA_UNABLE_TO_COMPLY" },
-       { 5014, "PANA_INVALID_AVP_LENGTH" },
-       { 5015, "PANA_INVALID_MESSAGE_LENGTH" },
-       { 5016, "PANA_PROTECTION_CAPABILITY_UNSUPPORTED" },
-       { 5017, "PANA_PPAC_CAPABILITY_UNSUPPORTED" },
+       { 0, "PANA_SUCCESS" },
+       { 1, "PANA_AUTHENTICATION_REJECTED" },
+       { 2, "PANA_AUTHORIZATION_REJECTED" },
        { 0, NULL }
 };
 
@@ -203,13 +165,12 @@ typedef enum {
   PANA_GROUPED,
   PANA_ENUMERATED,
   PANA_UTF8STRING,
-  PANA_IP_ADDRESS,
   PANA_EAP,
   PANA_RESULT_CODE
 } pana_avp_types;
 
 static const value_string avp_type_names[]={
-       { PANA_OCTET_STRING,"OctetString" },
+       { PANA_OCTET_STRING,    "OctetString" },
        { PANA_INTEGER32,       "Integer32" },
        { PANA_INTEGER64,       "Integer64" },
        { PANA_UNSIGNED32,      "Unsigned32" },
@@ -220,7 +181,6 @@ static const value_string avp_type_names[]={
        { PANA_GROUPED,         "Grouped" },
        { PANA_ENUMERATED,      "Enumerated" },
        { PANA_UTF8STRING,      "UTF8String" },
-       { PANA_IP_ADDRESS,      "IPAddress" },
        { PANA_EAP,             "OctetString" },
        { PANA_RESULT_CODE,     "Unsigned32" },
        { 0, NULL }
@@ -268,15 +228,20 @@ dissect_pana_flags(proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint16 f
        proto_tree_add_boolean(flags_tree, hf_pana_flag_s, tvb, offset, 2, flags);
        if (flags & PANA_FLAG_S)
                proto_item_append_text(flags_item, ", S flag set");
-       proto_tree_add_boolean(flags_tree, hf_pana_flag_n, tvb, offset, 2, flags);
-       if (flags & PANA_FLAG_N)
-               proto_item_append_text(flags_item, ", N flag set");
-       proto_tree_add_boolean(flags_tree, hf_pana_flag_l, tvb, offset, 2, flags);
-       if (flags & PANA_FLAG_L)
-               proto_item_append_text(flags_item, ", L flag set");
+       proto_tree_add_boolean(flags_tree, hf_pana_flag_c, tvb, offset, 2, flags);
+       if (flags & PANA_FLAG_C)
+               proto_item_append_text(flags_item, ", C flag set");
+       proto_tree_add_boolean(flags_tree, hf_pana_flag_a, tvb, offset, 2, flags);
+       if (flags & PANA_FLAG_A)
+               proto_item_append_text(flags_item, ", A flag set");
+       proto_tree_add_boolean(flags_tree, hf_pana_flag_p, tvb, offset, 2, flags);
+       if (flags & PANA_FLAG_P)
+               proto_item_append_text(flags_item, ", P flag set");
+       proto_tree_add_boolean(flags_tree, hf_pana_flag_i, tvb, offset, 2, flags);
+       if (flags & PANA_FLAG_I)
+               proto_item_append_text(flags_item, ", I flag set");
 }
 
-
 /*
  * Function for AVP flags dissector.
  */
@@ -295,9 +260,6 @@ dissect_pana_avp_flags(proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint
        proto_tree_add_boolean(avp_flags_tree, hf_pana_avp_flag_v, tvb, offset, 2, flags);
        if (flags & PANA_AVP_FLAG_V)
                proto_item_append_text(avp_flags_item, ", Vendor");
-       proto_tree_add_boolean(avp_flags_tree, hf_pana_avp_flag_m, tvb, offset, 2, flags);
-       if (flags & PANA_AVP_FLAG_M)
-               proto_item_append_text(avp_flags_item, ", Mandatory");
 }
 
 
@@ -307,28 +269,17 @@ dissect_pana_avp_flags(proto_tree *parent_tree, tvbuff_t *tvb, int offset, guint
 static pana_avp_types
 pana_avp_get_type(guint16 avp_code, guint32 vendor_id)
 {
-
        if(vendor_id == 0) {
                switch(avp_code) {
-                       case 1: return PANA_UNSIGNED32;         /* Algorithm AVP */
-                       case 2: return PANA_OCTET_STRING;       /* AUTH AVP */
-                       case 3: return PANA_OCTET_STRING;       /* Cookie AVP */
-                       case 4: return PANA_UNSIGNED64;         /* Device-Id AVP, it should be PANA_IP_ADDRESS*/
-                       case 5: return PANA_EAP;                        /* EAP-Payload AVP */
-                       case 6: return PANA_GROUPED;            /* Failed-AVP AVP */
-                       case 7: return PANA_GROUPED;            /* ISP-Information AVP */
-                       case 8: return PANA_INTEGER32;          /* Key-Id AVP */
-                       case 9: return PANA_GROUPED;            /* NAP-Information AVP */
-                       case 10: return PANA_OCTET_STRING;      /* Nonce AVP */
-                       case 11: return PANA_OCTET_STRING;      /* Notification AVP */
-                       case 12: return PANA_UNSIGNED32;        /* Post-PANA-Address-Configuration (PPAC) AVP */
-                       case 13: return PANA_UNSIGNED32;        /* Protection-Capability AVP */
-                       case 14: return PANA_UNSIGNED32;        /* Provider-Identifier AVP */
-                       case 15: return PANA_UTF8STRING;        /* Provider-Name AVP */
-                       case 16: return PANA_RESULT_CODE;       /* Result-Code AVP */
-                       case 17: return PANA_UTF8STRING;        /* Session-Id AVP */
-                       case 18: return PANA_UNSIGNED32;        /* Session-Lifetime AVP */
-                       case 19: return PANA_ENUMERATED;        /* Termination-Cause AVP */
+                       case 1: return PANA_OCTET_STRING;       /* AUTH AVP */
+                       case 2: return PANA_EAP;                /* EAP-Payload AVP */
+                       case 3: return PANA_UNSIGNED32;         /* Integrity-Algorithm AVP */
+                       case 4: return PANA_INTEGER32;          /* Key-Id AVP */
+                       case 5: return PANA_OCTET_STRING;       /* Nonce AVP */
+                       case 6: return PANA_UNSIGNED32;         /* PRF-Algorithm AVP */
+                       case 7: return PANA_RESULT_CODE;        /* Result-Code AVP */
+                       case 8: return PANA_UNSIGNED32;         /* Session-Lifetime AVP */
+                       case 9: return PANA_ENUMERATED;         /* Termination-Cause AVP */
                        default: return PANA_OCTET_STRING;
                }
        } else {
@@ -494,29 +445,11 @@ dissect_avps(tvbuff_t *tvb, packet_info *pinfo, proto_tree *avp_tree)
                                                        offset, 4, FALSE);
                                        break;
                                }
-                               case PANA_IP_ADDRESS: {
-                                       proto_tree_add_item(single_avp_tree, hf_pana_avp_data_addrfamily, tvb,
-                                                       offset, 2, FALSE);
-                                       if (tvb_get_ntohs(tvb, offset) == 0x0001) {
-                                               proto_tree_add_item(single_avp_tree, hf_pana_avp_data_ipv4, tvb,
-                                                               offset+2, avp_data_length-2, FALSE);
-                                       } else if (tvb_get_ntohs(tvb, offset) == 0x0002) {
-                                               proto_tree_add_item(single_avp_tree, hf_pana_avp_data_ipv6, tvb,
-                                                               offset+2, avp_data_length-2, FALSE);
-                                       } else {
-                                               proto_tree_add_bytes_format(single_avp_tree, hf_pana_avp_data_bytes, tvb,
-                                                               offset, avp_data_length,
-                                                               tvb_get_ptr(tvb, offset, avp_data_length),
-                                                               "Error! Cannot Parse Address Family %d",
-                                                               tvb_get_ntohs(tvb, offset));
-                                       }
-                                       break;
-                               }
                                case PANA_RESULT_CODE: {
                                        proto_tree_add_text(single_avp_tree, tvb, offset, avp_data_length,
                                                                "Value: %d (%s)",
                                                                tvb_get_ntohl(tvb, offset),
-                                                               val_to_str(tvb_get_ntohs(tvb, offset), avp_code_names, "Unknown"));
+                                                               val_to_str(tvb_get_ntohs(tvb, offset), avp_code_names, "Unknown (%d)"));
                                        break;
                                }
                                case PANA_EAP: {
@@ -561,6 +494,7 @@ dissect_pana_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
        guint16 msg_type;
        gint16 msg_length;
        gint16 avp_length;
+       guint32 session_id;
        guint32 seq_num;
        conversation_t *conversation;
        pana_conv_info_t *pana_info;
@@ -572,8 +506,9 @@ dissect_pana_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
        msg_length = tvb_get_ntohs(tvb, 2);
        flags = tvb_get_ntohs(tvb, 4);
        msg_type = tvb_get_ntohs(tvb, 6);
-       seq_num = tvb_get_ntohl(tvb, 8);
-       avp_length = msg_length-12;
+       session_id = tvb_get_ntohl(tvb, 8);
+       seq_num = tvb_get_ntohl(tvb, 12);
+       avp_length = msg_length-16;
 
        /* Make entries in Protocol column and Info column on summary display */
        if (check_col(pinfo->cinfo, COL_PROTOCOL))
@@ -582,8 +517,8 @@ dissect_pana_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
        if (check_col(pinfo->cinfo, COL_INFO)) {
                col_clear(pinfo->cinfo, COL_INFO);
                col_add_fstr(pinfo->cinfo, COL_INFO, "Type %s-%s",
-               val_to_str(msg_type, msg_type_names, "Unknown (%d)"),
-               match_strval(flags & PANA_FLAG_R, msg_subtype_names));
+			    val_to_str(msg_type, msg_type_names, "Unknown (%d)"),
+			    val_to_str(flags & PANA_FLAG_R, msg_subtype_names, "Unknown (%d)"));
        }
 
        /* Make the protocol tree */
@@ -593,20 +528,20 @@ dissect_pana_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
        }
 
 
-       /* 
+       /*
         * We need to track some state for this protocol on a per conversation
         * basis so we can do neat things like request/response tracking
         */
        /*
         * Do we have a conversation for this connection?
         */
-       conversation = find_conversation(pinfo->fd->num, 
+       conversation = find_conversation(pinfo->fd->num,
                                    &pinfo->src, &pinfo->dst,
-                                   pinfo->ptype, 
+                                   pinfo->ptype,
                                    pinfo->srcport, pinfo->destport, 0);
        if (conversation == NULL) {
              /* We don't yet have a conversation, so create one. */
-             conversation = conversation_new(pinfo->fd->num, 
+             conversation = conversation_new(pinfo->fd->num,
                                    &pinfo->src, &pinfo->dst,
     	                    	   pinfo->ptype,
                                    pinfo->srcport, pinfo->destport, 0);
@@ -672,17 +607,13 @@ dissect_pana_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                        it=proto_tree_add_time(pana_tree, hf_pana_time, tvb, 0, 0, &ns);
                        PROTO_ITEM_SET_GENERATED(it);
                }
-       }              
+       }
 
 
 
-       /* Version */
-       proto_tree_add_item(pana_tree, hf_pana_version_type, tvb, offset, 1, FALSE);
-       offset += 1;
-
        /* Reserved field */
-       proto_tree_add_item(pana_tree, hf_pana_reserved_type, tvb, offset, 1, FALSE);
-       offset += 1;
+       proto_tree_add_item(pana_tree, hf_pana_reserved_type, tvb, offset, 2, FALSE);
+       offset += 2;
 
        /* Length */
        proto_tree_add_item(pana_tree, hf_pana_length_type, tvb, offset, 2, FALSE);
@@ -695,10 +626,16 @@ dissect_pana_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
        /* Message Type */
        proto_tree_add_uint_format_value(pana_tree, hf_pana_msg_type, tvb,
                            offset, 2, msg_type, "%s-%s (%d)",
-                           val_to_str(msg_type, msg_type_names, "Unknown (%d)"), 
-                          match_strval(flags & PANA_FLAG_R, msg_subtype_names), msg_type);
+                           val_to_str(msg_type, msg_type_names, "Unknown (%d)"),
+			   val_to_str(flags & PANA_FLAG_R, msg_subtype_names, "Unknown (%d)"),
+			   msg_type);
        offset += 2;
 
+       /* Session ID */
+       proto_tree_add_item(pana_tree, hf_pana_session_id, tvb, offset, 4, FALSE);
+       offset += 4;
+
+       /* Sequence Number */
        proto_tree_add_item(pana_tree, hf_pana_seqnumber, tvb, offset, 4, FALSE);
        offset += 4;
 
@@ -723,12 +660,13 @@ static gboolean
 dissect_pana(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 
-       guint8 pana_version;
-       guint8 pana_res;
+       guint16 pana_res;
        guint16 msg_length;
        guint16 flags;
        guint32 buffer_length;
        guint16 msg_type;
+       guint16 avp_length;
+       guint16 avp_offset;
 
        /* Get buffer length */
        buffer_length = tvb_length(tvb);
@@ -739,16 +677,12 @@ dissect_pana(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
        }
 
        /* Get header fields */
-       pana_version = tvb_get_guint8(tvb, 0);
-       pana_res = tvb_get_guint8(tvb, 1);
+       pana_res = tvb_get_ntohs(tvb, 0);
        msg_length = tvb_get_ntohs(tvb, 2);
        flags = tvb_get_ntohs(tvb, 4);
        msg_type = tvb_get_ntohs(tvb, 6);
-
-       /* Check version */
-       if(pana_version != PANA_VERSION) {
-               return FALSE;
-       }
+       avp_length = msg_length-16;
+       avp_offset = 16;
 
        /* Check minimum packet length */
        if(msg_length < 12) {
@@ -760,18 +694,31 @@ dissect_pana(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
                return FALSE;
        }
 
-       /* check that the reserved byte is zero */
+       /* For bug 1908: check the length of the first AVP, too */
+       if (avp_length) {
+	       guint16 first_avp_length;
+
+               if (avp_length < MIN_AVP_SIZE)
+		    return FALSE;
+
+               first_avp_length = tvb_get_ntohs(tvb, avp_offset + 4);
+
+               if (first_avp_length < MIN_AVP_SIZE || first_avp_length > avp_length)
+		    return FALSE;
+       }
+
+       /* check that the reserved field is zero */
        if(pana_res!=0){
                return FALSE;
        }
 
        /* verify that none of the reserved bits are set */
-       if(flags&0x0fff){
+       if(flags&0x03ff){
                return FALSE;
        }
 
        /* verify that we recognize the message type */
-       if(msg_type>10 || msg_type==0){
+       if(msg_type>4 || msg_type==0){
                return FALSE;
        }
 
@@ -791,11 +738,6 @@ proto_register_pana(void)
   module_t *pana_module;
 
        static hf_register_info hf[] = {
-               { &hf_pana_version_type,
-                       { "PANA Version", "pana.version",
-                       FT_UINT8, BASE_DEC, NULL, 0x0,
-                       "", HFILL }
-               },
                { &hf_pana_response_in,
                        { "Response In", "pana.response_in",
                        FT_FRAMENUM, BASE_DEC, NULL, 0x0,
@@ -813,141 +755,136 @@ proto_register_pana(void)
                },
                { &hf_pana_reserved_type,
                        { "PANA Reserved", "pana.reserved",
-                       FT_UINT8, BASE_HEX, NULL, 0x0,
-                       "", HFILL }
+                       FT_UINT16, BASE_HEX, NULL, 0x0,
+                       NULL, HFILL }
                },
                { &hf_pana_length_type,
                        { "PANA Message Length", "pana.length",
                        FT_UINT16, BASE_DEC, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
                },
 
 
                { &hf_pana_flags,
                        { "Flags", "pana.flags",
                        FT_UINT8, BASE_HEX, NULL, 0x0,
-                   "", HFILL }
+                       NULL, HFILL }
                },
                { &hf_pana_flag_r,
                        { "Request", "pana.flags.r",
-                       FT_BOOLEAN, 16, TFS(&flags_set_truth), PANA_FLAG_R,
-                       "", HFILL }
+                       FT_BOOLEAN, 16, TFS(&tfs_set_notset), PANA_FLAG_R,
+                       NULL, HFILL }
                },
                { &hf_pana_flag_s,
-                       { "Separate", "pana.flags.s",
-                       FT_BOOLEAN, 16, TFS(&flags_set_truth), PANA_FLAG_S,
-                       "", HFILL }
+                       { "Start", "pana.flags.s",
+                       FT_BOOLEAN, 16, TFS(&tfs_set_notset), PANA_FLAG_S,
+                       NULL, HFILL }
                },
-               { &hf_pana_flag_n,
-                       { "NAP Auth","pana.flags.n",
-                       FT_BOOLEAN, 16, TFS(&flags_set_truth), PANA_FLAG_N,
-                       "", HFILL }
+               { &hf_pana_flag_c,
+                       { "Complete","pana.flags.c",
+                       FT_BOOLEAN, 16, TFS(&tfs_set_notset), PANA_FLAG_C,
+                       NULL, HFILL }
                },
-               { &hf_pana_flag_l,
-                       { "Stateless Discovery","pana.flags.l",
-                       FT_BOOLEAN, 16, TFS(&flags_set_truth), PANA_FLAG_L,
-                       "", HFILL }
+               { &hf_pana_flag_a,
+                       { "Auth","pana.flags.a",
+                       FT_BOOLEAN, 16, TFS(&tfs_set_notset), PANA_FLAG_A,
+                       NULL, HFILL }
+               },
+               { &hf_pana_flag_p,
+                       { "Ping","pana.flags.p",
+                       FT_BOOLEAN, 16, TFS(&tfs_set_notset), PANA_FLAG_P,
+                       NULL, HFILL }
+               },
+               { &hf_pana_flag_i,
+                       { "IP Reconfig","pana.flags.i",
+                       FT_BOOLEAN, 16, TFS(&tfs_set_notset), PANA_FLAG_I,
+                       NULL, HFILL }
                },
 
                { &hf_pana_msg_type,
                        { "PANA Message Type", "pana.type",
                        FT_UINT16, BASE_DEC, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
+               },
+               { &hf_pana_session_id,
+                       { "PANA Session ID", "pana.sid",
+                       FT_UINT32, BASE_HEX, NULL, 0x0,
+                       NULL, HFILL }
                },
                { &hf_pana_seqnumber,
                        { "PANA Sequence Number", "pana.seq",
                        FT_UINT32, BASE_HEX, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
                },
 
 
                { &hf_pana_avp_code,
                        { "AVP Code", "pana.avp.code",
                        FT_UINT16, BASE_DEC, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
                },
                { &hf_pana_avp_length,
                        { "AVP Length", "pana.avp.length",
                        FT_UINT16, BASE_DEC, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
                },
                { &hf_pana_avp_flags,
                        { "AVP Flags", "pana.avp.flags",
                        FT_UINT16, BASE_HEX, NULL, 0x0,
-                   "", HFILL }
+                       NULL, HFILL }
                },
                { &hf_pana_avp_flag_v,
                        { "Vendor", "pana.avp.flags.v",
-                       FT_BOOLEAN, 16, TFS(&flags_set_truth), PANA_AVP_FLAG_V,
-                       "", HFILL }
-               },
-               { &hf_pana_avp_flag_m,
-                       { "Mandatory", "pana.avp.flags.m",
-                       FT_BOOLEAN, 16, TFS(&flags_set_truth), PANA_AVP_FLAG_M,
-                       "", HFILL }
+                       FT_BOOLEAN, 16, TFS(&tfs_set_notset), PANA_AVP_FLAG_V,
+                       NULL, HFILL }
                },
                { &hf_pana_avp_reserved,
                        { "AVP Reserved", "pana.avp.reserved",
                        FT_UINT16, BASE_HEX, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
                },
                { &hf_pana_avp_vendorid,
                        { "AVP Vendor ID", "pana.avp.vendorid",
                        FT_UINT32, BASE_HEX, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
                },
 
 
                { &hf_pana_avp_data_uint64,
                        { "Value", "pana.avp.data.uint64",
                        FT_UINT64, BASE_HEX, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
                },
                { &hf_pana_avp_data_int64,
                        { "Value", "pana.avp.data.int64",
-                       FT_INT64, BASE_HEX, NULL, 0x0,
-                       "", HFILL }
+                       FT_INT64, BASE_DEC, NULL, 0x0,
+                       NULL, HFILL }
                },
                { &hf_pana_avp_data_uint32,
                        { "Value", "pana.avp.data.uint32",
                        FT_UINT32, BASE_HEX, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
                },
                { &hf_pana_avp_data_int32,
                        { "Value", "pana.avp.data.int32",
-                       FT_INT32, BASE_HEX, NULL, 0x0,
-                       "", HFILL }
+                       FT_INT32, BASE_DEC, NULL, 0x0,
+                       NULL, HFILL }
                },
                { &hf_pana_avp_data_bytes,
                        { "Value", "pana.avp.data.bytes",
                        FT_BYTES, BASE_NONE, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
                },
                { &hf_pana_avp_data_string,
                        { "Value", "pana.avp.data.string",
                        FT_STRING, BASE_NONE, NULL, 0x0,
-                       "", HFILL }
+                       NULL, HFILL }
                },
                { &hf_pana_avp_data_enumerated,
                        { "Value", "pana.avp.data.enum",
                        FT_INT32, BASE_DEC, NULL, 0x0,
-                       "", HFILL }
-               },
-               { &hf_pana_avp_data_addrfamily,
-                       { "Address Family", "pana.avp.data.addrfamily",
-                       FT_UINT16, BASE_DEC, NULL, 0x0,
-                       "", HFILL }
-               },
-               { &hf_pana_avp_data_ipv4,
-                       { "IPv4 Address", "pana.avp.data.ipv4",
-                       FT_IPv4, BASE_NONE, NULL, 0x0,
-                       "", HFILL }
-               },
-               { &hf_pana_avp_data_ipv6,
-                       { "IPv6 Address", "pana.avp.data.ipv6",
-                       FT_IPv6, BASE_NONE, NULL, 0x0,
-                       "", HFILL }
-               },
+                       NULL, HFILL }
+               }
 
        };
 
@@ -957,7 +894,7 @@ proto_register_pana(void)
                &ett_pana_flags,
                &ett_pana_avp,
                &ett_pana_avp_info,
-               &ett_pana_avp_flags,
+               &ett_pana_avp_flags
        };
 
        /* Register the protocol name and description */
