@@ -7,7 +7,7 @@ proper helper routines
  * Routines for dissection of ASN.1 Aligned PER
  * 2003  Ronnie Sahlberg
  *
- * $Id: packet-per.c 23333 2007-11-01 22:07:43Z etxrab $
+ * $Id: packet-per.c 24714 2008-03-22 05:45:59Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -45,6 +45,7 @@ proper helper routines
 #include <epan/prefs.h>
 #include <epan/emem.h>
 #include <epan/asn1.h>
+#include <epan/strutil.h>
 #include "packet-per.h"
 
 
@@ -122,21 +123,26 @@ static tvbuff_t *new_octet_aligned_subset(tvbuff_t *tvb, guint32 offset, guint32
   guint32 boffset = offset >> 3;
   unsigned int i, shift0, shift1;
   guint8 octet0, octet1, *buf;
+  guint32 actual_length;
+
+  actual_length = tvb_length_remaining(tvb,boffset);
+  if (length <= actual_length)
+	  actual_length = length;
 
   if (offset & 0x07) {  /* unaligned */
     shift1 = offset & 0x07;
     shift0 = 8 - shift1;
-    buf = ep_alloc(length);
+    buf = ep_alloc(actual_length);
     octet0 = tvb_get_guint8(tvb, boffset);
-    for (i=0; i<length; i++) {
+    for (i=0; i<actual_length; i++) {
       octet1 = octet0;
       octet0 = tvb_get_guint8(tvb, boffset + i + 1);
       buf[i] = (octet1 << shift1) | (octet0 >> shift0);
     }
-    sub_tvb = tvb_new_real_data(buf, length, length);
+    sub_tvb = tvb_new_real_data(buf, actual_length, length);
     tvb_set_child_real_data_tvbuff(tvb, sub_tvb);
   } else {  /* aligned */
-    sub_tvb = tvb_new_subset(tvb, boffset, length, length);
+    sub_tvb = tvb_new_subset(tvb, boffset, actual_length, length);
   }
   return sub_tvb;
 }
@@ -205,9 +211,6 @@ static tvbuff_t *new_octet_aligned_subset_bits(tvbuff_t *tvb, guint32 offset, gu
 
 /* 10 Encoding procedures -------------------------------------------------- */
 
-static guint32
-dissect_per_length_determinant(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index, guint32 *length);
-
 /* 10.2 Open type fields --------------------------------------------------- */
 static guint32 
 dissect_per_open_type_internal(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, void* type_cb, asn1_cb_variant variant)
@@ -304,7 +307,7 @@ dissect_per_open_type_pdu_new(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, p
 			field in the manner described above in Note 2.
 
  */
-static guint32
+guint32
 dissect_per_length_determinant(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx _U_, proto_tree *tree, int hf_index, guint32 *length)
 {
 	guint8 byte;
@@ -335,27 +338,27 @@ dissect_per_length_determinant(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx _
 		g_snprintf(str, 256, " ");
 		for(bit=0;bit<((int)(offset&0x07));bit++){
 			if(bit&&(!(bit%4))){
-				strcat(str, " ");
+				g_strlcat(str, " ", 256);
 			}
-			strcat(str,".");
+			g_strlcat(str,".", 256);
 		}
 		/* read the bits for the int */
 		num_bits = 8;
 		for(i=0;i<num_bits;i++){
 			if(bit&&(!(bit%4))){
-				strcat(str, " ");
+				g_strlcat(str, " ", 256);
 			}
 			if(bit&&(!(bit%8))){
-				strcat(str, " ");
+				g_strlcat(str, " ", 256);
 			}
 			bit++;
 			offset=dissect_per_boolean(tvb, offset, actx, tree, -1, &tmp);
 			val<<=1;
 			if(tmp){
 				val|=1;
-				strcat(str, "1");
+				g_strlcat(str, "1", 256);
 			} else {
-				strcat(str, "0");
+				g_strlcat(str, "0", 256);
 			}
 		}
 		if((val&0x80)==0){
@@ -868,7 +871,6 @@ dissect_per_object_identifier(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx _U
   guint length;
   const char *str;
   tvbuff_t *val_tvb = NULL;
-  proto_item *item = NULL;
   header_field_info *hfi;
 
 DEBUG_ENTRY("dissect_per_object_identifier");
@@ -879,10 +881,10 @@ DEBUG_ENTRY("dissect_per_object_identifier");
 	
   hfi = proto_registrar_get_nth(hf_index);
   if (hfi->type == FT_OID) {
-    item = proto_tree_add_item(tree, hf_index, val_tvb, 0, length, FALSE);
+    actx->created_item = proto_tree_add_item(tree, hf_index, val_tvb, 0, length, FALSE);
   } else if (IS_FT_STRING(hfi->type)) {
     str = oid_encoded2string(tvb_get_ptr(val_tvb, 0, length), length);
-    item = proto_tree_add_string(tree, hf_index, val_tvb, 0, length, str);
+    actx->created_item = proto_tree_add_string(tree, hf_index, val_tvb, 0, length, str);
   } else {
     DISSECTOR_ASSERT_NOT_REACHED();
   }
@@ -1015,8 +1017,59 @@ PER_NOT_DECODED_YET("too long integer");
 
 	return offset;
 }
+/* 64 bits experimental version, internal for now */
+static guint32
+dissect_per_integer64b(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, gint64 *value)
+{
+	guint32 i, length;
+	gint64 val;
+	proto_item *it=NULL;
+	header_field_info *hfi;
+
+	/* 12.2.6 b */
+	offset=dissect_per_length_determinant(tvb, offset, actx, tree, -1, &length);
+	/* gassert here? */
+	if(length>8){
+PER_NOT_DECODED_YET("too long integer");
+		length=4;
+	}
+
+	val=0;
+	for(i=0;i<length;i++){
+		if(i==0){
+			if(tvb_get_guint8(tvb, offset>>3)&0x80){
+				/* negative number */
+				val=G_GINT64_CONSTANT(0xffffffffffffffff);
+			} else {
+				/* positive number */
+				val=0;
+			}
+		}
+		val=(val<<8)|tvb_get_guint8(tvb,offset>>3);
+		offset+=8;
+	}
+
+	hfi = proto_registrar_get_nth(hf_index);
+	if (! hfi)
+		THROW(ReportedBoundsError);
+        if (IS_FT_INT(hfi->type)) {
+		it=proto_tree_add_int64(tree, hf_index, tvb, (offset>>3)-(length+1), length+1, val);
+        } else if (IS_FT_UINT(hfi->type)) {
+		it=proto_tree_add_uint64(tree, hf_index, tvb, (offset>>3)-(length+1), length+1, val);
+	} else {
+		proto_tree_add_text(tree, tvb, (offset>>3)-(length+1), length+1, "Field is not an integer: %s", hfi->abbrev);
+		REPORT_DISSECTOR_BUG("PER integer field that's not an FT_INT* or FT_UINT*");
+	}
 
 
+	actx->created_item = it;
+
+	if(value){
+		*value=val;
+	}
+
+	return offset;
+}
 /* this function reads a constrained integer  with or without a
    PER visible extension marker present
 
@@ -1131,34 +1184,34 @@ DEBUG_ENTRY("dissect_per_constrained_integer");
 		g_snprintf(str, 256, "%s: ", hfi->name);
 		for(bit=0;bit<((int)(offset&0x07));bit++){
 			if(bit&&(!(bit%4))){
-				strcat(str, " ");
+				g_strlcat(str, " ", 256);
 			}
-			strcat(str,".");
+			g_strlcat(str,".", 256);
 		}
 		/* read the bits for the int */
 		for(i=0;i<num_bits;i++){
 			if(bit&&(!(bit%4))){
-				strcat(str, " ");
+				g_strlcat(str, " ", 256);
 			}
 			if(bit&&(!(bit%8))){
 				length+=1;
-				strcat(str, " ");
+				g_strlcat(str, " ", 256);
 			}
 			bit++;
 			offset=dissect_per_boolean(tvb, offset, actx, tree, -1, &tmp);
 			val<<=1;
 			if(tmp){
 				val|=1;
-				strcat(str, "1");
+				g_strlcat(str, "1", 256);
 			} else {
-				strcat(str, "0");
+				g_strlcat(str, "0", 256);
 			}
 		}
 		for(;bit%8;bit++){
 			if(bit&&(!(bit%4))){
-				strcat(str, " ");
+				g_strlcat(str, " ", 256);
 			}
-			strcat(str,".");
+			g_strlcat(str,".", 256);
 		}
 		val_start = (offset-num_bits)>>3; val_length = length;
 		val+=min;
@@ -1231,6 +1284,202 @@ DEBUG_ENTRY("dissect_per_constrained_integer");
 	if (value) *value = val;
 	return offset;}
 
+guint32
+dissect_per_constrained_integer_64b(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, guint64 min, guint64 max, guint64 *value, gboolean has_extension)
+{
+	proto_item *it=NULL;
+	guint64 range, val;
+	gint val_start, val_length;
+	nstime_t timeval;
+	header_field_info *hfi;
+	int num_bits;
+	int pad;
+	gboolean tmp;
+
+DEBUG_ENTRY("dissect_per_constrained_integer_64b");
+	if(has_extension){
+		gboolean extension_present;
+		offset=dissect_per_boolean(tvb, offset, actx, tree, hf_per_extension_present_bit, &extension_present);
+		if (!display_internal_per_fields) PROTO_ITEM_SET_HIDDEN(actx->created_item);
+		if(extension_present){
+			offset = dissect_per_integer64b(tvb, offset, actx, tree, hf_index, (gint64*)value);
+			return offset;
+		}
+	}
+
+	hfi = proto_registrar_get_nth(hf_index);
+
+	/* 10.5.3 Let "range" be defined as the integer value ("ub" - "lb"   1), and let the value to be encoded be "n". 
+	 * 10.5.7	In the case of the ALIGNED variant the encoding depends on whether
+	 *			d)	"range" is greater than 64K (the indefinite length case).
+	 */
+	if(((max-min)>65536)&&(actx->aligned)){
+		/* just set range really big so it will fall through
+		   to the bottom of the encoding */
+		range=1000000;
+	} else {
+		/* Copied from the 32 bit version, asuming the same problem occures
+		 * at 64 bit boundary.
+		 * Really ugly hack.
+		 * We should really use guint64 as parameters for min/max.
+		 * This is to prevent range from being 0 if
+		 * the range for a signed integer spans the entire 32 bit range.
+		 * Special case the 2 common cases when this can happen until
+		 * a real fix is implemented.
+		 */
+		if( (max==G_GINT64_CONSTANT(0x7fffffffffffffff) && min==G_GINT64_CONSTANT(0x8000000000000000))
+		||  (max==G_GINT64_CONSTANT(0xffffffffffffffff) && min==0) ){
+			range=G_GINT64_CONSTANT(0xffffffffffffffff);
+		} else {
+			range=max-min+1;
+		}
+	}
+
+	num_bits=0;
+	pad=0;
+	val=0;
+	timeval.secs=0; timeval.nsecs=0;
+	/* 10.5.4 If "range" has the value 1, then the result of the encoding shall be an empty bit-field (no bits).*/
+
+	/* something is really wrong if range is 0 */
+	DISSECTOR_ASSERT(range!=0);
+
+	if(range==1){
+		val_start = offset>>3; val_length = 0;
+		val = min; 
+	} else if((range<=255)||(!actx->aligned)) {
+		/* 10.5.7.1 
+		 * 10.5.6	In the case of the UNALIGNED variant the value ("n" - "lb") shall be encoded
+		 * as a non-negative  binary integer in a bit field as specified in 10.3 with the minimum
+		 * number of bits necessary to represent the range.
+		 */
+		char *str;
+		int i, bit, length;
+		guint32 mask,mask2;
+		/* We only handle 32 bit integers */
+		mask  = 0x80000000;
+		mask2 = 0x7fffffff;
+		i = 32;
+		while ((range & mask)== 0){
+			i = i - 1;
+			mask = mask>>1;
+			mask2 = mask2>>1;
+		}
+		if ((range & mask2) == 0)
+			i = i-1;
+
+		num_bits = i;
+		length=1;
+		if(range<=2){
+			num_bits=1;
+		}
+
+		/* prepare the string */
+		str=ep_alloc(256);
+		g_snprintf(str, 256, "%s: ", hfi->name);
+		for(bit=0;bit<((int)(offset&0x07));bit++){
+			if(bit&&(!(bit%4))){
+				g_strlcat(str, " ", 256);
+			}
+			g_strlcat(str,".", 256);
+		}
+		/* read the bits for the int */
+		for(i=0;i<num_bits;i++){
+			if(bit&&(!(bit%4))){
+				g_strlcat(str, " ", 256);
+			}
+			if(bit&&(!(bit%8))){
+				length+=1;
+				g_strlcat(str, " ", 256);
+			}
+			bit++;
+			offset=dissect_per_boolean(tvb, offset, actx, tree, -1, &tmp);
+			val<<=1;
+			if(tmp){
+				val|=1;
+				g_strlcat(str, "1", 256);
+			} else {
+				g_strlcat(str, "0", 256);
+			}
+		}
+		for(;bit%8;bit++){
+			if(bit&&(!(bit%4))){
+				g_strlcat(str, " ", 256);
+			}
+			g_strlcat(str,".", 256);
+		}
+		val_start = (offset-num_bits)>>3; val_length = length;
+		val+=min;
+		if (display_internal_per_fields)
+			proto_tree_add_text(tree, tvb, val_start,val_length,"Range = (%" G_GINT64_MODIFIER "u) Bitfield length %u, %s",range, num_bits, str);
+	} else if(range==256){
+		/* 10.5.7.2 */
+		num_bits=8;
+		pad=7-(offset&0x07);
+
+		/* in the aligned case, align to byte boundary */
+		BYTE_ALIGN_OFFSET(offset);
+		val=tvb_get_guint8(tvb, offset>>3);
+		offset+=8;
+
+		val_start = (offset>>3)-1; val_length = 1;
+		val+=min;
+	} else if(range<=65536){
+		/* 10.5.7.3 */
+		num_bits=16;
+		pad=7-(offset&0x07);
+
+		/* in the aligned case, align to byte boundary */
+		BYTE_ALIGN_OFFSET(offset);
+		val=tvb_get_guint8(tvb, offset>>3);
+		val<<=8;
+		offset+=8;
+		val|=tvb_get_guint8(tvb, offset>>3);
+		offset+=8;
+
+		val_start = (offset>>3)-2; val_length = 2;
+		val+=min;
+	} else {
+		int i,num_bytes;
+		gboolean bit;
+
+		/* 10.5.7.4 */
+		/* 12.2.6 */
+		offset=dissect_per_boolean(tvb, offset, actx, tree, -1, &bit);
+		num_bytes=bit;
+		offset=dissect_per_boolean(tvb, offset, actx, tree, -1, &bit);
+		num_bytes=(num_bytes<<1)|bit;
+
+		num_bytes++;  /* lower bound for length determinant is 1 */
+		if (display_internal_per_fields)
+			proto_tree_add_uint(tree, hf_per_const_int_len, tvb, (offset>>3), 1, num_bytes);
+
+		/* byte aligned */
+		BYTE_ALIGN_OFFSET(offset);
+		val=0;
+		for(i=0;i<num_bytes;i++){
+			val=(val<<8)|tvb_get_guint8(tvb,offset>>3);
+			offset+=8;
+		}
+		val_start = (offset>>3)-(num_bytes+1); val_length = num_bytes+1;
+		val+=min;
+	}
+
+	
+	if (IS_FT_UINT(hfi->type)) {
+		it = proto_tree_add_uint64(tree, hf_index, tvb, val_start, val_length, val);
+	} else if (IS_FT_INT(hfi->type)) {
+		it = proto_tree_add_int64(tree, hf_index, tvb, val_start, val_length, val);
+	} else if (IS_FT_TIME(hfi->type)) {
+		timeval.secs = (guint32)val;
+		it = proto_tree_add_time(tree, hf_index, tvb, val_start, val_length, &timeval);
+	} else {
+		THROW(ReportedBoundsError);
+	}
+	actx->created_item = it;
+	if (value) *value = val;
+	return offset;}
+
 /* 13 Encoding the enumerated type */
 guint32
 dissect_per_enumerated(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, guint32 root_num, guint32 *value, gboolean has_extension, guint32 ext_num, guint32 *value_map)
@@ -1264,11 +1513,7 @@ dissect_per_enumerated(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tr
 			/* 13.3 ".. and the value shall be added to the field-list as a
 			 * normally small non-negative whole number whose value is the 
 			 * enumeration index of the additional enumeration and with "lb" set to 0.."
-			 *
-			 * Byte align after reading Extension bit 
-			 *
 			 */
-			BYTE_ALIGN_OFFSET(offset);
 			offset = dissect_per_normally_small_nonnegative_whole_number(tvb, offset, actx, tree, hf_per_enum_extension_index, &enum_index);
 		}
 		enum_index += root_num;
@@ -1662,6 +1907,8 @@ DEBUG_ENTRY("dissect_per_bit_string");
 			proto_item_append_text(actx->created_item, " [bit length %u]", max_len);
 		}
 		offset+=min_len;
+		if (value_tvb)
+			*value_tvb = out_tvb;
 		return offset;
 	}
 
@@ -1681,6 +1928,8 @@ DEBUG_ENTRY("dissect_per_bit_string");
 			proto_item_append_text(actx->created_item, " [bit length %u]",max_len);
 		}
 		offset+=min_len;
+		if (value_tvb)
+			*value_tvb = out_tvb;
 		return offset;
 	}
 
@@ -1884,6 +2133,44 @@ guint32 dissect_per_octet_string_containing_pdu_new(tvbuff_t *tvb, guint32 offse
 	return offset;
 }
 
+guint32 dissect_per_size_constrained_type(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, per_type_fn type_cb, const gchar *name, int min_len, int max_len, gboolean has_extension)
+{
+  asn1_stack_frame_push(actx, name);
+  asn1_param_push_integer(actx, min_len);
+  asn1_param_push_integer(actx, max_len);
+  asn1_param_push_boolean(actx, has_extension);
+
+  offset = type_cb(tvb, offset, actx, tree, hf_index);
+
+  asn1_stack_frame_pop(actx, name);
+
+  return offset;
+}
+
+gboolean get_size_constraint_from_stack(asn1_ctx_t *actx, const gchar *name, int *pmin_len, int *pmax_len, gboolean *phas_extension)
+{
+  asn1_par_t *par;
+
+  if (pmin_len) *pmin_len = NO_BOUND;
+  if (pmax_len) *pmax_len = NO_BOUND;
+  if (phas_extension) *phas_extension = FALSE;
+
+  if (!actx->stack) return FALSE;
+  if (strcmp(actx->stack->name, name)) return FALSE;
+
+  par = actx->stack->par;
+  if (!par || (par->ptype != ASN1_PAR_INTEGER)) return FALSE;
+  if (pmin_len) *pmin_len = par->value.v_integer;
+  par = par->next;
+  if (!par || (par->ptype != ASN1_PAR_INTEGER)) return FALSE;
+  if (pmax_len) *pmax_len = par->value.v_integer;
+  par = par->next;
+  if (!par || (par->ptype != ASN1_PAR_BOOLEAN)) return FALSE;
+  if (phas_extension) *phas_extension = par->value.v_boolean;
+
+  return TRUE;
+}
+
 
 /* 26 Encoding of a value of the external type */
 
@@ -1900,7 +2187,7 @@ guint32 dissect_per_octet_string_containing_pdu_new(tvbuff_t *tvb, guint32 offse
   } 
 }
 */
-/* NOTE  This sequence type differs from that in ITU-T Rec. X.680 | ISO/IEC 8824-1 for historical reasons. */
+/* NOTE: This sequence type differs from that in ITU-T Rec. X.680 | ISO/IEC 8824-1 for historical reasons. */
 
 static int
 dissect_per_T_direct_reference(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
