@@ -6,7 +6,7 @@
  * Copyright 2000, Philips Electronics N.V.
  * Written by Andreas Sikkema <h323@ramdyne.nl>
  *
- * $Id: packet-rtp.c 24045 2008-01-09 16:21:03Z martinm $
+ * $Id: packet-rtp.c 24583 2008-03-07 18:15:46Z martinm $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -75,6 +75,7 @@
 
 #include <epan/prefs.h>
 #include <epan/emem.h>
+#include <epan/strutil.h>
 
 #include "log.h"
 
@@ -340,7 +341,7 @@ const value_string rtp_payload_type_short_vals[] =
        { PT_H263,      "h263" },
        { 0,            NULL },
 };
-
+#if 0
 static const value_string srtp_encryption_alg_vals[] =
 {
 	{ SRTP_ENC_ALG_NULL,	"Null Encryption" },
@@ -355,7 +356,7 @@ static const value_string srtp_auth_alg_vals[] =
 	{ SRTP_AUTH_ALG_HMAC_SHA1,	"HMAC-SHA1" },
 	{ 0, NULL },
 };
-
+#endif
 
 /* initialisation routine */
 static void rtp_fragment_init(void)
@@ -446,8 +447,7 @@ void srtp_add_address(packet_info *pinfo,
 	/* Free the hash if already exists */
 	rtp_free_hash_dyn_payload(p_conv_data->rtp_dyn_payload);
 
-	strncpy(p_conv_data->method, setup_method, MAX_RTP_SETUP_METHOD_SIZE);
-	p_conv_data->method[MAX_RTP_SETUP_METHOD_SIZE] = '\0';
+	g_strlcpy(p_conv_data->method, setup_method, MAX_RTP_SETUP_METHOD_SIZE);
 	p_conv_data->frame_number = setup_frame_number;
 	p_conv_data->rtp_dyn_payload = rtp_dyn_payload;
 	p_conv_data->srtp_info = srtp_info;
@@ -478,23 +478,29 @@ dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	if (! global_rtp_heur)
 		return FALSE;
 
+	/* Was it sent between 2 even-numbered ports? */
+	if ((pinfo->srcport % 2) || (pinfo->destport % 2)) {
+		return FALSE;
+	}
+
 	/* Get the fields in the first octet */
 	octet1 = tvb_get_guint8( tvb, offset );
 	version = RTP_VERSION( octet1 );
 
 	if (version == 0) {
 		switch (global_rtp_version0_type) {
-		case RTP0_STUN:
-			call_dissector(stun_handle, tvb, pinfo, tree);
-			return TRUE;
+			case RTP0_STUN:
+				call_dissector(stun_handle, tvb, pinfo, tree);
+				return TRUE;
 
-		case RTP0_T38:
-			call_dissector(t38_handle, tvb, pinfo, tree);
-			return TRUE;
+			case RTP0_T38:
+				call_dissector(t38_handle, tvb, pinfo, tree);
+				return TRUE;
 
-		case RTP0_INVALID:
-		default:
-			return FALSE; /* Unknown or unsupported version */
+			case RTP0_INVALID:
+
+			default:
+				return FALSE; /* Unknown or unsupported version */
 		}
 	} else if (version != 2) {
 		/* Unknown or unsupported version */
@@ -504,12 +510,12 @@ dissect_rtp_heur( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 	/* Get the fields in the second octet */
 	octet2 = tvb_get_guint8( tvb, offset + 1 );
 	payload_type = RTP_PAYLOAD_TYPE( octet2 );
-	/*      if (payload_type == PT_PCMU ||
-	 *		     payload_type == PT_PCMA)
-	 *	     payload_type == PT_G729)
-	 *	 */
-	if (payload_type <= PT_H263) {
- 		dissect_rtp( tvb, pinfo, tree );
+
+	/* Check for a sensible payload type
+	   (recognised static and preferred dynamic ranges) */
+	if ((payload_type <= PT_H263) ||
+	    (payload_type >= 96 && payload_type <= 127)) {
+		dissect_rtp( tvb, pinfo, tree );
 		return TRUE;
 	}
 	else {
@@ -538,12 +544,15 @@ process_rtp_payload(tvbuff_t *newtvb, packet_info *pinfo, proto_tree *tree,
 	if (p_conv_data && p_conv_data->srtp_info) {
 		srtp_info = p_conv_data->srtp_info;
 		payload_len -= srtp_info->mki_len + srtp_info->auth_tag_len;
-
+#if 0
+#error Currently the srtp_info structure contains no cypher data, see packet-sdp.c adding dummy_srtp_info structure
 		if (p_conv_data->srtp_info->encryption_algorithm==SRTP_ENC_ALG_NULL) {
 			if (rtp_tree)
 				proto_tree_add_text(rtp_tree, newtvb, offset, payload_len, "SRTP Payload with NULL encryption");
 		}
-		else {
+		else
+#endif
+		{
 			if (rtp_tree)
 				proto_tree_add_item(rtp_tree, hf_srtp_encrypted_payload, newtvb, offset, payload_len, FALSE);
 			found_match = TRUE;	/* use this flag to prevent dissection below */
