@@ -2,7 +2,7 @@
  * Ronnie Sahlberg 2006
  * copied from packet-scsi.c
  *
- * $Id: packet-scsi-mmc.c 20100 2006-12-10 13:52:30Z sahlberg $
+ * $Id: packet-scsi-mmc.c 21414 2007-04-13 18:08:27Z standel $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -246,27 +246,31 @@ static const value_string scsi_feature_val[] = {
 };
 
 static void
-dissect_mmc4_getconfiguration (tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree,
-                     guint offset, gboolean isreq, gboolean iscdb,
-                     guint payload_len _U_, scsi_task_data_t *cdata _U_)
+dissect_mmc4_getconfiguration (tvbuff_t *tvb, packet_info *pinfo _U_,
+			       proto_tree *tree, guint offset,
+			       gboolean isreq, gboolean iscdb,
+			       guint payload_len _U_,
+			       scsi_task_data_t *cdata _U_)
 
 {
     guint8 flags;
     gint32 len;
     guint old_offset;
+    tvbuff_t *volatile tvb_v = tvb;
+    volatile guint offset_v = offset;
 
     if (tree && isreq && iscdb) {
-        proto_tree_add_item (tree, hf_scsi_mmc_getconf_rt, tvb, offset+0, 1, 0);
-        proto_tree_add_item (tree, hf_scsi_mmc_getconf_starting_feature, tvb, offset+1, 2, 0);
+        proto_tree_add_item (tree, hf_scsi_mmc_getconf_rt, tvb_v, offset_v+0, 1, 0);
+        proto_tree_add_item (tree, hf_scsi_mmc_getconf_starting_feature, tvb_v, offset_v+1, 2, 0);
 
-        proto_tree_add_item (tree, hf_scsi_alloclen16, tvb, offset+6, 2, 0);
+        proto_tree_add_item (tree, hf_scsi_alloclen16, tvb_v, offset_v+6, 2, 0);
 	/* we need the alloc_len in the response */
 	if(cdata){
-		cdata->itlq->alloc_len=tvb_get_ntohs(tvb, offset+6);
+		cdata->itlq->alloc_len=tvb_get_ntohs(tvb_v, offset_v+6);
 	}
 
-        flags = tvb_get_guint8 (tvb, offset+8);
-        proto_tree_add_uint_format (tree, hf_scsi_control, tvb, offset+8, 1,
+        flags = tvb_get_guint8 (tvb_v, offset_v+8);
+        proto_tree_add_uint_format (tree, hf_scsi_control, tvb_v, offset_v+8, 1,
                                     flags,
                                     "Vendor Unique = %u, NACA = %u, Link = %u",
                                     flags & 0xC0, flags & 0x4, flags & 0x1);
@@ -276,53 +280,53 @@ dissect_mmc4_getconfiguration (tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree
 		return;
 	}
 
-	TRY_SCSI_CDB_ALLOC_LEN(pinfo, tvb, offset, cdata->itlq->alloc_len);
+	TRY_SCSI_CDB_ALLOC_LEN(pinfo, tvb_v, offset_v, cdata->itlq->alloc_len);
 
-        len=tvb_get_ntohl(tvb, offset+0);
-        proto_tree_add_item (tree, hf_scsi_mmc_data_length, tvb, offset, 4, 0);
-        proto_tree_add_item (tree, hf_scsi_mmc_getconf_current_profile, tvb, offset+6, 2, 0);
-	offset+=8;
+        len=tvb_get_ntohl(tvb_v, offset_v+0);
+        proto_tree_add_item (tree, hf_scsi_mmc_data_length, tvb_v, offset_v, 4, 0);
+        proto_tree_add_item (tree, hf_scsi_mmc_getconf_current_profile, tvb_v, offset_v+6, 2, 0);
+	offset_v+=8;
         len-=4;
         while(len>0){
                 guint16 feature;
                 guint8 additional_length;
 		guint8 num_linksize;
 
-                feature=tvb_get_ntohs(tvb, offset);
-	        proto_tree_add_item (tree, hf_scsi_mmc_feature, tvb, offset, 2, 0);
-                offset+=2;
-	        proto_tree_add_item (tree, hf_scsi_mmc_feature_version, tvb, offset, 1, 0);
-	        proto_tree_add_item (tree, hf_scsi_mmc_feature_persistent, tvb, offset, 1, 0);
-	        proto_tree_add_item (tree, hf_scsi_mmc_feature_current, tvb, offset, 1, 0);
-                offset+=1;
-                additional_length=tvb_get_guint8(tvb, offset);
-	        proto_tree_add_item (tree, hf_scsi_mmc_feature_additional_length, tvb, offset, 1, 0);
-                offset+=1;
-                old_offset=offset;
+                feature=tvb_get_ntohs(tvb_v, offset_v);
+	        proto_tree_add_item (tree, hf_scsi_mmc_feature, tvb_v, offset_v, 2, 0);
+                offset_v+=2;
+	        proto_tree_add_item (tree, hf_scsi_mmc_feature_version, tvb_v, offset_v, 1, 0);
+	        proto_tree_add_item (tree, hf_scsi_mmc_feature_persistent, tvb_v, offset_v, 1, 0);
+	        proto_tree_add_item (tree, hf_scsi_mmc_feature_current, tvb_v, offset_v, 1, 0);
+                offset_v+=1;
+                additional_length=tvb_get_guint8(tvb_v, offset_v);
+	        proto_tree_add_item (tree, hf_scsi_mmc_feature_additional_length, tvb_v, offset_v, 1, 0);
+                offset_v+=1;
+                old_offset=offset_v;
                 switch(feature){
                 case 0x0000: /* profile list */
-                    while(offset<(old_offset+additional_length)){
+                    while(offset_v<(old_offset+additional_length)){
 			proto_item *it=NULL;
 			proto_tree *tr=NULL;
 			guint16 profile;
 			guint8  cur_profile;
 
 			if(tree){
-				it=proto_tree_add_text(tree, tvb, offset, 4, "Profile:");
+				it=proto_tree_add_text(tree, tvb_v, offset_v, 4, "Profile:");
 				tr=proto_item_add_subtree(it, ett_scsi_mmc_profile);
 			}
 
-			profile=tvb_get_ntohs(tvb, offset);
-                        proto_tree_add_item (tr, hf_scsi_mmc_feature_profile, tvb, offset, 2, 0);
+			profile=tvb_get_ntohs(tvb_v, offset_v);
+                        proto_tree_add_item (tr, hf_scsi_mmc_feature_profile, tvb_v, offset_v, 2, 0);
 			proto_item_append_text(it, "%s", val_to_str(profile, scsi_getconf_current_profile_val, "Unknown 0x%04x"));
 
-			cur_profile=tvb_get_guint8(tvb, offset+2);
-                        proto_tree_add_item (tr, hf_scsi_mmc_feature_profile_current, tvb, offset+2, 1, 0);
+			cur_profile=tvb_get_guint8(tvb_v, offset_v+2);
+                        proto_tree_add_item (tr, hf_scsi_mmc_feature_profile_current, tvb_v, offset_v+2, 1, 0);
 			if(cur_profile&0x01){
 				proto_item_append_text(it, "  [CURRENT PROFILE]");
 			}
 
-                        offset+=4;
+                        offset_v+=4;
                     }
                     break;
                 case 0x001d: /* multi-read */
@@ -330,60 +334,60 @@ dissect_mmc4_getconfiguration (tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree
                     /* no data for this one */
                     break;
                 case 0x001e: /* cd read */
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_cdread_dap, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_cdread_c2flag, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_cdread_cdtext, tvb, offset, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_cdread_dap, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_cdread_c2flag, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_cdread_cdtext, tvb_v, offset_v, 1, 0);
                     break;
                 case 0x0021: /* incremental streaming writeable */
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dts, tvb, offset, 2, 0);
-                    offset+=2;
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_isw_buf, tvb, offset, 1, 0);
-                    offset+=1;
-                    num_linksize=tvb_get_guint8(tvb, offset);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_isw_num_linksize, tvb, offset, 1, 0);
-                    offset+=1;
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dts, tvb_v, offset_v, 2, 0);
+                    offset_v+=2;
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_isw_buf, tvb_v, offset_v, 1, 0);
+                    offset_v+=1;
+                    num_linksize=tvb_get_guint8(tvb_v, offset_v);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_isw_num_linksize, tvb_v, offset_v, 1, 0);
+                    offset_v+=1;
                     while(num_linksize--){
-                        proto_tree_add_item (tree, hf_scsi_mmc_feature_isw_linksize, tvb, offset, 1, 0);
-                        offset+=1;
+                        proto_tree_add_item (tree, hf_scsi_mmc_feature_isw_linksize, tvb_v, offset_v, 1, 0);
+                        offset_v+=1;
                     }
                     break;
                 case 0x002a: /* dvd-rw */
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdrw_write, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdrw_quickstart, tvb, offset, 2, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdrw_closeonly, tvb, offset, 2, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdrw_write, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdrw_quickstart, tvb_v, offset_v, 2, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdrw_closeonly, tvb_v, offset_v, 2, 0);
                     break;
                 case 0x002b: /* dvd-r */
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdr_write, tvb, offset, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdr_write, tvb_v, offset_v, 1, 0);
                     break;
                 case 0x002d: /* track at once */
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_buf, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_rwraw, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_rwpack, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_testwrite, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_cdrw, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_rwsubcode, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dts, tvb, offset+2, 2, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_buf, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_rwraw, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_rwpack, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_testwrite, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_cdrw, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_tao_rwsubcode, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dts, tvb_v, offset_v+2, 2, 0);
                     break;
                 case 0x002e: /* session at once */
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_buf, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_sao, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_rawms, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_raw, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_testwrite, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_cdrw, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_rw, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_mcsl, tvb, offset+1, 3, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_buf, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_sao, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_rawms, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_raw, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_testwrite, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_cdrw, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_rw, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_sao_mcsl, tvb_v, offset_v+1, 3, 0);
                     break;
                 case 0x002f: /* dvd-r/-rw*/
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdr_buf, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdr_testwrite, tvb, offset, 1, 0);
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdr_dvdrw, tvb, offset, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdr_buf, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdr_testwrite, tvb_v, offset_v, 1, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_dvdr_dvdrw, tvb_v, offset_v, 1, 0);
                     break;
                 case 0x0108: /* logical unit serial number */
-                    proto_tree_add_item (tree, hf_scsi_mmc_feature_lun_sn, tvb, offset, additional_length, 0);
+                    proto_tree_add_item (tree, hf_scsi_mmc_feature_lun_sn, tvb_v, offset_v, additional_length, 0);
                     break;
                 default:
-		    proto_tree_add_text (tree, tvb, offset, additional_length,
+		    proto_tree_add_text (tree, tvb_v, offset_v, additional_length,
 			"SCSI/MMC Unknown Feature:0x%04x",feature);
 		    break;
                 }
@@ -1028,10 +1032,10 @@ const value_string scsi_mmc_vals[] = {
     {SCSI_MMC4_GETCONFIGURATION      , "Get Configuraion"},
     {SCSI_MMC4_GETEVENTSTATUSNOTIFY  , "Get Event Status Notification"},
     {SCSI_MMC4_GETPERFORMANCE        , "Get Performance"},
-    {SCSI_SPC2_INQUIRY               , "Inquiry"},
-    {SCSI_SPC2_MODESELECT10          , "Mode Select(10)"},
-    {SCSI_SPC2_MODESENSE10           , "Mode Sense(10)"},
-    {SCSI_SPC2_PREVMEDREMOVAL        , "Prevent/Allow Medium Removal"},
+    {SCSI_SPC_INQUIRY               , "Inquiry"},
+    {SCSI_SPC_MODESELECT10          , "Mode Select(10)"},
+    {SCSI_SPC_MODESENSE10           , "Mode Sense(10)"},
+    {SCSI_SPC_PREVMEDREMOVAL        , "Prevent/Allow Medium Removal"},
     {SCSI_MMC4_READ10                , "Read(10)"},
     {SCSI_MMC4_READ12                , "Read(12)"},
     {SCSI_MMC4_READBUFFERCAPACITY    , "Read Buffer Capacity"},
@@ -1041,26 +1045,26 @@ const value_string scsi_mmc_vals[] = {
     {SCSI_MMC4_READTOCPMAATIP        , "Read TOC/PMA/ATIP"},
     {SCSI_MMC4_READTRACKINFORMATION  , "Read Track Information"},
     {SCSI_MMC4_REPORTKEY             , "Report Key"},
-    {SCSI_SPC2_REPORTLUNS            , "Report LUNs"},
-    {SCSI_SPC2_REQSENSE              , "Request Sense"},
+    {SCSI_SPC_REPORTLUNS            , "Report LUNs"},
+    {SCSI_SPC_REQSENSE              , "Request Sense"},
     {SCSI_MMC4_RESERVETRACK          , "Reserve Track"},
     {SCSI_MMC4_SETCDSPEED            , "Set CD Speed"},
     {SCSI_MMC4_SETSTREAMING          , "Set Streaming"},
     {SCSI_SBC_STARTSTOPUNIT          , "Start Stop Unit"},
     {SCSI_MMC4_SYNCHRONIZECACHE      , "Synchronize Cache"},
-    {SCSI_SPC2_TESTUNITRDY           , "Test Unit Ready"},
+    {SCSI_SPC_TESTUNITRDY           , "Test Unit Ready"},
     {SCSI_MMC4_WRITE10               , "Write(10)"},
     {SCSI_MMC4_WRITE12               , "Write(12)"},
-    {SCSI_SPC2_WRITEBUFFER           , "Write Buffer"},
+    {SCSI_SPC_WRITEBUFFER           , "Write Buffer"},
     {0, NULL},
 };
 
 
 scsi_cdb_table_t scsi_mmc_table[256] = {
-/*SPC 0x00*/{dissect_spc3_testunitready},
+/*SPC 0x00*/{dissect_spc_testunitready},
 /*MMC 0x01*/{NULL},
 /*MMC 0x02*/{NULL},
-/*SPC 0x03*/{dissect_spc3_requestsense},
+/*SPC 0x03*/{dissect_spc_requestsense},
 /*MMC 0x04*/{NULL},
 /*MMC 0x05*/{NULL},
 /*MMC 0x06*/{NULL},
@@ -1075,7 +1079,7 @@ scsi_cdb_table_t scsi_mmc_table[256] = {
 /*MMC 0x0f*/{NULL},
 /*MMC 0x10*/{NULL},
 /*MMC 0x11*/{NULL},
-/*SPC 0x12*/{dissect_spc3_inquiry},
+/*SPC 0x12*/{dissect_spc_inquiry},
 /*MMC 0x13*/{NULL},
 /*MMC 0x14*/{NULL},
 /*MMC 0x15*/{NULL},
@@ -1087,7 +1091,7 @@ scsi_cdb_table_t scsi_mmc_table[256] = {
 /*MMC 0x1b*/{dissect_sbc_startstopunit},
 /*MMC 0x1c*/{NULL},
 /*MMC 0x1d*/{NULL},
-/*MMC 0x1e*/{dissect_spc3_preventallowmediaremoval},
+/*MMC 0x1e*/{dissect_spc_preventallowmediaremoval},
 /*MMC 0x1f*/{NULL},
 /*MMC 0x20*/{NULL},
 /*MMC 0x21*/{NULL},
@@ -1116,7 +1120,7 @@ scsi_cdb_table_t scsi_mmc_table[256] = {
 /*MMC 0x38*/{NULL},
 /*MMC 0x39*/{NULL},
 /*MMC 0x3a*/{NULL},
-/*SPC 0x3b*/{dissect_spc3_writebuffer},
+/*SPC 0x3b*/{dissect_spc_writebuffer},
 /*MMC 0x3c*/{NULL},
 /*MMC 0x3d*/{NULL},
 /*MMC 0x3e*/{NULL},
@@ -1142,12 +1146,12 @@ scsi_cdb_table_t scsi_mmc_table[256] = {
 /*MMC 0x52*/{dissect_mmc4_readtrackinformation},
 /*MMC 0x53*/{dissect_mmc4_reservetrack},
 /*MMC 0x54*/{NULL},
-/*SPC 0x55*/{dissect_spc3_modeselect10},
+/*SPC 0x55*/{dissect_spc_modeselect10},
 /*MMC 0x56*/{NULL},
 /*MMC 0x57*/{NULL},
 /*MMC 0x58*/{NULL},
 /*MMC 0x59*/{NULL},
-/*SPC 0x5a*/{dissect_spc3_modesense10},
+/*SPC 0x5a*/{dissect_spc_modesense10},
 /*MMC 0x5b*/{NULL},
 /*MMC 0x5c*/{dissect_mmc4_readbuffercapacity},
 /*MMC 0x5d*/{NULL},
@@ -1217,7 +1221,7 @@ scsi_cdb_table_t scsi_mmc_table[256] = {
 /*MMC 0x9d*/{NULL},
 /*MMC 0x9e*/{NULL},
 /*MMC 0x9f*/{NULL},
-/*SPC 0xa0*/{dissect_spc3_reportluns},
+/*SPC 0xa0*/{dissect_spc_reportluns},
 /*MMC 0xa1*/{NULL},
 /*MMC 0xa2*/{NULL},
 /*MMC 0xa3*/{NULL},
