@@ -4,7 +4,7 @@
  * Ronnie sahlberg 2006
  * Joe Breher 2006
  *
- * $Id: packet-scsi-osd.c 19817 2006-11-06 09:30:15Z sahlberg $
+ * $Id: packet-scsi-osd.c 24627 2008-03-14 07:45:30Z jake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -214,7 +214,7 @@ typedef void (*attribute_dissector)(tvbuff_t *tvb, packet_info *pinfo, proto_tre
 
 typedef struct _attribute_page_numbers_t {
     guint32	number;
-    char	*name;
+    const char	*name;
     attribute_dissector dissector;
 } attribute_page_numbers_t;
 static const attribute_page_numbers_t user_object_info_attributes[] = {
@@ -561,7 +561,7 @@ dissect_osd_attribute_data_out(packet_info *pinfo, tvbuff_t *tvb, int offset _U_
 
 
 static void
-dissect_osd_attribute_data_in(packet_info *pinfo, tvbuff_t *tvb, int offset, proto_tree *tree, scsi_task_data_t *cdata)
+dissect_osd_attribute_data_in(packet_info *pinfo, tvbuff_t *tvb, int offset _U_, proto_tree *tree, scsi_task_data_t *cdata)
 {
 	guint8 gsatype=0;
 	scsi_osd_extra_data_t *extra_data=NULL;
@@ -1912,9 +1912,8 @@ dissect_osd_append(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		dissect_osd_option(tvb, offset, tree);
 		offset++;
 
-		/* getset attributes byte / sort order */
+		/* getset attributes byte */
 		dissect_osd_getsetattrib(tvb, offset, tree, cdata);
-		dissect_osd_sortorder(tvb, offset, tree);
 		offset++;
 
 		/* timestamps control */
@@ -2635,7 +2634,7 @@ dissect_osd_opcode(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 
 
 		if(check_col(pinfo->cinfo, COL_INFO)){
-			col_append_fstr(pinfo->cinfo, COL_INFO, "%s",
+			col_append_str(pinfo->cinfo, COL_INFO, 
 				val_to_str(svcaction, scsi_osd_svcaction_vals, "Unknown OSD Serviceaction"));
 		}
 		dissector=find_svcaction_dissector(svcaction);
@@ -2653,7 +2652,7 @@ dissect_osd_opcode(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		svcaction=extra_data->svcaction;
 	}
 	if(check_col(pinfo->cinfo, COL_INFO)){
-		col_append_fstr(pinfo->cinfo, COL_INFO, "%s",
+		col_append_str(pinfo->cinfo, COL_INFO, 
 			val_to_str(svcaction, scsi_osd_svcaction_vals, "Unknown OSD Serviceaction"));
 	}
 	if(svcaction){
@@ -2671,14 +2670,14 @@ dissect_osd_opcode(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 
 /* OSD Commands */
 const value_string scsi_osd_vals[] = {
-    {SCSI_SPC2_INQUIRY			, "Inquiry"},
-    {SCSI_SPC2_LOGSELECT		, "Log Select"},
-    {SCSI_SPC2_LOGSENSE			, "Log Sense"},
-    {SCSI_SPC2_MODESELECT10		, "Mode Select(10)"},
-    {SCSI_SPC2_MODESENSE10		, "Mode Sense(10)"},
-    {SCSI_SPC2_PERSRESVIN		, "Persistent Reserve In"},
-    {SCSI_SPC2_PERSRESVOUT		, "Persistent Reserve Out"},
-    {SCSI_SPC2_REPORTLUNS		, "Report LUNs"},
+    {SCSI_SPC_INQUIRY			, "Inquiry"},
+    {SCSI_SPC_LOGSELECT			, "Log Select"},
+    {SCSI_SPC_LOGSENSE			, "Log Sense"},
+    {SCSI_SPC_MODESELECT10		, "Mode Select(10)"},
+    {SCSI_SPC_MODESENSE10		, "Mode Sense(10)"},
+    {SCSI_SPC_PERSRESVIN		, "Persistent Reserve In"},
+    {SCSI_SPC_PERSRESVOUT		, "Persistent Reserve Out"},
+    {SCSI_SPC_REPORTLUNS		, "Report LUNs"},
     {SCSI_OSD_OPCODE			, "OSD Command" },
     {0, NULL},
 };
@@ -2704,7 +2703,7 @@ scsi_cdb_table_t scsi_osd_table[256] = {
 /*OSD 0x0f*/{NULL},
 /*OSD 0x10*/{NULL},
 /*OSD 0x11*/{NULL},
-/*OSD 0x12*/{dissect_spc3_inquiry},
+/*OSD 0x12*/{dissect_spc_inquiry},
 /*OSD 0x13*/{NULL},
 /*OSD 0x14*/{NULL},
 /*OSD 0x15*/{NULL},
@@ -2762,8 +2761,8 @@ scsi_cdb_table_t scsi_osd_table[256] = {
 /*OSD 0x49*/{NULL},
 /*OSD 0x4a*/{NULL},
 /*OSD 0x4b*/{NULL},
-/*OSD 0x4c*/{dissect_spc3_logselect},
-/*OSD 0x4d*/{dissect_spc3_logsense},
+/*OSD 0x4c*/{dissect_spc_logselect},
+/*OSD 0x4d*/{dissect_spc_logsense},
 /*OSD 0x4e*/{NULL},
 /*OSD 0x4f*/{NULL},
 /*OSD 0x50*/{NULL},
@@ -2771,17 +2770,17 @@ scsi_cdb_table_t scsi_osd_table[256] = {
 /*OSD 0x52*/{NULL},
 /*OSD 0x53*/{NULL},
 /*OSD 0x54*/{NULL},
-/*OSD 0x55*/{dissect_spc3_modeselect10},
+/*OSD 0x55*/{dissect_spc_modeselect10},
 /*OSD 0x56*/{NULL},
 /*OSD 0x57*/{NULL},
 /*OSD 0x58*/{NULL},
 /*OSD 0x59*/{NULL},
-/*OSD 0x5a*/{dissect_spc3_modesense10},
+/*OSD 0x5a*/{dissect_spc_modesense10},
 /*OSD 0x5b*/{NULL},
 /*OSD 0x5c*/{NULL},
 /*OSD 0x5d*/{NULL},
-/*OSD 0x5e*/{dissect_spc3_persistentreservein},
-/*OSD 0x5f*/{dissect_spc3_persistentreserveout},
+/*OSD 0x5e*/{dissect_spc_persistentreservein},
+/*OSD 0x5f*/{dissect_spc_persistentreserveout},
 /*OSD 0x60*/{NULL},
 /*OSD 0x61*/{NULL},
 /*OSD 0x62*/{NULL},
@@ -2846,7 +2845,7 @@ scsi_cdb_table_t scsi_osd_table[256] = {
 /*OSD 0x9d*/{NULL},
 /*OSD 0x9e*/{NULL},
 /*OSD 0x9f*/{NULL},
-/*OSD 0xa0*/{dissect_spc3_reportluns},
+/*OSD 0xa0*/{dissect_spc_reportluns},
 /*OSD 0xa1*/{NULL},
 /*OSD 0xa2*/{NULL},
 /*OSD 0xa3*/{NULL},
