@@ -1,7 +1,7 @@
 /* packet-scsi-osd.h
  * Ronnie sahlberg 2006
  *
- * $Id: packet-scsi-osd.h 19360 2006-09-29 10:40:51Z sahlberg $
+ * $Id: packet-scsi-osd.h 19977 2006-11-24 18:47:50Z wmeier $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -29,6 +29,6 @@
 
 extern int hf_scsi_osd_opcode;
 extern scsi_cdb_table_t scsi_osd_table[256];
-extern const value_string scsi_osd_vals[];
+WS_VAR_IMPORT const value_string scsi_osd_vals[];
 
 #endif
