@@ -2,7 +2,7 @@
  * Routines for sFlow dissection
  * Copyright 2003, Jeff Rizzo <riz@boogers.sf.ca.us>
  *
- * $Id: packet-sflow.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-sflow.c 24565 2008-03-05 20:33:19Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -32,7 +32,6 @@
  *   make routines more consistent as to whether they return
  *     'offset' or bytes consumed ('len')
  *   implement sampled_ipv4 and sampled_ipv6 packet data types
- *   implement extended_gateway
  *   implement extended_user
  *   implement extended_url
  *   implement non-generic counters sampling
@@ -55,9 +54,26 @@
 #include <glib.h>
 
 #include <epan/packet.h>
+#include <epan/prefs.h>
+
 /*#include "packet-sflow.h"*/
 
-#define UDP_PORT_SFLOW 6343
+#define SFLOW_UDP_PORTS "6343"
+
+static dissector_handle_t sflow_handle;
+
+/*
+ *	global_sflow_ports : holds the configured range of ports for sflow
+ */
+static range_t *global_sflow_ports = NULL;
+
+/*
+ *	sflow_ports : holds the currently used range of ports for sflow
+ */
+static range_t *sflow_ports = NULL;
+static gboolean global_dissect_samp_headers = TRUE;
+static gboolean global_analyze_samp_ip_headers = FALSE;
+
 
 #define ADDRESS_IPV4 1
 #define ADDRESS_IPV6 2
@@ -151,6 +167,16 @@ static const value_string sflow_extended_data_types[] = {
 };
 
 
+#define SFLOW_AS_SET 1
+#define SFLOW_AS_SEQUENCE 2
+
+static const value_string sflow_as_types[] = {
+	{ SFLOW_AS_SET, "AS Set" },
+	{ SFLOW_AS_SEQUENCE, "AS Sequence" },
+	{ 0, NULL }
+};
+
+
 /* flow sample header */
 struct sflow_flow_sample_header {
 	guint32 	sequence_number;
@@ -285,6 +311,18 @@ static int hf_sflow_nexthop_v4 = -1;   /* nexthop address */
 static int hf_sflow_nexthop_v6 = -1;   /* nexthop address */
 static int hf_sflow_nexthop_src_mask = -1;
 static int hf_sflow_nexthop_dst_mask = -1;
+
+/* extended gateway (all versions) */
+static int hf_sflow_as = -1;
+static int hf_sflow_src_as = -1;
+static int hf_sflow_src_peer_as = -1;
+static int hf_sflow_dst_as_entries = -1; /* aka length */
+static int hf_sflow_dst_as = -1;
+/* extended gateway (>= version 4) */
+static int hf_sflow_community_entries = -1;
+static int hf_sflow_community = -1;
+static int hf_sflow_localpref = -1;
+
 static int hf_sflow_ifindex = -1;
 static int hf_sflow_iftype = -1;
 static int hf_sflow_ifspeed = -1;
@@ -309,6 +347,9 @@ static int hf_sflow_ifpromisc = -1;
 static gint ett_sflow = -1;
 static gint ett_sflow_sample = -1;
 static gint ett_sflow_extended_data = -1;
+static gint ett_sflow_gw_as_dst = -1;
+static gint ett_sflow_gw_as_dst_seg = -1;
+static gint ett_sflow_gw_community = -1;
 static gint ett_sflow_sampled_header = -1;
 
 /* dissectors for other protocols */
@@ -323,11 +364,15 @@ static dissector_handle_t aal5_handle;
 static dissector_handle_t ipv4_handle;
 static dissector_handle_t ipv6_handle;
 static dissector_handle_t mpls_handle;
+/* don't dissect */
+static dissector_handle_t data_handle;
+
+void proto_reg_handoff_sflow(void);
 
 /* dissect a sampled header - layer 2 protocols */
 static gint
 dissect_sflow_sampled_header(tvbuff_t *tvb, packet_info *pinfo,
-							 proto_tree *tree, volatile gint offset)
+			     proto_tree *tree, volatile gint offset)
 {
 	guint32 	header_proto, frame_length;
 	volatile 	guint32 	header_length;
@@ -336,7 +381,8 @@ dissect_sflow_sampled_header(tvbuff_t *tvb, packet_info *pinfo,
 	proto_item 	*ti;
 	/* stuff for saving column state before calling other dissectors.
 	 * Thanks to Guy Harris for the tip. */
-	gboolean 			save_writable;
+	gboolean save_writable;
+	gboolean save_in_error_pkt;
 	volatile address 	save_dl_src;
 	volatile address 	save_dl_dst;
 	volatile address 	save_net_src;
@@ -369,6 +415,31 @@ dissect_sflow_sampled_header(tvbuff_t *tvb, packet_info *pinfo,
 
 	/* save some state */
 	save_writable = col_get_writable(pinfo->cinfo);
+
+	/* 
+	   If sFlow samples a TCP packet it is very likely that the
+	   TCP analysis will flag the packet as having some error with
+	   the sequence numbers.  sFlow only report on a "sample" of
+	   traffic so many packets will not be reported on.  This is
+	   most obvious if the colorizing rules are on, but will also
+	   cause confusion if you attempt to filter on
+	   "tcp.analysis.flags".
+
+	   The following only works to suppress IP/TCP errors, but
+	   it is a start anyway.  Other protocols carried as payloads
+	   may exhibit similar issues.
+
+	   I think what is really needed is a more general
+	   "protocol_as_payload" flag.  Of course then someone has to
+	   play whack-a-mole and add code to implement it to any
+	   protocols that could be carried as a payload.  In the case
+	   of sFlow that pretty much means anything on your network.
+	*/
+	save_in_error_pkt = pinfo->in_error_pkt;
+	if (!global_analyze_samp_ip_headers) {
+		pinfo->in_error_pkt = TRUE;
+	}
+
 	col_set_writable(pinfo->cinfo, FALSE);
 	save_dl_src = pinfo->dl_src;
 	save_dl_dst = pinfo->dl_dst;
@@ -427,6 +498,8 @@ dissect_sflow_sampled_header(tvbuff_t *tvb, packet_info *pinfo,
 
 	/* restore saved state */
 	col_set_writable(pinfo->cinfo, save_writable);
+	pinfo->in_error_pkt = save_in_error_pkt;
+
 	pinfo->dl_src = save_dl_src;
 	pinfo->dl_dst = save_dl_dst;
 	pinfo->net_src = save_net_src;
@@ -446,10 +519,10 @@ dissect_sflow_extended_switch(tvbuff_t *tvb, proto_tree *tree, gint offset)
 	
 	proto_tree_add_item(tree, hf_sflow_vlan_in, tvb, offset + len, 4, FALSE);
 	len += 4;
-	proto_tree_add_item(tree, hf_sflow_vlan_out, tvb, offset + len, 4, FALSE);
-	len += 4;
 	proto_tree_add_item(tree, hf_sflow_pri_in, tvb, offset + len, 4, FALSE);
 	len += 4;
+	proto_tree_add_item(tree, hf_sflow_vlan_out, tvb, offset + len, 4, FALSE);
+	len += 4;
 	proto_tree_add_item(tree, hf_sflow_pri_out, tvb, offset + len, 4, FALSE);
 	len += 4;
 
@@ -481,7 +554,6 @@ dissect_sflow_extended_router(tvbuff_t *tvb, proto_tree *tree, gint offset)
 							"Unknown address type (%d)", address_type);
 		len += 4;  /* not perfect, but what else to do? */
 		return len;  /* again, this is wrong.  but... ? */
-		break;
 	};
 	
 	proto_tree_add_item(tree, hf_sflow_nexthop_src_mask, tvb, offset + len,
@@ -493,6 +565,107 @@ dissect_sflow_extended_router(tvbuff_t *tvb, proto_tree *tree, gint offset)
 	return len;
 }
 
+
+/* extended router data, after the packet data */
+static gint
+dissect_sflow_extended_gateway(tvbuff_t *tvb, proto_tree *tree, gint offset)
+{
+	gint32 	len = 0;
+	gint32 	i, j, comm_len, dst_len, dst_seg_len;
+	guint32 path_type;
+	gint32  kludge;
+
+	guint32 version = tvb_get_ntohl(tvb, 0); /* HACK */
+	proto_item *ti;
+	proto_tree *sflow_dst_as_tree;
+	proto_tree *sflow_comm_tree;
+	proto_tree *sflow_dst_as_seg_tree;
+
+	proto_tree_add_item(tree, hf_sflow_as, tvb, offset + len,
+							4, FALSE);
+	len += 4;
+
+	proto_tree_add_item(tree, hf_sflow_src_as, tvb, offset + len,
+							4, FALSE);
+	len += 4;
+
+	proto_tree_add_item(tree, hf_sflow_src_peer_as, tvb, offset + len,
+							4, FALSE);
+	len += 4;
+
+	dst_len = tvb_get_ntohl(tvb, offset+len);
+	ti = proto_tree_add_uint(tree, hf_sflow_dst_as_entries, tvb, offset + len, 4, dst_len);
+	sflow_dst_as_tree = proto_item_add_subtree(ti, ett_sflow_gw_as_dst);
+	len += 4;
+
+	for (i = 0; i < dst_len; i++) {
+		if( version < 4 ) {
+			/* Version 2 AS paths are different than versions >= 4 as
+			   follows:
+
+			   There is no type encoded in the packet.  
+
+			   The destination ASs are encoded as an array of integers
+			   rather as an array of arrays of integers.  I just
+			   pretended they were encoded as an array of arrays with
+			   an implicit length of 1 to not have to do two
+			   completely separate blocks for the different versions.
+
+			   Having a subtree for "arrays" guaranteed to have only a
+			   single element proved cumbersome to navigate so I moved
+			   the creation of the subtree to only happen for versions
+			   >= 4.
+			 */
+			dst_seg_len = 1;
+			path_type = 0;
+			kludge = 0;
+			sflow_dst_as_seg_tree = sflow_dst_as_tree;
+		} else {
+			path_type = tvb_get_ntohl(tvb, offset+len);
+			len += 4;
+			dst_seg_len = tvb_get_ntohl(tvb, offset+len);
+			len += 4;
+			kludge = 8;
+			ti = proto_tree_add_text(tree, tvb, offset+len-kludge, kludge,
+									 "%s, (%d entries)",
+									 val_to_str(path_type, 
+												sflow_as_types,
+												"Unknown AS type"),
+									 dst_seg_len);
+			sflow_dst_as_seg_tree = proto_item_add_subtree(ti, ett_sflow_gw_as_dst_seg);
+		}
+
+		for (j = 0; j < dst_seg_len; j++) {
+			proto_tree_add_item(sflow_dst_as_seg_tree, 
+								hf_sflow_dst_as, tvb, offset + len,
+								4, FALSE);
+			len += 4;
+		}
+	}
+
+		
+	if( version >= 4 ) {
+		comm_len = tvb_get_ntohl(tvb, offset+len);
+
+		ti = proto_tree_add_uint(tree, hf_sflow_community_entries, tvb, offset + len, 4, comm_len);
+		sflow_comm_tree = proto_item_add_subtree(ti, ett_sflow_gw_community);
+		len += 4;
+		for (i = 0; i < comm_len; i++) {
+			proto_tree_add_item(sflow_comm_tree, 
+								hf_sflow_dst_as, tvb, offset + len,
+								4, FALSE);
+			len += 4;
+		}
+		
+		proto_tree_add_item(tree, hf_sflow_localpref, tvb, offset + len,
+							4, FALSE);
+		len += 4;
+
+	}
+
+	return len;
+}
+
 /* dissect a flow sample */
 static gint
 dissect_sflow_flow_sample(tvbuff_t *tvb, packet_info *pinfo,
@@ -582,6 +755,8 @@ dissect_sflow_flow_sample(tvbuff_t *tvb, packet_info *pinfo,
 													offset);
 			break;
 		case SFLOW_EXTENDED_GATEWAY:
+			offset += dissect_sflow_extended_gateway(tvb, extended_data_tree,
+													offset);
 			break;
 		case SFLOW_EXTENDED_USER:
 			break;
@@ -754,7 +929,7 @@ dissect_sflow_samples(tvbuff_t *tvb, packet_info *pinfo,
 }
 
 /* Code to actually dissect the packets */
-static void
+static int
 dissect_sflow(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
 
@@ -814,7 +989,7 @@ dissect_sflow(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		break;
 	default:
 		/* unknown address.  this will cause a malformed packet.  */
-		break;
+		return 0;
 	};
 
 	if (version == 5) {
@@ -857,6 +1032,38 @@ dissect_sflow(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			    offset);
 		}
 	}
+
+	return tvb_length(tvb);
+}
+
+
+static void
+sflow_delete_callback(guint32 port)
+{
+    if ( port ) {
+		dissector_delete("udp.port", port, sflow_handle);
+    }
+}
+static void
+sflow_add_callback(guint32 port)
+{
+    if ( port ) {
+		dissector_add("udp.port", port, sflow_handle);
+    }
+}
+
+
+static void
+sflow_reinit(void)
+{
+	if (sflow_ports) {
+		range_foreach(sflow_ports, sflow_delete_callback);
+		g_free(sflow_ports);
+	}
+
+	sflow_ports = range_copy(global_sflow_ports);
+
+	range_foreach(sflow_ports, sflow_add_callback);
 }
 
 
@@ -868,97 +1075,98 @@ dissect_sflow(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 void
 proto_register_sflow(void)
-{                 
+{
+	module_t *sflow_module;
 
 /* Setup list of header fields  See Section 1.6.1 for details*/
 	static hf_register_info hf[] = {
 		{ &hf_sflow_version,
-			{ "datagram version", "sflow.version",
+		  { "datagram version", "sflow.version",
 			FT_UINT32, BASE_DEC, NULL, 0x0,          
 			"sFlow datagram version", HFILL }
 		},
 		{ &hf_sflow_agent_address_v4,
-			{ "agent address", "sflow.agent",
+		  { "agent address", "sflow.agent",
 			FT_IPv4, BASE_NONE, NULL, 0x0,          
 			"sFlow Agent IP address", HFILL }
 		},
 		{ &hf_sflow_agent_address_v6,
-			{ "agent address", "sflow.agent.v6",
+		  { "agent address", "sflow.agent.v6",
 			FT_IPv6, BASE_NONE, NULL, 0x0,          
 			"sFlow Agent IPv6 address", HFILL }
 		},
 		{ &hf_sflow_sub_agent_id,
-			{ "Sub-agent ID", "sflow.sub_agent_id",
+		  { "Sub-agent ID", "sflow.sub_agent_id",
 			FT_UINT32, BASE_DEC, NULL, 0x0,          
 			"sFlow sub-agent ID", HFILL }
 		},
 		{ &hf_sflow_seqnum,
-			{ "Sequence number", "sflow.sequence_number",
+		  { "Sequence number", "sflow.sequence_number",
 			FT_UINT32, BASE_DEC, NULL, 0x0,          
 			"sFlow datagram sequence number", HFILL }
 		},
 		{ &hf_sflow_sysuptime,
-			{ "SysUptime", "sflow.sysuptime",
+		  { "SysUptime", "sflow.sysuptime",
 			FT_UINT32, BASE_DEC, NULL, 0x0,          
 			"System Uptime", HFILL }
 		},
 		{ &hf_sflow_numsamples,
-			{ "NumSamples", "sflow.numsamples",
+		  { "NumSamples", "sflow.numsamples",
 			FT_UINT32, BASE_DEC, NULL, 0x0,          
 			"Number of samples in sFlow datagram", HFILL }
 		},
 		{ &hf_sflow_sampletype,
-			{ "sFlow sample type", "sflow.sampletype",
+		  { "sFlow sample type", "sflow.sampletype",
 			FT_UINT32, BASE_DEC, VALS(sflow_sampletype), 0x0,          
 			"Type of sFlow sample", HFILL }
 		},
 		{ &hf_sflow_header_protocol,
-			{ "Header protocol", "sflow.header_protocol",
+		  { "Header protocol", "sflow.header_protocol",
 			FT_UINT32, BASE_DEC, VALS(sflow_header_protocol), 0x0,          
 			"Protocol of sampled header", HFILL }
 		},
 		{ &hf_sflow_header,
-			{ "Header of sampled packet", "sflow.header",
+		  { "Header of sampled packet", "sflow.header",
 			FT_BYTES, BASE_HEX, NULL, 0x0,          
 			"Data from sampled header", HFILL }
 		},
 		{ &hf_sflow_packet_information_type,
-			{ "Sample type", "sflow.packet_information_type",
+		  { "Sample type", "sflow.packet_information_type",
 			FT_UINT32, BASE_DEC, VALS(sflow_packet_information_type), 0x0,
 			"Type of sampled information", HFILL }
 		},
 		{ &hf_sflow_extended_information_type,
-			{ "Extended information type", "sflow.extended_information_type",
+		  { "Extended information type", "sflow.extended_information_type",
 			FT_UINT32, BASE_DEC, VALS(sflow_extended_data_types), 0x0,
 			"Type of extended information", HFILL }
 		},
 		{ &hf_sflow_vlan_in,
-			{ "Incoming 802.1Q VLAN", "sflow.vlan.in",
+		  { "Incoming 802.1Q VLAN", "sflow.vlan.in",
 			FT_UINT32, BASE_DEC, NULL, 0x0,
 			"Incoming VLAN ID", HFILL }
 		},
 		{ &hf_sflow_vlan_out,
-			{ "Outgoing 802.1Q VLAN", "sflow.vlan.out",
+		  { "Outgoing 802.1Q VLAN", "sflow.vlan.out",
 			FT_UINT32, BASE_DEC, NULL, 0x0,
 			"Outgoing VLAN ID", HFILL }
 		},
 		{ &hf_sflow_pri_in,
-			{ "Incoming 802.1p priority", "sflow.pri.in",
+		  { "Incoming 802.1p priority", "sflow.pri.in",
 			FT_UINT32, BASE_DEC, NULL, 0x0,
 			"Incoming 802.1p priority", HFILL }
 		},
 		{ &hf_sflow_pri_out,
-			{ "Outgoing 802.1p priority", "sflow.pri.out",
+		  { "Outgoing 802.1p priority", "sflow.pri.out",
 			FT_UINT32, BASE_DEC, NULL, 0x0,
 			"Outgoing 802.1p priority", HFILL }
 		},
 		{ &hf_sflow_nexthop_v4,
-			{ "Next hop", "sflow.nexthop",
+		  { "Next hop", "sflow.nexthop",
 			FT_IPv4, BASE_DEC, NULL, 0x0,
 			"Next hop address", HFILL }
 		},
 		{ &hf_sflow_nexthop_v6,
-			{ "Next hop", "sflow.nexthop",
+		  { "Next hop", "sflow.nexthop",
 			FT_IPv6, BASE_HEX, NULL, 0x0,
 			"Next hop address", HFILL }
 		},
@@ -977,6 +1185,48 @@ proto_register_sflow(void)
 			FT_UINT32, BASE_DEC, NULL, 0x0,
 			"Interface Index", HFILL }
 		},
+		{ &hf_sflow_as,
+		  { "AS Router", "sflow.as",
+			FT_UINT32, BASE_DEC, NULL, 0x0,
+			"Autonomous System of Router", HFILL }
+		},
+		{ &hf_sflow_src_as,
+		  { "AS Source", "sflow.srcAS",
+			FT_UINT32, BASE_DEC, NULL, 0x0,
+			"Autonomous System of Source", HFILL }
+		},
+		{ &hf_sflow_src_peer_as,
+		  { "AS Peer", "sflow.peerAS",
+			FT_UINT32, BASE_DEC, NULL, 0x0,
+			"Autonomous System of Peer", HFILL }
+		},
+		{ &hf_sflow_dst_as_entries,
+		  { "AS Destinations", "sflow.dstASentries",
+			FT_UINT32, BASE_DEC, NULL, 0x0,
+			"Autonomous System destinations", HFILL }
+		},
+		{ &hf_sflow_dst_as,
+		  { "AS Destination", "sflow.dstAS",
+			FT_UINT32, BASE_DEC, NULL, 0x0,
+			"Autonomous System destination", HFILL }
+		},
+		/* Needed for sFlow >= 4.  If I had a capture to test... */
+		{ &hf_sflow_community_entries,
+		  { "Gateway Communities", "sflow.communityEntries",
+			FT_UINT32, BASE_DEC, NULL, 0x0,
+			"Gateway Communities", HFILL }
+		},
+		{ &hf_sflow_community,
+		  { "Gateway Community", "sflow.community",
+			FT_UINT32, BASE_DEC, NULL, 0x0,
+			"Gateway Communities", HFILL }
+		},
+		{ &hf_sflow_localpref,
+ 		  { "localpref", "sflow.localpref",
+			FT_UINT32, BASE_DEC, NULL, 0x0,
+			"Local preferences of AS route", HFILL }
+		},
+		/**/
 		{ &hf_sflow_iftype,
 		  { "Interface Type", "sflow.iftype",
 			FT_UINT32, BASE_DEC, NULL, 0x0,
@@ -1074,6 +1324,9 @@ proto_register_sflow(void)
 		&ett_sflow,
 		&ett_sflow_sample,
 		&ett_sflow_extended_data,
+		&ett_sflow_gw_as_dst,
+		&ett_sflow_gw_as_dst_seg,
+		&ett_sflow_gw_community,
 		&ett_sflow_sampled_header,
 	};
 
@@ -1084,44 +1337,120 @@ proto_register_sflow(void)
 /* Required function calls to register the header fields and subtrees used */
 	proto_register_field_array(proto_sflow, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
+
+/* Register our configuration options for sFlow */
+	sflow_module = prefs_register_protocol(proto_sflow,
+					       proto_reg_handoff_sflow);
+
+/* Set default Neflow port(s) */
+	range_convert_str(&global_sflow_ports, SFLOW_UDP_PORTS,
+			  MAX_UDP_PORT);
+
+	prefs_register_obsolete_preference(sflow_module, "udp.port");
+
+	prefs_register_range_preference(sflow_module, "ports",
+					"sFlow UDP Port(s)",
+					"Set the port(s) for sFlow messages"
+					" (default: " SFLOW_UDP_PORTS ")",
+					&global_sflow_ports, MAX_UDP_PORT);
+
+	/* 
+	   If I use a filter like "ip.src == 10.1.1.1" this will, in
+	   addition to the usual suspects, find every sFlow packet
+	   where *any* of the payload headers contain 10.1.1.1 as a
+	   src addr.  I think this may not be the desired behavior.
+	   It can certainly be confusing since the ip.src being found
+	   is buried about 3 subtrees deep and the subtrees might be
+	   under any one of the sampled (payload) header trees. It is
+	   certainly not quickly obvious why the filter matched.
+	*/
+	prefs_register_bool_preference(sflow_module, "enable_dissection",
+				       "Dissect data in sampled headers",
+				       "Enabling dissection makes it easy to view protocol details in each of the sampled headers.  Disabling dissection may reduce noise caused when display filters match the contents of any sampled header(s).",
+					&global_dissect_samp_headers);
+	/* 
+	   It is not clear to me that it *ever* makes sense to enable
+	   this option.  However, it was previously the default
+	   behavior so I'll leave it as an option if someone thinks
+	   they have a use for it.
+	*/
+	prefs_register_bool_preference(sflow_module, "enable_analysis",
+				       "Analyze data in sampled IP headers",
+				       "This option only makes sense if dissection of sampled headers is enabled and probably not even then.",
+					&global_analyze_samp_ip_headers );
+
+
+	register_init_routine(&sflow_reinit);
 }
 
 
-/* If this dissector uses sub-dissector registration add a registration routine.
-   This format is required because a script is used to find these routines and
-   create the code that calls these routines.
+/* If this dissector uses sub-dissector registration add a
+   registration routine.  This format is required because a script is
+   used to find these routines and create the code that calls these
+   routines.
 */
 void
 proto_reg_handoff_sflow(void)
 {
-	dissector_handle_t sflow_handle;
+	static int sflow_prefs_initialized = FALSE;
+	if (!sflow_prefs_initialized) {
+		sflow_handle = new_create_dissector_handle(dissect_sflow,
+							   proto_sflow);
+
+		sflow_prefs_initialized = TRUE;
+	}
+
+
+	sflow_reinit();
+
+	/*dissector_handle_t sflow_handle;*/
 
 	/*
 	 * XXX - should this be done with a dissector table?
 	 */
-	eth_withoutfcs_handle = find_dissector("eth_withoutfcs");
-	tr_handle = find_dissector("tr");
-	fddi_handle = find_dissector("fddi");
-	fr_handle = find_dissector("fr");
-	x25_handle = find_dissector("x.25");
-	ppp_handle = find_dissector("ppp");
+	data_handle = find_dissector("data");
+
+	if (global_dissect_samp_headers) {
+	    eth_withoutfcs_handle = find_dissector("eth_withoutfcs");
+	    tr_handle = find_dissector("tr");
+	    fddi_handle = find_dissector("fddi");
+	    fr_handle = find_dissector("fr");
+	    x25_handle = find_dissector("x.25");
+	    ppp_handle = find_dissector("ppp");
 #if 0
-	smds_handle = find_dissector("smds");
+	    smds_handle = find_dissector("smds");
 #else
-	/* We don't have an SMDS dissector yet */
-	smds_handle = find_dissector("data");
+	    /* We don't have an SMDS dissector yet */
+	    smds_handle = data_handle;
 #endif
 #if 0
-	aal5_handle = find_dissector("atm");
+	    aal5_handle = find_dissector("atm");
 #else
-	/* What dissector should be used here? */
-	aal5_handle = find_dissector("data");
+	    /* What dissector should be used here? */
+	    aal5_handle = data_handle;
 #endif
-	ipv4_handle = find_dissector("ip");
-	ipv6_handle = find_dissector("ipv6");
-	mpls_handle = find_dissector("mpls");
+	    ipv4_handle = find_dissector("ip");
+	    ipv6_handle = find_dissector("ipv6");
+	    mpls_handle = find_dissector("mpls");
+	} else {
+	    eth_withoutfcs_handle = data_handle;
+	    tr_handle = data_handle;
+	    fddi_handle = data_handle;
+	    fr_handle = data_handle;
+	    x25_handle = data_handle;
+	    ppp_handle = data_handle;
+	    smds_handle = data_handle;
+	    aal5_handle = data_handle;
+	    ipv4_handle = data_handle;
+	    ipv6_handle = data_handle;
+	    mpls_handle = data_handle;
+	}
 
-	sflow_handle = create_dissector_handle(dissect_sflow,
-	    proto_sflow);
-	dissector_add("udp.port", UDP_PORT_SFLOW, sflow_handle);
 }
+
+
+/* Local Variables: ***/
+/* mode:C ***/
+/* c-basic-offset:4 ***/
+/* tab-width:4 ***/
+/* End: ***/
