@@ -2,7 +2,7 @@
  * ssl manipulation functions
  * By Paolo Abeni <paolo.abeni@email.com>
  *
- * $Id: packet-ssl-utils.c 24301 2008-02-11 17:04:51Z wmeier $
+ * $Id: packet-ssl-utils.c 24525 2008-03-01 17:23:39Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -2091,14 +2091,14 @@ ssl_load_pkcs12(FILE* fp, const gchar *cert_passwd) {
 
           buf_len = sizeof(buf_name);
           ret = gnutls_x509_crt_get_dn_by_oid(ssl_cert, GNUTLS_OID_X520_COMMON_NAME, 0, 0, buf_name, &buf_len);
-          if (ret < 0) { strncpy(buf_name, "<ERROR>", 256); }
+          if (ret < 0) { g_strlcpy(buf_name, "<ERROR>", 256); }
           buf_len = sizeof(buf_email);
           ret = gnutls_x509_crt_get_dn_by_oid(ssl_cert, GNUTLS_OID_PKCS9_EMAIL, 0, 0, buf_email, &buf_len);
-          if (ret < 0) { strncpy(buf_email, "<ERROR>", 256); }
+          if (ret < 0) { g_strlcpy(buf_email, "<ERROR>", 128); }
 
           buf_len = sizeof(buf_keyid);
           ret = gnutls_x509_crt_get_key_id(ssl_cert, 0, buf_keyid, &buf_len); 
-          if (ret < 0) { strncpy(buf_keyid, "<ERROR>", 256); }
+          if (ret < 0) { g_strlcpy(buf_keyid, "<ERROR>", 32); }
 
           private_key->x509_cert = ssl_cert;
           ssl_debug_printf( "Certificate imported: %s <%s>, KeyID %s\n", buf_name, buf_email, bytes_to_str(buf_keyid, buf_len));
@@ -2355,7 +2355,7 @@ ssl_association_add(GTree* associations, dissector_handle_t handle, guint port,
   assoc->tcp = tcp;
   assoc->ssl_port = port;
   assoc->info=g_malloc(strlen(protocol)+1);
-  strncpy(assoc->info, protocol, strlen(protocol)+1);
+  g_strlcpy(assoc->info, protocol, strlen(protocol)+1);
   assoc->handle = find_dissector(protocol);
   assoc->from_key_list = from_key_list;
 
