@@ -5,7 +5,7 @@
  * Full Tacacs+ parsing with decryption by
  *   Emanuele Caratti <wiz@iol.it>
  *
- * $Id: packet-tacacs.c 20277 2007-01-02 22:49:57Z gerald $
+ * $Id: packet-tacacs.c 24282 2008-02-07 12:14:40Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -87,6 +87,8 @@ static int hf_tacacs_result3 = -1;
 
 static gint ett_tacacs = -1;
 
+static gboolean tacplus_preference_desegment = TRUE;
+
 static const char *tacplus_opt_key;
 static GSList *tacplus_keys = NULL;
 
@@ -350,7 +352,7 @@ tacplus_decrypted_tvb_setup( tvbuff_t *tvb, tvbuff_t **dst_tvb, packet_info *pin
 	/* TODO Check the possibility to use pinfo->decrypted_data */
 /* session_id is in NETWORK Byte Order, and is used as byte array in the md5_xor */
 
-	tvb_memcpy(tvb, (guint8*)session_id, 4,4);
+	tvb_memcpy(tvb, session_id, 4,4);
 
 	buff = tvb_memdup(tvb, TAC_PLUS_HDR_SIZE, len);
 
@@ -769,24 +771,23 @@ dissect_tacplus_body(tvbuff_t * hdr_tvb, tvbuff_t * tvb, proto_tree * tree )
 		} else {
 			dissect_tacplus_body_authen_rep( tvb, tree );
 		}
-		return;
 		break;
 	  case TAC_PLUS_AUTHOR:
 		if ( seq_no & 0x01)
 			dissect_tacplus_body_author_req( tvb, tree );
 		else
 			dissect_tacplus_body_author_rep( tvb, tree );
-		return;
 		break;
 	  case TAC_PLUS_ACCT:
 		if ( seq_no & 0x01)
 			dissect_tacplus_body_acct_req( tvb, tree );
 		else
 			dissect_tacplus_body_acct_rep( tvb, tree );
-		return;
+		break;
+	  default:
+		proto_tree_add_text( tree, tvb, 0, tvb_length( tvb ), "Bogus..");
 		break;
 	}
-	proto_tree_add_text( tree, tvb, 0, tvb_length( tvb ), "Bogus..");
 }
 
 #ifdef DEB_TACPLUS
@@ -855,7 +856,7 @@ mkipv4_address( address **addr, const char *str_addr )
 	inet_pton( AF_INET, str_addr, addr_data );
 	(*addr)->type=AT_IPv4;
 	(*addr)->len=4;
-	(*addr)->data=addr_data;
+	(*addr)->data=(guint8*)addr_data;
 }
 static void
 parse_tuple( char *key_from_option )
@@ -926,6 +927,15 @@ dissect_tacplus(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	gboolean	request=( pinfo->destport == TCP_PORT_TACACS );
 	const char	*key=NULL;
 
+	len = tvb_get_ntohl(tvb, 8);
+	
+	if(len > (guint)tvb_length_remaining(tvb, 12) &&
+	   pinfo->can_desegment && tacplus_preference_desegment) {
+		pinfo->desegment_offset = 0;
+		pinfo->desegment_len = len;
+		return;
+	}
+	
 	if( request ) {
 		key=find_key( &pinfo->dst, &pinfo->src );
 	} else {
@@ -985,7 +995,7 @@ dissect_tacplus(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		    tvb, 3, 1, flags);
 		proto_tree_add_item(tacplus_tree, hf_tacplus_session_id, tvb, 4, 4,
 		    FALSE);
-		len = tvb_get_ntohl(tvb,8);
+
 		if ((gint) len < 1) {
 			proto_tree_add_text(tacplus_tree, tvb, 8, 4,
 				"Invalid length: %u", len);
@@ -1088,6 +1098,9 @@ proto_register_tacplus(void)
 	proto_register_field_array(proto_tacplus, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
 	tacplus_module = prefs_register_protocol (proto_tacplus, tacplus_pref_cb );
+
+        prefs_register_bool_preference(tacplus_module, "desegment", "Reassemble TACACS+ messages spanning multiple TCP segments.", "Whether the TACACS+ dissector should reasssemble messages spanning multiple TCP segments.  To use this option, you must also enable \"Allow subdissectors to reassemble TCP streams\" in the TCP protocol settings.", &tacplus_preference_desegment);
+
 	prefs_register_string_preference ( tacplus_module, "key",
 	"TACACS+ Encryption Key", "TACACS+ Encryption Key", &tacplus_opt_key );
 }
@@ -1121,7 +1134,7 @@ md5_xor( guint8 *data, const char *key, int data_len, guint8 *session_id, guint8
 
 
 	mdp = md5_buff;
-	*(guint32*)mdp = *(guint32*)session_id;
+	memcpy(mdp, session_id, 4);
 	mdp += 4 ;
 	memcpy(mdp, key, strlen(key));
 	mdp += strlen(key);
