@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2007 by Valery Sigalov <valery.sigalov@audiocodes.com>
  *
- * $Id: packet-tpncp.c 24280 2008-02-07 11:42:11Z stig $
+ * $Id: packet-tpncp.c 24614 2008-03-13 04:49:37Z sfisher $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.com>
@@ -43,6 +43,7 @@
 #include <epan/emem.h>
 #include <epan/filesystem.h>
 #include <epan/dissectors/packet-tcp.h>
+#include <epan/strutil.h>
 
 /*-------------------------------------------------------------------------------------------------------------------------------------------*/
 
@@ -53,7 +54,6 @@
 #define UDP_PORT_TPNCP_HOST BASE_TPNCP_PORT
 
 #define BASE_TPNCP_DATA_LEN 256
-#define MAX_TPNCP_DATA_FIELD_NAME_LEN BASE_TPNCP_DATA_LEN
 #define MAX_TPNCP_DAT_FILE_PATH_LEN BASE_TPNCP_DATA_LEN
 #define MAX_TPNCP_DB_ENTRY_LEN BASE_TPNCP_DATA_LEN
 #define MAX_TPNCP_HEADER_LEN BASE_TPNCP_DATA_LEN
@@ -67,7 +67,7 @@
 /* The linked list for storing information about specific data fields. */
 typedef struct tpncp_data_field_info
 {
-    gchar tpncp_data_field_name[MAX_TPNCP_DATA_FIELD_NAME_LEN];
+    gchar *tpncp_data_field_name;
     gint tpncp_data_field_descr;
     gint tpncp_data_field_sign;
     gint tpncp_data_field_size;
@@ -518,8 +518,7 @@ static gint fill_enums_id_vals(FILE *file) {
                 else
                     first_entry = 0;
                 tpncp_enums_name_vals[enum_val] = g_strdup(enum_name);
-                strncpy(enum_type, enum_name, MAX_TPNCP_DB_ENTRY_LEN);
-		enum_type[MAX_TPNCP_DB_ENTRY_LEN-1] = '\0';
+                g_strlcpy(enum_type, enum_name, MAX_TPNCP_DB_ENTRY_LEN);
             }
             tpncp_enums_id_vals[enum_val][i].strptr = g_strdup(enum_str);
             tpncp_enums_id_vals[enum_val][i].value = enum_id;
@@ -640,8 +639,7 @@ static gint init_tpncp_data_fields_info(tpncp_data_field_info *data_fields_info,
         }
         current_tpncp_data_field_info->tpncp_data_field_descr = -1;
         hf_entr.p_id = &current_tpncp_data_field_info->tpncp_data_field_descr;
-        strncpy(current_tpncp_data_field_info->tpncp_data_field_name, tpncp_data_field_name, MAX_TPNCP_DATA_FIELD_NAME_LEN);
-	current_tpncp_data_field_info->tpncp_data_field_name[MAX_TPNCP_DATA_FIELD_NAME_LEN-1] = '\0';
+	current_tpncp_data_field_info->tpncp_data_field_name = g_strdup(tpncp_data_field_name);
         hf_entr.hfinfo.name = current_tpncp_data_field_info->tpncp_data_field_name;
         hf_entr.hfinfo.abbrev = current_tpncp_data_field_info->tpncp_data_field_name;
         switch (tpncp_data_field_size) {
