@@ -1,7 +1,7 @@
 /* packet-udp.c
  * Routines for UDP/UDPLite packet disassembly
  *
- * $Id: packet-udp.c 23346 2007-11-03 04:45:35Z sfisher $
+ * $Id: packet-udp.c 24043 2008-01-09 13:30:31Z kukosa $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -122,8 +122,8 @@ decode_udp_ports(tvbuff_t *tvb, int offset, packet_info *pinfo,
 /* determine if this packet is part of a conversation and call dissector */
 /* for the conversation if available */
 
-  if (try_conversation_dissector(&pinfo->src, &pinfo->dst, PT_UDP,
-		uh_sport, uh_dport, next_tvb, pinfo, tree)){
+  if (try_conversation_dissector(&pinfo->dst, &pinfo->src, PT_UDP,
+		uh_dport, uh_sport, next_tvb, pinfo, tree)){
     return;
   }
 
@@ -301,12 +301,17 @@ dissect(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, guint32 ip_proto)
     } else {
       item = proto_tree_add_uint_format(udp_tree, hf_udp_checksum, tvb, offset + 6, 2, 0,
         "Checksum: 0x%04x (Illegal)", 0);
+      expert_add_info_format(pinfo, item, PI_CHECKSUM, PI_ERROR, "Illegal Checksum value (0)");
+      if (check_col(pinfo->cinfo, COL_INFO))
+        col_append_fstr(pinfo->cinfo, COL_INFO, " [ILLEGAL CHECKSUM (0)]");
 
       checksum_tree = proto_item_add_subtree(item, ett_udp_checksum);
-      proto_tree_add_boolean(checksum_tree, hf_udp_checksum_good, tvb,
+      item = proto_tree_add_boolean(checksum_tree, hf_udp_checksum_good, tvb,
                              offset + 6, 2, FALSE);
-      proto_tree_add_boolean(checksum_tree, hf_udp_checksum_bad, tvb,
+      PROTO_ITEM_SET_GENERATED(item);
+      item = proto_tree_add_boolean(checksum_tree, hf_udp_checksum_bad, tvb,
                              offset + 6, 2, TRUE);
+      PROTO_ITEM_SET_GENERATED(item);
     }
   } else if (!pinfo->fragmented && len >= reported_len &&
              len >= udph->uh_sum_cov && reported_len >= udph->uh_sum_cov &&
