@@ -1,6 +1,6 @@
 /* epan.h
  *
- * $Id: epan.h 20261 2007-01-02 06:49:40Z guy $
+ * $Id: epan.h 21716 2007-05-07 17:55:42Z gal $
  *
  * Wireshark Protocol Analyzer Library
  *
@@ -27,14 +27,17 @@
 #include <glib.h>
 #include "frame_data.h"
 #include "column_info.h"
+#include "register.h"
 
 typedef struct _epan_dissect_t epan_dissect_t;
 
 #include "dfilter/dfilter.h"
 
 /* init the whole epan module, this is used to be called only once in a program */
-void epan_init(void (*register_all_protocols)(void),
-	       void (*register_all_handoffs)(void),
+void epan_init(void (*register_all_protocols)(register_cb cb, gpointer client_data),
+	       void (*register_all_handoffs)(register_cb cb, gpointer client_data),
+	       register_cb cb,
+	       void *client_data,
 	       void (*report_failure)(const char *, va_list),
 	       void (*report_open_failure)(const char *, int, gboolean),
 	       void (*report_read_failure)(const char *, int));
