@@ -1,6 +1,6 @@
 /* follow.h
  *
- * $Id: follow.h 21870 2007-05-22 00:00:09Z guy $
+ * $Id: follow.h 23878 2007-12-15 23:25:05Z sake $
  *
  * Copyright 1998 Mike Hall <mlh@io.com>
  *
@@ -42,17 +42,17 @@ typedef struct _tcp_stream_chunk {
 } tcp_stream_chunk;
 
 char* build_follow_filter( packet_info * );
-void reassemble_tcp( gulong, gulong, const char*, gulong, int,
+void reassemble_tcp( gulong, gulong, gulong, const char*, gulong, int,
 		     address *, address *, guint, guint );
 void  reset_tcp_reassembly( void );
 
 typedef struct {
 	guint8		ip_address[2][MAX_IPADDR_LEN];
-	guint32		tcp_port[2];
+	guint32		port[2];
 	unsigned int	bytes_written[2];
 	gboolean        is_ipv6;
-} follow_tcp_stats_t;
+} follow_stats_t;
 
-void follow_tcp_stats(follow_tcp_stats_t* stats);
+void follow_stats(follow_stats_t* stats);
 
 #endif
