@@ -3,7 +3,7 @@
  *
  * (c) 2007, Luis E. Garcia Ontanon <luis.ontanon@gmail.com>
  *
- * $Id: oids.c 23842 2007-12-11 21:24:53Z gerald $
+ * $Id: oids.c 24717 2008-03-23 02:42:30Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -101,7 +101,7 @@ static oid_info_t* add_oid(const char* name, oid_kind_t kind, const oid_value_ty
 			if (i == oid_len) {
 				if (n->name) {
 					if (!g_str_equal(n->name,name)) {
-						D(2,("Renaming Oid from: %s -> %s, this menas the same oid is registered more than once",n->name,name));
+						D(2,("Renaming Oid from: %s -> %s, this means the same oid is registered more than once",n->name,name));
 					}
 					/* XXX - Don't free n->name here. It may be part of an hf_register_info
                                          * struct that has been appended to the hfa GArray. */
@@ -180,6 +180,28 @@ extern void oid_add_from_encoded(const char* name, const guint8 *oid, gint oid_l
 }
 
 #ifdef HAVE_LIBSMI
+/* de-allocate storage mallocated by libsmi                            */
+/*                                                                     */
+/* XXX: On Windows we can use free() only if the Windows libsmi.dll    */
+/*      being used is linked with the same CRTL as wireshark.          */
+/*      As a temporary hack we'll assume on Windows that the Wireshark */
+/*      build of libsmi.dll was done with VC6/msvcrt.dll (as is        */
+/*      currently the case).  If wireshark is being built with         */
+/*      vc6 (which is also currently the case for the standard         */
+/*      wireshark release). we can use free().                         */
+/*      If vc6 is not being used here, then we'll just have to live    */
+/*      with a memory leak for now.                                    */
+/*      Note: A permanent fix would probably be for libsmi to include  */
+/*       its' existing smiFree function as part of the libsmi API so   */
+/*       wireshark can call it to free storage mallocated by libsmi.   */
+
+static void smi_free(void *ptr) {
+#if !defined _WIN32 || (_MSC_VER == 1200)
+	free(ptr);
+#endif
+}
+
+
 typedef struct smi_module_t {
 	char* name;
 } smi_module_t;
@@ -289,10 +311,14 @@ const oid_value_type_t* get_typedata(SmiType* smiType) {
 
 	do {
 		for (t = types; t->type ; t++ ) {
-			const char* name = smiRenderType(sT, SMI_RENDER_NAME);
+			char* name = smiRenderType(sT, SMI_RENDER_NAME);
 			if (name && t->name && g_str_equal(name, t->name )) {
+				smi_free(name);
 				return t->type;
 			}
+			if (name) {
+				smi_free (name);
+			}
 		}
 	} while(( sT  = smiGetParentType(sT) ));
 
@@ -367,6 +393,7 @@ static inline oid_kind_t smikind(SmiNode* sN, oid_key_t** key_p) {
 				SmiType* elType = smiGetNodeType(elNode);
 				oid_key_t* k;
 				guint non_implicit_size = 0;
+				char *oid1, *oid2;
 
 				if (elType) {
 					non_implicit_size = get_non_implicit_size(elType);
@@ -376,9 +403,12 @@ static inline oid_kind_t smikind(SmiNode* sN, oid_key_t** key_p) {
 
 				k = g_malloc(sizeof(oid_key_t));
 
-				k->name = g_strdup_printf("%s.%s",
-										  smiRenderOID(sN->oidlen, sN->oid, SMI_RENDER_QUALIFIED),
-										  smiRenderOID(elNode->oidlen, elNode->oid, SMI_RENDER_NAME));
+				oid1 = smiRenderOID(sN->oidlen, sN->oid, SMI_RENDER_QUALIFIED);
+				oid2 = smiRenderOID(elNode->oidlen, elNode->oid, SMI_RENDER_NAME);
+				k->name = g_strdup_printf("%s.%s", oid1, oid2);
+				smi_free (oid1);
+				smi_free (oid2);
+
 				k->hfid = -2;
 				k->ft_type = typedata ? typedata->ft_type : FT_BYTES;
 				k->display = typedata ? typedata->display : BASE_NONE;
@@ -451,22 +481,11 @@ static inline oid_kind_t smikind(SmiNode* sN, oid_key_t** key_p) {
 						   || (ft == FT_INT8) || (ft == FT_INT16) || (ft == FT_INT24) || (ft == FT_INT32) \
 						   || (ft == FT_UINT64) || (ft == FT_INT64) )
 
-#ifdef WIN32
-#define PATH_SEPARATOR ";"
-#define DEFAULT_PATH_FMT "%s;%s"
-#define DEFAULT_PATH_ARGS get_datafile_path("snmp\\mibs"), get_persconffile_path("snmp\\mibs", FALSE)
-#else
-#define PATH_SEPARATOR ":"
-#define DEFAULT_PATH_FMT  "%s"
-#define DEFAULT_PATH_ARGS smiGetPath()
-#endif
-
 void register_mibs(void) {
 	SmiModule *smiModule;
 	SmiNode *smiNode;
 	guint i;
 	int proto_mibs = -1;
-	module_t* mibs_module;
 	GArray* hfa = g_array_new(FALSE,TRUE,sizeof(hf_register_info));
 	GArray* etta = g_array_new(FALSE,TRUE,sizeof(gint*));
 	static uat_field_t smi_fields[] = {
@@ -483,6 +502,7 @@ void register_mibs(void) {
 	smi_modules_uat = uat_new("SMI Modules",
 							  sizeof(smi_module_t),
 							  "smi_modules",
+							  FALSE,
 							  (void*)&smi_modules,
 							  &num_smi_modules,
 							  UAT_CAT_GENERAL,
@@ -495,6 +515,7 @@ void register_mibs(void) {
 	smi_paths_uat = uat_new("SMI Paths",
 							  sizeof(smi_module_t),
 							  "smi_paths",
+							  FALSE,
 							  (void*)&smi_paths,
 							  &num_smi_paths,
 							  UAT_CAT_GENERAL,
@@ -569,13 +590,14 @@ void register_mibs(void) {
 			const oid_value_type_t* typedata =  get_typedata(smiType);
 			oid_key_t* key;
 			oid_kind_t kind = smikind(smiNode,&key);
-			oid_info_t* oid_data = add_oid(smiRenderOID(smiNode->oidlen, smiNode->oid, SMI_RENDER_QUALIFIED),
-										   kind,
-										   typedata,
-										   key,
-										   smiNode->oidlen,
-										   smiNode->oid);
-
+			char *oid = smiRenderOID(smiNode->oidlen, smiNode->oid, SMI_RENDER_QUALIFIED);
+			oid_info_t* oid_data = add_oid(oid,
+						       kind,
+						       typedata,
+						       key,
+						       smiNode->oidlen,
+						       smiNode->oid);
+			smi_free (oid);
 
 			D(4,("\t\tNode: kind=%d oid=%s name=%s ",
 				 oid_data->kind, oid_subid2string(smiNode->oid, smiNode->oidlen), oid_data->name ));
@@ -589,7 +611,11 @@ void register_mibs(void) {
 					typedata->display,
 					NULL,
 					0,
-					g_strdup(smiRenderOID(smiNode->oidlen, smiNode->oid, SMI_RENDER_ALL)),
+#if !defined _WIN32 || (_MSC_VER == 1200)
+					smiRenderOID(smiNode->oidlen, smiNode->oid, SMI_RENDER_ALL),
+#else
+					g_strdup (smiRenderOID(smiNode->oidlen, smiNode->oid, SMI_RENDER_ALL)),
+#endif
 					HFILL }};
 
 				oid_data->value_hfid = -1;
@@ -607,7 +633,7 @@ void register_mibs(void) {
 					hf.hfinfo.strings = VALS(vals->data);
 					g_array_free(vals,FALSE);
 				}
-#if 0 /* packet-snmp does not hanldle bits yet */
+#if 0 /* packet-snmp does not handle bits yet */
 			} else if (smiType->basetype == SMI_BASETYPE_BITS && ( smiEnum = smiGetFirstNamedNumber(smiType) )) {
 				guint n = 0;
 				oid_bits_info_t* bits = g_malloc(sizeof(oid_bits_info_t));
@@ -667,24 +693,12 @@ void register_mibs(void) {
 					}
 				}
 			}
-
 		}
 	}
 
 	proto_mibs = proto_register_protocol("MIBs", "MIBS", "mibs");
 
 	proto_register_field_array(proto_mibs, (hf_register_info*)hfa->data, hfa->len);
-	mibs_module = prefs_register_protocol(proto_mibs, NULL);
-
-	prefs_register_uat_preference(mibs_module, "smi_paths",
-								  "MIB paths",
-								  "List of directories where MIBs are to be looked for",
-								  smi_paths_uat);
-
-	prefs_register_uat_preference(mibs_module, "smi_modules",
-											   "MIB modules",
-											   "List of MIB modules to be loaded",
-											   smi_modules_uat);
 
 	proto_register_subtree_array((gint**)etta->data, etta->len);
 
@@ -1011,8 +1025,8 @@ const gchar *oid_resolved(guint32 num_subids, guint32* subids) {
 
 	if (left) {
 		return ep_strdup_printf("%s.%s",
-								oid->name ? oid->name : oid_subid2string(subids,matched),
-								oid_subid2string(&(subids[matched]),left));
+					oid->name ? oid->name : oid_subid2string(subids,matched),
+					oid_subid2string(&(subids[matched]),left));
 	} else {
 		return oid->name ? oid->name : oid_subid2string(subids,matched);
 	}
@@ -1037,18 +1051,33 @@ extern void oid_both_from_string(const gchar *oid_str, char** resolved_p, char**
 	*numeric_p = (void*)oid_subid2string(subids,subids_len);
 }
 
-#ifdef HAVE_LIBSMI
 /**
  * Fetch the default OID path.
  */
 extern gchar *
 oid_get_default_mib_path(void) {
+#ifdef HAVE_LIBSMI
 	GString* path_str;
 	gchar *path_ret;
+	char *path;
 	guint i;
 
 	path_str = g_string_new("");
-	g_string_sprintfa(path_str, DEFAULT_PATH_FMT,  DEFAULT_PATH_ARGS);
+#ifdef _WIN32
+#define PATH_SEPARATOR ";"
+	path = get_datafile_path("snmp\\mibs");
+	g_string_sprintfa(path_str, "%s;", path);
+	g_free (path);
+
+	path = get_persconffile_path("snmp\\mibs", FALSE, FALSE);
+	g_string_sprintfa(path_str, "%s", path);
+	g_free (path);
+#else
+#define PATH_SEPARATOR ":"
+	path = smiGetPath();
+	g_string_sprintfa(path_str, "%s", path);
+	free (path);
+#endif
 
 	for(i=0;i<num_smi_paths;i++) {
 		if (!( smi_paths[i].name && *smi_paths[i].name))
@@ -1060,8 +1089,10 @@ oid_get_default_mib_path(void) {
 	path_ret = path_str->str;
 	g_string_free(path_str, FALSE);
 	return path_ret;
-}
+#else /* HAVE_LIBSMI */
+        return g_strdup("");
 #endif
+}
 
 #ifdef DEBUG_OIDS
 char* oid_test_a2b(guint32 num_subids, guint32* subids) {
