@@ -1,7 +1,7 @@
 /* packet_info.h
  * Definitions for packet info structures and routines
  *
- * $Id: packet_info.h 22067 2007-06-08 17:06:13Z guy $
+ * $Id: packet_info.h 24600 2008-03-10 17:33:53Z jake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -62,8 +62,8 @@ typedef struct _packet_info {
   port_type ptype;		/* type of the following two port numbers */
   guint32 srcport;		/* source port */
   guint32 destport;		/* destination port */
-  guint32 match_port;
-  const char *match_string;	/* Subdissectors with string dissector tables use this */
+  guint32 match_port;           /* matched port for calling subdissector from table */
+  const char *match_string;	/* matched string for calling subdissector from table */
   guint16 can_desegment;	/* >0 if this segment could be desegmented.
 				   A dissector that can offer this API (e.g.
 				   TCP) sets can_desegment=2, then
@@ -118,8 +118,8 @@ typedef struct _packet_info {
   guint32 bytes_until_next_pdu;
 
 
-  int     iplen;
-  int     iphdrlen;
+  int     iplen;                /* total length of IP packet */
+  int     iphdrlen;             /* length of IP header */
   int	  p2p_dir;              /* Packet was captured as an 
                                        outbound (P2P_DIR_SENT) 
                                        inbound (P2P_DIR_RECV) 
@@ -170,12 +170,15 @@ typedef struct _packet_info {
   guint16 link_number;
   guint8  annex_a_used;
   guint16 profinet_type; 	/* the type of PROFINET packet (0: not a PROFINET packet) */
+  void *profinet_conv; 	    /* the PROFINET conversation data (NULL: not a PROFINET packet) */
   void *usb_conv_info;
   void *tcp_tree;		/* proto_tree for the tcp layer */
 
   const char *dcerpc_procedure_name;	/* Used by PIDL to store the name of the current dcerpc procedure */
 
   struct _sccp_msg_info_t* sccp_info;
+  guint16 clnp_srcref;      /* clnp/cotp source reference (can't use srcport, this would confuse tpkt) */
+  guint16 clnp_dstref;      /* clnp/cotp destination reference (can't use dstport, this would confuse tpkt) */
 } packet_info;
 
 #endif /* __PACKET_INFO_H__ */
