@@ -1,7 +1,7 @@
 /* reassemble.c
  * Routines for {fragment,segment} reassembly
  *
- * $Id: reassemble.c 19953 2006-11-21 21:49:58Z guy $
+ * $Id: reassemble.c 23422 2007-11-10 16:08:14Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -53,16 +53,45 @@ static GMemChunk *fragment_key_chunk = NULL;
 static GMemChunk *fragment_data_chunk = NULL;
 static int fragment_init_count = 200;
 
-#define LINK_FRAG(fd_head,fd)					\
-	{ 	fragment_data *fd_i;				\
-		/* add fragment to list, keep list sorted */		\
-		for(fd_i=(fd_head);fd_i->next;fd_i=fd_i->next){	\
-			if( ((fd)->offset) < (fd_i->next->offset) )	\
-				break;					\
-		}							\
-		(fd)->next=fd_i->next;				\
-		fd_i->next=(fd);					\
+static void LINK_FRAG(fragment_data *fd_head,fragment_data *fd)
+{ 	
+	fragment_data *fd_i;
+
+	/* add fragment to list, keep list sorted */
+	for(fd_i= fd_head; fd_i->next;fd_i=fd_i->next) {
+		if (fd->offset < fd_i->next->offset )
+			break;
 	}
+	fd->next=fd_i->next;
+	fd_i->next=fd;
+}
+
+/* copy a fragment key to heap store to insert in the hash */
+static void *fragment_key_copy(const void *k)
+{
+	const fragment_key* key = (const fragment_key*) k;
+	fragment_key *new_key = g_mem_chunk_alloc(fragment_key_chunk);
+
+	COPY_ADDRESS(&new_key->src, &key->src);
+	COPY_ADDRESS(&new_key->dst, &key->dst);
+	new_key->id = key->id;
+	return new_key;
+}
+
+/* copy a dcerpc fragment key to heap store to insert in the hash */
+static void *dcerpc_fragment_key_copy(const void *k)
+{
+	const dcerpc_fragment_key* key = (const dcerpc_fragment_key*) k;
+	dcerpc_fragment_key *new_key = se_alloc(sizeof(dcerpc_fragment_key));
+
+	COPY_ADDRESS(&new_key->src, &key->src);
+	COPY_ADDRESS(&new_key->dst, &key->dst);
+	new_key->id = key->id;
+	new_key->act_id = key->act_id;
+    
+	return new_key;
+}
+
 
 static gint
 fragment_equal(gconstpointer k1, gconstpointer k2)
@@ -203,6 +232,21 @@ free_all_fragments(gpointer key_arg, gpointer value, gpointer user_data _U_)
 	return TRUE;
 }
 
+/* ------------------------- */
+static fragment_data *new_head(guint32 flags)
+{
+	fragment_data *fd_head;
+	/* head/first structure in list only holds no other data than
+         * 'datalen' then we don't have to change the head of the list
+         * even if we want to keep it sorted
+         */
+	fd_head=g_mem_chunk_alloc(fragment_data_chunk);
+	memset(fd_head, 0, sizeof(fragment_data));
+        
+	fd_head->flags=flags;
+	return fd_head;
+}
+
 /*
  * For a reassembled-packet hash table entry, free the fragment data
  * to which the value refers.
@@ -388,7 +432,7 @@ fragment_get(packet_info *pinfo, guint32 id, GHashTable *fragment_table)
 
 /* id *must* be the frame number for this to work! */
 fragment_data *
-fragment_get_reassembled(packet_info *pinfo, guint32 id, GHashTable *reassembled_table)
+fragment_get_reassembled(packet_info *pinfo _U_, guint32 id, GHashTable *reassembled_table)
 {
 	fragment_data *fd_head;
 	reassembled_key key;
@@ -445,6 +489,7 @@ fragment_set_tot_len(packet_info *pinfo, guint32 id, GHashTable *fragment_table,
 
 	if(fd_head){
 		fd_head->datalen = tot_len;
+		fd_head->flags |= FD_DATALEN_SET;
 	}
 
 	return;
@@ -607,6 +652,8 @@ fragment_add_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 	fd->next = NULL;
 	fd->flags = 0;
 	fd->frame = pinfo->fd->num;
+	if (fd->frame > fd_head->frame)
+	    fd_head->frame = fd->frame;
 	fd->offset = frag_offset;
 	fd->len  = frag_data_len;
 	fd->data = NULL;
@@ -624,7 +671,7 @@ fragment_add_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 			}
 			fd_i->flags &= (~FD_TOOLONGFRAGMENT) & (~FD_MULTIPLETAILS);
 		}
-		fd_head->flags ^= FD_DEFRAGMENTED|FD_PARTIAL_REASSEMBLY;
+		fd_head->flags &= ~(FD_DEFRAGMENTED|FD_PARTIAL_REASSEMBLY|FD_DATALEN_SET);
 		fd_head->flags &= (~FD_TOOLONGFRAGMENT) & (~FD_MULTIPLETAILS);
 		fd_head->datalen=0;
 		fd_head->reassembled_in=0;
@@ -634,7 +681,7 @@ fragment_add_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 		/*
 		 * This is the tail fragment in the sequence.
 		 */
-		if (fd_head->datalen) {
+		if (fd_head->flags & FD_DATALEN_SET) {
 			/* ok we have already seen other tails for this packet
 			 * it might be a duplicate.
 			 */
@@ -650,6 +697,7 @@ fragment_add_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 			 * length of the packet
 			 */
 			fd_head->datalen = fd->offset + fd->len;
+			fd_head->flags |= FD_DATALEN_SET;
 		}
 	}
 
@@ -668,16 +716,12 @@ fragment_add_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 		if (fd->offset + fd->len > fd_head->datalen) {
 			fd->flags      |= FD_TOOLONGFRAGMENT;
 			fd_head->flags |= FD_TOOLONGFRAGMENT;
-			LINK_FRAG(fd_head,fd);
-			return TRUE;
 		}
 		/* make sure it doesnt conflict with previous data */
-		if ( memcmp(fd_head->data+fd->offset,
+		else if ( memcmp(fd_head->data+fd->offset,
 			tvb_get_ptr(tvb,offset,fd->len),fd->len) ){
 			fd->flags      |= FD_OVERLAPCONFLICT;
 			fd_head->flags |= FD_OVERLAPCONFLICT;
-			LINK_FRAG(fd_head,fd);
-			return TRUE;
 		}
 		/* it was just an overlap, link it and return */
 		LINK_FRAG(fd_head,fd);
@@ -696,7 +740,7 @@ fragment_add_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 	LINK_FRAG(fd_head,fd);
 
 
-	if( !(fd_head->datalen) ){
+	if( !(fd_head->flags & FD_DATALEN_SET) ){
 		/* if we dont know the datalen, there are still missing
 		 * packets. Cheaper than the check below.
 		 */
@@ -867,9 +911,11 @@ fragment_add_common(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 	 * are in one PDU.
 	 */
 	if (!already_added && check_already_added && fd_head != NULL) {
-		for(fd_item=fd_head->next;fd_item;fd_item=fd_item->next){
-			if(pinfo->fd->num==fd_item->frame && frag_offset==fd_item->offset){
-				already_added=TRUE;
+		if (pinfo->fd->num <= fd_head->frame) {
+			for(fd_item=fd_head->next;fd_item;fd_item=fd_item->next){
+				if(pinfo->fd->num==fd_item->frame && frag_offset==fd_item->offset){
+					already_added=TRUE;
+				}
 			}
 		}
 	}
@@ -886,19 +932,7 @@ fragment_add_common(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 		/* not found, this must be the first snooped fragment for this
                  * packet. Create list-head.
 		 */
-		fd_head=g_mem_chunk_alloc(fragment_data_chunk);
-
-		/* head/first structure in list only holds no other data than
-                 * 'datalen' then we don't have to change the head of the list
-                 * even if we want to keep it sorted
-                 */
-		fd_head->next=NULL;
-		fd_head->datalen=0;
-		fd_head->offset=0;
-		fd_head->len=0;
-		fd_head->flags=0;
-		fd_head->data=NULL;
-		fd_head->reassembled_in=0;
+		fd_head = new_head(0);
 
 		/*
 		 * We're going to use the key to insert the fragment,
@@ -981,19 +1015,7 @@ fragment_add_check(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		/* not found, this must be the first snooped fragment for this
                  * packet. Create list-head.
 		 */
-		fd_head=g_mem_chunk_alloc(fragment_data_chunk);
-
-		/* head/first structure in list only holds no other data than
-                 * 'datalen' then we don't have to change the head of the list
-                 * even if we want to keep it sorted
-                 */
-		fd_head->next=NULL;
-		fd_head->datalen=0;
-		fd_head->offset=0;
-		fd_head->len=0;
-		fd_head->flags=0;
-		fd_head->data=NULL;
-		fd_head->reassembled_in=0;
+		fd_head = new_head(0);
 
 		/*
 		 * We're going to use the key to insert the fragment,
@@ -1051,6 +1073,68 @@ fragment_add_check(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	}
 }
 
+static void
+fragment_defragment_and_free (fragment_data *fd_head, fragment_data *fd,
+			      packet_info *pinfo)
+{
+	fragment_data *fd_i = NULL;
+	fragment_data *last_fd = NULL;
+	guint32  dfpos = 0, size = 0;
+	void *old_data = NULL;
+	
+	for(fd_i=fd_head->next;fd_i;fd_i=fd_i->next) {
+		if(!last_fd || last_fd->offset!=fd_i->offset){
+			size+=fd_i->len;
+		}
+		last_fd=fd_i;
+	}
+
+	/* store old data in case the fd_i->data pointers refer to it */
+	old_data=fd_head->data;
+	fd_head->data = g_malloc(size);
+	fd_head->len = size;		/* record size for caller	*/
+
+	/* add all data fragments */
+	last_fd=NULL;
+	for (fd_i=fd_head->next;fd_i && fd_i->len + dfpos <= size;fd_i=fd_i->next) {
+		if (fd_i->len) {
+			if(!last_fd || last_fd->offset!=fd_i->offset){
+				memcpy(fd_head->data+dfpos,fd_i->data,fd_i->len);
+				dfpos += fd_i->len;
+			} else {
+				/* duplicate/retransmission/overlap */
+				fd_i->flags    |= FD_OVERLAP;
+				fd_head->flags |= FD_OVERLAP;
+				if( (last_fd->len!=fd_i->len)
+				    || memcmp(last_fd->data, fd_i->data, last_fd->len) ) {
+					if (fd) {
+						fd->flags |= FD_OVERLAPCONFLICT;
+					}
+					fd_head->flags |= FD_OVERLAPCONFLICT;
+				}
+			}
+		}
+		last_fd=fd_i;
+	}
+	
+	/* we have defragmented the pdu, now free all fragments*/
+	for (fd_i=fd_head->next;fd_i;fd_i=fd_i->next) {
+		if( fd_i->flags & FD_NOT_MALLOCED )
+			fd_i->flags &= ~FD_NOT_MALLOCED;
+		else
+			g_free(fd_i->data);
+		fd_i->data=NULL;
+	}
+	
+	if( old_data )
+		g_free(old_data);
+	
+	/* mark this packet as defragmented.
+           allows us to skip any trailing fragments */
+	fd_head->flags |= FD_DEFRAGMENTED;
+	fd_head->reassembled_in=pinfo->fd->num;
+}
+
 /*
  * This function adds a new fragment to the entry for a reassembly
  * operation.
@@ -1066,12 +1150,42 @@ fragment_add_check(tvbuff_t *tvb, int offset, packet_info *pinfo,
 static gboolean
 fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 	     packet_info *pinfo, guint32 frag_number,
-	     guint32 frag_data_len, gboolean more_frags)
+	     guint32 frag_data_len, gboolean more_frags,
+	     guint32 flags _U_)
 {
 	fragment_data *fd;
 	fragment_data *fd_i;
 	fragment_data *last_fd;
-	guint32 max, dfpos, size;
+	guint32 max, dfpos;
+
+	/* if the partial reassembly flag has been set, and we are extending
+	 * the pdu, un-reassemble the pdu. This means pointing old fds to malloc'ed data.
+	 */
+	if(fd_head->flags & FD_DEFRAGMENTED && frag_number >= fd_head->datalen &&
+		fd_head->flags & FD_PARTIAL_REASSEMBLY){
+		guint32 lastdfpos = 0;
+		dfpos = 0;
+		for(fd_i=fd_head->next; fd_i; fd_i=fd_i->next){
+			if( !fd_i->data ) {
+				if( fd_i->flags & FD_OVERLAP ) {
+					/* this is a duplicate of the previous
+					 * fragment. */
+					fd_i->data = fd_head->data + lastdfpos;
+				} else {
+					fd_i->data = fd_head->data + dfpos;
+					lastdfpos = dfpos;
+					dfpos += fd_i->len;
+				}
+				fd_i->flags |= FD_NOT_MALLOCED;
+			}
+			fd_i->flags &= (~FD_TOOLONGFRAGMENT) & (~FD_MULTIPLETAILS);
+		}
+		fd_head->flags &= ~(FD_DEFRAGMENTED|FD_PARTIAL_REASSEMBLY|FD_DATALEN_SET);
+		fd_head->flags &= (~FD_TOOLONGFRAGMENT) & (~FD_MULTIPLETAILS);
+		fd_head->datalen=0;
+		fd_head->reassembled_in=0;
+	}
+
 
 	/* create new fd describing this fragment */
 	fd = g_mem_chunk_alloc(fragment_data_chunk);
@@ -1086,7 +1200,7 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 		/*
 		 * This is the tail fragment in the sequence.
 		 */
-		if (fd_head->datalen) {
+		if (fd_head->flags&FD_DATALEN_SET) {
 			/* ok we have already seen other tails for this packet
 			 * it might be a duplicate.
 			 */
@@ -1103,6 +1217,7 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 			 * the length of the packet!)
 			 */
 			fd_head->datalen = fd->offset;
+			fd_head->flags |= FD_DATALEN_SET;
 		}
 	}
 
@@ -1209,7 +1324,7 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 	LINK_FRAG(fd_head,fd);
 
 
-	if( !(fd_head->datalen) ){
+	if( !(fd_head->flags & FD_DATALEN_SET) ){
 		/* if we dont know the sequence number of the last fragment,
 		 * there are definitely still missing packets. Cheaper than
 		 * the check below.
@@ -1220,6 +1335,7 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 
 	/* check if we have received the entire fragment
 	 * this is easy since the list is sorted and the head is faked.
+	 * common case the whole list is scanned.
 	 */
 	max = 0;
 	for(fd_i=fd_head->next;fd_i;fd_i=fd_i->next) {
@@ -1245,51 +1361,7 @@ fragment_add_seq_work(fragment_data *fd_head, tvbuff_t *tvb, int offset,
 	/* we have received an entire packet, defragment it and
          * free all fragments
          */
-	size=0;
-	last_fd=NULL;
-	for(fd_i=fd_head->next;fd_i;fd_i=fd_i->next) {
-	  if(!last_fd || last_fd->offset!=fd_i->offset){
-	    size+=fd_i->len;
-	  }
-	  last_fd=fd_i;
-	}
-	fd_head->data = g_malloc(size);
-	fd_head->len = size;		/* record size for caller	*/
-
-	/* add all data fragments */
-	dfpos = 0;
-	last_fd=NULL;
-	for (fd_i=fd_head->next;fd_i && fd_i->len + dfpos <= size;fd_i=fd_i->next) {
-	  if (fd_i->len) {
-	    if(!last_fd || last_fd->offset!=fd_i->offset){
-	      memcpy(fd_head->data+dfpos,fd_i->data,fd_i->len);
-	      dfpos += fd_i->len;
-	    } else {
-	      /* duplicate/retransmission/overlap */
-	      fd_i->flags    |= FD_OVERLAP;
-	      fd_head->flags |= FD_OVERLAP;
-	      if( (last_fd->len!=fd_i->datalen)
-		  || memcmp(last_fd->data, fd_i->data, last_fd->len) ){
-			fd->flags      |= FD_OVERLAPCONFLICT;
-			fd_head->flags |= FD_OVERLAPCONFLICT;
-	      }
-	    }
-	  }
-	  last_fd=fd_i;
-	}
-
-	/* we have defragmented the pdu, now free all fragments*/
-	for (fd_i=fd_head->next;fd_i;fd_i=fd_i->next) {
-	  if(fd_i->data){
-	    g_free(fd_i->data);
-	    fd_i->data=NULL;
-	  }
-	}
-
-	/* mark this packet as defragmented.
-           allows us to skip any trailing fragments */
-	fd_head->flags |= FD_DEFRAGMENTED;
-	fd_head->reassembled_in=pinfo->fd->num;
+	fragment_defragment_and_free(fd_head, fd, pinfo);
 
 	return TRUE;
 }
@@ -1311,15 +1383,50 @@ fragment_add_seq(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 	     GHashTable *fragment_table, guint32 frag_number,
 	     guint32 frag_data_len, gboolean more_frags)
 {
-	fragment_key key, *new_key;
-	fragment_data *fd_head;
+	fragment_key key;
 
 	/* create key to search hash with */
 	key.src = pinfo->src;
 	key.dst = pinfo->dst;
 	key.id  = id;
 
-	fd_head = g_hash_table_lookup(fragment_table, &key);
+	return fragment_add_seq_key(tvb, offset, pinfo,
+				    &key, fragment_key_copy,
+				    fragment_table, frag_number,
+				    frag_data_len, more_frags, 0);
+}
+                                    
+
+fragment_data *
+fragment_add_dcerpc_dg(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
+                    void *v_act_id,
+                    GHashTable *fragment_table, guint32 frag_number,
+                    guint32 frag_data_len, gboolean more_frags)
+{
+	e_uuid_t *act_id = (e_uuid_t *)v_act_id;
+	dcerpc_fragment_key key;
+
+	/* create key to search hash with */
+	key.src = pinfo->src;
+	key.dst = pinfo->dst;
+	key.id  = id;
+	key.act_id  = *act_id;
+
+	return fragment_add_seq_key(tvb, offset, pinfo,
+				    &key, dcerpc_fragment_key_copy,
+				    fragment_table, frag_number,
+				    frag_data_len, more_frags, 0);
+}
+
+fragment_data *
+fragment_add_seq_key(tvbuff_t *tvb, int offset, packet_info *pinfo,
+                     void *key, fragment_key_copier key_copier,
+                    GHashTable *fragment_table, guint32 frag_number,
+                    guint32 frag_data_len, gboolean more_frags,
+                    guint32 flags)
+{
+	fragment_data *fd_head;
+	fd_head = g_hash_table_lookup(fragment_table, key);
 
 	/* have we already seen this frame ?*/
 	if (pinfo->fd->flags.visited) {
@@ -1334,35 +1441,91 @@ fragment_add_seq(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 		/* not found, this must be the first snooped fragment for this
                  * packet. Create list-head.
 		 */
-		fd_head=g_mem_chunk_alloc(fragment_data_chunk);
-
-		/* head/first structure in list only holds no other data than
-                 * 'datalen' then we don't have to change the head of the list
-                 * even if we want to keep it sorted
-                 */
-		fd_head->next=NULL;
-		fd_head->datalen=0;
-		fd_head->offset=0;
-		fd_head->len=0;
-		fd_head->flags=FD_BLOCKSEQUENCE;
-		fd_head->data=NULL;
-		fd_head->reassembled_in=0;
+		fd_head= new_head(FD_BLOCKSEQUENCE);
 
+		if((flags & (REASSEMBLE_FLAGS_NO_FRAG_NUMBER|REASSEMBLE_FLAGS_802_11_HACK))
+		   && !more_frags) {
+			/*
+			 * This is the last fragment for this packet, and
+			 * is the only one we've seen.
+			 *
+			 * Either we don't have sequence numbers, in which
+			 * case we assume this is the first fragment for
+			 * this packet, or we're doing special 802.11
+			 * processing, in which case we assume it's one
+			 * of those reassembled packets with a non-zero
+			 * fragment number (see packet-80211.c); just
+			 * return a pointer to the head of the list;
+			 * fragment_add_seq_check will then add it to the table
+			 * of reassembled packets.
+			 */
+			fd_head->reassembled_in=pinfo->fd->num;
+			return fd_head;
+		}
+		       
 		/*
 		 * We're going to use the key to insert the fragment,
-		 * so allocate a structure for it, and copy the
-		 * addresses, allocating new buffers for the address
-		 * data.
+		 * so copy it to a long-term store.
 		 */
-		new_key = g_mem_chunk_alloc(fragment_key_chunk);
-		COPY_ADDRESS(&new_key->src, &key.src);
-		COPY_ADDRESS(&new_key->dst, &key.dst);
-		new_key->id = key.id;
-		g_hash_table_insert(fragment_table, new_key, fd_head);
+		if(key_copier != NULL)
+			key = key_copier(key);
+		g_hash_table_insert(fragment_table, key, fd_head);
+		
+		/*
+		 * If we weren't given an initial fragment number,
+		 * make it 0.
+		 */
+		if (flags & REASSEMBLE_FLAGS_NO_FRAG_NUMBER)
+			frag_number = 0;
+	} else {
+		if (flags & REASSEMBLE_FLAGS_NO_FRAG_NUMBER) {
+			fragment_data *fd;
+			/*
+			 * If we weren't given an initial fragment number,
+			 * use the next expected fragment number as the fragment
+			 * number for this fragment.
+			 */
+			for (fd = fd_head; fd != NULL; fd = fd->next) {
+				if (fd->next == NULL)
+					frag_number = fd->offset + 1;
+			}
+		}
 	}
 
+	/*
+	 * XXX I've copied this over from the old separate
+	 * fragment_add_seq_check_work, but I'm not convinced it's doing the
+	 * right thing -- rav
+	 *
+	 * If we don't have all the data that is in this fragment,
+	 * then we can't, and don't, do reassembly on it.
+	 *
+	 * If it's the first frame, handle it as an unfragmented packet.
+	 * Otherwise, just handle it as a fragment.
+	 *
+	 * If "more_frags" isn't set, we get rid of the entry in the
+	 * hash table for this reassembly, as we don't need it any more.
+	 */
+	if ((flags & REASSEMBLE_FLAGS_CHECK_DATA_PRESENT) &&
+		!tvb_bytes_exist(tvb, offset, frag_data_len)) {
+		if (!more_frags) {
+			gpointer orig_key;
+			/*
+			 * Remove this from the table of in-progress
+			 * reassemblies, and free up any memory used for
+			 * it in that table.
+			 */
+			if (g_hash_table_lookup_extended(fragment_table, key,
+							 &orig_key, NULL)) {
+				fragment_unhash(fragment_table, (fragment_key *)orig_key);
+			}
+		}
+		fd_head -> flags |= FD_DATA_NOT_PRESENT;
+		return frag_number == 0 ? fd_head : NULL;
+	}
+	
 	if (fragment_add_seq_work(fd_head, tvb, offset, pinfo,
-				  frag_number, frag_data_len, more_frags)) {
+				  frag_number, frag_data_len, more_frags, flags)) {
 		/*
 		 * Reassembly is complete.
 		 */
@@ -1375,79 +1538,6 @@ fragment_add_seq(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
 	}
 }
 
-fragment_data *
-fragment_add_dcerpc_dg(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id,
-                    void *v_act_id,
-                    GHashTable *fragment_table, guint32 frag_number,
-                    guint32 frag_data_len, gboolean more_frags)
-{
-       dcerpc_fragment_key key, *new_key;
-       fragment_data *fd_head;
-       e_uuid_t *act_id = (e_uuid_t *)v_act_id;
-
-       /* create key to search hash with */
-       key.src = pinfo->src;
-       key.dst = pinfo->dst;
-       key.id  = id;
-       key.act_id  = *act_id;
-
-       fd_head = g_hash_table_lookup(fragment_table, &key);
-
-       /* have we already seen this frame ?*/
-       if (pinfo->fd->flags.visited) {
-               if (fd_head != NULL && fd_head->flags & FD_DEFRAGMENTED) {
-                       return fd_head;
-               } else {
-                       return NULL;
-               }
-       }
-
-       if (fd_head==NULL){
-               /* not found, this must be the first snooped fragment for this
-                 * packet. Create list-head.
-                */
-               fd_head=g_mem_chunk_alloc(fragment_data_chunk);
-
-               /* head/first structure in list only holds no other data than
-                 * 'datalen' then we don't have to change the head of the list
-                 * even if we want to keep it sorted
-                 */
-               fd_head->next=NULL;
-               fd_head->datalen=0;
-               fd_head->offset=0;
-               fd_head->len=0;
-               fd_head->flags=FD_BLOCKSEQUENCE;
-               fd_head->data=NULL;
-               fd_head->reassembled_in=0;
-
-               /*
-                * We're going to use the key to insert the fragment,
-                * so allocate a structure for it, and copy the
-                * addresses, allocating new buffers for the address
-                * data.
-                */
-               new_key = se_alloc(sizeof(dcerpc_fragment_key));
-               COPY_ADDRESS(&new_key->src, &key.src);
-               COPY_ADDRESS(&new_key->dst, &key.dst);
-               new_key->id = key.id;
-               new_key->act_id = key.act_id;
-               g_hash_table_insert(fragment_table, new_key, fd_head);
-       }
-
-       if (fragment_add_seq_work(fd_head, tvb, offset, pinfo,
-                                 frag_number, frag_data_len, more_frags)) {
-               /*
-                * Reassembly is complete.
-                */
-               return fd_head;
-       } else {
-               /*
-                * Reassembly isn't complete.
-                */
-               return NULL;
-       }
-}
-
 /*
  * This does the work for "fragment_add_seq_check()" and
  * "fragment_add_seq_next()".
@@ -1463,31 +1553,21 @@ fragment_add_dcerpc_dg(tvbuff_t *tvb, int offset, packet_info *pinfo, guint32 id
  * the fragment number.
  *
  * If this is the first fragment seen for this datagram, a new
- * "fragment_data" structure is allocated to refer to the reassembled,
- * packet, and:
+ * "fragment_data" structure is allocated to refer to the reassembled
+ * packet.
  *
- *	if "more_frags" is false and "frag_802_11_hack" is true, the
- *	structure is not added to the hash table, and not given any
- *	fragments to refer to, but is just returned - this is a special
- *	hack for the 802.11 dissector (see packet-80211.c);
+ * This fragment is added to the linked list of fragments for this packet.
  *
- *	otherwise, this fragment is added to the linked list of fragments
- *	for this packet, and the "fragment_data" structure is put into
- *	the hash table.
- *
- * Otherwise, this fragment is just added to the linked list of fragments
- * for this packet.
+ * If "more_frags" is false and REASSEMBLE_FLAGS_802_11_HACK (as the name
+ * implies, a special hack for 802.11) or REASSEMBLE_FLAGS_NO_FRAG_NUMBER
+ * (implying messages must be in order since there's no sequence number) are
+ * set in "flags", then this (one element) list is returned.
  *
  * If, after processing this fragment, we have all the fragments,
  * "fragment_add_seq_check_work()" removes that from the fragment hash
  * table if necessary and adds it to the table of reassembled fragments,
  * and returns a pointer to the head of the fragment list.
  *
- * If this is the first fragment we've seen, and "more_frags" is false
- * and "frag_802_11_hack" is true, "fragment_add_seq_check_work()" does
- * nothing to the fragment data list, and returns a pointer to the head
- * of that (empty) list.
- *
  * Otherwise, it returns NULL.
  *
  * XXX - Should we simply return NULL for zero-length fragments?
@@ -1497,12 +1577,11 @@ fragment_add_seq_check_work(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	     guint32 id, GHashTable *fragment_table,
 	     GHashTable *reassembled_table, guint32 frag_number,
 	     guint32 frag_data_len, gboolean more_frags,
-	     gboolean no_frag_number, gboolean frag_802_11_hack)
+	     guint32 flags)
 {
 	reassembled_key reass_key;
-	fragment_key key, *new_key, *old_key;
-	gpointer orig_key, value;
-	fragment_data *fd_head, *fd;
+	fragment_key key;
+	fragment_data *fd_head;
 
 	/*
 	 * Have we already seen this frame?
@@ -1519,122 +1598,34 @@ fragment_add_seq_check_work(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	key.dst = pinfo->dst;
 	key.id  = id;
 
-	if (!g_hash_table_lookup_extended(fragment_table, &key,
-					  &orig_key, &value)) {
-		/* not found, this must be the first snooped fragment for this
-                 * packet. Create list-head.
-		 */
-		fd_head=g_mem_chunk_alloc(fragment_data_chunk);
+	fd_head = fragment_add_seq_key(tvb, offset, pinfo,
+				       &key, fragment_key_copy,
+				       fragment_table, frag_number,
+				       frag_data_len, more_frags, flags|REASSEMBLE_FLAGS_CHECK_DATA_PRESENT);
+	if (fd_head) {
+		gpointer orig_key;
 
-		/* head/first structure in list only holds no other data than
-                 * 'datalen' then we don't have to change the head of the list
-                 * even if we want to keep it sorted
-                 */
-		fd_head->next=NULL;
-		fd_head->datalen=0;
-		fd_head->offset=0;
-		fd_head->len=0;
-		fd_head->flags=FD_BLOCKSEQUENCE;
-		fd_head->data=NULL;
-		fd_head->reassembled_in=0;
-
-		if ((no_frag_number || frag_802_11_hack) && !more_frags) {
-			/*
-			 * This is the last fragment for this packet, and
-			 * is the only one we've seen.
-			 *
-			 * Either we don't have sequence numbers, in which
-			 * case we assume this is the first fragment for
-			 * this packet, or we're doing special 802.11
-			 * processing, in which case we assume it's one
-			 * of those reassembled packets with a non-zero
-			 * fragment number (see packet-80211.c); just
-			 * add the fragment to the table of reassembled
-			 * packets, and return a pointer to the head of
-			 * the list.
-			 */
-			fragment_reassembled(fd_head, pinfo,
-			       reassembled_table, id);
+		if(fd_head->flags & FD_DATA_NOT_PRESENT) {
+			/* this is the first fragment of a datagram with
+			 * truncated fragments. Don't move it to the
+			 * reassembled table. */
 			return fd_head;
 		}
-
-		/*
-		 * We're going to use the key to insert the fragment,
-		 * so allocate a structure for it, and copy the
-		 * addresses, allocating new buffers for the address
-		 * data.
-		 */
-		new_key = g_mem_chunk_alloc(fragment_key_chunk);
-		COPY_ADDRESS(&new_key->src, &key.src);
-		COPY_ADDRESS(&new_key->dst, &key.dst);
-		new_key->id = key.id;
-		g_hash_table_insert(fragment_table, new_key, fd_head);
-
-		orig_key = new_key;	/* for unhashing it later */
-
-		/*
-		 * If we weren't given an initial fragment number,
-		 * make it 0.
-		 */
-		if (no_frag_number)
-			frag_number = 0;
-	} else {
-		/*
-		 * We found it.
-		 */
-		fd_head = value;
-
-		/*
-		 * If we weren't given an initial fragment number,
-		 * use the next expected fragment number as the fragment
-		 * number for this fragment.
-		 */
-		if (no_frag_number) {
-			for (fd = fd_head; fd != NULL; fd = fd->next) {
-				if (fd->next == NULL)
-					frag_number = fd->offset + 1;
-			}
-		}
-	}
-
-	/*
-	 * If we don't have all the data that is in this fragment,
-	 * then we can't, and don't, do reassembly on it.
-	 *
-	 * If it's the first frame, handle it as an unfragmented packet.
-	 * Otherwise, just handle it as a fragment.
-	 *
-	 * If "more_frags" isn't set, we get rid of the entry in the
-	 * hash table for this reassembly, as we don't need it any more.
-	 */
-	if (!tvb_bytes_exist(tvb, offset, frag_data_len)) {
-		if (!more_frags) {
-			/*
-			 * Remove this from the table of in-progress
-			 * reassemblies, and free up any memory used for
-			 * it in that table.
-			 */
-			old_key = orig_key;
-			fragment_unhash(fragment_table, old_key);
-		}
-		return frag_number == 0 ? fd_head : NULL;
-	}
-
-	if (fragment_add_seq_work(fd_head, tvb, offset, pinfo,
-				  frag_number, frag_data_len, more_frags)) {
+		
 		/*
 		 * Reassembly is complete.
 		 * Remove this from the table of in-progress
 		 * reassemblies, add it to the table of
 		 * reassembled packets, and return it.
 		 */
-
-		/*
-		 * Remove this from the table of in-progress reassemblies,
-		 * and free up any memory used for it in that table.
-		 */
-		old_key = orig_key;
-		fragment_unhash(fragment_table, old_key);
+		if (g_hash_table_lookup_extended(fragment_table, &key,
+						 &orig_key, NULL)) {
+			/*
+			 * Remove this from the table of in-progress reassemblies,
+			 * and free up any memory used for it in that table.
+			 */
+			fragment_unhash(fragment_table, (fragment_key *)orig_key);
+		}
 
 		/*
 		 * Add this item to the table of reassembled packets.
@@ -1657,7 +1648,7 @@ fragment_add_seq_check(tvbuff_t *tvb, int offset, packet_info *pinfo,
 {
 	return fragment_add_seq_check_work(tvb, offset, pinfo, id,
 	    fragment_table, reassembled_table, frag_number, frag_data_len,
-	    more_frags, FALSE, FALSE);
+	    more_frags, 0);
 }
 
 fragment_data *
@@ -1668,7 +1659,7 @@ fragment_add_seq_802_11(tvbuff_t *tvb, int offset, packet_info *pinfo,
 {
 	return fragment_add_seq_check_work(tvb, offset, pinfo, id,
 	    fragment_table, reassembled_table, frag_number, frag_data_len,
-	    more_frags, FALSE, TRUE);
+	    more_frags, REASSEMBLE_FLAGS_802_11_HACK);
 }
 
 fragment_data *
@@ -1679,9 +1670,122 @@ fragment_add_seq_next(tvbuff_t *tvb, int offset, packet_info *pinfo,
 {
 	return fragment_add_seq_check_work(tvb, offset, pinfo, id,
 	    fragment_table, reassembled_table, 0, frag_data_len,
-	    more_frags, TRUE, FALSE);
+	    more_frags, REASSEMBLE_FLAGS_NO_FRAG_NUMBER);
 }
 
+void
+fragment_start_seq_check(packet_info *pinfo, guint32 id, GHashTable *fragment_table,
+			 guint32 tot_len)
+{
+	fragment_key key, *new_key;
+	fragment_data *fd_head;
+
+	/* Have we already seen this frame ?*/
+	if (pinfo->fd->flags.visited) {
+		return;
+	}
+
+	/* Create key to search hash with */
+	key.src = pinfo->src;
+	key.dst = pinfo->dst;
+	key.id  = id;
+
+	/* Check if fragment data exist for this key */
+	fd_head = g_hash_table_lookup(fragment_table, &key);
+
+	if (fd_head == NULL) {
+		/* Create list-head. */
+		fd_head = g_mem_chunk_alloc(fragment_data_chunk);
+	  
+		fd_head->next = NULL;
+		fd_head->datalen = tot_len;
+		fd_head->offset = 0;
+		fd_head->len = 0;
+		fd_head->flags = FD_BLOCKSEQUENCE|FD_DATALEN_SET;	  
+		fd_head->data = NULL;
+		fd_head->reassembled_in = 0;
+		/*
+		 * We're going to use the key to insert the fragment,
+		 * so copy it to a long-term store.
+		 */
+		new_key = fragment_key_copy(&key);
+		g_hash_table_insert(fragment_table, new_key, fd_head);
+	}
+}
+
+fragment_data *
+fragment_end_seq_next(packet_info *pinfo, guint32 id, GHashTable *fragment_table,
+		      GHashTable *reassembled_table)
+{
+	reassembled_key reass_key;
+	reassembled_key *new_key;
+	fragment_key key;
+	fragment_data *fd_head;
+
+	/*
+	 * Have we already seen this frame?
+	 * If so, look for it in the table of reassembled packets.
+	 */
+	if (pinfo->fd->flags.visited) {
+		reass_key.frame = pinfo->fd->num;
+		reass_key.id = id;
+		return g_hash_table_lookup(reassembled_table, &reass_key);
+	}
+
+	/* create key to search hash with */
+	key.src = pinfo->src;
+	key.dst = pinfo->dst;
+	key.id  = id;
+
+	fd_head = g_hash_table_lookup (fragment_table, &key);
+
+	if (fd_head) {
+		gpointer orig_key;
+		
+		if (fd_head->flags & FD_DATA_NOT_PRESENT) {
+			/* No data added */
+			return NULL;
+		}
+
+		fd_head->datalen = fd_head->offset;
+		fd_head->flags |= FD_DATALEN_SET;
+		
+		fragment_defragment_and_free (fd_head, NULL, pinfo);
+
+		/*
+		 * Remove this from the table of in-progress
+		 * reassemblies, add it to the table of
+		 * reassembled packets, and return it.
+		 */
+		if (g_hash_table_lookup_extended(fragment_table, &key,
+						 &orig_key, NULL)) {
+			/*
+			 * Remove this from the table of in-progress reassemblies,
+			 * and free up any memory used for it in that table.
+			 */
+			fragment_unhash(fragment_table, (fragment_key *)orig_key);
+		}
+
+		/*
+		 * Add this item to the table of reassembled packets.
+		 */
+		fragment_reassembled(fd_head, pinfo, reassembled_table, id);
+		if (fd_head->next != NULL) {
+			new_key = se_alloc(sizeof(reassembled_key));
+			new_key->frame = pinfo->fd->num;
+			new_key->id = id;
+			g_hash_table_insert(reassembled_table, new_key, fd_head);
+		}
+
+		return fd_head;	
+	} else {
+		/*
+		 * Fragment data not found.
+		 */
+		return NULL;
+	}
+}
+		      
 /*
  * Process reassembled data; if we're on the frame in which the data
  * was reassembled, put the fragment information into the protocol
@@ -1935,3 +2039,11 @@ show_fragment_seq_tree(fragment_data *fd_head, const fragment_items *fit,
 
 	return show_fragment_errs_in_col(fd_head, fit, pinfo);
 }
+
+/*
+ * Local Variables:
+ * c-basic-offset: 8
+ * indent-tabs-mode: t
+ * tab-width: 8
+ * End:
+ */
