@@ -1,7 +1,7 @@
 /* tap.c
  * packet tap interface   2002 Ronnie Sahlberg
  *
- * $Id: tap.c 20106 2006-12-11 02:58:38Z guy $
+ * $Id: tap.c 24368 2008-02-17 00:35:55Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -268,7 +268,7 @@ tap_push_tapped_queue(epan_dissect_t *edt)
  * use "filters" and should specify the "filter" as NULL when registering
  * the tap listener.
  */
-void * 
+const void * 
 fetch_tapped_data(int tap_id, int idx)
 {
 	tap_packet_t *tp;
@@ -411,6 +411,53 @@ register_tap_listener(const char *tapname, void *tapdata, const char *fstring, t
 	return NULL;
 }
 
+/* this function sets a new dfilter to a tap listener
+ */
+GString *
+set_tap_dfilter(void *tapdata, const char *fstring)
+{
+	tap_listener_t *tl=NULL,*tl2;
+	GString *error_string;
+
+	if(!tap_listener_queue){
+		return NULL;
+	}
+
+	if(tap_listener_queue->tapdata==tapdata){
+		tl=(tap_listener_t *)tap_listener_queue;
+	} else {
+		for(tl2=(tap_listener_t *)tap_listener_queue;tl2->next;tl2=tl2->next){
+			if(tl2->next->tapdata==tapdata){
+				tl=tl2->next;
+				break;
+			}
+			
+		}
+	}
+
+	if(tl){
+		if(tl->code){
+			dfilter_free(tl->code);
+			num_tap_filters--;
+			tl->code=NULL;
+		}
+		tl->needs_redraw=1;
+		if(fstring){
+			if(!dfilter_compile(fstring, &tl->code)){
+				error_string = g_string_new("");
+				g_string_sprintf(error_string,
+						 "Filter \"%s\" is invalid - %s",
+						 fstring, dfilter_error_msg);
+				return error_string;
+			} else {
+				num_tap_filters++;
+			}
+		}
+	}
+
+	return NULL;
+}
+
 /* this function removes a tap listener
  */
 void
@@ -460,3 +507,19 @@ have_tap_listeners(void)
 {
 	return tap_listener_queue != NULL;
 }
+
+/* Returns TRUE there is an active tap listener for the specified tap id. */
+gboolean
+have_tap_listener(int tap_id)
+{
+	volatile tap_listener_t *tap_queue = tap_listener_queue;
+
+	while(tap_queue) {
+		if(tap_queue->tap_id == tap_id)
+			return TRUE;
+
+		tap_queue = tap_queue->next;
+	}
+
+	return FALSE;
+}
