@@ -9,7 +9,7 @@
  * 		the data of a backing tvbuff, or can be a composite of
  * 		other tvbuffs.
  *
- * $Id: tvbuff.c 23901 2007-12-17 20:42:10Z gerald $
+ * $Id: tvbuff.c 24559 2008-03-04 22:56:26Z stig $
  *
  * Copyright (c) 2000 by Gilbert Ramirez <gram@alumni.rice.edu>
  *
@@ -1523,9 +1523,14 @@ tvb_get_bits8(tvbuff_t *tvb, gint bit_offset, gint no_of_bits)
 	 */
 	bit_offset = bit_offset & 0x7;
 	tot_no_bits = bit_offset+no_of_bits;
-	/* Read two octets, mask off bit_offset bits and left shift out the unused bits */
-	value = tvb_get_ntohs(tvb,offset) & bit_mask16[bit_offset];
-	value = value >> (16 - tot_no_bits);
+	if(tot_no_bits<=8){
+		value = tvb_get_guint8(tvb,offset) & bit_mask8[bit_offset];
+		value = value >> (8-tot_no_bits);
+	}else{
+		/* Read two octets, mask off bit_offset bits and left shift out the unused bits */
+		value = tvb_get_ntohs(tvb,offset) & bit_mask16[bit_offset];
+		value = value >> (16 - tot_no_bits);
+	}
 
 	return (guint8)value;
 
@@ -2918,6 +2923,7 @@ tvb_uncompress(tvbuff_t *tvb, int offset, int comprlen)
 			strm->next_in = next;
 			strm->avail_in = comprlen;
 
+			inflateEnd(strm);
 			memset(strmbuf, '\0', bufsiz);
 			strm->next_out = strmbuf;
 			strm->avail_out = bufsiz;
