@@ -1,7 +1,7 @@
 /* unicode-utils.c
  * Unicode utility routines
  *
- * $Id: unicode-utils.c 20122 2006-12-12 02:24:51Z wmeier $
+ * $Id: unicode-utils.c 22829 2007-09-09 13:05:32Z ulfl $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -125,6 +125,4 @@ gchar * utf_16to8(const wchar_t *utf16str) {
   return utf8buf[idx];
 }
 
-#else
-	;
-#endif
+#endif	/* _WIN32 */
