@@ -5,7 +5,7 @@
  *
  * (c) 2006, Luis E. Garcia Ontanon <luis.ontanon@gmail.com>
  *
- * $Id: wslua_field.c 19992 2006-11-26 18:31:53Z wmeier $
+ * $Id: wslua_field.c 22373 2007-07-21 18:23:08Z lego $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -68,11 +68,13 @@ WSLUA_METAMETHOD FieldInfo__call(lua_State* L) {
 		case FT_UINT24:
 		case FT_UINT32:
 		case FT_FRAMENUM:
+			lua_pushnumber(L,(lua_Number)fvalue_get_uinteger(&(fi->value)));
+			return 1;
 		case FT_INT8:
 		case FT_INT16:
 		case FT_INT24:
 		case FT_INT32:
-			lua_pushnumber(L,(lua_Number)fvalue_get_integer(&(fi->value)));
+			lua_pushnumber(L,(lua_Number)fvalue_get_sinteger(&(fi->value)));
 			return 1;
 		case FT_FLOAT:
 		case FT_DOUBLE:
@@ -148,7 +150,7 @@ WSLUA_METAMETHOD FieldInfo__tostring(lua_State* L) {
 	return 1;
 }
 
-WSLUA_ATTR_GET FieldInfo_get_range(lua_State* L) {
+int FieldInfo_get_range(lua_State* L) {
 	/* the TvbRange covering this field */
 	FieldInfo fi = checkFieldInfo(L,1);
 	TvbRange r = ep_alloc(sizeof(struct _wslua_tvbrange));
@@ -161,14 +163,14 @@ WSLUA_ATTR_GET FieldInfo_get_range(lua_State* L) {
 	return 1;
 }
 
-WSLUA_ATTR_GET FieldInfo_get_generated(lua_State* L) {
+int FieldInfo_get_generated(lua_State* L) {
 	/* Whether this field was marked as generated. */
 	FieldInfo fi = checkFieldInfo(L,1);
 	lua_pushboolean(L,FI_GET_FLAG(fi, FI_GENERATED));
 	return 1;
 }
 
-WSLUA_ATTR_GET FieldInfo_get_name(lua_State* L) {
+int FieldInfo_get_name(lua_State* L) {
 	/* the filter name of this field. */
 	FieldInfo fi = checkFieldInfo(L,1);
 	lua_pushstring(L,fi->hfinfo->abbrev);
@@ -180,10 +182,16 @@ static const luaL_reg FieldInfo_get[] = {
     {"range", FieldInfo_get_range},
 /*    {"hidden", FieldInfo_get_hidden}, */
     {"generated", FieldInfo_get_generated},
+	
+	/* WSLUA_ATTRIBUTE FieldInfo_name RO The name of this field */
     {"name", FieldInfo_get_name},
+	/* WSLUA_ATTRIBUTE FieldInfo_label RO The string representing this field */
     {"label", FieldInfo__tostring},
+	/* WSLUA_ATTRIBUTE FieldInfo_value RO The value of this field */
     {"value", FieldInfo__call},
+	/* WSLUA_ATTRIBUTE FieldInfo_len RO The length of this field */
     {"len", FieldInfo__len},
+	/* WSLUA_ATTRIBUTE FieldInfo_offset RO The offset of this field */
     {"offset", FieldInfo__unm},
     {0, 0}
 };
@@ -275,10 +283,16 @@ int FieldInfo_register(lua_State* L) {
 
 WSLUA_FUNCTION wslua_all_field_infos(lua_State* L) {
 	/* obtain all fields from the current tree */
-	GPtrArray* found = lua_tree->tree ? proto_all_finfos(lua_tree->tree) : NULL;
+	GPtrArray* found;
 	int items_found = 0;
 	guint i;
 
+	if (! lua_tree || ! lua_tree->tree ) {
+		WSLUA_ERROR(wslua_all_field_infos,"Cannot be called outside a listener or dissector");
+	}
+	
+	found = proto_all_finfos(lua_tree->tree);
+	
 	if (found) {
 		for (i=0; i<found->len; i++) {
 			pushFieldInfo(L,g_ptr_array_index(found,i));
