@@ -5,7 +5,7 @@
  *
  * (c) 2006, Luis E. Garcia Ontanon <luis.ontanon@gmail.com>
  *
- * $Id: wslua_tree.c 19992 2006-11-26 18:31:53Z wmeier $
+ * $Id: wslua_tree.c 24433 2008-02-22 21:02:10Z morriss $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -119,12 +119,20 @@ static int TreeItem_add_item_any(lua_State *L, gboolean little_endian) {
                 case FT_STRINGZ:
                     item = proto_tree_add_string(tree_item->tree,hfid,tvbr->tvb,tvbr->offset,tvbr->len,luaL_checkstring(L,1));
                     break;
-                case FT_UINT64:
-                case FT_INT64:
-                case FT_ETHER:
                 case FT_BYTES:
-                case FT_UINT_BYTES:
+                    item = proto_tree_add_bytes(tree_item->tree,hfid,tvbr->tvb,tvbr->offset,tvbr->len, (const guint8*) luaL_checkstring(L,1));
+                    break;
+                case FT_UINT64:
+                    item = proto_tree_add_uint64(tree_item->tree,hfid,tvbr->tvb,tvbr->offset,tvbr->len,(guint64)luaL_checknumber(L,1));
+                    break;
+                case FT_INT64:
+                    item = proto_tree_add_int64(tree_item->tree,hfid,tvbr->tvb,tvbr->offset,tvbr->len,(gint64)luaL_checknumber(L,1));
+                    break;
                 case FT_IPv4:
+                    item = proto_tree_add_ipv4(tree_item->tree,hfid,tvbr->tvb,tvbr->offset,tvbr->len,*((guint32*)(checkAddress(L,1)->data)));
+                    break;
+                case FT_ETHER:
+                case FT_UINT_BYTES:
                 case FT_IPv6:
                 case FT_IPXNET:
                 case FT_GUID:
