@@ -1,6 +1,6 @@
 /* about_dlg.c
  *
- * $Id: about_dlg.c 20525 2007-01-22 22:55:08Z gerald $
+ * $Id: about_dlg.c 24089 2008-01-14 16:40:23Z stig $
  *
  * Ulf Lamping <ulf.lamping@web.de>
  *
@@ -28,15 +28,20 @@
 #endif
 
 #include <gtk/gtk.h>
+#include <string.h>
 
 #include <epan/filesystem.h>
 #include <epan/plugins.h>
+#ifdef HAVE_LIBSMI
+#include <epan/oids.h>
+#endif
 #include "about_dlg.h"
 #include "gui_utils.h"
 #include "dlg_utils.h"
 #include "file_dlg.h"
 #include "compat_macros.h"
 #include "globals.h"
+#include "log.h"
 #if GTK_MAJOR_VERSION >= 2 || GTK_MINOR_VERSION >= 3
 #include "text_page.h"
 #endif
@@ -60,12 +65,13 @@ static GtkWidget *about_wireshark_w;
 
 
 static void
-about_wireshark(GtkWidget *parent, GtkWidget *main_vb, const char *title)
+about_wireshark(GtkWidget *parent, GtkWidget *main_vb)
 {
   GtkWidget   *msg_label, *icon;
 #if GTK_MAJOR_VERSION >= 2
   gchar       *message;
 #endif
+  const char  *title = "Network Protocol Analyzer";
 
   icon = xpm_to_widget_from_parent(parent, wssplash_xpm);
   gtk_container_add(GTK_CONTAINER(main_vb), icon);
@@ -79,14 +85,31 @@ about_wireshark(GtkWidget *parent, GtkWidget *main_vb, const char *title)
   gtk_container_add(GTK_CONTAINER(main_vb), msg_label);
 }
 
+static void
+splash_update_label(GtkWidget *win, const char *message)
+{
+    GtkWidget *main_lb;
+
+    if (win == NULL) return;
+
+    main_lb = OBJECT_GET_DATA(win, "splash_label");
+    gtk_label_set_text(GTK_LABEL(main_lb), message);
+
+    /* Process all pending GUI events before continuing, so that
+       the splash screen window gets updated. */
+    while (gtk_events_pending()) gtk_main_iteration();
+}
 
 GtkWidget*
-splash_new(char *message)
+splash_new(const char *message)
 {
     GtkWidget *win;
     GtkWidget *main_lb;
 
     GtkWidget *main_vb;
+    GtkWidget *percentage_hb;
+    GtkWidget *prog_bar;
+    GtkWidget *percentage_lb;
 
     win = splash_window_new();
 
@@ -100,32 +123,152 @@ splash_new(char *message)
     gtk_container_border_width(GTK_CONTAINER(main_vb), 24);
     gtk_container_add(GTK_CONTAINER(win), main_vb);
 
-    about_wireshark(win, main_vb, "Network Protocol Analyzer");
+    about_wireshark(win, main_vb);
 
     main_lb = gtk_label_new(message);
     gtk_container_add(GTK_CONTAINER(main_vb), main_lb);
     OBJECT_SET_DATA(win, "splash_label", main_lb);
 
+    main_lb = gtk_label_new("");
+    gtk_container_add(GTK_CONTAINER(main_vb), main_lb);
+    OBJECT_SET_DATA(win, "protocol_label", main_lb);
+
+    percentage_hb = gtk_hbox_new(FALSE, 1);
+    gtk_box_pack_start(GTK_BOX(main_vb), percentage_hb, TRUE, TRUE, 3);
+
+    prog_bar = gtk_progress_bar_new();
+#if GTK_MAJOR_VERSION < 2
+    gtk_progress_set_activity_mode(GTK_PROGRESS(prog_bar), FALSE);
+#endif
+    gtk_box_pack_start(GTK_BOX(percentage_hb), prog_bar, TRUE, TRUE, 3);
+    OBJECT_SET_DATA(win, "progress_bar", prog_bar);
+
+    percentage_lb = gtk_label_new("  0%");
+    gtk_misc_set_alignment(GTK_MISC(percentage_lb), 0.0, 0.0);
+    gtk_box_pack_start(GTK_BOX(percentage_hb), percentage_lb, FALSE, TRUE, 3);
+    OBJECT_SET_DATA(win, "percentage_label", percentage_lb);
+
     gtk_widget_show_all(win);
 
-    splash_update(win, message);
+    splash_update_label(win, message);
 
     return win;
 }
 
+#define REGISTER_FREQ 100 /* Milliseconds */
+
 void
-splash_update(GtkWidget *win, char *message)
+splash_update(register_action_e action, const char *message, gpointer client_data)
 {
-    GtkWidget *main_lb;
+    GtkWidget  *win;
+    GtkWidget  *main_lb;
+    GtkWidget  *prog_bar;
+    GtkWidget  *percentage_lb;
+    gfloat     percentage;
+    gulong     ul_percentage;
+    gchar      tmp[100];
+    const char *action_msg;
+
+    static gulong ul_sofar = 0;
+    static gulong ul_count = 0;
+
+    static register_action_e last_action = RA_NONE;
+
+    static GTimeVal cur_tv;
+    static GTimeVal next_tv = {0, 0};
+
+    win = (GtkWidget *)client_data;
 
     if (win == NULL) return;
 
-    main_lb = OBJECT_GET_DATA(win, "splash_label");
-    gtk_label_set_text(GTK_LABEL(main_lb), message);
+    g_get_current_time(&cur_tv);
+    if (cur_tv.tv_sec <= next_tv.tv_sec && cur_tv.tv_usec <= next_tv.tv_usec && ul_sofar < ul_count - 1) {
+      /* Only update every REGISTER_FREQ milliseconds */
+      ul_sofar++;
+      return;
+    }
+    memcpy(&next_tv, &cur_tv, sizeof(next_tv));
+    next_tv.tv_usec += REGISTER_FREQ * 1000;
+    if (next_tv.tv_usec >= 1000000) {
+        next_tv.tv_sec++;
+        next_tv.tv_usec -= 1000000;
+    }
+
+    if(last_action != action) {
+      /* the action has changed */
+      switch(action) {
+      case RA_DISSECTORS:
+	action_msg = "Initializing dissectors ...";
+	break;
+      case RA_LISTENERS:
+	action_msg = "Initializing tap listeners ...";
+	break;
+      case RA_REGISTER:
+	action_msg = "Registering dissector ...";
+	break;
+      case RA_PLUGIN_REGISTER:
+	action_msg = "Registering plugins ...";
+	break;
+      case RA_HANDOFF:
+	action_msg = "Handing off dissector ...";
+	break;
+      case RA_PLUGIN_HANDOFF:
+	action_msg = "Handing off plugins ...";
+	break;
+      case RA_PREFERENCES:
+	action_msg = "Loading module preferences ...";
+	break;
+      case RA_CONFIGURATION:
+	action_msg = "Loading configuration files ...";
+	break;
+      default:
+	action_msg = "(Unknown action)";;
+	break;
+      }
+      splash_update_label(win, action_msg);
+      last_action = action;
+    }
+
+    if(ul_count == 0) /* get the count of dissectors */
+      ul_count = register_count() + 6; /* additional 6 for:
+					  dissectors, listeners,
+					  registering plugins, handingoff plugins,
+					  preferences and configuration */
+
+    main_lb = OBJECT_GET_DATA(win, "protocol_label");
+    /* make_dissector_reg.py changed -
+       so we need to strip off the leading elements to get back to the protocol */
+    if(message) {
+      if(!strncmp(message, "proto_register_", 15))
+	message += 15;
+      else if(!strncmp(message, "proto_reg_handoff_", 18))
+	message += 18;
+    }
+    gtk_label_set_text(GTK_LABEL(main_lb), message ? message : "");
+
+    ul_sofar++;
+
+    g_assert (ul_sofar <= ul_count);
+
+    percentage = (gfloat)ul_sofar/(gfloat)ul_count;
+    ul_percentage = (gulong)(percentage * 100);
+
+    /* update progress bar */
+    prog_bar = OBJECT_GET_DATA(win, "progress_bar");
+#if GTK_MAJOR_VERSION < 2
+    gtk_progress_bar_update(GTK_PROGRESS_BAR(prog_bar), percentage);
+#else
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(prog_bar), percentage);
+#endif
+
+    percentage_lb = OBJECT_GET_DATA(win, "percentage_label");
+    g_snprintf(tmp, sizeof(tmp), "%lu%%", ul_percentage);
+    gtk_label_set_text((GtkLabel*)percentage_lb, tmp);
 
     /* Process all pending GUI events before continuing, so that
        the splash screen window gets updated. */
     while (gtk_events_pending()) gtk_main_iteration();
+
 }
 
 guint
@@ -143,12 +286,11 @@ about_wireshark_page_new(void)
 {
   GtkWidget   *main_vb, *msg_label /*, *icon*/;
   gchar       *message;
-  const char   title[] = "Network Protocol Analyzer";
 
   main_vb = gtk_vbox_new(FALSE, 6);
   gtk_container_border_width(GTK_CONTAINER(main_vb), 12);
 
-  about_wireshark(top_level, main_vb, title);
+  about_wireshark(top_level, main_vb);
 
   /* Construct the message string */
   message = g_strdup_printf(
@@ -163,7 +305,7 @@ about_wireshark_page_new(void)
        "Wireshark is Open Source Software released under the GNU General Public License.\n"
        "\n"
        "Check the man page and http://www.wireshark.org for more information.",
-       svnversion, get_copyright_info(), comp_info_str->str,
+       wireshark_svnversion, get_copyright_info(), comp_info_str->str,
        runtime_info_str->str);
 
   msg_label = gtk_label_new(message);
@@ -227,7 +369,7 @@ about_folders_page_new(void)
   g_free((void *) path);
 
   /* pers conf */
-  path = get_persconffile_path("", FALSE);
+  path = get_persconffile_path("", FALSE, FALSE);
   about_folders_row(table, "Personal configuration", path,
       "\"dfilters\", \"preferences\", \"ethers\", ...");
   g_free((void *) path);
@@ -261,6 +403,14 @@ about_folders_page_new(void)
       "dissector plugins");
 #endif
 
+#ifdef HAVE_LIBSMI
+  /* SMI MIBs/PIBs */
+  path = oid_get_default_mib_path();
+  about_folders_row(table, "MIB/PIB paths", path,
+      "SMI MIB/PIB search path");
+  g_free((void *) path);
+#endif
+
   gtk_container_add(GTK_CONTAINER(scrolledwindow), table);
 
   return scrolledwindow;
@@ -284,8 +434,12 @@ void
 about_wireshark_cb( GtkWidget *w _U_, gpointer data _U_ )
 {
   GtkWidget   *main_vb, *main_nb, *bbox, *ok_btn;
+  GtkWidget   *page_lb, *about_page, *folders_page;
+
+#ifdef HAVE_PLUGINS
+  GtkWidget   *plugins_page;
+#endif
 
-  GtkWidget   *page_lb, *about_page, *folders_page, *plugins_page;
 #if GTK_MAJOR_VERSION >= 2 || GTK_MINOR_VERSION >= 3
   GtkWidget   *authors_page, *license_page;
 #endif
