@@ -1,7 +1,7 @@
 /* capture_if_details_dlg.c
  * Routines for capture interface details window (only Win32!)
  *
- * $Id: capture_if_details_dlg.c 20624 2007-01-30 17:14:37Z gerald $
+ * $Id: capture_if_details_dlg.c 24504 2008-02-29 13:33:37Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -52,10 +52,14 @@
 #include "gtkvumeter.h"
 #endif
 
-#ifndef HAVE_SOCKADDR_STORAGE
-/* packet32.h requires sockaddr_storage (usually defined in Platform SDK)
+/* packet32.h requires sockaddr_storage 
+ * wether sockaddr_storage is defined or not depends on the Platform SDK 
+ * version installed. The only one not defining it is the SDK that comes
+ * with MSVC 6.0 (WINVER 0x0400).
+ *
  * copied from RFC2553 (and slightly modified because of datatypes) ...
  * XXX - defined more than once, move this to a header file */
+#if (WINVER <= 0x0400) && defined(_MSC_VER)
 typedef unsigned short eth_sa_family_t;
 
 /*
@@ -71,13 +75,6 @@ typedef unsigned short eth_sa_family_t;
 #define ETH_SS_PAD2SIZE   (ETH_SS_MAXSIZE - (sizeof (eth_sa_family_t) + \
                               ETH_SS_PAD1SIZE + ETH_SS_ALIGNSIZE))
 
-/* sockaddr_storage problem with different MSVC versions
- * - MSVC 6   (1200) doesn't define this
- * - MSVC 7   (1300) does
- * - MSVC 7.1 (1310) does
- * - MSVC 8   (1400) does */
-/* we might need to tweak this #if, see version_info for _MSC_VER values */
-#if _MSC_VER < 1300
 struct sockaddr_storage {
     eth_sa_family_t  __ss_family;     /* address family */
     /* Following fields are implementation specific */
@@ -93,9 +90,7 @@ struct sockaddr_storage {
               /* __ss_pad1, __ss_align fields is 112 */
 };
 /* ... copied from RFC2553 */
-#endif /* _MSC_VER */
-#endif
-
+#endif /* WINVER */
 
 #include <Packet32.h>
 #include <windows.h>
@@ -1171,9 +1166,9 @@ capture_if_details_802_11_bssid_list(GtkWidget *main_vb, struct ndis_bssid_list
             /* Vendor */
             manuf_name = get_manuf_name_if_known(mac);
             if(manuf_name != NULL) {
-                strcpy(vendor_buff, manuf_name);
+                g_strlcpy(vendor_buff, manuf_name, DETAILS_STR_MAX);
             } else {
-                strcpy(vendor_buff, "");
+                vendor_buff[0] = '\0';
             }
 
             /* Supported Rates */
