@@ -1,7 +1,7 @@
 /* filter_dlg.c
  * Dialog boxes for (display and capture) filter editing
  *
- * $Id: filter_dlg.c 19418 2006-10-03 21:03:08Z ulfl $
+ * $Id: filter_dlg.c 24315 2008-02-13 01:18:40Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -293,23 +293,25 @@ get_filter_dialog_list(filter_list_type_t list_type)
 }
 
 
-static void
-fill_list(GtkWidget  *main_w, filter_list_type_t list_type)
+#if GTK_MAJOR_VERSION < 2
+static GtkWidget *
+#else
+static GtkTreeIter *
+#endif
+fill_list(GtkWidget  *main_w, filter_list_type_t list_type, const gchar *filter_te_str)
 {
     GList      *fl_entry;
     filter_def *filt;
-    const gchar *filter_te_str = NULL;
 #if GTK_MAJOR_VERSION < 2
     GtkWidget  *nl_item,
                *nl_lb,
                *l_select = NULL;
     GtkWidget  *filter_l = OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY);
 #else
-    gboolean           l_select = FALSE;
-    GtkTreeView        *filter_l;
-    GtkListStore       *store;
-    GtkTreeIter        iter;
-    GtkTreeIter        sel_iter;
+    GtkTreeView       *filter_l;
+    GtkListStore      *store;
+    GtkTreeIter       iter;
+    GtkTreeIter       *l_select = NULL;
 
     filter_l = GTK_TREE_VIEW(OBJECT_GET_DATA(main_w, E_FILT_FILTER_L_KEY));
     store = GTK_LIST_STORE(gtk_tree_view_get_model(filter_l));
@@ -341,17 +343,22 @@ fill_list(GtkWidget  *main_w, filter_list_type_t list_type)
 
         if (filter_te_str && filt->strval) {
             if (strcmp(filter_te_str, filt->strval) == 0) {
-#if GTK_MAJOR_VERSION < 2 
+#if GTK_MAJOR_VERSION < 2
                 l_select = nl_item;
 #else
-                sel_iter = iter;
-                l_select = TRUE;
+                /*
+                 * XXX - We're assuming that we can just copy a GtkTreeIter
+                 * and use it later without any crashes.  This may not be a
+                 * valid assumption.
+                 */
+                l_select = g_memdup(&iter, sizeof(iter));
 #endif
             }
         }
 
         fl_entry = fl_entry->next;
     }
+    return l_select;
 }
 
 #if 0
@@ -400,6 +407,7 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
                *filter_fr,
                *edit_fr,
                *props_fr;
+    GdkWindow  *parent;
     GtkTooltips *tooltips;
     static filter_list_type_t cfilter_list_type = CFILTER_EDITED_LIST;
     static filter_list_type_t dfilter_list_type = DFILTER_EDITED_LIST;
@@ -409,12 +417,11 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 #if GTK_MAJOR_VERSION < 2
     GtkWidget   *l_select = NULL;
 #else
-    gboolean           l_select = FALSE;
     GtkListStore      *store;
     GtkCellRenderer   *renderer;
     GtkTreeViewColumn *column;
     GtkTreeSelection  *sel;
-    GtkTreeIter        sel_iter;
+    GtkTreeIter       *l_select;
 #endif
 
     /* Get a pointer to a static variable holding the type of filter on
@@ -447,12 +454,6 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 	gtk_window_set_default_size(GTK_WINDOW(main_w), 400, 400);
     OBJECT_SET_DATA(main_w, E_FILT_CONSTRUCT_ARGS_KEY, construct_args);
 
-    if(construct_args->modal_and_transient) {
-        GdkWindow*  parent = gtk_widget_get_parent_window(parent_filter_te);
-        gtk_window_set_transient_for(GTK_WINDOW(main_w), GTK_WINDOW(parent));
-        gtk_window_set_modal(GTK_WINDOW(main_w), TRUE);
-    }
-
     main_vb = gtk_vbox_new(FALSE, 0);
     gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
     gtk_container_add(GTK_CONTAINER(main_w), main_vb);
@@ -489,7 +490,7 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 #endif
     gtk_widget_show(new_bt);
     gtk_box_pack_start (GTK_BOX (list_bb), new_bt, FALSE, FALSE, 0);
-    gtk_tooltips_set_tip (tooltips, new_bt, 
+    gtk_tooltips_set_tip (tooltips, new_bt,
         "Create a new filter at the end of the list (with the current properties)", NULL);
 
     del_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_DELETE);
@@ -509,7 +510,7 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 
     filter_sc = scrolled_window_new(NULL, NULL);
 #if GTK_MAJOR_VERSION >= 2
-    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(filter_sc), 
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(filter_sc),
                                    GTK_SHADOW_IN);
 #endif
 
@@ -554,9 +555,9 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
                     construct_args->activate_on_ok ? "" : NULL);
 
     /* fill in data */
-    fill_list(main_w, list_type);
+    l_select = fill_list(main_w, list_type, filter_te_str);
 
-#if GTK_MAJOR_VERSION >= 2 
+#if GTK_MAJOR_VERSION >= 2
     g_object_unref(G_OBJECT(store));
 #endif
 
@@ -603,9 +604,9 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     OBJECT_SET_DATA(main_w, E_FILT_PARENT_FILTER_TE_KEY, parent_filter_te);
 
     if (list_type == DFILTER_EDITED_LIST) {
-        gtk_tooltips_set_tip(tooltips, filter_te, 
+        gtk_tooltips_set_tip(tooltips, filter_te,
             "Enter a display filter. "
-            "The background color of this field is changed by a continuous syntax check (green is valid, red is invalid).", 
+            "The background color of this field is changed by a continuous syntax check (green is valid, red is invalid, yellow may have unexpected results).",
             NULL);
 
         /* Create the "Add Expression..." button, to pop up a dialog
@@ -648,6 +649,7 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
     cancel_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_CANCEL);
     gtk_tooltips_set_tip (tooltips, cancel_bt, ("Cancel the changes"), NULL);
     SIGNAL_CONNECT(cancel_bt, "clicked", filter_dlg_cancel_cb, filter_list_type_p);
+    window_set_cancel_button(main_w, cancel_bt, NULL);
 
     help_bt = OBJECT_GET_DATA(bbox, GTK_STOCK_HELP);
     if (list_type == CFILTER_EDITED_LIST) {
@@ -675,27 +677,29 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 
     /* DO SELECTION THINGS *AFTER* SHOWING THE DIALOG! */
     /* otherwise the updatings can get confused */
-#if GTK_MAJOR_VERSION < 2 
     if (l_select) {
+#if GTK_MAJOR_VERSION < 2
         gtk_list_select_child(GTK_LIST(filter_l), l_select);
-    } else if (filter_te_str && filter_te_str[0]) {
-        gtk_entry_set_text(GTK_ENTRY(name_te), "New filter");
-        gtk_entry_set_text(GTK_ENTRY(filter_te), filter_te_str);
-    }
 #else
-    if (l_select) {
-        gtk_tree_selection_select_iter(sel, &sel_iter);        
+        gtk_tree_selection_select_iter(sel, l_select);
+        g_free(l_select);
+#endif
     } else if (filter_te_str && filter_te_str[0]) {
         gtk_entry_set_text(GTK_ENTRY(name_te), "New filter");
         gtk_entry_set_text(GTK_ENTRY(filter_te), filter_te_str);
     }
-#endif
 
     SIGNAL_CONNECT(main_w, "delete_event", filter_dlg_delete_event_cb, filter_list_type_p);
     SIGNAL_CONNECT(main_w, "destroy", filter_dlg_destroy_cb, filter_list_type_p);
 
     gtk_widget_show(main_w);
 
+    if(construct_args->modal_and_transient) {
+        parent = gtk_widget_get_parent_window(parent_filter_te);
+        gdk_window_set_transient_for(main_w->window, parent);
+        gtk_window_set_modal(GTK_WINDOW(main_w), TRUE);
+    }
+
     /* hide the Ok button, if we don't have to apply it and our caller wants a Save button */
     if (parent_filter_te == NULL && prefs.gui_use_pref_save) {
         gtk_widget_hide(ok_bt);
@@ -772,7 +776,7 @@ filter_dlg_dclick(GtkWidget *filter_l, gpointer main_w_arg, gpointer activate)
                     /*
                      * Yes - do so.
                      */
-                    SIGNAL_EMIT_BY_NAME(parent_filter_te, "activate", NULL);
+                    SIGNAL_EMIT_BY_NAME(SIGNAL_EMIT_OBJECT(parent_filter_te), "activate", NULL);
                 }
             }
         }
@@ -864,7 +868,7 @@ filter_apply(GtkWidget *main_w, gboolean destroy)
 		 * to destroy it.
 		 */
 		if (construct_args->activate_on_ok) {
-			SIGNAL_EMIT_BY_NAME(parent_filter_te, "activate", NULL);
+			SIGNAL_EMIT_BY_NAME(SIGNAL_EMIT_OBJECT(parent_filter_te), "activate", NULL);
 		}
 	}
 }
@@ -936,7 +940,7 @@ static void filter_dlg_update_list_cb(gpointer data, gpointer user_data)
 
     /* refill the list */
     clear_list(main_w);
-    fill_list(main_w, list_type);
+    fill_list(main_w, list_type, NULL);
 }
 #endif
 
@@ -1426,10 +1430,25 @@ void
 filter_add_expr_bt_cb(GtkWidget *w _U_, gpointer main_w_arg)
 {
 	GtkWidget  *main_w = GTK_WIDGET(main_w_arg);
-	GtkWidget  *filter_te;
+	GtkWidget  *filter_te, *dfilter_w;
 
 	filter_te = OBJECT_GET_DATA(main_w, E_FILT_FILTER_TE_KEY);
-	dfilter_expr_dlg_new(filter_te);
+	dfilter_w = dfilter_expr_dlg_new(filter_te);
+
+#if GTK_MAJOR_VERSION >= 2
+	/* If we're opening a series of modal dialogs (such as when going
+	 * through file->open, make the latest dialog modal also so that it
+	 * takes over "control" from the other modal dialogs.  Also set
+	 * the transient property of the new dialog so the user doesn't try
+	 * to interact with the previous window when they can't.
+         * XXX: containing widget might be the Filter Toolbar */
+
+	if ( GTK_IS_WINDOW(main_w) && gtk_window_get_modal(GTK_WINDOW(main_w))) {
+		gtk_window_set_modal(GTK_WINDOW(dfilter_w), TRUE);
+		gtk_window_set_transient_for(GTK_WINDOW(dfilter_w),
+					     GTK_WINDOW(main_w));
+	}
+#endif
 }
 
 static void
@@ -1463,6 +1482,13 @@ colorize_filter_te_as_invalid(GtkWidget *w)
     color_filter_te(w, 0xFFFF, 0xAFFF, 0xAFFF);
 }
 
+void
+colorize_filter_te_as_deprecated(GtkWidget *w)
+{
+    /* light yellow */
+    color_filter_te(w, 0xFFFF, 0xFFFF, 0xAFFF);
+}
+
 void
 colorize_filter_te_as_valid(GtkWidget *w)
 {
@@ -1475,19 +1501,39 @@ filter_te_syntax_check_cb(GtkWidget *w)
 {
     const gchar *strval;
     dfilter_t   *dfp;
+    GPtrArray   *depr = NULL;
+    gchar       *msg;
+
+    statusbar_pop_filter_msg();
 
     strval = gtk_entry_get_text(GTK_ENTRY(w));
 
     /* colorize filter string entry */
     if (strval && dfilter_compile(strval, &dfp)) {
-    	if (dfp != NULL)
-    	  dfilter_free(dfp);
-        if (strlen(strval) == 0)
+    	if (dfp != NULL) {
+          depr = dfilter_deprecated_tokens(dfp);
+          dfilter_free(dfp);
+        }
+        if (strlen(strval) == 0) {
             colorize_filter_te_as_empty(w);
-        else
+        } else if (depr) {
+            /* You keep using that word. I do not think it means what you think it means. */
+            colorize_filter_te_as_deprecated(w);
+            /*
+             * We're being lazy and only printing the first "problem" token.
+             * Would it be better to print all of them?
+             */
+            msg = g_strdup_printf(" \"%s\" may have unexpected results (see the User's Guide)",
+                (const char *) g_ptr_array_index(depr, 0));
+            statusbar_push_filter_msg(msg);
+            g_free(msg);
+        } else {
             colorize_filter_te_as_valid(w);
-    } else
+        }
+    } else {
         colorize_filter_te_as_invalid(w);
+        statusbar_push_filter_msg(" Invalid filter");
+    }
 }
 
 
