@@ -1,7 +1,7 @@
 /* follow_tcp.c
  * TCP specific routines for following traffic streams
  *
- * $Id: follow_tcp.c 23605 2007-11-26 18:35:37Z morriss $
+ * $Id: follow_tcp.c 24745 2008-03-28 15:32:15Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -71,14 +71,6 @@
 
 #include "follow_stream.h"
 
-/* This is backwards-compatibility code for old versions of GTK+ (2.2.1 and
- * earlier).  It defines the new wrap behavior (unknown in earlier versions)
- * as the old (slightly buggy) wrap behavior.
- */
-#ifndef GTK_WRAP_WORD_CHAR
-#define GTK_WRAP_WORD_CHAR GTK_WRAP_WORD
-#endif
-
 /* With MSVC and a libwireshark.dll, we need a special declaration. */
 WS_VAR_IMPORT FILE *data_out_file;
 
@@ -262,7 +254,7 @@ follow_tcp_stream_cb(GtkWidget * w, gpointer data _U_)
 	port1 = get_tcp_port(stats.port[1]);
 
 	/* Host 0 --> Host 1 */
-	if(sc.src_port == strtol(port0, NULL, 10)) {
+	if(sc.src_port == stats.port[0]) {
 		server_to_client_string =
 			g_strdup_printf("%s:%s --> %s:%s (%u bytes)",
 					hostname0, port0,
@@ -277,18 +269,17 @@ follow_tcp_stream_cb(GtkWidget * w, gpointer data _U_)
 	}
 
 	/* Host 1 --> Host 0 */
-	if(sc.src_port == strtol(port0, NULL, 10)) {
+	if(sc.src_port == stats.port[1]) {
 		client_to_server_string =
 			g_strdup_printf("%s:%s --> %s:%s (%u bytes)",
-					hostname1, port1,
 					hostname0, port0,
+					hostname1, port1,
 					stats.bytes_written[1]);
-
 	} else {
 		client_to_server_string =
 			g_strdup_printf("%s:%s --> %s:%s (%u bytes)",
-					hostname0, port0,
 					hostname1, port1,
+					hostname0, port0,
 					stats.bytes_written[1]);
 	}
 
@@ -335,6 +326,8 @@ follow_read_tcp_stream(follow_info_t *follow_info,
     guint16		client_port = 0;
     gboolean		is_server;
     guint32		global_client_pos = 0, global_server_pos = 0;
+    guint32		server_packet_count = 0;
+    guint32		client_packet_count = 0;
     guint32		*global_pos;
     gboolean		skip;
     char                buffer[FLT_BUF_SIZE+1]; /* +1 to fix ws bug 1043 */
@@ -393,8 +386,9 @@ follow_read_tcp_stream(follow_info_t *follow_info,
 
 	    if (!skip) {
 		    frs_return = follow_show(follow_info, print_line, buffer,
-					     nchars, is_server, arg,
-					     global_pos);
+					     nchars, is_server, arg, global_pos, 
+					     &server_packet_count,
+					     &client_packet_count);
 		    if(frs_return == FRS_PRINT_ERROR) {
 			    fclose(data_out_file);
 			    data_out_file = NULL;
