@@ -1,7 +1,7 @@
 /* graph_analysis.c
  * Graphic Analysis addition for Wireshark
  *
- * $Id: graph_analysis.c 19499 2006-10-11 23:25:51Z gerald $
+ * $Id: graph_analysis.c 23770 2007-12-05 13:32:09Z stig $
  *
  * Copyright 2004, Verso Technologies Inc.
  * By Alejandro Vaquero <alejandrovaquero@yahoo.com>
@@ -32,8 +32,6 @@
 #include "config.h"
 #endif
 
-#include "graph_analysis.h"
-
 #include <epan/epan_dissect.h>
 
 #include "util.h"
@@ -70,6 +68,8 @@
 
 #include "simple_dialog.h"
 
+#include "graph_analysis.h"
+
 
 /* XXX FIXME */
 /* GTK-1.x does not have GDK_IS_DRAWABLE so just define is as always being true
@@ -170,6 +170,7 @@ static void graph_analysis_init_dlg(graph_analysis_data_t* user_data)
 	user_data->dlg.left_x_border=0;
 	user_data->dlg.selected_item=0xFFFFFFFF;    /*not item selected */
 	user_data->dlg.window=NULL;
+	user_data->dlg.parent_w=NULL;
 	user_data->dlg.inverse = FALSE;
 	user_data->dlg.title=NULL;
 }
@@ -244,7 +245,12 @@ static void enlarge_string(GString *gstr, guint32 length, char pad){
 
 static void overwrite (GString *gstr, char *text_to_insert, guint32 p1, guint32 p2){
 
-	guint32 pos, len;
+	guint32 len;
+#if GTK_MAJOR_VERSION < 2
+	gint pos;
+#else
+	gsize pos;
+#endif
 
 	if (p1 == p2)
 		return;
@@ -262,10 +268,12 @@ static void overwrite (GString *gstr, char *text_to_insert, guint32 p1, guint32
 		len = strlen(text_to_insert);
 	}
 
-	if (pos > gstr->len)
+	/* ouch this is ugly but gtk1 needs it */
+	if ((guint32)pos > (guint32)gstr->len)
 		pos = gstr->len;
 
-	if (pos + len > gstr->len)
+	/* ouch this is ugly but gtk1 needs it */
+	if ((guint32)(pos + len) > (guint32)gstr->len)
 		g_string_truncate(gstr, pos);
 	else
 		g_string_erase(gstr, pos, len);
@@ -286,7 +294,7 @@ static gboolean dialog_graph_dump_to_file(graph_analysis_data_t* user_data)
 	guint32 start_position, end_position, item_width, header_length;
         guint32 current_item;
 	graph_analysis_item_t *gai;
-	guint16 first_conv_num;
+	guint16 first_conv_num = 0;
 	gboolean several_convs = FALSE;
 	gboolean first_packet = TRUE;
 
@@ -564,7 +572,7 @@ on_save_bt_clicked                    (GtkButton       *button _U_,
 		return;
 	}
 
-	save_to_file_w = gtk_file_selection_new("Wireshark: Save graph to file");
+	save_to_file_w = gtk_file_selection_new("Wireshark: Save graph to plain text file");
 
 	/* Container for each row of widgets */
 	vertb = gtk_vbox_new(FALSE, 0);
@@ -617,7 +625,7 @@ static void dialog_graph_draw(graph_analysis_data_t* user_data)
         PangoLayout  *big_layout;
         PangoLayout  *small_layout;
 #endif
-        guint32 label_width, label_height;
+        gint label_width, label_height;
         guint32 draw_width, draw_height;
         char label_string[MAX_COMMENT];
 	GList* list;
@@ -989,7 +997,7 @@ static void dialog_graph_draw(graph_analysis_data_t* user_data)
 			label_x = arrow_width/2+end_arrow;
 		}
 
-		if (label_width>arrow_width) arrow_width = label_width;
+		if (label_width>(gint)arrow_width) arrow_width = label_width;
 
 		if ((int)left_x_border > ((int)label_x-(int)label_width/2))
 			label_x = left_x_border + label_width/2;
@@ -1150,6 +1158,7 @@ static gint button_press_event(GtkWidget *widget, GdkEventButton *event _U_)
 
 	/* get the item clicked */
 	item = ((guint32)event->y - TOP_Y_BORDER) / ITEM_HEIGHT;
+	if (item >= user_data->num_items) return TRUE;
 	user_data->dlg.selected_item = item + user_data->dlg.first_item;
 
 	user_data->dlg.needs_redraw=TRUE;
@@ -1609,12 +1618,17 @@ static void dialog_graph_create_window(graph_analysis_data_t* user_data)
     	GtkWidget *bt_close;
     	GtkWidget *bt_save;
 	GtkTooltips *tooltips = gtk_tooltips_new();
+	const gchar *title_name_ptr;
+	gchar   *win_name;
+
+	title_name_ptr = cf_get_display_name(&cfile);
+	win_name = g_strdup_printf("%s - Graph Analysis", title_name_ptr);
 
         /* create the main window */
 		if (user_data->dlg.title)
 	        user_data->dlg.window=window_new(GTK_WINDOW_TOPLEVEL, user_data->dlg.title);
 		else
-	        user_data->dlg.window=window_new(GTK_WINDOW_TOPLEVEL, "Graph Analysis");
+	        user_data->dlg.window=window_new(GTK_WINDOW_TOPLEVEL, win_name);
 
 
         vbox=gtk_vbox_new(FALSE, 0);
@@ -1648,102 +1662,64 @@ static void dialog_graph_create_window(graph_analysis_data_t* user_data)
 
         gtk_widget_show(user_data->dlg.window);
         window_present(user_data->dlg.window);
+
+	/* Destroy our graph window with our parent if the caller specified the parent */
+	if(user_data->dlg.parent_w) {
+		gtk_window_set_transient_for(GTK_WINDOW(user_data->dlg.window),
+					     GTK_WINDOW(user_data->dlg.parent_w));
+#if GTK_MAJOR_VERSION >= 2
+		/* Destruction of this child window in GTK1 is handled by a call
+		 * to destroy_window() in flow_graph.c. */
+		gtk_window_set_destroy_with_parent(GTK_WINDOW(user_data->dlg.window), TRUE);
+#endif
+	}
+	g_free(win_name);
 }
 
 /* Return the index array if the node is in the array. Return -1 if there is room in the array
  * and Return -2 if the array is full
  */
 /****************************************************************************/
-static gint is_node_array(graph_analysis_data_t* user_data, address* node)
-{
-	int i;
-	for (i=0; i<MAX_NUM_NODES; i++){
-		if (user_data->nodes[i].type == AT_NONE)	return -1;	/* it is not in the array */
-		if (ADDRESSES_EQUAL((&user_data->nodes[i]),node)) return i;	/* it is in the array */
+static gint add_or_get_node(graph_analysis_data_t* user_data, address* node) {
+	guint i;
+	
+	if (node->type == AT_NONE) return NODE_OVERFLOW;
+		
+	for (i=0; i<MAX_NUM_NODES && i < user_data->num_nodes ; i++){
+		if ( CMP_ADDRESS(&(user_data->nodes[i]), node) == 0 ) return i;	/* it is in the array */
+	}
+	
+	if (i == MAX_NUM_NODES) {
+		return  NODE_OVERFLOW;
+	} else {
+		user_data->num_nodes++;
+		COPY_ADDRESS(&(user_data->nodes[i]),node);
+		return i;
 	}
-	return -2;		/* array full */
 }
 
-
 /* Get the nodes from the list */
 /****************************************************************************/
 static void get_nodes(graph_analysis_data_t* user_data)
 {
 	GList* list;
 	graph_analysis_item_t *gai;
-	gint index;
 
 	/* fill the node array */
 	list = g_list_first(user_data->graph_info->list);
 	while (list)
 	{
 		gai = list->data;
-		if (gai->display){
+		if (gai->display) {
 			user_data->num_items++;
 			if (!user_data->dlg.inverse) {
-				/* check source node address */
-				index = is_node_array(user_data, &(gai->src_addr));
-				switch(index){
-					case -2: /* array full */
-						gai->src_node = NODE_OVERFLOW;
-						break;
-					case -1: /* not in array */
-						COPY_ADDRESS(&(user_data->nodes[user_data->num_nodes]),&(gai->src_addr));
-						gai->src_node = user_data->num_nodes;
-						user_data->num_nodes++;
-						break;
-					default: /* it is in the array, just update the src_node */
-						gai->src_node = (guint16)index;
-				}
-
-				/* check destination node address*/
-				index = is_node_array(user_data, &(gai->dst_addr));
-				switch(index){
-					case -2: /* array full */
-						gai->dst_node = NODE_OVERFLOW;
-						break;
-					case -1: /* not in array */
-						COPY_ADDRESS(&(user_data->nodes[user_data->num_nodes]),&(gai->dst_addr));
-						gai->dst_node = user_data->num_nodes;
-						user_data->num_nodes++;
-						break;
-					default: /* it is in the array, just update the dst_node */
-						gai->dst_node = (guint16)index;
-				}
+				gai->src_node = (guint16)add_or_get_node(user_data, &(gai->src_addr));
+				gai->dst_node = (guint16)add_or_get_node(user_data, &(gai->dst_addr));
 			} else {
-				/* check destination node address*/
-				index = is_node_array(user_data, &(gai->dst_addr));
-				switch(index){
-					case -2: /* array full */
-						gai->dst_node = NODE_OVERFLOW;
-						break;
-					case -1: /* not in array */
-						COPY_ADDRESS(&(user_data->nodes[user_data->num_nodes]),&(gai->dst_addr));
-						gai->dst_node = user_data->num_nodes;
-						user_data->num_nodes++;
-						break;
-					default: /* it is in the array, just update the dst_node */
-						gai->dst_node = (guint16)index;
-				}
-
-				/* check source node address */
-				index = is_node_array(user_data, &(gai->src_addr));
-				switch(index){
-					case -2: /* array full */
-						gai->src_node = NODE_OVERFLOW;
-						break;
-					case -1: /* not in array */
-						COPY_ADDRESS(&(user_data->nodes[user_data->num_nodes]),&(gai->src_addr));
-						gai->src_node = user_data->num_nodes;
-						user_data->num_nodes++;
-						break;
-					default: /* it is in the array, just update the src_node */
-						gai->src_node = (guint16)index;
-				}
-
+				gai->dst_node = (guint16)add_or_get_node(user_data, &(gai->src_addr));
+				gai->src_node = (guint16)add_or_get_node(user_data, &(gai->dst_addr));
 			}
 		}
-
 		list = g_list_next(list);
 	}
 }
