@@ -4,6 +4,8 @@
  * By Jakob Bratkovic <j.bratkovic@iskratel.si> and
  * Miha Jemec <m.jemec@iskratel.si>
  *
+ * $Id: mcast_stream_dlg.c 24513 2008-03-01 10:23:57Z tuexen $
+ *
  * based on rtp_stream_dlg.c
  * Copyright 2003, Alcatel Business Systems
  * By Lars Ruoff <lars.ruoff@gmx.net>
@@ -53,6 +55,7 @@
 #include <string.h>
 #include <locale.h>
 #include <epan/addr_resolv.h>
+#include <epan/strutil.h>
 
 /* Capture callback data keys */
 #define E_MCAST_ENTRY_1     "burst_interval"
@@ -187,10 +190,10 @@ mcaststream_on_filter                    (GtkButton       *button _U_,
 	if (selected_stream_fwd)
 	{
 		if (selected_stream_fwd->src_addr.type==AT_IPv6){
-			strncpy(ip_version,"v6",3);
+			g_strlcpy(ip_version,"v6",3);
 		}		
 		else{
-			strncpy(ip_version,"",3);
+			ip_version[0] = '\0';
 		}
 		filter_string_fwd = g_strdup_printf(
 			"(ip%s.src==%s && udp.srcport==%u && ip%s.dst==%s && udp.dstport==%u)",
