@@ -1,7 +1,7 @@
 /* proto_draw.c
  * Routines for GTK+ packet display
  *
- * $Id: proto_draw.c 23720 2007-12-03 21:57:57Z guy $
+ * $Id: proto_draw.c 24625 2008-03-14 06:15:06Z sfisher $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1820,8 +1820,7 @@ static void tree_cell_renderer(GtkTreeViewColumn *tree_column _U_,
     /* for each field, we have to reset the renderer attributes */
     g_object_set (cell, "foreground-set", FALSE, NULL);
 
-    g_object_set (cell, "background", "white", NULL);
-    g_object_set (cell, "background-set", TRUE, NULL);
+    g_object_set (cell, "background-set", FALSE, NULL);
 
     g_object_set (cell, "underline", PANGO_UNDERLINE_NONE, NULL);
     g_object_set (cell, "underline-set", FALSE, NULL);
