@@ -1,6 +1,6 @@
  /* rtp_player.c
  *
- * $Id: rtp_player.c 19687 2006-10-25 19:48:54Z guy $
+ * $Id: rtp_player.c 24112 2008-01-16 07:36:34Z jake $
  *
  *  Copyright 2006, Alejandro Vaquero <alejandrovaquero@yahoo.com>
  *
@@ -56,12 +56,14 @@
 #endif
 
 #ifdef HAVE_LIBPORTAUDIO
+/* TODO: The RTP Player it is only supported for GTK >=2 */
+#include <gtk/gtk.h>
+#if GTK_MAJOR_VERSION >= 2
 
 #include <epan/stats_tree.h>
 #include <epan/addr_resolv.h>
 #include <string.h>
 #include <glib.h>
-#include <gtk/gtk.h>
 #include "globals.h"
 #include "portaudio.h"
 #include "simple_dialog.h"
@@ -76,11 +78,14 @@
 
 
 #include <epan/dissectors/packet-rtp.h>
+#include <epan/rtp_pt.h>
+#include <epan/codecs.h>
 
 #include "rtp_player.h"
 #include "codecs/G711a/G711adecode.h"
 #include "codecs/G711u/G711udecode.h"
 #include <math.h>
+#include <epan/prefs.h>       /* For prefs.rtp_player_max_visible and RTP_PLAYER_DEFAULT_VISIBLE */
 
 #ifndef min
 #define min(a,b) (((a)<(b))?(a):(b))
@@ -161,9 +166,6 @@ PortAudioStream *pa_stream;
 PaStream *pa_stream;
 #endif /* PORTAUDIO_API_1 */
 
-/* TODO: The RTP Player it is only supported for GTK >=2 */
-#if GTK_MAJOR_VERSION >= 2
-
 /* defines a RTP stream */
 typedef struct _rtp_stream_info {
 	address src_addr;
@@ -238,6 +240,11 @@ typedef struct _rtp_play_channles {
 /* The two RTP channles to play */
 static rtp_play_channles_t *rtp_channels = NULL;
 
+typedef struct _rtp_decoder_t {
+	codec_handle_t handle;
+	void *context;
+} rtp_decoder_t;
+
 
 /****************************************************************************/
 static void 
@@ -282,6 +289,17 @@ rtp_stream_value_destroy(gpointer rsi_arg)
 	rsi = NULL;
 }
 
+/****************************************************************************/
+static void 
+rtp_decoder_value_destroy(gpointer dec_arg)
+{
+	rtp_decoder_t *dec = dec_arg;
+
+	if (dec->handle)
+		codec_release(dec->handle, dec->context);
+	g_free(dec_arg);
+}
+
 /****************************************************************************/
 static void
 set_sensitive_check_bt(gchar *key _U_ , rtp_channel_info_t *rci, guint *stop _U_ ) 
@@ -417,7 +435,7 @@ mark_rtp_stream_to_play(gchar *key _U_ , rtp_stream_info_t *rsi, gpointer ptr _U
 		if (rsi->first_frame_number == graph_item->frame_num) {
 			rsi->call_num = graph_item->conv_num;
 			/* if it is in the graph list, then check if the voip_call is selected */
-			voip_calls_list = g_list_first(voip_calls->strinfo_list);
+			voip_calls_list = g_list_first(voip_calls->callsinfo_list);
 			while (voip_calls_list)
 			{
 				tmp_voip_call = voip_calls_list->data;
@@ -440,10 +458,13 @@ mark_rtp_stream_to_play(gchar *key _U_ , rtp_stream_info_t *rsi, gpointer ptr _U
  * Return the number of decoded bytes
  */
 static int 
-decode_rtp_packet(rtp_packet_t *rp, rtp_channel_info_t *rci, SAMPLE **out_buff)
+decode_rtp_packet(rtp_packet_t *rp, SAMPLE **out_buff, GHashTable *decoders_hash)
 {
 	unsigned int  payload_type;
+	const gchar *p;
+	rtp_decoder_t *decoder;
 	SAMPLE *tmp_buff = NULL;
+	int tmp_buff_len;
 	int decoded_bytes = 0;
 
 	if ((rp->payload_data == NULL) || (rp->info->info_payload_len == 0) ) {
@@ -451,29 +472,59 @@ decode_rtp_packet(rtp_packet_t *rp, rtp_channel_info_t *rci, SAMPLE **out_buff)
 	}
 
 	payload_type = rp->info->info_payload_type;
+
+	/* Look for registered codecs */
+	decoder = g_hash_table_lookup(decoders_hash, GUINT_TO_POINTER(payload_type));
+	if (!decoder) {  /* Put either valid or empty decoder into the hash table */
+		decoder = g_malloc(sizeof(rtp_decoder_t));
+		decoder->handle = NULL;
+		decoder->context = NULL;
+		p = match_strval(payload_type, rtp_payload_type_short_vals);
+		if (p) {
+			decoder->handle = find_codec(p);
+			if (decoder->handle)
+				decoder->context = codec_init(decoder->handle);
+		}
+		g_hash_table_insert(decoders_hash, GUINT_TO_POINTER(payload_type), decoder);
+	}
+	if (decoder->handle) {  /* Decode with registered codec */
+		tmp_buff_len = codec_decode(decoder->handle, decoder->context, rp->payload_data, rp->info->info_payload_len, NULL, NULL);
+		tmp_buff = g_malloc(tmp_buff_len);
+		decoded_bytes = codec_decode(decoder->handle, decoder->context, rp->payload_data, rp->info->info_payload_len, tmp_buff, &tmp_buff_len);
+		*out_buff = tmp_buff;
+		return decoded_bytes;
+	}
+
+	/* Try to decode with built-in codec */
+
 	switch (payload_type) {
-	case 0:	/* G711 Ulaw */
-		tmp_buff = malloc(sizeof(SAMPLE) * rp->info->info_payload_len * 1);
+
+	case PT_PCMU:	/* G.711 u-law */
+		tmp_buff = g_malloc(sizeof(SAMPLE) * rp->info->info_payload_len * 1);
 		decodeG711u(rp->payload_data, rp->info->info_payload_len,
 			  tmp_buff, &decoded_bytes);
 		break; 
-	case 8:	/* G711 Alaw */
-		tmp_buff = malloc(sizeof(SAMPLE) * rp->info->info_payload_len * 1);
+
+	case PT_PCMA:	/* G.711 A-law */
+		tmp_buff = g_malloc(sizeof(SAMPLE) * rp->info->info_payload_len * 1);
 		decodeG711a(rp->payload_data, rp->info->info_payload_len,
 			  tmp_buff, &decoded_bytes);
 		break; 
+
 #ifdef HAVE_G729_G723
-	case 18:	/* G729 */
-		tmp_buff = malloc(sizeof(SAMPLE) * rp->info->info_payload_len * 8); /* G729 8kbps => 64kbps/8kbps = 8  */
+	case PT_G729:	/* G.729 */
+		/* G729 8kbps => 64kbps/8kbps = 8  */
+		/* Compensate for possible 2 octet SID frame (G.729B) */
+		tmp_buff = g_malloc(sizeof(SAMPLE) * ((rp->info->info_payload_len + 8) / 10) * 80);
 		decodeG729(rp->payload_data, rp->info->info_payload_len,
 			  tmp_buff, &decoded_bytes);
 		break; 
-	case 4:	/* G723 */
 
+	case PT_G723:	/* G.723 */
 		if (rp->info->info_payload_len%24 == 0)	/* G723 High 6.4kbps */
-			tmp_buff = malloc(sizeof(SAMPLE) * rp->info->info_payload_len * 10); /* G723 High 64kbps/6.4kbps = 10  */	
+			tmp_buff = g_malloc(sizeof(SAMPLE) * rp->info->info_payload_len * 10); /* G723 High 64kbps/6.4kbps = 10  */	
 		else if (rp->info->info_payload_len%20 == 0)    /* G723 Low 5.3kbps */
-			tmp_buff = malloc(sizeof(SAMPLE) * rp->info->info_payload_len * 13); /* G723 High 64kbps/5.3kbps = 13  */	
+			tmp_buff = g_malloc(sizeof(SAMPLE) * rp->info->info_payload_len * 13); /* G723 High 64kbps/5.3kbps = 13  */	
 		else {
 			return 0;
 		}
@@ -481,7 +532,14 @@ decode_rtp_packet(rtp_packet_t *rp, rtp_channel_info_t *rci, SAMPLE **out_buff)
 			  tmp_buff, &decoded_bytes);
 		break;
 #endif /* HAVE_G729_G723 */
-	} 
+
+	default:
+		/*
+		 * XXX - return an error here, so the user gets told that
+		 * we don't support this codec!
+		 */
+		break;
+	}
 
 	*out_buff = tmp_buff;
 	return decoded_bytes;
@@ -517,7 +575,7 @@ decode_rtp_stream(rtp_stream_info_t *rsi, gpointer ptr _U_)
 	double arrive_time;
 	double arrive_time_prev;
 	double start_time;
-	double start_rtp_time;
+	double start_rtp_time = 0;
 	double diff;
 	double pack_period;
 	double total_time;
@@ -536,6 +594,7 @@ decode_rtp_stream(rtp_stream_info_t *rsi, gpointer ptr _U_)
 	sample_t sample;
 	guint8 status;
 	guint32 start_timestamp; 
+	GHashTable *decoders_hash = NULL;
 
 	guint32 progbar_nextstep;
 	int progbar_quantum;
@@ -570,7 +629,7 @@ decode_rtp_stream(rtp_stream_info_t *rsi, gpointer ptr _U_)
 
 	/* ..if it is not in the hash, create an entry */
 	if (rci == NULL) {
-		rci = malloc(sizeof(rtp_channel_info_t));
+		rci = g_malloc(sizeof(rtp_channel_info_t));
 		rci->call_num = rsi->call_num;
 		rci->start_time = rsi->start_time;
 		rci->end_time = rsi->start_time;		
@@ -603,6 +662,7 @@ decode_rtp_stream(rtp_stream_info_t *rsi, gpointer ptr _U_)
 	/* decode the RTP stream */
 	first = TRUE;
 	rtp_time = 0;
+	rtp_time_prev = 0;
 	decoded_bytes = 0;
 	decoded_bytes_prev = 0;
 	silence_frames = 0;
@@ -617,6 +677,7 @@ decode_rtp_stream(rtp_stream_info_t *rsi, gpointer ptr _U_)
 	mean_delay = 0;
 	variation = 0;
 	start_timestamp = 0;
+	decoders_hash = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, rtp_decoder_value_destroy);
 
 	/* we update the progress bar 100 times */
 
@@ -652,7 +713,7 @@ decode_rtp_stream(rtp_stream_info_t *rsi, gpointer ptr _U_)
 			seq = rp->info->info_seq_num - 1;
 		}
 
-		decoded_bytes = decode_rtp_packet(rp, rci, &out_buff);
+		decoded_bytes = decode_rtp_packet(rp, &out_buff, decoders_hash);
 		if (decoded_bytes == 0) {
 			seq = rp->info->info_seq_num;
 		}
@@ -675,21 +736,25 @@ decode_rtp_stream(rtp_stream_info_t *rsi, gpointer ptr _U_)
 		if (diff<0) diff = -diff;
   
 		total_time = (double)rp->arrive_offset/1000;
-		
+#if DEBUG		
 		printf("seq = %d arr = %f abs_diff = %f index = %d tim = %f ji=%d jb=%f\n",rp->info->info_seq_num, 
 			total_time, diff, rci->samples->len, ((double)rci->samples->len/8000 - total_time)*1000, 0,
 				(mean_delay + 4*variation)*1000);
 		fflush(stdout);
-
+#endif
 		/* if the jitter buffer was exceeded */	
 		if ( diff*1000 > jitter_buff ) {
+#if DEBUG
 			printf("Packet drop by jitter buffer exceeded\n");
+#endif
 			rci->drop_by_jitter_buff++;
 			status = S_DROP_BY_JITT;
 
 			/* if there was a silence period (more than two packetization period) resync the source */
 			if ( (rtp_time - rtp_time_prev) > pack_period*2 ){
+#if DEBUG
 				printf("Resync...\n");
+#endif
 
 				silence_frames = (gint32)((arrive_time - arrive_time_prev)*SAMPLE_RATE - decoded_bytes_prev/2);
 				for (i = 0; i< silence_frames; i++) {
@@ -733,6 +798,10 @@ decode_rtp_stream(rtp_stream_info_t *rsi, gpointer ptr _U_)
 
 		}
 
+		if (out_buff) {
+			g_free(out_buff);
+			out_buff = NULL;
+		}
 		rtp_packets_list = g_list_next (rtp_packets_list);
 		progbar_count++;
 	}
@@ -740,6 +809,7 @@ decode_rtp_stream(rtp_stream_info_t *rsi, gpointer ptr _U_)
 	rci->end_time = rci->start_time + ((double)rci->samples->len/SAMPLE_RATE)*1000;
 
 	g_string_free(key_str, TRUE);
+	g_hash_table_destroy(decoders_hash);
 }
 
 /****************************************************************************/
@@ -942,7 +1012,7 @@ draw_channel_cursor(rtp_channel_info_t *rci, guint32 start_index)
 /* Move and draw the cursor in the graph 
  */
 static gboolean 
-draw_cursors(gpointer data)
+draw_cursors(gpointer data _U_)
 {
 	if (!rtp_channels) return FALSE;
 
@@ -1011,13 +1081,12 @@ static int paCallback(   void *inputBuffer, void *outputBuffer,
                              PaTimestamp outTime, void *userData)
 {
 #else /* PORTAUDIO_API_1 */
-static int paCallback( void *inputBuffer, void *outputBuffer,
+static int paCallback( const void *inputBuffer, void *outputBuffer,
                              unsigned long framesPerBuffer,
 							 const PaStreamCallbackTimeInfo* outTime,
-							 PaStreamCallbackFlags statusFlags,
+							 PaStreamCallbackFlags statusFlags _U_,
                              void *userData)
 {
-/*	(void) statusFlags;*/
 #endif /* PORTAUDIO_API_1 */
 	rtp_play_channles_t *rpci = (rtp_play_channles_t*)userData;
 	SAMPLE *wptr = (SAMPLE*)outputBuffer;
@@ -1691,9 +1760,13 @@ decode_streams(void)
 	if (rtp_channels_hash)
 		g_hash_table_foreach( rtp_channels_hash, (GHFunc)add_channel_to_window, &counter);	
 
-	/* Resize the main scroll window to display no more than 5 channels, otherwise the scroll bar need to be used */
+	/* Resize the main scroll window to display no more than preferred (or default) max channels, scroll bar will be used if needed */
+
+        if (prefs.rtp_player_max_visible < 1 || prefs.rtp_player_max_visible > 10)
+                prefs.rtp_player_max_visible = RTP_PLAYER_DEFAULT_VISIBLE;
+
 	WIDGET_SET_SIZE(main_scrolled_window, CHANNEL_WIDTH, 
-		min(counter, 5) * (CHANNEL_HEIGHT+60));
+		min(counter, prefs.rtp_player_max_visible) * (CHANNEL_HEIGHT+60));
 
 	gtk_widget_show_all(main_scrolled_window);
 
@@ -1764,7 +1837,7 @@ rtp_player_on_destroy(GtkObject *object _U_, gpointer user_data _U_)
 
 /****************************************************************************/
 static void
-jitter_spinner_value_changed (GtkSpinButton *spinner, gpointer user_data _U_)
+jitter_spinner_value_changed (GtkSpinButton *spinner _U_, gpointer user_data _U_)
 {
 	/* set the sensitive state of the buttons (decode, play, pause, stop) */
 	bt_state(TRUE, TRUE, FALSE, FALSE);
@@ -1780,12 +1853,16 @@ rtp_player_dlg_create(void)
 	GtkWidget *bt_close;
 	GtkAdjustment *jitter_spinner_adj;
 	GtkWidget *label;
+	const gchar *title_name_ptr;
+	gchar   *win_name;
 
 	GtkTooltips *tooltips = gtk_tooltips_new();
 
 	rtp_player_dlg_w=gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	
-	gtk_window_set_title(GTK_WINDOW(rtp_player_dlg_w), "Wireshark: RTP Player");
+
+	title_name_ptr = cf_get_display_name(&cfile);
+	win_name = g_strdup_printf("%s - VoIP - RTP Player", title_name_ptr);	
+	gtk_window_set_title(GTK_WINDOW(rtp_player_dlg_w), win_name);
 	gtk_window_set_position(GTK_WINDOW(rtp_player_dlg_w), GTK_WIN_POS_NONE);
 
 	gtk_window_set_default_size(GTK_WINDOW(rtp_player_dlg_w), 400, 50);
@@ -1874,6 +1951,8 @@ rtp_player_dlg_create(void)
 
 	/* Force gtk to redraw the window before starting decoding the packet */
 	while (g_main_context_iteration(NULL, FALSE));
+
+	g_free(win_name);
 }
 
 /****************************************************************************/
