@@ -1,7 +1,7 @@
 /* rtp_stream_dlg.c
  * RTP streams summary addition for Wireshark
  *
- * $Id: rtp_stream_dlg.c 18205 2006-05-22 07:29:40Z guy $
+ * $Id: rtp_stream_dlg.c 24513 2008-03-01 10:23:57Z tuexen $
  *
  * Copyright 2003, Alcatel Business Systems
  * By Lars Ruoff <lars.ruoff@gmx.net>
@@ -30,8 +30,6 @@
 #endif
 
 #include "rtp_stream_dlg.h"
-#include "rtp_stream.h"
-#include "rtp_analysis.h"
 
 #include "globals.h"
 #include "epan/filesystem.h"
@@ -54,7 +52,10 @@
 #include <string.h>
 #include <locale.h>
 #include <epan/addr_resolv.h>
+#include <epan/strutil.h>
 
+#include "rtp_stream.h"
+#include "rtp_analysis.h"
 
 static const gchar FWD_LABEL_TEXT[] = "Select a forward stream with left mouse button";
 static const gchar REV_LABEL_TEXT[] = "Select a reverse stream with SHIFT + left mouse button";
@@ -102,9 +103,18 @@ static void add_to_clist(rtp_stream_info_t* strinfo)
 	data[1] = g_strdup_printf("%u", strinfo->src_port);
 	data[2] = g_strdup(get_addr_name(&(strinfo->dest_addr)));
 	data[3] = g_strdup_printf("%u", strinfo->dest_port);
-	data[4] = g_strdup_printf("%u", strinfo->ssrc);
-	data[5] = g_strdup(val_to_str(strinfo->pt, rtp_payload_type_vals,
-		"Unknown (%u)"));
+	data[4] = g_strdup_printf("0x%X", strinfo->ssrc);
+	if(strinfo->pt>95){
+		if(strinfo->info_payload_type_str != NULL){
+			data[5] = g_strdup(strinfo->info_payload_type_str);
+		}else{
+			data[5] = g_strdup_printf("Unknown(%u)",strinfo->pt);
+		}
+
+	}else{
+		data[5] = g_strdup(val_to_str(strinfo->pt, rtp_payload_type_vals,
+			"Unknown (%u)"));
+	}
 	data[6] = g_strdup_printf("%u", strinfo->npackets);
 
 	expected = (strinfo->rtp_stats.stop_seq_nr + strinfo->rtp_stats.cycles*65536)
@@ -135,7 +145,7 @@ static void add_to_clist(rtp_stream_info_t* strinfo)
 	gtk_clist_set_row_data(GTK_CLIST(clist), added_row, strinfo);
 
 	/* Update the top label with the number of detected streams */
-	sprintf(label_text,
+	g_snprintf(label_text, 256,
 	        "Detected %d RTP streams. Choose one for forward and reverse direction for analysis",
 	        ++streams_nb);
 	gtk_label_set(GTK_LABEL(top_label), label_text);
@@ -366,13 +376,13 @@ rtpstream_on_filter                    (GtkButton       *button _U_,
 	if (selected_stream_fwd)
 	{
 		if (selected_stream_fwd->src_addr.type==AT_IPv6){
-			strcpy(ip_version,"v6");
+			g_strlcpy(ip_version,"v6",3);
 		}		
 		else{
-			strcpy(ip_version,"");
+			ip_version[0] = '\0';
 		}
 		filter_string_fwd = g_strdup_printf(
-			"(ip%s.src==%s && udp.srcport==%u && ip%s.dst==%s && udp.dstport==%u && rtp.ssrc==%u)",
+			"(ip%s.src==%s && udp.srcport==%u && ip%s.dst==%s && udp.dstport==%u && rtp.ssrc==0x%X)",
 			ip_version,
 			address_to_str(&(selected_stream_fwd->src_addr)),
 			selected_stream_fwd->src_port,
@@ -386,13 +396,13 @@ rtpstream_on_filter                    (GtkButton       *button _U_,
 	if (selected_stream_rev)
 	{
 		if (selected_stream_fwd->src_addr.type==AT_IPv6){
-			strcpy(ip_version,"v6");
+			g_strlcpy(ip_version,"v6",3);
 		}		
 		else{
-			strcpy(ip_version,"");
+			ip_version[0] = '\0';
 		}
 		filter_string_rev = g_strdup_printf(
-			"(ip%s.src==%s && udp.srcport==%u && ip%s.dst==%s && udp.dstport==%u && rtp.ssrc==%u)",
+			"(ip%s.src==%s && udp.srcport==%u && ip%s.dst==%s && udp.dstport==%u && rtp.ssrc==0x%X)",
 			ip_version,
 			address_to_str(&(selected_stream_rev->src_addr)),
 			selected_stream_rev->src_port,
@@ -528,7 +538,7 @@ rtpstream_on_select_row(GtkCList *clist,
 	/* update the labels */
 	if (event==NULL || event->state & GDK_SHIFT_MASK) {
 		selected_stream_rev = gtk_clist_get_row_data(GTK_CLIST(clist), row);
-		g_snprintf(label_text, 80, "Reverse: %s:%u -> %s:%u, SSRC=%u",
+		g_snprintf(label_text, 80, "Reverse: %s:%u -> %s:%u, SSRC=0x%X",
 			get_addr_name(&(selected_stream_rev->src_addr)),
 			selected_stream_rev->src_port,
 			get_addr_name(&(selected_stream_rev->dest_addr)),
@@ -539,7 +549,7 @@ rtpstream_on_select_row(GtkCList *clist,
 	}
 	else {
 		selected_stream_fwd = gtk_clist_get_row_data(GTK_CLIST(clist), row);
-		g_snprintf(label_text, 80, "Forward: %s:%u -> %s:%u, SSRC=%u",
+		g_snprintf(label_text, 80, "Forward: %s:%u -> %s:%u, SSRC=0x%X",
 			get_addr_name(&(selected_stream_fwd->src_addr)),
 			selected_stream_fwd->src_port,
 			get_addr_name(&(selected_stream_fwd->dest_addr)),
@@ -686,7 +696,7 @@ static void rtpstream_dlg_create (void)
     gtk_clist_set_column_width (GTK_CLIST (clist), 4, 64);
     gtk_clist_set_column_width (GTK_CLIST (clist), 5, 96);
     gtk_clist_set_column_width (GTK_CLIST (clist), 6, 50);
-    gtk_clist_set_column_width (GTK_CLIST (clist), 7, 50);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 7, 64);
     gtk_clist_set_column_width (GTK_CLIST (clist), 8, 80);
     gtk_clist_set_column_width (GTK_CLIST (clist), 9, 80);
     gtk_clist_set_column_width (GTK_CLIST (clist), 10, 80);
