@@ -1,7 +1,7 @@
 /* 
  * Copyright 2004, Irene Ruengeler <i.ruengeler [AT] fh-muenster.de>
  *
- * $Id: sctp_assoc_analyse.c 24239 2008-02-01 01:07:58Z stig $
+ * $Id: sctp_assoc_analyse.c 24513 2008-03-01 10:23:57Z tuexen $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -39,6 +39,7 @@
 #include "gui_utils.h"
 #include "main.h"
 #include "compat_macros.h"
+#include <epan/strutil.h>
 
 #include "sctp_stat.h"
 #include "gtkglobals.h"
@@ -834,7 +835,7 @@ static void sctp_analyse_cb(struct sctp_analyse* u_data, gboolean ext)
 	int i;
 	guint32 *fn;
 
-	strncpy(filter_text,"sctp",250);
+	g_strlcpy(filter_text,"sctp",250);
 	if (!dfilter_compile(filter_text, &sfcode)) {
 		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, dfilter_error_msg);
 		return;
