@@ -1,7 +1,7 @@
 /*
  * Copyright 2004, Irene Ruengeler <i.ruengeler [AT] fh-muenster.de>
  *
- * $Id: sctp_byte_graph_dlg.c 18267 2006-05-31 15:26:41Z tuexen $
+ * $Id: sctp_byte_graph_dlg.c 24513 2008-03-01 10:23:57Z tuexen $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -41,6 +41,7 @@
 #include "compat_macros.h"
 #include "simple_dialog.h"
 #include "sctp_stat.h"
+#include <epan/strutil.h>
 
 #define DEFAULT_PIXELS_PER_TICK 2
 #define MAX_PIXELS_PER_TICK     4
@@ -311,8 +312,8 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 	}
 	else
 	{
-		u_data->io->min_x=u_data->io->x1_tmp_sec*1000000.0+u_data->io->x1_tmp_usec;
-		u_data->io->max_x=u_data->io->x2_tmp_sec*1000000.0+u_data->io->x2_tmp_usec;		
+		u_data->io->min_x=((guint32) (u_data->io->x1_tmp_sec*1000000.0))+u_data->io->x1_tmp_usec;
+		u_data->io->max_x=((guint32) (u_data->io->x2_tmp_sec*1000000.0))+u_data->io->x2_tmp_usec;		
 		u_data->io->uoff = FALSE;
 	}	
 
@@ -551,7 +552,7 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 		}
 	}
 
-	strcpy(label_string, "sec");
+	g_strlcpy(label_string, "sec", 15);
 
 #if GTK_MAJOR_VERSION < 2
 	lwidth = gdk_string_width(font, label_string);
@@ -940,12 +941,14 @@ static gint
 on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_udata *u_data)
 {
 	sctp_graph_t *ios;
-	guint32 helpx, helpy, x1_tmp, x2_tmp, y_value;
+	guint32 helpx, helpy, x1_tmp, x2_tmp, y_value, frame;
 	gint label_width, label_height;
-	gdouble x_value, position;
+	gdouble x_value, position, tfirst;
 	gint lwidth;
 	char label_string[30];
 	GdkGC *text_color;
+	GList *tsnlist=NULL;
+	tsn_t *tsn, *tmptsn;
 	#if GTK_MAJOR_VERSION < 2
 		GdkFont *font;
 #else
@@ -973,10 +976,10 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 
 	if (abs((long)(event->x-u_data->io->x_old))>10 || abs((long)(event->y-u_data->io->y_old))>10)
 	{
-		u_data->io->rect_x_min = (gint)floor(MIN(u_data->io->x_old,event->x));
-		u_data->io->rect_x_max = (gint)ceil(MAX(u_data->io->x_old,event->x));
-		u_data->io->rect_y_min = (gint)floor(MIN(u_data->io->y_old,event->y));
-		u_data->io->rect_y_max = (gint)ceil(MAX(u_data->io->y_old,event->y));
+		u_data->io->rect_x_min = (guint32) floor(MIN(u_data->io->x_old,event->x));
+		u_data->io->rect_x_max = (guint32) ceil(MAX(u_data->io->x_old,event->x));
+		u_data->io->rect_y_min = (guint32) floor(MIN(u_data->io->y_old,event->y));
+		u_data->io->rect_y_max = (guint32) ceil(MAX(u_data->io->y_old,event->y));
 		gdk_draw_rectangle(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
  		                   FALSE,
  		                   u_data->io->rect_x_min, u_data->io->rect_y_min,
@@ -996,8 +999,8 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 		                u_data->io->draw_area->allocation.width,
 		                u_data->io->draw_area->allocation.height);
 
-		x1_tmp=(unsigned int)floor(u_data->io->min_x+((u_data->io->x_old-LEFT_BORDER-u_data->io->offset)*u_data->io->tmp_width/u_data->io->axis_width));
-		x2_tmp=(unsigned int)floor(u_data->io->min_x+((event->x-LEFT_BORDER-u_data->io->offset)*u_data->io->tmp_width/u_data->io->axis_width));
+		x1_tmp=(guint32) floor(u_data->io->min_x+((u_data->io->x_old-LEFT_BORDER-u_data->io->offset)*u_data->io->tmp_width/u_data->io->axis_width));
+		x2_tmp=(guint32) floor(u_data->io->min_x+((event->x-LEFT_BORDER-u_data->io->offset)*u_data->io->tmp_width/u_data->io->axis_width));
 		helpx=MIN(x1_tmp, x2_tmp);
 		if (helpx==x2_tmp)
 		{
@@ -1059,14 +1062,47 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 		}
 		else
 		{
-			x_value = ((event->x-LEFT_BORDER-u_data->io->offset) * ((u_data->io->x2_tmp_sec+u_data->io->x2_tmp_usec/1000000.0)-(u_data->io->x1_tmp_sec+u_data->io->x1_tmp_usec/1000000.0)) / (u_data->io->pixmap_width-LEFT_BORDER-u_data->io->offset))+u_data->io->x1_tmp_sec+u_data->io->x1_tmp_usec/1000000.0;
-			y_value = floor((u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-event->y) * (max_tsn - min_tsn) / (u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset)) + min_tsn;
+			x_value = ((event->x-LEFT_BORDER-u_data->io->offset) * ((u_data->io->x2_tmp_sec+u_data->io->x2_tmp_usec/1000000.0)-(u_data->io->x1_tmp_sec+u_data->io->x1_tmp_usec/1000000.0)) / (u_data->io->pixmap_width-LEFT_BORDER-RIGHT_BORDER-u_data->io->offset))+u_data->io->x1_tmp_sec+u_data->io->x1_tmp_usec/1000000.0;
+			y_value = (guint32) floor((u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-event->y) * (max_tsn - min_tsn) / (u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset)) + min_tsn;
 			text_color = u_data->io->draw_area->style->black_gc;
-			g_snprintf(label_string, 30, "(%.6lf, %u)", x_value, y_value);
+
+			if (u_data->dir == 1)
+				tsnlist = g_list_last(u_data->assoc->tsn1);
+			else
+				tsnlist = g_list_last(u_data->assoc->tsn2);
+
+			tsn = (tsn_t*) (tsnlist->data);
+			tmptsn =(tsn_t*)(tsnlist->data);
+			tfirst = tsn->secs + tsn->usecs/1000000.0;
+			frame = tsn->frame_number;
+			
+			while (tsnlist)
+			{
+				tsnlist = g_list_previous(tsnlist);
+				tsn = (tsn_t*) (tsnlist->data);
+				if (tsn->secs+tsn->usecs/1000000.0<x_value)
+				{
+					tfirst = tsn->secs+tsn->usecs/1000000.0;
+					tmptsn =tsn;
+				}
+				else
+				{
+					if ((tfirst+tsn->secs+tsn->usecs/1000000.0)/2.0<x_value)
+					{
+						x_value = tsn->secs+tsn->usecs/1000000.0;
+						tmptsn = tsn;
+					}
+					else
+						x_value = tmptsn->secs+tmptsn->usecs/1000000.0;
+					break;
+				}
+			}
+			cf_goto_frame(&cfile, tmptsn->frame_number);
+			g_snprintf(label_string, 30, "(%.6f, %u)", x_value, y_value);
 			label_set = TRUE;
 
-			gdk_draw_line(u_data->io->pixmap,text_color, event->x-2, event->y, event->x+2, event->y);
-			gdk_draw_line(u_data->io->pixmap,text_color, event->x, event->y-2, event->x, event->y+2);
+			gdk_draw_line(u_data->io->pixmap,text_color, (gint)(event->x-2), (gint)(event->y), (gint)(event->x+2), (gint)(event->y));
+			gdk_draw_line(u_data->io->pixmap,text_color, (gint)(event->x), (gint)(event->y-2), (gint)(event->x), (gint)(event->y+2));
 			if (event->x+150>=u_data->io->pixmap_width)
 				position = event->x - 150;
 			else
@@ -1075,8 +1111,8 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 #if GTK_MAJOR_VERSION < 2
 			lwidth=gdk_string_width(font, label_string);
 		                            gdk_draw_string(u_data->io->pixmap,font,text_color,
-		                            position,
-		                            event->y-10,
+		                            (gint)(position),
+		                            (gint)(event->y-10),
 		                            label_string);
 #else
 			memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
@@ -1084,10 +1120,10 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 			pango_layout_get_pixel_size(layout, &lwidth, NULL);
 
 			gdk_draw_layout(u_data->io->pixmap,text_color,
-							position,
-							event->y-10,
+							(gint)(position),
+							(gint)(event->y-10),
 							layout);
-	#endif
+#endif
 			ios=(sctp_graph_t *)OBJECT_GET_DATA(u_data->io->draw_area, "sctp_graph_t");
 
 			if(!ios){
