@@ -1,7 +1,7 @@
 /* summary_dlg.c
  * Routines for capture file summary window
  *
- * $Id: summary_dlg.c 23447 2007-11-14 10:15:20Z stig $
+ * $Id: summary_dlg.c 24513 2008-03-01 10:23:57Z tuexen $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -49,6 +49,7 @@
 #include "gui_utils.h"
 #include "compat_macros.h"
 #include "help_dlg.h"
+#include <epan/strutil.h>
 
 #define SUM_STR_MAX     1024
 #define FILTER_SNIP_LEN 50
@@ -87,9 +88,9 @@ add_string_to_table(GtkWidget *list, guint *row, gchar *title, gchar *value)
 
 
 static void
-add_string_to_list(GtkWidget *list, gchar *title, gchar *captured, gchar *displayed)
+add_string_to_list(GtkWidget *list, gchar *title, gchar *captured, gchar *displayed, gchar *marked)
 {
-    simple_list_append(list, 0, title, 1, captured, 2, displayed, -1);
+    simple_list_append(list, 0, title, 1, captured, 2, displayed, 3, marked, -1);
 }
 
 void
@@ -100,13 +101,15 @@ summary_open_cb(GtkWidget *w _U_, gpointer d _U_)
                 *main_vb, *bbox, *close_bt, *help_bt;
   GtkWidget     *table;
   GtkWidget     *list;
-  static const char *titles[] = { "Traffic", "Captured", "Displayed" };
+  static const char *titles[] = { "Traffic", "Captured", "Displayed", "Marked" };
 
   gchar         string_buff[SUM_STR_MAX];
   gchar         string_buff2[SUM_STR_MAX];
+  gchar         string_buff3[SUM_STR_MAX];
 
   double        seconds;
   double        disp_seconds;
+  double        marked_seconds;
   guint         offset;
   guint         snip;
   guint         row;
@@ -124,6 +127,7 @@ summary_open_cb(GtkWidget *w _U_, gpointer d _U_)
 #endif
   seconds = summary.stop_time - summary.start_time;
   disp_seconds = summary.filtered_stop - summary.filtered_start;
+  marked_seconds = summary.marked_stop - summary.marked_start;
 
   sum_open_w = window_new(GTK_WINDOW_TOPLEVEL, "Wireshark: Summary");
 
@@ -267,101 +271,138 @@ summary_open_cb(GtkWidget *w _U_, gpointer d _U_)
     add_string_to_table(table, &row, "Display filter:", "none");
   }
 
-  /* Marked Packet count */
-  g_snprintf(string_buff, SUM_STR_MAX, "%i", summary.marked_count);
-  add_string_to_table(table, &row, "Marked packets:", string_buff);
-
-
   /* Traffic */
-  list = simple_list_new(3, titles);
+  list = simple_list_new(4, titles);
   gtk_container_add(GTK_CONTAINER(main_vb), list);
 
-  g_snprintf(string_buff, SUM_STR_MAX, "%.3f sec", seconds);
-  if(summary.dfilter) {
-    g_snprintf(string_buff2, SUM_STR_MAX, "%.3f sec", disp_seconds);
-  } else {
-    strcpy(string_buff2, "");
-  }
-  add_string_to_list(list, "Between first and last packet", string_buff, string_buff2);
-
   /* Packet count */
   g_snprintf(string_buff, SUM_STR_MAX, "%i", summary.packet_count);
-  if(summary.dfilter) {
+  if (summary.dfilter) {
     g_snprintf(string_buff2, SUM_STR_MAX, "%i", summary.filtered_count);
   } else {
-    strcpy(string_buff2, "");
+    g_strlcpy(string_buff2, string_buff, SUM_STR_MAX);
   }
-  add_string_to_list(list, "Packets", string_buff, string_buff2);
+  g_snprintf(string_buff3, SUM_STR_MAX, "%i", summary.marked_count);
+  add_string_to_list(list, "Packets", string_buff, string_buff2, string_buff3);
+
+  /* Time between first and last */
+  if (seconds > 0) {
+    g_snprintf(string_buff, SUM_STR_MAX, "%.3f sec", seconds);
+  } else {
+    string_buff[0] = '\0';
+  }
+  if (summary.dfilter && disp_seconds > 0) {
+    g_snprintf(string_buff2, SUM_STR_MAX, "%.3f sec", disp_seconds);
+  } else {
+    string_buff2[0] = '\0';
+  }
+  if (summary.marked_count && marked_seconds > 0) {
+    g_snprintf(string_buff3, SUM_STR_MAX, "%.3f sec", marked_seconds);
+  } else {
+    string_buff3[0] = '\0';
+  }
+  add_string_to_list(list, "Between first and last packet", string_buff, string_buff2, string_buff3);
 
   /* Packets per second */
-  if (seconds > 0){
+  if (seconds > 0) {
     g_snprintf(string_buff, SUM_STR_MAX, "%.3f", summary.packet_count/seconds);
   } else {
-    strcpy(string_buff, "");
+    string_buff[0] = '\0';
   }
-  if(summary.dfilter && disp_seconds > 0){
+  if(summary.dfilter && disp_seconds > 0) {
     g_snprintf(string_buff2, SUM_STR_MAX, "%.3f", summary.filtered_count/disp_seconds);
   } else {
-    strcpy(string_buff2, "");
+    string_buff2[0] = '\0';
   }
-  add_string_to_list(list, "Avg. packets/sec", string_buff, string_buff2);
+  if(summary.marked_count && marked_seconds > 0) {
+    g_snprintf(string_buff3, SUM_STR_MAX, "%.3f", summary.marked_count/marked_seconds);
+  } else {
+    string_buff3[0] = '\0';
+  }
+  add_string_to_list(list, "Avg. packets/sec", string_buff, string_buff2, string_buff3);
 
   /* Packet size */
-  if (summary.packet_count > 0){
+  if (summary.packet_count > 1) {
     g_snprintf(string_buff, SUM_STR_MAX, "%.3f bytes",
-      /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
-      (float) ((gint64) summary.bytes)/summary.packet_count);
+	       /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
+	       (float) ((gint64) summary.bytes)/summary.packet_count);
   } else {
-    strcpy(string_buff, "");
+    string_buff[0] = '\0';
   }
-  if (summary.dfilter && summary.filtered_count > 0){
+  if (summary.dfilter && summary.filtered_count > 1) {
     g_snprintf(string_buff2, SUM_STR_MAX, "%.3f bytes",
-      /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
-      (float) ((gint64) summary.filtered_bytes)/summary.filtered_count);
+	       /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
+	       (float) ((gint64) summary.filtered_bytes)/summary.filtered_count);
   } else {
-    strcpy(string_buff2, "");
+    string_buff2[0] = '\0';
   }
-  add_string_to_list(list, "Avg. packet size", string_buff, string_buff2);
+  if (summary.marked_count > 1) {
+    g_snprintf(string_buff3, SUM_STR_MAX, "%.3f bytes",
+	       /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
+	       (float) ((gint64) summary.marked_bytes)/summary.marked_count);
+  } else {
+    string_buff3[0] = '\0';
+  }
+  add_string_to_list(list, "Avg. packet size", string_buff, string_buff2, string_buff3);
 
   /* Byte count */
   g_snprintf(string_buff, SUM_STR_MAX, "%" G_GINT64_MODIFIER "u", summary.bytes);
-  if (summary.dfilter && summary.filtered_count > 0){
+  if (summary.dfilter && summary.filtered_count > 0) {
     g_snprintf(string_buff2, SUM_STR_MAX, "%" G_GINT64_MODIFIER "u", summary.filtered_bytes);
   } else {
-    strcpy(string_buff2, "");
+    string_buff2[0] = '\0';
   }
-  add_string_to_list(list, "Bytes", string_buff, string_buff2);
+  if (summary.marked_count) {
+    g_snprintf(string_buff3, SUM_STR_MAX, "%" G_GINT64_MODIFIER "u", summary.marked_bytes);
+  } else {
+    string_buff3[0] = '\0';
+  }
+  add_string_to_list(list, "Bytes", string_buff, string_buff2, string_buff3);
 
   /* Bytes per second */
   if (seconds > 0){
     /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
     g_snprintf(string_buff, SUM_STR_MAX, "%.3f", ((gint64) summary.bytes)/seconds);
   } else {
-    strcpy(string_buff, "");
+    string_buff[0] = '\0';
   }
-  if (summary.dfilter && disp_seconds > 0){
+  if (summary.dfilter && disp_seconds > 0) {
     /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
     g_snprintf(string_buff2, SUM_STR_MAX, "%.3f", ((gint64) summary.filtered_bytes)/disp_seconds);
   } else {
-    strcpy(string_buff2, "");
+    string_buff2[0] = '\0';
   }
-  add_string_to_list(list, "Avg. bytes/sec", string_buff, string_buff2);
+  if (summary.marked_count && marked_seconds > 0) {
+    /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
+    g_snprintf(string_buff3, SUM_STR_MAX, "%.3f", ((gint64) summary.marked_bytes)/marked_seconds);
+  } else {
+    string_buff3[0] = '\0';
+  }
+  add_string_to_list(list, "Avg. bytes/sec", string_buff, string_buff2, string_buff3);
 
   /* MBit per second */
-  if (seconds > 0){
-    /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
-    g_snprintf(string_buff, SUM_STR_MAX, "%.3f", ((gint64) summary.bytes) * 8.0 / (seconds * 1000.0 * 1000.0));
+  if (seconds > 0) {
+    g_snprintf(string_buff, SUM_STR_MAX, "%.3f", 
+	       /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
+	       ((gint64) summary.bytes) * 8.0 / (seconds * 1000.0 * 1000.0));
   } else {
-    strcpy(string_buff, "");
+    string_buff[0] = '\0';
   }
-  if (summary.dfilter && disp_seconds > 0){
+  if (summary.dfilter && disp_seconds > 0) {
     g_snprintf(string_buff2, SUM_STR_MAX, "%.3f",
-          /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
-          ((gint64) summary.filtered_bytes) * 8.0 / (disp_seconds * 1000.0 * 1000.0));
+	       /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
+	       ((gint64) summary.filtered_bytes) * 8.0 / (disp_seconds * 1000.0 * 1000.0));
   } else {
-    strcpy(string_buff2, "");
+    string_buff2[0] = '\0';
   }
-  add_string_to_list(list, "Avg. MBit/sec", string_buff, string_buff2);
+  if (summary.marked_count && marked_seconds > 0) {
+    g_snprintf(string_buff3, SUM_STR_MAX, "%.3f",
+	       /* MSVC cannot convert from unsigned __int64 to float, so first convert to signed __int64 */
+	       ((gint64) summary.marked_bytes) * 8.0 / (marked_seconds * 1000.0 * 1000.0));
+  } else {
+    string_buff3[0] = '\0';
+  }
+  add_string_to_list(list, "Avg. MBit/sec", string_buff, string_buff2, string_buff3);
 
 
   /* Button row. */
