@@ -3,7 +3,7 @@
  * By Steve Limkemann <stevelim@dgtech.com>
  * Copyright 1998 Steve Limkemann
  *
- * $Id: packet-gryphon.c 18197 2006-05-21 05:12:17Z sahlberg $
+ * $Id: packet-gryphon.c 24274 2008-02-05 22:30:32Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -47,12 +47,12 @@
 
 static int proto_gryphon = -1;
 
-static int hf_gryph_src = -1;
-static int hf_gryph_srcchan = -1;
-static int hf_gryph_dest = -1;
-static int hf_gryph_destchan= -1;
-static int hf_gryph_type = -1;
-static int hf_gryph_cmd = -1;
+static int hf_gryphon_src = -1;
+static int hf_gryphon_srcchan = -1;
+static int hf_gryphon_dest = -1;
+static int hf_gryphon_destchan= -1;
+static int hf_gryphon_type = -1;
+static int hf_gryphon_cmd = -1;
 
 static gint ett_gryphon = -1;
 static gint ett_gryphon_header = -1;
@@ -148,7 +148,7 @@ static const char *frame_type[] = {
 #define FRAME_HEADER_LEN	8
 
 static guint
-get_gryphon_pdu_len(tvbuff_t *tvb, int offset)
+get_gryphon_pdu_len(packet_info *pinfo _U_, tvbuff_t *tvb, int offset)
 {
     guint16 plen;
     int padded_len;
@@ -264,18 +264,18 @@ dissect_gryphon_message(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	"Source: %s, channel %u",
 	val_to_str(src, src_dest, "Unknown (0x%02x)"),
 	tvb_get_guint8(tvb, offset + 1));
-    proto_tree_add_uint_hidden(header_tree, hf_gryph_src, tvb,
+    proto_tree_add_uint_hidden(header_tree, hf_gryphon_src, tvb,
 	offset, 1, src);
-    proto_tree_add_uint_hidden(header_tree, hf_gryph_srcchan, tvb,
+    proto_tree_add_uint_hidden(header_tree, hf_gryphon_srcchan, tvb,
 	offset+1, 1, tvb_get_guint8(tvb, offset + 1));
 
     proto_tree_add_text(header_tree, tvb, offset+2, 2,
 	"Destination: %s, channel %u",
 	val_to_str(dest, src_dest, "Unknown (0x%02x)"),
 	tvb_get_guint8(tvb, offset + 3));
-    proto_tree_add_uint_hidden(header_tree, hf_gryph_dest, tvb,
+    proto_tree_add_uint_hidden(header_tree, hf_gryphon_dest, tvb,
 	offset+2, 1, dest);
-    proto_tree_add_uint_hidden(header_tree, hf_gryph_destchan, tvb,
+    proto_tree_add_uint_hidden(header_tree, hf_gryphon_destchan, tvb,
 	offset+3, 1, tvb_get_guint8(tvb, offset + 3));
 
     proto_tree_add_text(header_tree, tvb, offset+4, 2,
@@ -296,7 +296,7 @@ dissect_gryphon_message(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     }
     proto_tree_add_text(header_tree, tvb, offset+7, 1, "reserved");
 
-    proto_tree_add_uint_hidden(header_tree, hf_gryph_type, tvb,
+    proto_tree_add_uint_hidden(header_tree, hf_gryphon_type, tvb,
 	offset+6, 1, frmtyp);
     msgpad = 3 - (msglen + 3) % 4;
     msgend = offset + msglen + msgpad + MSG_HDR_SZ;
@@ -638,7 +638,7 @@ decode_command(tvbuff_t *tvb, int offset, int dst, proto_tree *pt)
 
     msglen = tvb_reported_length_remaining(tvb, offset);
     cmd = tvb_get_guint8(tvb, offset);
-    proto_tree_add_uint_hidden(pt, hf_gryph_cmd, tvb, offset, 1, cmd);
+    proto_tree_add_uint_hidden(pt, hf_gryphon_cmd, tvb, offset, 1, cmd);
     if (cmd > 0x3F)
     	cmd += dst * 256;
 
@@ -937,11 +937,11 @@ cmd_setfilt(tvbuff_t *tvb, int offset, proto_tree *pt)
     length =  tvb_get_guint8(tvb, offset+4) + tvb_get_guint8(tvb, offset+5)
 	+ tvb_get_ntohs(tvb, offset+6);
     if (flag)
-    	strcpy (mode, "Pass");
+       strncpy (mode, "Pass", 10);
     else
-    	strcpy (mode, "Block");
+       strncpy (mode, "Block", 10);
     if (length == 0)
-    	strcat (mode, " all");
+       strncat (mode, " all", 10);
     proto_tree_add_text(pt, tvb, offset, 4, "Pass/Block flag: %s", mode);
     proto_tree_add_text(pt, tvb, offset+4, 4, "Length of Pattern & Mask: %d", length);
     offset += 8;
@@ -2197,21 +2197,21 @@ blm_mode(tvbuff_t *tvb, int offset, proto_tree *pt)
     switch (x) {
     case 0:
     	mode = "Off";
-	sprintf (line, "reserved");
+	g_snprintf (line, 50, "reserved");
     	break;
     case 1:
     	mode = "Average over time";
 	seconds = y / 1000;
 	y = y % 1000;
-	sprintf (line, "Averaging period: %d.%03d seconds", seconds, y);
+	g_snprintf (line, 50, "Averaging period: %d.%03d seconds", seconds, y);
     	break;
     case 2:
     	mode = "Average over frame count";
-	sprintf (line, "Averaging period: %d frames", y);
+	g_snprintf (line, 50, "Averaging period: %d frames", y);
     	break;
     default:
     	mode = "- unknown -";
-	sprintf (line, "reserved");
+	g_snprintf (line, 50, "reserved");
     }
     proto_tree_add_text(pt, tvb, offset, 4, "Mode: %s", mode);
     offset += 4;
@@ -2224,23 +2224,23 @@ void
 proto_register_gryphon(void)
 {
     static hf_register_info hf[] = {
-	{ &hf_gryph_src,
-	{ "Source",           "gryph.src", FT_UINT8, BASE_HEX, VALS(src_dest), 0x0,
+	{ &hf_gryphon_src,
+	{ "Source",           "gryphon.src", FT_UINT8, BASE_HEX, VALS(src_dest), 0x0,
 	    	"", HFILL }},
-	{ &hf_gryph_srcchan,
-	{ "Source channel",   "gryph.srcchan", FT_UINT8, BASE_DEC, NULL, 0x0,
+	{ &hf_gryphon_srcchan,
+	{ "Source channel",   "gryphon.srcchan", FT_UINT8, BASE_DEC, NULL, 0x0,
 	    	"", HFILL }},
-	{ &hf_gryph_dest,
-	{ "Destination",      "gryph.dest", FT_UINT8, BASE_HEX, VALS(src_dest), 0x0,
+	{ &hf_gryphon_dest,
+	{ "Destination",      "gryphon.dest", FT_UINT8, BASE_HEX, VALS(src_dest), 0x0,
 	    	"", HFILL }},
-	{ &hf_gryph_destchan,
-	{ "Destination channel", "gryph.dstchan", FT_UINT8, BASE_DEC, NULL, 0x0,
+	{ &hf_gryphon_destchan,
+	{ "Destination channel", "gryphon.destchan", FT_UINT8, BASE_DEC, NULL, 0x0,
 	    	"", HFILL }},
-	{ &hf_gryph_type,
-	{ "Frame type",       "gryph.type", FT_UINT8, BASE_DEC, NULL, 0x0,
+	{ &hf_gryphon_type,
+	{ "Frame type",       "gryphon.type", FT_UINT8, BASE_DEC, NULL, 0x0,
 	    	"", HFILL }},
-	{ &hf_gryph_cmd,
-	{ "Command",          "gryph.cmd.cmd", FT_UINT8, BASE_DEC, NULL, 0x0,
+	{ &hf_gryphon_cmd,
+	{ "Command",          "gryphon.cmd", FT_UINT8, BASE_DEC, NULL, 0x0,
 	    	"", HFILL }},
     };
 
