@@ -2,7 +2,7 @@
  * h225 message counter for wireshark
  * Copyright 2003 Lars Roland
  *
- * $Id: tap-h225counter.c 23226 2007-10-18 13:35:57Z kukosa $
+ * $Id: tap-h225counter.c 24283 2008-02-07 12:52:08Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -225,7 +225,6 @@ h225counter_packet(void *phs, packet_info *pinfo _U_, epan_dissect_t *edt _U_, c
 
 	default:
 		return 0;
-		break;
 	}
 
 	return 1;
@@ -363,8 +362,7 @@ h225counter_init(const char *optarg, void* userdata _U_)
 	}
 
 	hs = g_malloc(sizeof(h225counter_t));
-	hs->filter=g_malloc(strlen(filter)+1);
-	strcpy(hs->filter, filter);
+	hs->filter=g_strdup(filter);
 
 	h225counter_reset(hs);
 
