@@ -1,7 +1,7 @@
 /* tap-protocolinfo.c
  * protohierstat   2002 Ronnie Sahlberg
  *
- * $Id: tap-protocolinfo.c 19976 2006-11-24 18:25:23Z guy $
+ * $Id: tap-protocolinfo.c 24531 2008-03-02 21:12:24Z stig $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -40,6 +40,7 @@
 #include "epan/proto.h"
 #include <epan/tap.h>
 #include <epan/stat_cmd_args.h>
+#include <epan/strutil.h>
 #include "register.h"
 
 typedef struct _pci_t {
@@ -117,8 +118,7 @@ protocolinfo_init(const char *optarg, void* userdata _U_)
 	rs->hf_index=hfi->id;
 	if((field-filter)>1){
 		rs->filter=g_malloc(field-filter);
-		strncpy(rs->filter,filter,(field-filter)-1);
-		rs->filter[(field-filter)-1]=0;
+		g_strlcpy(rs->filter,filter,(field-filter));
 	} else {
 		rs->filter=NULL;
 	}
