@@ -1,6 +1,6 @@
 /* catapult_dct2000.c
  *
- * $Id: catapult_dct2000.c 21986 2007-05-29 14:38:40Z martinm $
+ * $Id: catapult_dct2000.c 24258 2008-02-03 15:38:20Z stig $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -43,7 +43,7 @@
 #define MAX_PROTOCOL_NAME          64
 #define MAX_PORT_DIGITS            2
 #define MAX_VARIANT_DIGITS         32
-#define MAX_OUTHDR_NAME            64
+#define MAX_OUTHDR_NAME            256
 #define AAL_HEADER_CHARS           12
 
 /* TODO:
@@ -60,12 +60,17 @@ typedef enum packet_direction_t
 } packet_direction_t;
 
 
+/***********************************************************************/
+/* For each line, store (in case we need to dump):                     */
+/* - String before time field                                          */
+/* - String beween time field and data (if NULL assume " l ")          */
 typedef struct
 {
     gchar *before_time;
-    gchar *after_time; /* If NULL assume " l " */
+    gchar *after_time;
 } line_prefix_info_t;
 
+
 /*******************************************************************/
 /* Information stored external to a file (wtap) needed for dumping */
 typedef struct dct2000_file_externals
@@ -79,13 +84,13 @@ typedef struct dct2000_file_externals
     gint  secondline_length;
 
     /* Hash table to store text prefix data part of displayed packets.
-       Records (file offset -> pre-data-prefix-string)
+       Records (file offset -> line_prefix_info_t)
        N.B. This is only needed for dumping
     */
     GHashTable *packet_prefix_table;
 } dct2000_file_externals_t;
 
-/* This global table maps wtap -> file_external structs */
+/* This global table maps wtap -> dct2000_file_externals_t structs */
 static GHashTable *file_externals_table = NULL;
 
 
@@ -112,7 +117,7 @@ static gchar outhdr_name[MAX_OUTHDR_NAME+1];
 
 
 /************************************************************/
-/* Functions called from wiretap                            */
+/* Functions called from wiretap core                       */
 static gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info,
                                       gint64 *data_offset);
 static gboolean catapult_dct2000_seek_read(wtap *wth, gint64 seek_off,
@@ -164,7 +169,7 @@ static gboolean free_line_prefix_info(gpointer key, gpointer value, gpointer use
 
 
 /********************************************/
-/* Open file                                */
+/* Open file (for reading)                 */
 /********************************************/
 int catapult_dct2000_open(wtap *wth, int *err, gchar **err_info _U_)
 {
@@ -178,16 +183,6 @@ int catapult_dct2000_open(wtap *wth, int *err, gchar **err_info _U_)
     errno = 0;
 
 
-    /*********************************************************************/
-    /* Need entry in file_externals table                                */
-
-    /* Create file externals table if it doesn't yet exist */
-    if (file_externals_table == NULL)
-    {
-        file_externals_table = g_hash_table_new(wth_hash_func, wth_equal);
-    }
-
-
     /********************************************************************/
     /* First line needs to contain at least as many characters as magic */
 
@@ -205,7 +200,16 @@ int catapult_dct2000_open(wtap *wth, int *err, gchar **err_info _U_)
     }
 
 
-    /* Allocate a new file_externals structure */
+    /*********************************************************************/
+    /* Need entry in file_externals table                                */
+
+    /* Create file externals table if it doesn't yet exist */
+    if (file_externals_table == NULL)
+    {
+        file_externals_table = g_hash_table_new(wth_hash_func, wth_equal);
+    }
+
+    /* Allocate a new file_externals structure for this file */
     file_externals = g_malloc(sizeof(dct2000_file_externals_t));
     memset((void*)file_externals, '\0', sizeof(dct2000_file_externals_t));
 
@@ -227,6 +231,7 @@ int catapult_dct2000_open(wtap *wth, int *err, gchar **err_info _U_)
         return 0;
     }
 
+    /* Allocate struct and fill in timestamp */
     wth->capture.catapult_dct2000 = g_malloc(sizeof(catapult_dct2000_t));
     wth->capture.catapult_dct2000->start_secs = timestamp;
     wth->capture.catapult_dct2000->start_usecs = usecs;
@@ -268,7 +273,7 @@ int catapult_dct2000_open(wtap *wth, int *err, gchar **err_info _U_)
 
 
 /**************************************************/
-/* Read function.                                 */
+/* Read packet function.                          */
 /* Look for and read the next usable packet       */
 /* - return TRUE and details if found             */
 /**************************************************/
@@ -290,7 +295,7 @@ gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info _U_,
         return FALSE;
     }
 
-    /* Search for a line containing a usable message */
+    /* Search for a line containing a usable packet */
     while (1)
     {
         int line_length, seconds, useconds, data_chars;
@@ -309,11 +314,11 @@ gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info _U_,
         /* Read a new line from file into linebuff */
         if (read_new_line(wth->fh, &offset, &line_length) == FALSE)
         {
-            /* Get out when no more lines to be read */
+            /* Get out if no more lines can be read */
             break;
         }
 
-        /* Try to parse the line as a message */
+        /* Try to parse the line as a frame record */
         if (parse_line(line_length, &seconds, &useconds,
                        &before_time_offset, &after_time_offset,
                        &dollar_offset,
@@ -326,7 +331,7 @@ gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info _U_,
             char timestamp_string[32];
             gint64 *pkey = NULL;
 
-            sprintf(timestamp_string, "%d.%04d", seconds, useconds/100);
+            g_snprintf(timestamp_string, 32, "%d.%04d", seconds, useconds/100);
 
             /* All packets go to Catapult DCT2000 stub dissector */
             wth->phdr.pkt_encap = WTAP_ENCAP_CATAPULT_DCT2000;
@@ -399,6 +404,7 @@ gboolean catapult_dct2000_read(wtap *wth, int *err, gchar **err_info _U_,
             }
             else
             {
+                /* Allocate & write buffer for line between timestamp and data */
                 line_prefix_info->after_time = g_malloc(dollar_offset - after_time_offset);
                 strncpy(line_prefix_info->after_time, linebuff+after_time_offset,
                         dollar_offset - after_time_offset);
@@ -449,7 +455,7 @@ catapult_dct2000_seek_read(wtap *wth, gint64 seek_off,
         return FALSE;
     }
 
-    /* Re-read whole line (this should succeed) */
+    /* Re-read whole line (this really should succeed) */
     if (read_new_line(wth->random_fh, &offset, &length) == FALSE)
     {
         return FALSE;
@@ -464,7 +470,7 @@ catapult_dct2000_seek_read(wtap *wth, gint64 seek_off,
         int n;
         int stub_offset = 0;
         char timestamp_string[32];
-        sprintf(timestamp_string, "%d.%04d", seconds, useconds/100);
+        g_snprintf(timestamp_string, 32, "%d.%04d", seconds, useconds/100);
 
         /* Make sure all packets go to catapult dct2000 dissector */
         wth->phdr.pkt_encap = WTAP_ENCAP_CATAPULT_DCT2000;
@@ -500,9 +506,9 @@ catapult_dct2000_seek_read(wtap *wth, gint64 seek_off,
 }
 
 
-/******************************************/
-/* Free dct2000-specific capture info     */
-/******************************************/
+/***************************************************************************/
+/* Free dct2000-specific capture info from file that was open for reading  */
+/***************************************************************************/
 void catapult_dct2000_close(wtap *wth)
 {
     /* Look up externals for this file */
@@ -562,7 +568,9 @@ int catapult_dct2000_dump_can_write_encap(int encap)
         case WTAP_ENCAP_CATAPULT_DCT2000:
             /* We support this */
             return 0;
+
         default:
+            /* But don't write to any other formats... */
             return WTAP_ERR_UNSUPPORTED_ENCAP;
     }
 }
@@ -571,9 +579,30 @@ int catapult_dct2000_dump_can_write_encap(int encap)
 /*****************************************/
 /* Write a single packet out to the file */
 /*****************************************/
+
+static gboolean do_fwrite(const void *data, size_t size, size_t count, FILE *stream, int *err_p)
+{
+    size_t nwritten;
+
+    nwritten = fwrite(data, size, count, stream);
+    if (nwritten != count) {
+        if (nwritten == 0 && ferror(stream))
+        {
+            *err_p = errno;
+        }
+        else
+        {
+            *err_p = WTAP_ERR_SHORT_WRITE;
+        }
+
+        return FALSE;
+    }
+    return TRUE;
+}
+
 gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
                                const union wtap_pseudo_header *pseudo_header,
-                               const guchar *pd, int *err _U_)
+                               const guchar *pd, int *err)
 {
     guint32 n;
     line_prefix_info_t *prefix = NULL;
@@ -592,14 +621,26 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
         wdh->dump.dct2000 = g_malloc(sizeof(catapult_dct2000_t));
 
         /* Write out saved first line */
-        fwrite(file_externals->firstline, 1, file_externals->firstline_length, wdh->fh);
-        fwrite("\n", 1, 1, wdh->fh);
+        if (! do_fwrite(file_externals->firstline, 1, file_externals->firstline_length, wdh->fh, err))
+        {
+            return FALSE;
+        }
+        if (! do_fwrite("\n", 1, 1, wdh->fh, err))
+        {
+            return FALSE;
+        }
 
         /* Also write out saved second line with timestamp corresponding to the
            opening time of the log.
         */
-        fwrite(file_externals->secondline, 1, file_externals->secondline_length, wdh->fh);
-        fwrite("\n", 1, 1, wdh->fh);
+        if (! do_fwrite(file_externals->secondline, 1, file_externals->secondline_length, wdh->fh, err))
+        {
+            return FALSE;
+        }
+        if (! do_fwrite("\n", 1, 1, wdh->fh, err))
+        {
+            return FALSE;
+        }
 
         /* Allocate the dct2000-specific dump structure */
         wdh->dump.dct2000 = g_malloc(sizeof(catapult_dct2000_t));
@@ -623,7 +664,10 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
                                                       (const void*)&(pseudo_header->dct2000.seek_off));
 
     /* Write out text before timestamp */
-    fwrite(prefix->before_time, 1, strlen(prefix->before_time), wdh->fh);
+    if (! do_fwrite(prefix->before_time, 1, strlen(prefix->before_time), wdh->fh, err))
+    {
+        return FALSE;
+    }
 
     /* Calculate time of this packet to write, relative to start of dump */
     if (phdr->ts.nsecs >= wdh->dump.dct2000->start_time.nsecs)
@@ -640,16 +684,25 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     }
 
     /* Write out the calculated timestamp */
-    fwrite(time_string, 1, strlen(time_string), wdh->fh);
+    if (! do_fwrite(time_string, 1, strlen(time_string), wdh->fh, err))
+    {
+        return FALSE;
+    }
 
     /* Write out text between timestamp and start of hex data */
     if (prefix->after_time == NULL)
     {
-        fwrite(" l ", 1, strlen(" l "), wdh->fh);
+        if (! do_fwrite(" l ", 1, strlen(" l "), wdh->fh, err))
+        {
+            return FALSE;
+        }
     }
     else
     {
-        fwrite(prefix->after_time, 1, strlen(prefix->after_time), wdh->fh);
+        if (! do_fwrite(prefix->after_time, 1, strlen(prefix->after_time), wdh->fh, err))
+        {
+            return FALSE;
+        }
     }
 
 
@@ -685,7 +738,10 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 
     /**************************************/
     /* Remainder is encapsulated protocol */
-    fwrite("$", 1, 1, wdh->fh);
+    if (! do_fwrite("$", 1, 1, wdh->fh, err))
+    {
+        return FALSE;
+    }
 
     /* Each binary byte is written out as 2 hex string chars */ 
     for (; n < phdr->len; n++)
@@ -695,11 +751,17 @@ gboolean catapult_dct2000_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
         c[1] = char_from_hex((guchar)(pd[n] & 0x0f));
 
         /* Write both hex chars of byte together */
-        fwrite(c, 1, 2, wdh->fh);
+        if (! do_fwrite(c, 1, 2, wdh->fh, err))
+        {
+            return FALSE;
+        }
     }
 
     /* End the line */
-    fwrite("\n", 1, 1, wdh->fh);
+    if (! do_fwrite("\n", 1, 1, wdh->fh, err))
+    {
+        return FALSE;
+    }
 
     return TRUE;
 }
@@ -785,7 +847,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
     /* Read context name until find '.' */
     for (n=0; linebuff[n] != '.' && (n < MAX_CONTEXT_NAME) && (n+1 < line_length); n++)
     {
-	if (!isalnum((int)linebuff[n]) && (linebuff[n] != '_'))
+        if (!isalnum((guchar)linebuff[n]) && (linebuff[n] != '_'))
         {
             return FALSE;
         }
@@ -811,7 +873,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
          (linebuff[n] != '/') && (port_digits <= MAX_PORT_DIGITS) && (n+1 < line_length);
          n++, port_digits++)
     {
-	if (!isdigit((int)linebuff[n]))
+        if (!isdigit((guchar)linebuff[n]))
         {
             return FALSE;
         }
@@ -838,7 +900,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
          (linebuff[n] != '/') && (protocol_chars < MAX_PROTOCOL_NAME) && (n < line_length);
          n++, protocol_chars++)
     {
-	if (!isalnum((int)linebuff[n]) && linebuff[n] != '_')
+        if (!isalnum((guchar)linebuff[n]) && linebuff[n] != '_')
         {
             return FALSE;
         }
@@ -862,10 +924,10 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
 
     /* Following the / is the variant number.  No digits indicate 1 */
     for (variant_digits = 0;
-         (isdigit((int)linebuff[n])) && (variant_digits <= MAX_VARIANT_DIGITS) && (n+1 < line_length);
+         (isdigit((guchar)linebuff[n])) && (variant_digits <= MAX_VARIANT_DIGITS) && (n+1 < line_length);
          n++, variant_digits++)
     {
-	if (!isdigit((int)linebuff[n]))
+        if (!isdigit((guchar)linebuff[n]))
         {
             return FALSE;
         }
@@ -894,11 +956,11 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
         n++;
 
         for (outhdr_chars = 0;
-             (isdigit((int)linebuff[n]) || linebuff[n] == ',') &&
+             (isdigit((guchar)linebuff[n]) || linebuff[n] == ',') &&
              (outhdr_chars <= MAX_OUTHDR_NAME) && (n+1 < line_length);
              n++, outhdr_chars++)
         {
-	    if (!isdigit((int)linebuff[n]) && (linebuff[n] != ','))
+            if (!isdigit((guchar)linebuff[n]) && (linebuff[n] != ','))
             {
                 return FALSE;
             }
@@ -920,7 +982,9 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
 
     if ((strcmp(protocol_name, "ip") == 0) ||
         (strcmp(protocol_name, "sctp") == 0) ||
-        (strcmp(protocol_name, "mipv6") == 0))
+        (strcmp(protocol_name, "gre") == 0) ||
+        (strcmp(protocol_name, "mipv6") == 0) ||
+        (strcmp(protocol_name, "igmp") == 0))
     {
         *encap = WTAP_ENCAP_RAW_IP;
     }
@@ -1022,19 +1086,17 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
 
         /* Read consecutive hex chars into atm header buffer */
         for (;
-             (isalnum((int)linebuff[n]) &&
+             ((linebuff[n] >= '0') && (linebuff[n] <= '?') &&
               (n < line_length) &&
               (header_chars_seen < AAL_HEADER_CHARS));
              n++, header_chars_seen++)
         {
             aal_header_chars[header_chars_seen] = linebuff[n];
-        }
-
-        /* Sometimes see strange encoding of cid in last (non-digit) character */
-        if (header_chars_seen == (AAL_HEADER_CHARS-1))
-        {
-            aal_header_chars[AAL_HEADER_CHARS-1] = linebuff[n];
-            header_chars_seen++;
+            /* Next 6 characters after '9' are mapped to a->f */
+            if (!isdigit((guchar)linebuff[n]))
+            {
+                aal_header_chars[header_chars_seen] = 'a' + (linebuff[n] - '9') -1;
+            }
         }
 
         if (header_chars_seen != AAL_HEADER_CHARS || n >= line_length)
@@ -1073,7 +1135,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
     /* Find and read the timestamp                                       */
 
     /* Now scan to the next digit, which should be the start of the timestamp */
-    for (; !isdigit((int)linebuff[n]) && (n < line_length); n++);
+    for (; !isdigit((guchar)linebuff[n]) && (n < line_length); n++);
     if (n >= line_length)
     {
         return FALSE;
@@ -1088,7 +1150,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
          (n < line_length);
          n++, seconds_chars++)
     {
-        if (!isdigit((int)linebuff[n]))
+        if (!isdigit((guchar)linebuff[n]))
         {
             /* Found a non-digit before decimal point. Fail */
             return FALSE;
@@ -1120,7 +1182,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
          (n < line_length);
          n++, subsecond_decimals_chars++)
     {
-        if (!isdigit((int)linebuff[n]))
+        if (!isdigit((guchar)linebuff[n]))
         {
             return FALSE;
         }
@@ -1286,7 +1348,7 @@ void set_aal_info(union wtap_pseudo_header *pseudo_header, packet_direction_t di
 
     /* cid is usually last byte.  Unless last char is not hex digit, in which
        case cid is derived from last char in ascii */
-    if (isalnum((int)aal_header_chars[11]))
+    if (isalnum((guchar)aal_header_chars[11]))
     {
         pseudo_header->dct2000.inner_pseudo_header.atm.aal2_cid =
             ((hex_from_char(aal_header_chars[10]) << 4) |
