@@ -1,6 +1,6 @@
 /* dbs-etherwatch.c
  *
- * $Id: dbs-etherwatch.c 19814 2006-11-05 22:46:44Z ulfl $
+ * $Id: dbs-etherwatch.c 23790 2007-12-07 01:18:17Z guy $
  *
  * Wiretap Library
  * Copyright (c) 2001 by Marc Milgram <ethereal@mmilgram.NOSPAMmail.net>
@@ -38,23 +38,27 @@
  */
 
 /*
-   Example 'ETHERWATCH' output data:
+   Example 'ETHERWATCH' output data (with "printable" characters in the
+   "printable characters" section of the output replaced by "." if they have
+   the 8th bit set, so as not to upset compilers that are expecting text
+   in comments to be in a particular character encoding that can't handle
+   those values):
 ETHERWATCH  X5-008
 42 names and addresses were loaded
 Reading recorded data from PERSISTENCE
 ------------------------------------------------------------------------------
 From 00-D0-C0-D2-4D-60 [MF1] to AA-00-04-00-FC-94 [PSERVB]
 Protocol 08-00 00 00-00-00-00-00,   60 byte buffer at 10-OCT-2001 10:20:45.16
-  [E..<8..........]-    0-[45 00 00 3C 38 93 00 00 1D 06 D2 12 80 93 11 1A]
-  [.......(.....]-   16-[80 93 80 D6 02 D2 02 03 00 28 A4 90 00 00 00 00]
-  [..............]-   32-[A0 02 FF FF 95 BD 00 00 02 04 05 B4 03 03 04 01]
-  [...........    ]-   48-[01 01 08 0A 90 90 E5 14 00 00 00 00]
+  [E..<8...........]-    0-[45 00 00 3C 38 93 00 00 1D 06 D2 12 80 93 11 1A]
+  [.........(......]-   16-[80 93 80 D6 02 D2 02 03 00 28 A4 90 00 00 00 00]
+  [................]-   32-[A0 02 FF FF 95 BD 00 00 02 04 05 B4 03 03 04 01]
+  [............    ]-   48-[01 01 08 0A 90 90 E5 14 00 00 00 00]
 ------------------------------------------------------------------------------
 From 00-D0-C0-D2-4D-60 [MF1] to AA-00-04-00-FC-94 [PSERVB]
 Protocol 08-00 00 00-00-00-00-00,   50 byte buffer at 10-OCT-2001 10:20:45.17
-  [E..(8.....%....]-    0-[45 00 00 28 38 94 00 00 1D 06 D2 25 80 93 11 1A]
-  [.......(.Z.4w]-   16-[80 93 80 D6 02 D2 02 03 00 28 A4 91 5A 1C 34 77]
-  [P.#(s.........]-   32-[50 10 23 28 C1 73 00 00 02 04 05 B4 03 03 00 00]
+  [E..(8......%....]-    0-[45 00 00 28 38 94 00 00 1D 06 D2 25 80 93 11 1A]
+  [.........(..Z.4w]-   16-[80 93 80 D6 02 D2 02 03 00 28 A4 91 5A 1C 34 77]
+  [P.#(.s..........]-   32-[50 10 23 28 C1 73 00 00 02 04 05 B4 03 03 00 00]
   [..              ]-   48-[02 04]
 
 
@@ -513,9 +517,9 @@ parse_dbs_etherwatch_packet(wtap *wth, FILE_T fh, guint8* buf, int *err,
 
           1         2         3         4
 0123456789012345678901234567890123456789012345
-  [E..(8..........]-    0-[45 00 00 28 38 9B 00 00 1D 06 D2 1E 80 93 11 1A]
-  [.......(Z.4y]-   16-[80 93 80 D6 02 D2 02 03 00 28 A4 BF 5A 1C 34 79]
-  [P.#(C...00000..]-   32-[50 10 23 28 C1 43 00 00 03 30 30 30 30 30 00 00]
+  [E..(8...........]-    0-[45 00 00 28 38 9B 00 00 1D 06 D2 1E 80 93 11 1A]
+  [.........(..Z.4y]-   16-[80 93 80 D6 02 D2 02 03 00 28 A4 BF 5A 1C 34 79]
+  [P.#(.C...00000..]-   32-[50 10 23 28 C1 43 00 00 03 30 30 30 30 30 00 00]
   [.0              ]-   48-[03 30]
 
 /DISPLAY=HEXADECIMAL output:
