@@ -1,6 +1,6 @@
 /* iseries.c
  *
- * $Id: iseries.c 19814 2006-11-05 22:46:44Z ulfl $
+ * $Id: iseries.c 24295 2008-02-09 23:35:28Z wmeier $
  *
  * Wiretap Library
  * Copyright (c) 2005 by Martin Warnes <Martin_Warnes@Stercomm.com>
@@ -254,8 +254,8 @@ iseries_check_file_type (wtap * wth, int *err, int format)
 {
   guint line;
   int num_items_scanned;
-  char buf[ISERIES_LINE_LENGTH], protocol[8], tcpformat[1];
-  guint8 *sdate;
+  char buf[ISERIES_LINE_LENGTH], protocol[9], tcpformat[2];
+  char *sdate;
 
   /* Save trace format for passing between packets */
   sdate = g_malloc (10);
@@ -273,10 +273,11 @@ iseries_check_file_type (wtap * wth, int *err, int format)
 	   */
 	  if (wth->capture.iseries->format == ISERIES_FORMAT_UNICODE)
 	    {
-	      iseries_UNICODE_to_ASCII (buf, ISERIES_LINE_LENGTH);
+             iseries_UNICODE_to_ASCII ((guint8 *)buf, ISERIES_LINE_LENGTH);
 	    }
+	  g_strup(buf);
 	  num_items_scanned = sscanf (buf,
-				      "   Object protocol  . . . . . . :  %8s",
+				      "   OBJECT PROTOCOL  . . . . . . :  %8s",
 				      protocol);
 	  if (num_items_scanned == 1)
 	    {
@@ -288,7 +289,7 @@ iseries_check_file_type (wtap * wth, int *err, int format)
 	   * Determine if the data has been formatted or not
 	   */
 	  num_items_scanned = sscanf (buf,
-				      "   Format TCP/IP data only  . . :  %1s",
+				      "   FORMAT TCP/IP DATA ONLY  . . :  %1s",
 				      tcpformat);
 	  if (num_items_scanned == 1)
 	    {
@@ -307,12 +308,13 @@ iseries_check_file_type (wtap * wth, int *err, int format)
 	   * extract it here and store for all packets to access
 	   */
 	  num_items_scanned = sscanf (buf,
-				      "   Start date/Time  . . . . . . :  %8s",
+				      "   START DATE/TIME  . . . . . . :  %8s",
 				      sdate);
 	  if (num_items_scanned == 1)
 	    {
 	      wth->capture.iseries->sdate = sdate;
 	    }
+
 	}
       else
 	{
@@ -384,7 +386,7 @@ iseries_seek_next_packet (wtap * wth, int *err)
 	  if (wth->capture.iseries->format == ISERIES_FORMAT_UNICODE)
 	    {
 	      /* buflen is #bytes to 1st 0x0A */
-	      buflen = iseries_UNICODE_to_ASCII (buf, ISERIES_LINE_LENGTH);
+             buflen = iseries_UNICODE_to_ASCII ((guint8 *)buf, ISERIES_LINE_LENGTH);
 	    }
 	  else
 	    {
@@ -472,13 +474,15 @@ iseries_parse_packet (wtap * wth, FILE_T fh,
 {
   gint64 cur_off;
   gboolean isValid, isCurrentPacket, IPread, TCPread, isDATA;
-  int num_items_scanned, line, pktline, buflen;
-  int pkt_len, cap_len, pktnum, month, day, year, hr, min, sec, csec;
+  int num_items_scanned, line, pktline, buflen, i;
+  guint32 pkt_len;
+  int cap_len, pktnum, month, day, year, hr, min, sec, csec;
   char direction[2], destmac[13], srcmac[13], type[5], ipheader[41],
     tcpheader[81];
   char hex1[17], hex2[17], hex3[17], hex4[17];
   char data[ISERIES_LINE_LENGTH * 2];
-  guint8 *buf, *asciibuf, *tcpdatabuf, *workbuf;
+  guint8 *buf;
+  char   *asciibuf, *tcpdatabuf, *workbuf;
   struct tm tm;
 
   /*
@@ -502,12 +506,16 @@ iseries_parse_packet (wtap * wth, FILE_T fh,
       /* Convert UNICODE data to ASCII */
       if (wth->capture.iseries->format == ISERIES_FORMAT_UNICODE)
 	{
-	  iseries_UNICODE_to_ASCII (data, ISERIES_LINE_LENGTH);
+         iseries_UNICODE_to_ASCII ((guint8 *)data, ISERIES_LINE_LENGTH);
 	}
       /* look for packet header */
+      for (i=0; i<8; i++) {      
+	if (strncmp(data+i,"*",1) == 0)
+	  strncpy(data+i," ",1);
+      }
       num_items_scanned =
 	sscanf (data,
-		"%6d   %1s   %6d  %d:%d:%d.%d               %12s  %12s  ETHV2   Type: %s",
+		"%6d   %1s   %6d  %d:%d:%d.%d               %12s  %12s  ETHV2   Type: %4s",
 		&pktnum, direction, &cap_len, &hr, &min, &sec, &csec, destmac,
 		srcmac, type);
       if (num_items_scanned == 10)
@@ -536,7 +544,7 @@ iseries_parse_packet (wtap * wth, FILE_T fh,
   /*
    * If we have Wiretap Header then populate it here
    *
-   * XXX - Timer resolution on the iSeries is hardware dependant, the value for csec may be
+   * XXX - Timer resolution on the iSeries is hardware dependant; the value for csec may be
    * different on other platforms though all the traces I've seen seem to show resolution
    * to Milliseconds (i.e HH:MM:SS.nnnnn) or Nanoseconds (i.e HH:MM:SS.nnnnnn)
    */
@@ -607,7 +615,7 @@ iseries_parse_packet (wtap * wth, FILE_T fh,
       /* Convert UNICODE data to ASCII and determine line length */
       if (wth->capture.iseries->format == ISERIES_FORMAT_UNICODE)
 	{
-	  buflen = iseries_UNICODE_to_ASCII (data, ISERIES_LINE_LENGTH);
+         buflen = iseries_UNICODE_to_ASCII ((guint8 *)data, ISERIES_LINE_LENGTH);
 	}
       else
 	{
@@ -738,11 +746,18 @@ iseries_parse_packet (wtap * wth, FILE_T fh,
     }
 
   /*
-   * Extract the packet length from the actual IP header, this may differ from the capture length
-   * reported by the formatted trace
+   * Extract the packet length from the actual IP header; this may
+   * differ from the capture length reported by the formatted trace.
+   * Note: if the entire Ethernet packet is present, but the IP
+   * packet is less than 46 bytes long, there will be padding, and
+   * the length in the IP header won't include the padding; if
+   * the packet length is less than the captured length, set the
+   * packet length to the captured length.
    */
   num_items_scanned = sscanf (asciibuf + 32, "%4x", &pkt_len);
   wth->phdr.len = pkt_len + 14;
+  if (wth->phdr.caplen > wth->phdr.len)
+    wth->phdr.len = wth->phdr.caplen;
 
   /* Make sure we have enough room for the packet, only create buffer if none supplied */
   if (pd == NULL)
@@ -760,9 +775,9 @@ iseries_parse_packet (wtap * wth, FILE_T fh,
 
   /* free buffers allocs and return */
   *err = 0;
-  free (asciibuf);
-  free (tcpdatabuf);
-  free (workbuf);
+  g_free (asciibuf);
+  g_free (tcpdatabuf);
+  g_free (workbuf);
   return wth->phdr.len;
 }
 
